<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods Custom Controllers Controller
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Andrew Palfrey
 * @since: 22/01/2019
 */
class Ctrl extends Ctrl_MVC_Admin_Controller {

	const PAGE_SLUG = 'simpods-custom-controllers';
	const DB_TABLE   = 'simpods_custom_controllers';

	public $main_form_items    = array();
	public $main_form_settings = array();
	public $add_form_items     = array();
	public $add_form_settings  = array();
	public $edit_form_items    = array();
	public $edit_form_settings = array();

	public $notice = '';
	public $wp_errors  = false;

	public function __construct() {

		$this->actions();
		$this->filters();

	}

	/**
	 * actions_fn: WP actions to globally hook when the admin controller is loaded
	 */
	public function actions() {
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );
	}

	/**
	 * filters_fn: WP filters to globally hook when the admin controller is loaded
	 */
	public function filters() {}

	/**
	 * enqueue_fn: Scripts and styles to globally enqueue when the admin controller is loaded
	 */
	public function enqueue() {}

	/**
	 * Sets up & processes all custom controller forms
	 *
	 * @return null
	 */
	public function forms() {
		$this->update_settings();

		$this->main_form_items = array(
			'spcc_disable'            => array(
				'type'    => 'toggle',
				'title'   => esc_html__( 'Disable Custom Controllers', 'simpods-mvc' ),
				'value'   => get_option( 'spcc_disable' ),
				'options' => array(
					'required' => false,
				),
			),
			'spcc_admin_show_ctrl'    => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Show Admin Controller Path', 'simpods-mvc' ),
				'value'   => get_option( 'spcc_admin_show_ctrl' ),
				'choices' => array(
					'0' => esc_html__( 'No', 'simpods-mvc' ),
					'1' => esc_html__( 'Yes', 'simpods-mvc' ),
					'2' => esc_html__( 'Yes - Admins Only', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
			),
			'spcc_frontend_show_ctrl' => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Show Frontend Controller Path', 'simpods-mvc' ),
				'value'   => get_option( 'spcc_frontend_show_ctrl' ),
				'choices' => array(
					'0' => esc_html__( 'No', 'simpods-mvc' ),
					'1' => esc_html__( 'Yes', 'simpods-mvc' ),
					'2' => esc_html__( 'Yes - Admins Only', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
			),
		);

		$this->main_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-cc-settings-submit',
				'label' => esc_html__( 'Update Settings', 'simpods-mvc' ),
			),
			'form' => array(
				'id'     => 'simpods-cc-settings-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'update_settings',
			'action' => 'update_settings',
		);

		$this->add_form_items = array(
			'spcc_where'           => array(
				'type'     => 'select',
				'title'    => esc_html__( 'Area', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Where would you like the controller to be used?', 'simpods-mvc' ),
				'choices'  => array(
					'frontend' => esc_html__( 'Frontend', 'simpods-mvc' ),
					'admin'    => esc_html__( 'Admin', 'simpods-mvc' ),
				),
				'options'  => array(
					'required' => true,
				),
				'errors'   => array(
					'error'   => esc_html__( 'You must select an area for this controller.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'You must select a valid controller area.', 'simpods-mvc' ),
				),
			),
			'spcc_look_up'         => array(
				'type'     => 'text',
				'title'    => esc_html__( 'URL Path', 'simpods-mvc' ),
				'subtitle' => sprintf(
					/* translators: %1$s: Example URL, %2$s: Example input, $3$s: Example with wildcard, %4$s: Example with multiple wildcards */
					esc_html__( 'The URL path to apply the controller to. e.g for %1$s, enter %2$s or use wild card like %3$s or %4$s. Wild cards have lower priority.', 'simpods-mvc' ),
					'<code>[domain]/events/list/</code>',
					'<code>events/list</code>',
					'<code>events/*</code>',
					'<code>events/*/*</code>'
				),
				'options'  => array(
					'required' => true,
				),
				'errors'   => array(
					'error' => esc_html__( 'You must enter a URL path for this controller.', 'simpods-mvc' ),
				),
			),
			'spcc_controller_path' => array(
				'type'     => 'text',
				'title'    => esc_html__( 'Controller Path', 'simpods-mvc' ),
				'subtitle' => sprintf(
					/* translators: Example filepath, %2$s: Example input */
					esc_html__( 'The path of the controller file. e.g for %1$s, enter %2$s.', 'simpods-mvc' ),
					'<code>[theme_directory/mvc/control]/events/template-event.php</code>',
					'<code>events/template-event.php</code>'
				),
				'options'  => array(
					'required' => true,
				),
				'errors'   => array(
					'error' => esc_html__( 'You must enter the path of the controller file.', 'simpods-mvc' ),
				),
			),
			'spcc_note'            => array(
				'type'  => 'textarea',
				'title' => esc_html__( 'Note', 'simpods-mvc' ),
			),
		);

		$this->add_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-cc-add-submit',
				'label' => esc_html__( 'Create Controller', 'simpods-mvc' ),
			),
			'form' => array(
				'id'     => 'simpods-cc-add-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'add_new_entry',
			'action' => 'add_new_entry',
		);

		$this->edit_form_items = array(
			'spcc_where'           => array(
				'type'     => 'select',
				'title'    => esc_html__( 'Area', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Where would you like the controller to be used?', 'simpods-mvc' ),
				'choices'  => array(
					'frontend' => esc_html__( 'Frontend', 'simpods-mvc' ),
					'admin'    => esc_html__( 'Admin', 'simpods-mvc' ),
				),
				'options'  => array(
					'required' => true,
					'id'       => 'edit-spcc_where',
				),
				'errors'   => array(
					'error'   => esc_html__( 'You must select an area for this controller.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'You must select a valid controller area.', 'simpods-mvc' ),
				),
			),
			'spcc_look_up'         => array(
				'type'     => 'text',
				'title'    => esc_html__( 'URL Path', 'simpods-mvc' ),
				'subtitle' => sprintf(
					/* translators: %1$s: Example URL, %2$s: Example input, $3$s: Example with wildcard, %4$s: Example with multiple wildcards */
					esc_html__( 'The URL path to apply the controller to. e.g for %1$s, enter %2$s or use wild card like %3$s or %4$s. Wild cards have lower priority.', 'simpods-mvc' ),
					'<code>[domain]/events/list/</code>',
					'<code>events/list</code>',
					'<code>events/*</code>',
					'<code>events/*/*</code>'
				),
				'options'  => array(
					'required' => true,
					'id'       => 'edit-spcc_look_up',
				),
				'errors'   => array(
					'error' => esc_html__( 'You must enter a URL path for this controller.', 'simpods-mvc' ),
				),
			),
			'spcc_controller_path' => array(
				'type'     => 'text',
				'title'    => esc_html__( 'Controller Path', 'simpods-mvc' ),
				'subtitle' => sprintf(
					/* translators: Example filepath, %2$s: Example input */
					esc_html__( 'The path of the controller file. e.g for %1$s, enter %2$s.', 'simpods-mvc' ),
					'<code>[theme_directory/mvc/control]/events/template-event.php</code>',
					'<code>events/template-event.php</code>'
				),
				'options'  => array(
					'required' => true,
					'id'       => 'edit-spcc_controller_path',
				),
				'errors'   => array(
					'error' => esc_html__( 'You must enter the path of the controller file.', 'simpods-mvc' ),
				),
			),
			'spcc_note'            => array(
				'type'    => 'textarea',
				'title'   => esc_html__( 'Note', 'simpods-mvc' ),
				'options' => array(
					'id' => 'edit-spcc_note',
				),
			),
			'id'                   => array(
				'type'    => 'number',
				'title'   => esc_html__( 'ID', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'readonly' => true,
					'id'       => 'edit-id',
				),
				'errors'  => array(
					'error' => esc_html__( 'There should be an ID in here...', 'simpods-mvc' ),
				),
			),
		);

		$this->edit_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'form'   => false,
			'submit' => false,
			'nonce'  => 'edit_entry',
			'action' => 'edit_entry',
		);

		$this->add_controller();
		$this->edit_controller();
	}

	/**
	 * Updates the settings for the custom controller system
	 *
	 * @return null
	 */
	private function update_settings() {
		global $funs, $cross_admin, $current_user;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'update_settings' ) {
			return;
		}

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'update_settings' ) ) {
			$message 		= esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     	= 'error';
			$this->notice  .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );

		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );

		if ( !isset( $sanitized_data['spcc_disable'] ) ) {
			$sanitized_data['spcc_disable'] = '0';
		}

		$successful = false;

		foreach ( $sanitized_data as $key => $value ) {

			if ( strpos( $key, 'spcc_' ) === 0 ) {
				$is_done = update_option( esc_sql( $key ) , $value );

				if ( $is_done ) {
					$successful = true;
				}

			}

		}

		// if one works, we can assume that both work
		$message = '';
		$css_class     = '';

		if ( $successful ) {
			$message = esc_html__( 'Settings Updated.', 'simpods-mvc' );
			$css_class     = 'success';
		} else {
			$message = esc_html__( 'Settings Not Changed.', 'simpods-mvc' );
			$css_class     = 'error';
		}

		$this->notice .= $cross_admin->notice( $message, $css_class );
	}

	/**
	 * Adds new controllers into the system
	 *
	 * @return null
	 */
	private function add_controller() {
		global $funs, $cross_admin, $current_user;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'add_new_entry' ) {
			return;
		}

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'add_new_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$wp_errors = new WP_Error();

		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );

		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );

		$defaults = array(			
			'spcc_where'   => 'frontend',
			'spcc_look_up' => '',
			'spcc_note'    => '',
		);
	
		$sanitized_data = wp_parse_args( $sanitized_data, $defaults );

		foreach ( $sanitized_data as $key => $value ) {
			if ( $key == 'spcc_note' ) {
				continue;
			}

			if ( $value === '' ) {

				$wp_errors->add( $key . '_error', esc_html__( 'This field is required.', 'simpods-mvc' ), $key . '-error' );
			
			}
		}

		if ( !in_array( $sanitized_data['spcc_where'], array( 'frontend', 'admin' ) ) ) {
			$wp_errors->add( 'spcc_where_invalid', esc_html__( 'This is not a valid option.', 'simpods-mvc' ), 'spcc_where-invalid' );
		}

		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;

			$message = esc_html__( 'Unable to create custom controller.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$now_str = date( 'Y-m-d H:i:s' );

		$sanitized_data['spcc_author']          = $current_user->ID;
		$sanitized_data['spcc_created']         = $now_str;
		$sanitized_data['spcc_modified']        = $now_str;
		$sanitized_data['spcc_modified_author'] = $current_user->ID;

		$successful = $funs->general_insert( self::DB_TABLE, $sanitized_data );

		$message = '';
		$css_class     = '';

		if ( $successful ) {
			$message = esc_html__( 'Controller Added', 'simpods-mvc' );
			$css_class     = 'success';
		} else {
			$message = esc_html__( 'Controller Not Added', 'simpods-mvc' );
			$css_class     = 'error';
		}

		$this->notice .= $cross_admin->notice( $message, $css_class );
	}

	/**
	 * Updates controllers within the system
	 *
	 * @return null
	 */
	private function edit_controller() {
		global $funs, $cross_admin, $current_user;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'edit_entry' ) {
			return;
		}

		$wp_errors = new WP_Error();

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'edit_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );

		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );

		$sanitized_data['id'] = intval( $sanitized_data['id'] );

		$defaults = array(
			'id'           => 0,
			'spcc_where'   => 'frontend',
			'spcc_look_up' => '',
			'spcc_note'    => '',
		);

		$sanitized_data = wp_parse_args( $sanitized_data, $defaults );

		foreach ( $sanitized_data as $key => $value ) {
			if ( $key == 'spcc_note' ) {
				continue;
			}

			if ( $value == '' || $value === 0 ) {
				$wp_errors->add( $key . '_error', esc_html__( 'This field is required.', 'simpods-mvc' ), $key . '-error' );
			}
		}

		if ( !in_array( $sanitized_data['spcc_where'], array( 'frontend', 'admin' ) ) ) {
			$wp_errors->add( 'spcc_where_invalid', esc_html__( 'This is not a valid option.', 'simpods-mvc' ), 'spcc_where-invalid' );
		}

		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;

			$message = esc_html__( 'Unable to edit custom controller.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$sanitized_data['spcc_modified']        = date( 'Y-m-d H:i:s' );
		$sanitized_data['spcc_modified_author'] = $current_user->ID;

		$id_int = $sanitized_data['id'];
		unset( $sanitized_data['id'] );

		$funs->general_update( self::DB_TABLE, $sanitized_data, array( 'id' => $id_int ) );

		$message = esc_html__( 'Controller Edited', 'simpods-mvc' );
		$css_class     = 'success';

		$this->notice .= $cross_admin->notice( $message, $css_class );
	}

}
