<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods Guest Accounts Controller
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Andrew Palfrey
 * @since: 17/01/2019
 */
class Ctrl extends Ctrl_MVC_Admin_Controller {

	const PAGE_SLUG = 'simpods-guest-accounts';
	const DB_TABLE   = 'simpods_guest_accounts';

	public $add_form_items     = array();
	public $add_form_settings  = array();
	public $edit_form_items    = array();
	public $edit_form_settings = array();

	public $notice = '';
	public $wp_errors  = false;

	public function __construct() {

		$this->actions();
		$this->filters();

	}

	/**
	 * actions_fn: WP actions to globally hook when the admin controller is loaded
	 */
	public function actions() {
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );
	}

	/**
	 * filters_fn: WP filters to globally hook when the admin controller is loaded
	 */
	public function filters() {}

	/**
	 * enqueue_fn: Scripts and styles to globally enqueue when the admin controller is loaded
	 */
	public function enqueue() {}

	/**
	 * Sets up & processes all guest accounts forms
	 *
	 * @return null
	 */
	public function forms() {
		$this->add_form_items = array(
			'how_many'         => array(
				'type'     => 'number',
				'title'    => esc_html__( 'No. of Users', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'The number of users to add.', 'simpods-mvc' ),
				'options'  => array(
					'required' => true,
					'min'      => 1,
				),
				'errors'   => array(
					'error'   => esc_html__( 'You must enter how many accounts to create.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'You must enter a valid number of accounts to create.', 'simpods-mvc' ),
					'min-max' => esc_html__( 'You must enter a valid number of accounts to create.', 'simpods-mvc' ),
				),
			),
			'spga_user_name'   => array(
				'type'     => 'text',
				'title'    => esc_html__( 'Username', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'The username to create the accounts, e.g. "user". Numbers will be added on the end to make each unique, e.g. "user_1"', 'simpods-mvc' ),
				'options'  => array(
					'required' => true,
				),
				'errors'   => array(
					'error' => esc_html__( 'You must enter the base username format.', 'simpods-mvc' ),
				),
			),
			'spga_expiry_date' => array(
				'type'    => 'date',
				'title'   => esc_html__( 'Expiry Date', 'simpods-mvc' ),
				'options' => array(
					'required' => false,
					'format'   => get_option( 'date_format' ),
				),
				'errors'  => array(
					'invalid' => esc_html__( 'You must enter a valid expiry date.', 'simpods-mvc' ),
				),
			),
			'password'         => array(
				'type'     => 'text',
				'title'    => esc_html__( 'Generic Password', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Don\'t forget to make a note of this password.', 'simpods-mvc' ),
				'options'  => array(
					'required' => true,
				),
				'errors'   => array(
					'error' => esc_html__( 'You must enter a password for the accounts.', 'simpods-mvc' ),
				),
			),
			'spga_role'        => array(
				'type'    => 'role',
				'title'   => esc_html__( 'Role', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'style'    => 'select',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a user role.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'You must select a valid user role.', 'simpods-mvc' ),
				),
			),
		);

		$this->add_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-ga-add-submit',
				'label' => esc_html__( 'Create Guest Users', 'simpods-mvc' ),
			),
			'form' => array(
				'id'     => 'simpods-ga-add-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'add_new_entry',
			'action' => 'add_new_entry',
		);

		$this->edit_form_items = array(
			'spga_user_id'     => array(
				'type'    => 'number',
				'title'   => esc_html__( 'User ID', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'readonly' => true,
					'id'       => 'edit-spga_user_id',
				),
				'errors'  => array(
					'error' => esc_html__( 'There should be an ID in here...', 'simpods-mvc' ),
				),
			),
			'spga_expiry_date' => array(
				'type'    => 'date',
				'title'   => esc_html__( 'Expiry Date', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => false,
					'id'       => 'edit-spga_expiry_date',
					'format'   => get_option( 'date_format' ),
				),
				'errors'  => array(
					'invalid' => esc_html__( 'You must enter a valid expiry date.', 'simpods-mvc' ),
				),
			),
			'spga_role'        => array(
				'type'    => 'role',
				'title'   => esc_html__( 'Role', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'id'       => 'edit-spga_role',
					'style'    => 'select',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a user role.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'You must select a valid user role.', 'simpods-mvc' ),
				),
			),
			'id'               => array(
				'type'    => 'number',
				'title'   => esc_html__( 'ID', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'readonly' => true,
					'id'       => 'edit-id',
				),
				'errors'  => array(
					'error' => esc_html__( 'There should be an ID in here...', 'simpods-mvc' ),
				),
			),
		);

		$this->edit_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => false,
			'form'   => false,
			'nonce'  => 'edit_entry',
			'action' => 'edit_entry',
		);

		$this->add_guest_accounts();
		$this->edit_guest_account();
		$this->remove_guest_account();
	}

	/**
	 * Adds new guest accounts into the system
	 *
	 * @return null
	 */
	private function add_guest_accounts() {
		global $funs, $cross_admin, $current_user;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'add_new_entry' ) {
			return;
		}

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'add_new_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );

		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );

		if ( isset( $sanitized_data['spga_expiry_date'] ) && $sanitized_data['spga_expiry_date'] == '' ) {
			$sanitized_data['spga_expiry_date'] = '0000-00-00';
		}

		$wp_errors = new WP_Error();

		foreach ( $sanitized_data as $key => $value ) {
			if ( $value == '' && $key !== 'spga_expiry_date' ) {
				$wp_errors->add( $key . '_error', esc_html__( 'This field is required', 'simpods-mvc' ), $key . '-error' );
			}
		}

		if ( $sanitized_data['how_many'] !== '' && !is_numeric( $sanitized_data['how_many'] ) ) {
			$wp_errors->add( 'how_many_invalid', esc_html__( 'You must enter a valid number of users to create', 'simpods-mvc' ), 'how_many-invalid' );
		}

		if ( $sanitized_data['spga_expiry_date'] !== '0000-00-00' ) {
			$format = $this->add_form_items['spga_expiry_date']['options']['format'];

			if ( !$funs->validate_date( $sanitized_data['spga_expiry_date'], $format ) ) {
				$wp_errors->add( 'spga_expiry_date_invalid', esc_html__( 'This is not a valid date.', 'simpods-mvc' ), 'spga_expiry_date-invalid' );
			}

		}

		if ( $sanitized_data['spga_role'] !== '' && !array_key_exists( $sanitized_data['spga_role'], get_editable_roles() ) ) {
			$wp_errors->add( 'spga_role_invalid', esc_html__( 'You must select a valid user role', 'simpods-mvc' ), 'spga_role-invalid' );
		}

		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;

			$message = esc_html__( 'Unable to create guest accounts', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		if ( $sanitized_data['spga_expiry_date'] !== '0000-00-00' ) {
			$sanitized_data['spga_expiry_date'] = $funs->format_date( $sanitized_data['spga_expiry_date'], 'Y-m-d', $format );
		}

					

		$created_count = 0;
		$failed_count  = 0;

		for ( $i = 0; $i < intval( $sanitized_data['how_many'] ); $i++ ) {
			$user_name = $funs->unique_username( $sanitized_data['spga_user_name'] );

			$attributes = array(
				'user_login' => $user_name,
				'user_email' => $user_name . '@' . $user_name . 'com',
				'user_pass'  => $sanitized_data['password'],
				'role'       => $sanitized_data['spga_role'],
			);

			$user_id = wp_insert_user( $attributes );

			if ( is_numeric( $user_id ) ) {
				$created_count++;

				$now = date( 'Y-m-d H:i:s' );

				$data = array(
					'spga_user_id'         => $user_id,
					'spga_user_name'       => $user_name,
					'spga_expiry_date'     => $sanitized_data['spga_expiry_date'],
					'spga_last_login'      => '0000-00-00',
					'spga_role'            => $sanitized_data['spga_role'],
					'spga_created'         => $now,
					'spga_modified'        => $now,
					'spga_modified_author' => $current_user->ID,
					'spga_author'          => $current_user->ID,
					'spga_status'          => 'publish',
				);

				$funs->general_insert( self::DB_TABLE, $data );
			} else {
				$failed_count++;
			}

		}

		$message 	= '';
		$css_class  = '';

		if ( $created_count > 0 ) {
			$message .= esc_html( sprintf(
				/* translators: %d: number of accounts */
				_n(
					'%d account created successfully.',
					'%d accounts created successfully.',
					$created_count,
					'simpods-mvc'
				),
				$created_count
			) );
			$css_class     = 'success';
		}

		if ( $failed_count !== 0 ) {
			$message .= ' ' . esc_html( sprintf(
				/* translators: %d: number of accounts */
				_n(
					'Failed to create %d account.',
					'Failed to create %d accounts.',
					$failed_count,
					'simpods-mvc'
				),
				$failed_count
			) );
			$css_class     = 'error';
		}

		if ( $created_count > 0 && $failed_count > 0 ) {
			$css_class = 'warning';
		}

		$this->notice .= $cross_admin->notice( $message, $css_class );
		return true;
	}

	/**
	 * Updates guest accounts within the system
	 *
	 * @return null
	 */
	private function edit_guest_account() {
		global $funs, $cross_admin, $current_user;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'edit_entry' ) {
			return;
		}

		$wp_errors = new WP_Error();

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'edit_entry' ) ) {
			$wp_errors->add( 'invalid_nonce', esc_html__( 'There was an issue submitting your request. Please refresh the page and try again.', 'simpods-mvc' ), 'form-error' );
			$this->wp_errors = $wp_errors;

			$message 		= esc_html__( 'There was an issue submitting your request. Please refresh the page and try again.', 'simpods-mvc' );
			$css_class      = 'error';
			$this->notice  .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$post_data = $_POST;
		unset( $post_data['action'] );
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );

		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );

		$defaults = array(
			'id'               => 0,
			'spga_user_id'     => 0,
			'spga_role'        => '',
			'spga_expiry_date' => '0000-00-00',
		);

		$sanitized_data = wp_parse_args( $sanitized_data, $defaults );

		$sanitized_data['id']           = intval( $sanitized_data['id'] );
		$sanitized_data['spga_user_id'] = intval( $sanitized_data['spga_user_id'] );

		if ( isset( $sanitized_data['spga_expiry_date'] ) && $sanitized_data['spga_expiry_date'] == '' ) {
			$sanitized_data['spga_expiry_date'] = '0000-00-00';
		}

		foreach ( $sanitized_data as $key => $value ) {

			if ( $key == 'spga_expiry_date' ) {
				continue;
			}

			if ( $value == '' || $value === 0 ) {
				$wp_errors->add( $key . '_error', esc_html__( 'This field is required.', 'simpods-mvc' ), $key . '-error' );
			}

		}

		if ( $sanitized_data['spga_expiry_date'] !== '0000-00-00' ) {
			$format = $this->edit_form_items['spga_expiry_date']['options']['format'];

			if ( !$funs->validate_date( $sanitized_data['spga_expiry_date'], $format ) ) {
				$wp_errors->add( 'spga_expiry_date_invalid', esc_html__( 'This is not a valid date.', 'simpods-mvc' ), 'spga_expiry_date-invalid' );
			}

		}

		if ( $sanitized_data['spga_role'] !== '' ) {
			global $wp_roles;
			$roles    	= array();
			$all_roles 	= $wp_roles->roles;

			foreach ( $all_roles as $key => $info_arr ) {

				if ( $key == 'administrator' ) {
					continue;
				}

				$roles[] = $key;
			}

			if ( !in_array( $sanitized_data['spga_role'], $roles ) ) {
				$wp_errors->add( 'spga_role_invalid', esc_html__( 'This is not a valid role.', 'simpods-mvc' ), 'spga_role-invalid' );
			}

		}

		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;

			$message = esc_html__( 'Unable to edit guest accounts', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		if ( $sanitized_data['spga_expiry_date'] !== '0000-00-00' ) {
			$sanitized_data['spga_expiry_date'] = $funs->format_date( $sanitized_data['spga_expiry_date'], 'Y-m-d', $format );
		}

		$id      = $sanitized_data['id'];
		$user_id = $sanitized_data['spga_user_id'];

		unset( $sanitized_data['id'] );
		unset( $sanitized_data['spga_user_id'] );

		$user_obj = get_userdata( $user_id );

		if ( $user_obj === false ) {
			$message = esc_html__( 'This user does not exist.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$sanitized_data['spga_modified']        = date( 'Y-m-d H:i:s' );
		$sanitized_data['spga_modified_author'] = $current_user->ID;

		$funs->general_update( self::DB_TABLE, $sanitized_data, array( 'id' => $id ) );

		wp_update_user( array( 'ID' => $sanitized_data['spga_user_id'], 'role' => $sanitized_data['spga_role'] ) );

		$message = esc_html__( 'Guest account updated', 'simpods-mvc' );
		$css_class     = 'success';
		$this->notice .= $cross_admin->notice( $message, $css_class );
		return true;
	}

	/**
	 * Removes guest accounts within the system
	 *
	 * @return null
	 */
	private function remove_guest_account() {
		global $funs, $cross_admin, $current_user;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'remove_entry' ) {
			return;
		}

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'remove_entry' ) ) {
			return false;
		}

		if ( !is_array( $_POST['remove'] ) || count( $_POST['remove'] ) <= 0 ) {
			return false;
		}

		$removes = $funs->escape_attrs( $_POST['remove'] );

		foreach ( $removes as $item ) {

			if ( !is_numeric( $item ) ) {
				continue;
			}

			$results = $funs->simpods_select( array( 'main_tb.`id`' => $item ), array( 'target_tb' => self::DB_TABLE ) );

			if ( !is_array( $results ) || empty( $results ) ) {
				continue;
			}

			$results = reset( $results );

			wp_delete_user( intval( $results['spga_user_id'] ) );

		}

	}

}