<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods area categories controller
 *
 * @version: 1.1.4
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/05/2014
 */
class Ctrl extends Ctrl_MVC_Admin_Controller {

	const PAGE_SLUG = 'simpods-hierarchy';
	const DB_TABLE   = 'simpods_area_categories';

	public $add_form_items     = array();
	public $add_form_settings  = array();
	public $edit_form_items    = array();
	public $edit_form_settings = array();

	public $notice = '';
	public $wp_errors  = false;

	public $custom_fields = array(
		'sp_categories' => array( 'type' => 'text', 'settings' => '', 'default' => '' ),
		'sp_order'      => array( 'type' => 'text', 'settings' => '', 'default' => '' ),
		'sp_granditem'  => array( 'type' => 'text', 'settings' => '', 'default' => '' ), // overwrite granditem from the settings
	);

	public function __construct() {

		$this->actions();
		$this->filters();

	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {
		global $cross_admin;
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ), 11 );
	}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}

	/**
	 * Scripts & styles to register & enqueue when the admin controller is loaded
	 * @return null
	 */
	public function enqueue() {
		global $funs;
		// Prepare Ajax
		wp_localize_script(
			'simpods-hierarchy',
			'simpods_hierarchy_ajax',
			array(
				'ajaxurl' => admin_url( 'admin-ajax.php' ),
				'nonce'   => wp_create_nonce( 'simpods_hierarchy' ),
			)
		);

		wp_localize_script( 'simpods-hierarchy', 'sh_l10n', array(
			'child_table'  => esc_html__( 'Child Table Not Set', 'simpods-mvc' ),
			'parent_table' => esc_html__( 'Parent Table Not Set', 'simpods-mvc' ),
			'grandparent'  => esc_html__( 'No Grandparents', 'simpods-mvc' ),
			'select'       => esc_html__( 'Please Select One...', 'simpods-mvc' ),
			'all'          => esc_html__( 'All', 'simpods-mvc' ),
		) );

		$tables = array();

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {
			$values['type'] = str_replace( '_', ' ', $values['type'] );

			$tables[ $key ] = ucfirst( $values['type'] ) . ': ' . $values['title'];
		}

		asort( $tables );

		wp_localize_script( 'simpods-hierarchy', 'tables', $tables );

		$areas = $funs->simpods_select( array(), array( 'target_tb' => 'simpods_area_categories', 'order' => 'DESC' ) );
		wp_localize_script( 'simpods-hierarchy', 'areas', $areas );
	}

	/**
	 * Sets up & processes all hierarchies forms
	 *
	 * @return null
	 */
	public function forms() {
		$this->add_form_items = array(
			'sp_child'     => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Child Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Please Select One...', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a child table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid child table.', 'simpods-mvc' ),
				),
			),
			'sp_parent'    => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Parent Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Child Table Not Set', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					'disabled' => true,
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a parent table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid parent table.', 'simpods-mvc' ),
				),
			),
			'sp_granditem' => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Grandparent Item', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Parent Table Not Set', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => false,
					'disabled' => true,
				),
			),
		);

		$this->add_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-sph-add-submit',
				'label' => esc_html__( 'Add Hierarchy', 'simpods-mvc' ),
			),
			'form'   => array(
				'id'     => 'simpods-sph-add-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'add_new_entry',
			'action' => 'add_new_entry',
		);

		$this->edit_form_items = array(
			'sp_child'   => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Child Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Please Select One...', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					'id'       => 'edit-sp_child',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a main table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid main table.', 'simpods-mvc' ),
				),
			),
			'sp_parent' => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Parent Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Child Table Not Set', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					// 'disabled' => true,
					'id'       => 'edit-sp_parent',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a friend table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid friend table.', 'simpods-mvc' ),
				),
			),
			'sp_granditem'  => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Grandparent Item', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Parent Table Not Set', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => false,
					// 'disabled' => true,
					'id'       => 'edit-sp_granditem',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a parent table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid grandparent item.', 'simpods-mvc' ),
				),
			),
			'id'         => array(
				'type'    => 'number',
				'title'   => esc_html__( 'ID', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'readonly' => true,
					'id'       => 'edit-id',
				),
				'errors'  => array(
					'error' => esc_html__( 'There should be an ID in here...', 'simpods-mvc' ),
				),
			),
		);

		$this->edit_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => false,
			'form'   => false,
			'nonce'  => 'edit_entry',
			'action' => 'edit_entry',
		);

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {
			$values['type'] = str_replace( '_', ' ', $values['type'] );

			$this->add_form_items['sp_child']['choices'][ $key ] = ucfirst( $values['type'] ) . ': ' . $values['title'];
			$this->edit_form_items['sp_child']['choices'][ $key ] = ucfirst( $values['type'] ) . ': ' . $values['title'];
		}

		asort( $this->add_form_items['sp_child']['choices'] );
		asort( $this->edit_form_items['sp_child']['choices'] );

		$this->add_hierarchy();
		$this->edit_hierarchy();
	}

	/**
	 * Adds new hierarchies into the system
	 *
	 * @return null
	 */
	private function add_hierarchy() {
		global $funs, $current_user, $cross_admin;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'add_new_entry' ) {
			return;
		}

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'add_new_entry' ) ) {
			$message 		= esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     	= 'error';
			$this->notice  .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$tables = MVC_Admin_Factory::$act_pod_tables;
		$validate_data = $this->validate();

		if ( !$validate_data ) {
			return false;
		}

		$sanitized_data = $validate_data;

		$records = $funs->simpods_select(
			array(
				'sp_grandparent' => $sanitized_data['sp_grandparent'],
				'sp_granditem'   => $sanitized_data['sp_granditem'],
				'sp_parent'      => array( $sanitized_data['sp_parent'] ),
				'sp_child'       => array( $sanitized_data['sp_child'] ),
			),
			array(
				'target_tb' => self::DB_TABLE,
				'limit'     => 1,
			)
		);

		if ( $records != NULL && count( $records > 0 ) ) {
			$message = esc_html__( 'Item already exists.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$cross_admin->update_columns( $tables[$sanitized_data['sp_child']]['name'], $this->custom_fields );
						
		$now = date( 'Y-m-d H:i:s' );

		$sanitized_data['sp_author']          = $current_user->ID;
		$sanitized_data['sp_created']         = $now;
		$sanitized_data['sp_modified']        = $now;
		$sanitized_data['sp_modified_author'] = $current_user->ID;

		$funs->general_insert( self::DB_TABLE, $sanitized_data );
		$message = esc_html__( 'Hierarchy Created', 'simpods-mvc' );
		$css_class     = 'success';
		$this->notice .= $cross_admin->notice( $message, $css_class );
		return true;
	}

	/**
	 * Updates hierarchies within the system
	 *
	 * @return null
	 */
	private function edit_hierarchy() {
		global $funs, $current_user, $cross_admin;

		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'edit_entry' ) {
			return;
		}

		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'edit_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$tables = MVC_Admin_Factory::$act_pod_tables;
		$validate_data = $this->validate();

		if ( !$validate_data ) {
			return false;
		}

		$sanitized_data = $validate_data;

		$records = $funs->simpods_select(
			array(
				'sp_grandparent' => $sanitized_data['sp_grandparent'],
				'sp_granditem'   => $sanitized_data['sp_granditem'],
				'sp_parent'      => array( $sanitized_data['sp_parent'] ),
				'sp_child'       => array( $sanitized_data['sp_child'] ),
			),
			array(
				'target_tb' => self::DB_TABLE,
				'limit'     => 1,
			)
		);

		if ( $records != NULL && count( $records > 0 ) && $records[0]['id'] != $sanitized_data['id'] ) {
			$message = esc_html__( 'Item already exists.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		$cross_admin->update_columns( $tables[$sanitized_data['sp_child']]['name'], $this->custom_fields );

		$sanitized_data['sp_modified']        = date( 'Y-m-d H:i:s' );
		$sanitized_data['sp_modified_author'] = $current_user->ID;

		$id_int     = $sanitized_data['id'];
		unset( $sanitized_data['id'] );

		$funs->general_update( self::DB_TABLE, $sanitized_data, array( 'id' => $id_int ) );
		$message = esc_html__( 'Hierarchy Updated', 'simpods-mvc' );
		$css_class     = 'success';
		$this->notice .= $cross_admin->notice( $message, $css_class );
		return true;
	}

	/**
	 * Validates the forms for adding or updating hierarchies
	 *
	 * @return void
	 */
	private function validate() {
		global $funs, $cross_admin;

		$wp_errors = new WP_Error();
		$tables = MVC_Admin_Factory::$act_pod_tables;

		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );

		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );

		if ( !isset( $sanitized_data['sp_granditem'] ) || $sanitized_data['sp_granditem'] == '' ) {
			$sanitized_data['sp_granditem'] = '0';
		}

		if ( !isset( $sanitized_data['sp_input'] ) || $sanitized_data['sp_input'] == '' ) {
			$sanitized_data['sp_input'] = 'radio';
		}

		if ( !isset( $sanitized_data['sp_child'] ) || $sanitized_data['sp_child'] == '' ) {
			$wp_errors->add( 'sp_child_error', esc_html__( 'You must select a child table', 'simpods-mvc' ), 'sp_child-error' );
		}

		if ( !isset( $sanitized_data['sp_parent'] ) || $sanitized_data['sp_parent'] == '' ) {
			$wp_errors->add( 'sp_parent_error', esc_html__( 'You must select a parent table', 'simpods-mvc' ), 'sp_parent-error' );
		}

		if ( !isset( $tables[$sanitized_data['sp_child']] ) ) {
			$wp_errors->add( 'sp_child_invalid', esc_html__( 'This is not a valid child table', 'simpods-mvc' ), 'sp_child-invalid' );
		}

		if ( !isset( $tables[$sanitized_data['sp_parent']] ) || $sanitized_data['sp_parent'] == $sanitized_data['sp_child'] ) {
			$wp_errors->add( 'sp_parent_invalid', esc_html__( 'This is not a valid parent table', 'simpods-mvc' ), 'sp_parent-invalid' );
		}

		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;

			$message = esc_html__( 'Unable to create a hierarchy', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}

		// get grandparent table, then get its items as categories
		$grandRecord_arr = $funs->simpods_select(
			array(
				'sp_child' => array( $sanitized_data['sp_parent'] )
			),
			array(
				'target_tb' => self::DB_TABLE,
				'limit' => 1
			)
		);

		$sanitized_data['sp_grandparent'] = '';

		if ( $grandRecord_arr != NULL && count( $grandRecord_arr ) > 0 ) {
			$sanitized_data['sp_grandparent'] = $grandRecord_arr[0]['sp_parent'];
		}

		return $sanitized_data;
	}

}
