<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods Linked Areas controller
 *
 * @version: 1.1.4
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/05/2014
 */
class Ctrl extends Ctrl_MVC_Admin_Controller {
	const PAGE_SLUG = 'simpods-linked-areas';
	const DB_TABLE   = 'simpods_linked_areas';
	public $add_form_items     = array();
	public $add_form_settings  = array();
	public $edit_form_items    = array();
	public $edit_form_settings = array();
	public $notice = '';
	public $wp_errors  = '';
	public $custom_fields = array(
		'sp_linked_areas' => array( 'type' => 'text', 'settings' => '', 'default' => '' ),
	);
	public function __construct() {
		$this->actions();
		$this->filters();
	}
	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {
		global $cross_admin;
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ), 11 );
	}
	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}
	/**
	 * Scripts & styles to register & enqueue when the admin controller is loaded
	 * @return null
	 */
	public function enqueue() {
		global $funs;
		// Prepare Ajax
		wp_localize_script(
			'simpods-linked-areas',
			'simpods_linked_areas_ajax',
			array(
				'ajaxurl' => admin_url( 'admin-ajax.php' ),
				'nonce'   => wp_create_nonce( 'simpods_linked_areas' ),
			)
		);
		wp_localize_script( 'simpods-linked-areas', 'sh_l10n', array(
			'select'     => esc_html__( 'Please Select One...', 'simpods-mvc' ),
			'main_table' => esc_html__( 'Main Table Not Set', 'simpods-mvc' ),
		) );
		$tables = array();
		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {
			$values['type'] = str_replace( '_', ' ', $values['type'] );
			$tables[ $key ] = ucfirst( $values['type'] ) . ': ' . $values['title'];
		}
		asort( $tables );
		wp_localize_script( 'simpods-linked-areas', 'tables', $tables );
		$areas = $funs->simpods_select( array(), array( 'target_tb' => 'simpods_linked_areas', 'order' => 'DESC' ) );
		wp_localize_script( 'simpods-linked-areas', 'areas', $areas );
	}
	/**
	 * Sets up & processes all linked areas forms
	 *
	 * @return null
	 */
	public function forms() {
		$this->add_form_items = array(
			'spl_main'   => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Main Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Please Select One...', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a main table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid main table.', 'simpods-mvc' ),
				),
			),
			'spl_friend' => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Friend Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Main Table Not Set', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					'disabled' => true,
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a friend table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid friend table.', 'simpods-mvc' ),
				),
			),
			'spl_input'  => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Relationship Type', 'simpods-mvc' ),
				'choices' => array(
					'checkbox' => esc_html__( 'Many to many', 'simpods-mvc' ),
					'radio'    => esc_html__( 'One to many', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a relationship type.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid relationship type.', 'simpods-mvc' ),
				),
			),
			'spl_admin_only'  => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Admin Only', 'simpods-mvc' ),
				'choices' => array(
					'0' 	=> esc_html__( 'No', 'simpods-mvc' ),
					'1'    	=> esc_html__( 'Yes', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must choose an option for Admin Only.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid option.', 'simpods-mvc' ),
				),
			),			
		);
		$this->add_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-la-add-submit',
				'label' => esc_html__( 'Add Linked Area', 'simpods-mvc' ),
			),
			'form' => array(
				'id'     => 'simpods-la-add-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'add_new_entry',
			'action' => 'add_new_entry',
		);
		$this->edit_form_items = array(
			'spl_main'   => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Main Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Please Select One...', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					'id'       => 'edit-spl_main',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a main table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid main table.', 'simpods-mvc' ),
				),
			),
			'spl_friend' => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Friend Table', 'simpods-mvc' ),
				'choices' => array(
					'' => esc_html__( 'Main Table Not Set', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					// 'disabled' => true,
					'id'       => 'edit-spl_friend',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a friend table.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid friend table.', 'simpods-mvc' ),
				),
			),
			'spl_input'  => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Relationship Type', 'simpods-mvc' ),
				'choices' => array(
					'checkbox' => esc_html__( 'Many to many', 'simpods-mvc' ),
					'radio'    => esc_html__( 'One to many', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					'id'       => 'edit-spl_input',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must select a relationship type.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid relationship type.', 'simpods-mvc' ),
				),
			),
			'spl_admin_only'  => array(
				'type'    => 'select',
				'title'   => esc_html__( 'Admin Only', 'simpods-mvc' ),
				'choices' => array(
					'0' 	=> esc_html__( 'No', 'simpods-mvc' ),
					'1'    	=> esc_html__( 'Yes', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
					'id'       => 'edit-spl_admin_only',
				),
				'errors'  => array(
					'error'   => esc_html__( 'You must choose an option for Admin Only.', 'simpods-mvc' ),
					'invalid' => esc_html__( 'This is not a valid option.', 'simpods-mvc' ),
				),
			),			
			'id'         => array(
				'type'    => 'number',
				'title'   => esc_html__( 'ID', 'simpods-mvc' ),
				'value'   => '',
				'options' => array(
					'required' => true,
					'readonly' => true,
					'id'       => 'edit-id',
				),
				'errors'  => array(
					'error' => esc_html__( 'There should be an ID in here...', 'simpods-mvc' ),
				),
			),
		);
		$this->edit_form_settings = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => false,
			'form'   => false,
			'nonce'  => 'edit_entry',
			'action' => 'edit_entry',
		);
		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {
			$values['type'] = str_replace( '_', ' ', $values['type'] );
			$this->add_form_items['spl_main']['choices'][ $key ] = ucfirst( $values['type'] ) . ': ' . $values['title'];
			$this->edit_form_items['spl_main']['choices'][ $key ] = ucfirst( $values['type'] ) . ': ' . $values['title'];
		}
		asort( $this->add_form_items['spl_main']['choices'] );
		asort( $this->edit_form_items['spl_main']['choices'] );
		$this->add_linked_area();
		$this->edit_linked_area();
	}
	/**
	 * Adds new linked areas into the system
	 *
	 * @return null
	 */
	private function add_linked_area() {
		global $funs, $cross_admin, $current_user;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'add_new_entry' ) {
			return;
		}
		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'add_new_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}
		$tables = MVC_Admin_Factory::$act_pod_tables;
		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );
		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );
		$wp_errors = new WP_Error();
		// TODO: Validation
		if ( !isset( $sanitized_data['spl_main'] ) || $sanitized_data['spl_main'] == '' ) {
			$wp_errors->add( 'spl_main_error', esc_html__( 'You must select a main table', 'simpods-mvc' ), 'spl_main-error' );
		}
		if ( !isset( $sanitized_data['spl_friend'] ) || $sanitized_data['spl_friend'] == '' ) {
			$wp_errors->add( 'spl_friend_error', esc_html__( 'You must select a friend table', 'simpods-mvc' ), 'spl_friend-error' );
		}
		if ( !isset( $tables[$sanitized_data['spl_main']] ) ) {
			$wp_errors->add( 'spl_main_invalid', esc_html__( 'This is not a valid main table', 'simpods-mvc' ), 'spl_main-invalid' );
		}
		if ( !isset( $tables[$sanitized_data['spl_friend']] ) || $sanitized_data['spl_friend'] == $sanitized_data['spl_main'] ) {
			$wp_errors->add( 'spl_friend_invalid', esc_html__( 'This is not a valid friend table', 'simpods-mvc' ), 'spl_friend-invalid' );
		}
		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;
			$message   = esc_html__( 'Unable to create a linked area', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}
		$records = $funs->simpods_select(
			array(
				'spl_friend' => array( $sanitized_data['spl_friend'] ),
				'spl_main'   => array( $sanitized_data['spl_main'] ),
			),
			array(
				'target_tb' => self::DB_TABLE,
				'limit'     => 1,
			)
		);
		if ( $records != NULL && count( $records > 0 ) ) {
			$message = esc_html__( 'Item already exists.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}
		$cross_admin->update_columns( $tables[$sanitized_data['spl_main']]['name'], $this->custom_fields );
					
		$now = date( 'Y-m-d H:i:s' );
		$sanitized_data['spl_author']          = $current_user->ID;
		$sanitized_data['spl_created']         = $now;
		$sanitized_data['spl_modified']        = $now;
		$sanitized_data['spl_modified_author'] = $current_user->ID;
		$funs->general_insert( self::DB_TABLE, $sanitized_data );
		$message = esc_html__( 'Linked Area Created', 'simpods-mvc' );
		$css_class     = 'success';
		$this->notice .= $cross_admin->notice( $message, $css_class );
		return true;
	}
	/**
	 * Updates linked areas within the system
	 *
	 * @return null
	 */
	private function edit_linked_area() {
		global $funs, $cross_admin, $current_user;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'edit_entry' ) {
			return;
		}
		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'edit_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}
		$tables = MVC_Admin_Factory::$act_pod_tables;
		$post_data = $_POST;
		unset( $post_data['_wpnonce'] );
		unset( $post_data['_wp_http_referer'] );
		unset( $post_data['action'] );
		$sanitized_data = array_map( 'wp_strip_all_tags', $post_data );
		$sanitized_data = $funs->escape( $sanitized_data );
		$sanitized_data = array_map( 'trim', $sanitized_data );
		$sanitized_data['id'] = intval( $sanitized_data['id'] );
		$wp_errors = new WP_Error();
		// TODO: Validation
		if ( !isset( $sanitized_data['spl_main'] ) || $sanitized_data['spl_main'] == '' ) {
			$wp_errors->add( 'spl_main_error', esc_html__( 'You must select a main table', 'simpods-mvc' ), 'spl_main-error' );
		}
		if ( !isset( $sanitized_data['spl_friend'] ) || $sanitized_data['spl_friend'] == '' ) {
			$wp_errors->add( 'spl_friend_error', esc_html__( 'You must select a friend table', 'simpods-mvc' ), 'spl_friend-error' );
		}
		if ( !isset( $tables[$sanitized_data['spl_main']] ) ) {
			$wp_errors->add( 'spl_main_invalid', esc_html__( 'This is not a valid main table', 'simpods-mvc' ), 'spl_main-invalid' );
		}
		if ( !isset( $tables[$sanitized_data['spl_friend']] ) || $sanitized_data['spl_friend'] == $sanitized_data['spl_main'] ) {
			$wp_errors->add( 'spl_friend_invalid', esc_html__( 'This is not a valid friend table', 'simpods-mvc' ), 'spl_friend-invalid' );
		}
		if ( $wp_errors->get_error_code() ) {
			$this->wp_errors = $wp_errors;
			$message = esc_html__( 'Unable to edit a linked area', 'simpods-mvc' );
			$css_class     = 'error';
			$this->notice .= $cross_admin->notice( $message, $css_class );
			return false;
		}
		$records = $funs->simpods_select(
			array(
				'spl_friend' => array( $sanitized_data['spl_friend'] ),
				'spl_main'   => array( $sanitized_data['spl_main'] ),
			),
			array(
				'target_tb' => self::DB_TABLE,
				'limit'     => 1,
			)
		);
		if ( !empty( $records_arr ) ) {
			if ( count( $records_arr ) != 1 || $records_arr[0]['id'] != $sanitized_data['id'] ) {
				$this->wp_errors = $wp_errors;
				$message = esc_html__( 'This Linked Area already exists.', 'simpods-mvc' );
				$css_class     = 'error';
				$this->notice .= $cross_admin->notice( $message, $css_class );
				return false;
			}
		}
		$cross_admin->update_columns( $tables[ $sanitized_data['spl_main'] ]['name'], $this->custom_fields );
		$sanitized_data['spss_modified']        = date( 'Y-m-d H:i:s' );
		$sanitized_data['spss_modified_author'] = $current_user->ID;
		$id = $sanitized_data['id'];
		unset( $sanitized_data['id'] );
		$funs->general_update( self::DB_TABLE, $sanitized_data, array( 'id' => $id ) );
		$message = esc_html__( 'Linked Area Edited', 'simpods-mvc' );
		$css_class     = 'success';
		$this->notice .= $cross_admin->notice( $message, $css_class );
		return true;
	}
}