<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods Settings Controller
 *
 * @package Simpods MVC
 * @author Andrew Palfrey
 * @since 18/01/2019
 */
class Ctrl extends Ctrl_MVC_Admin_Controller {
	const PAGE_SLUG = 'simpods-settings';
	const DB_TABLE   = 'simpods_settings';
	public $defaults 			= array();
	public $form_items 			= array();
	public $form_settings 		= array();
	public $public_post_types 	= array();
	public $wp_notices 			= '';
	public function __construct() {
		$this->option_defaults();
	}
	private function option_defaults() {
		$this->defaults = array(
			'simpods_disable_feeds' 		=> 1,
			'simpods_login_errors' 			=> 1,
			'simpods_wp_version_meta' 		=> 1,
			'simpods_scripts_versions' 		=> 1,
			'simpods_styles_versions' 		=> 1,
			'simpods_remove_dashboard_meta' => 1,
			'simpods_feed_links' 			=> 0,
			'simpods_feed_links_extra' 		=> 0,
			'simpods_rsd_link' 				=> 0,
			'simpods_wlwmanifest_link' 		=> 0,
			'simpods_posts_rel_links' 		=> 0,
			'simpods_xhtml_generator' 		=> 0,
			'simpods_enable_sorting'		=> '',
			'simpods_single_login'			=> '',
			/* 'style_include_bootstrap' => 'bs4',
			'style_include_fontawesome' => 'fa5-js',
			'style_include_match_height' => 1,
			'style_compile_scss' => 0,
			'style_scss_compile_frequency' => 'watch',
			'style_scss_compile_type' => 'Expanded',
			'style_defer_scripts' => 1,
			'style_load_min' => 0, */
		);
	}
	/**
	 * Sets up & processes all simpods settings forms
	 *
	 * @return null
	 */
	public function forms() {
		$this->update_settings();
		global $funs, $wp_roles;
		$db_options = $funs->simpods_select( array( 'main_tb.`sps_option`' => 'simpods_%' ), array( 'target_tb' => self::DB_TABLE ) );
		$options = array();
		foreach ( $db_options as $option ) {
			$option_name  				= $option['sps_option'];
			$option_value 				= $option['sps_value'];
			$options[ $option_name ] 	= $option_value;
		}
		$options = wp_parse_args( $options, $this->defaults );
		$this->public_post_types	=	get_post_types( 
											array( 'public' => true ) 
										);
    	$user_roles = $wp_roles->get_names();
    
		// no need to sort attachement
		unset( $this->public_post_types[ 'attachment' ] );
		$this->form_items['main'] = array(
			'simpods_disable_feeds' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Disable WordPress Feeds', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Disable all WordPress subscription feeds.', 'simpods-mvc' ),
				'value' => $options['simpods_disable_feeds'],
			),
			'simpods_login_errors' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Generic Login Errors', 'simpods-mvc' ),
				'subtitle' => sprintf(
					esc_html__( 'Make all %s login failures return a generic error.', 'simpods-mvc' ),
					'<code>wp-admin</code>'
				),
				'value' => $options['simpods_login_errors'],
			),
			'simpods_wp_version_meta' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Remove WordPress Version Meta Tag', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Remove the meta tag on the front end containing the version of WordPress installed.', 'simpods-mvc' ),
				'value' => $options['simpods_wp_version_meta'],
			),
			'simpods_scripts_versions' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Remove Script Versions', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Remove the version parameter from enqueued scripts.', 'simpods-mvc' ),
				'value' => $options['simpods_scripts_versions'],
			),
			'simpods_styles_versions' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Remove Style Versions', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Remove the version parameter from enqueued scripts.', 'simpods-mvc' ),
				'value' => $options['simpods_styles_versions'],
			),
			'simpods_remove_dashboard_meta' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Remove Dashboard Meta', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Removes some useless modules from the Admin dashboard screen.', 'simpods-mvc' ),
				'value' => $options['simpods_remove_dashboard_meta'],
			),
			'simpods_feed_links' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Feed Links', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Display the links to the general feeds: Post and Comment Feed.', 'simpods-mvc' ),
				'value' => $options['simpods_feed_links'],
			),
			'simpods_feed_links_extra' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Feed Links (extra)', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Display the links to the extra feeds such as category feeds.', 'simpods-mvc' ),
				'value' => $options['simpods_feed_links_extra'],
			),
			'simpods_rsd_link' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Really Simple Discovery Links', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Really Simple Discovery Links', 'simpods-mvc' ),
				'value' => $options['simpods_rsd_link'],
			),
			'simpods_wlwmanifest_link' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Windows Live Writer Manifest', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Display the link to the Windows Live Writer manifest file.', 'simpods-mvc' ),
				'value' => $options['simpods_wlwmanifest_link'],
			),
			'simpods_posts_rel_links' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Post\'s Relational Links', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Display relational links for the posts adjacent to the current post.', 'simpods-mvc' ),
				'value' => $options['simpods_posts_rel_links'],
			),
			'simpods_xhtml_generator' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Display Wordpress XHTML generator', 'simpods-mvc' ),
				'subtitle' => sprintf(
					esc_html__( 'Display the XHTML generator that is generated on the %s hook.', 'simpods-mvc' ),
					'<code>wp_head</code>'
				),
				'value' => $options['simpods_xhtml_generator'],
			),
			'simpods_enable_sorting' => array(
				'type' => 'check',
				'title' => esc_html__( 'Enable Sorting Posts', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Enable drag and drop to sort posts on the required post types.', 'simpods-mvc' ),
				'value' => $options['simpods_enable_sorting'],
				'choices'	=> $this->public_post_types,
			),	
			'simpods_single_login' => array(
				'type' => 'check',
				'title' => esc_html__( 'Enable Single Login', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'The ticked user roles can only access the site on one session.', 'simpods-mvc' ),
				'value' => $options['simpods_single_login'],
				'choices'	=> $user_roles,
			),						
		);
		
		$this->form_settings['main'] = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-ss-settings-submit',
				'label' => esc_html__( 'Update Settings', 'simpods-mvc' ),
			),
			'form' => array(
				'id'     => 'simpods-ss-settings-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'update_settings',
			'action' => 'update_settings',
		);
	}
	/**
	 * Updates the settings for the simpods
	 *
	 * @return null
	 */
	private function update_settings() {
		global $funs, $cross_admin;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'update_settings' ) {
			return;
		}
		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'update_settings' ) ) {
			$this->wp_notices .= $cross_admin->notice( esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' ), 'error' );
			return false;
		}
		$post = $_POST;
		unset( $post['action'] );
		unset( $post['_wpnonce'] );
		unset( $post['_wp_http_referer'] );
		
		//convert array into string
		foreach ( $post as $key => $value ) {
			if( is_array( $value ) ){
				//Check if the array has string keys. If not, we can convert it into a string
				if( count( array_filter( array_keys( $value ), 'is_string' ) ) == 0 ){
					 $post[ $key ] = implode( ',', $value );
				}
			}
		}
		$sanitized = array();
		$sanitized = array_map( 'wp_strip_all_tags', $post );
		$sanitized = $funs->escape( $sanitized );
		$sanitized = array_map( 'trim', $sanitized );
		
		$fields = $funs->simpods_select( array( 'main_tb.`sps_option`' => 'simpods_%' ), array( 'target_tb' => self::DB_TABLE ) );
		foreach ( $fields as $field ) {
			if ( !array_key_exists( $field['sps_option'], $sanitized ) ) {
				$sanitized[$field['sps_option']] = 0;
			}
		}
		foreach ( $sanitized as $key => $value ) {
			$data = array(
				'sps_value' => $value,
				'sps_modified' => date( 'Y-m-d H:i:s' ),
			);
			$where = array(
				'sps_option' => $key
			);
			$funs->general_update( self::DB_TABLE, $data, $where );
		}
		$this->wp_notices .= $cross_admin->notice( esc_html__( 'Settings Updated.', 'simpods-mvc' ), 'success' );
	}


}
