<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods Style & Script Controller
 *
 * @package Simpods MVC
 * @author Andrew Palfrey
 * @since 18/01/2019
 */
class Ctrl extends Ctrl_MVC_Admin_Controller {
	const PAGE_SLUG = 'simpods-style-script';
	const DB_TABLE   = 'simpods_style_script';
	public $defaults = array();
	public $form_items = array();
	public $form_settings = array();
	public $records = array();
	
	public $edit_form_items    = array();
	public $edit_form_settings = array();
	public $wp_notices = '';
	public $wp_errors  = false;
	public function __construct() {
		$this->option_defaults();
		//$this->deprecation_notice();
	}
	private function option_defaults() {
		$this->defaults = array(
			'style_include_bootstrap' => 'bs4',
			'style_include_fontawesome' => 'fa5-js',
			'style_include_match_height' => 1,
			'style_compile_scss' => 0,
			'style_scss_compile_frequency' => 'watch',
			'style_scss_compile_type' => 'Expanded',
			'style_defer_scripts' => 0,
			'style_load_min' => 0,
		);
	}
	private function deprecation_notice() {
		global $cross_admin;
		$this->wp_notices .= $cross_admin->notice( esc_html__( 'The SCSS compiler will be removed in version 3.0.0', 'simpods-mvc' ), 'warning', false );
	}
	/**
	 * Sets up & processes all style & script forms
	 *
	 * @return null
	 */
	public function forms() {
		global $funs, $cross_admin;
		$this->update_settings();
		$db_options = $funs->simpods_select( array( 'main_tb.`sps_option`' => 'style_%' ), array( 'target_tb' => 'simpods_settings' ) );
		$options = array();
		foreach ( $db_options as $option ) {
			$option_name  = $option['sps_option'];
			$option_value = $option['sps_value'];
			$options[$option_name] = $option_value;
		}
		$options = wp_parse_args( $options, $this->defaults );
		$this->form_items['main'] = array(
			'style_include_bootstrap' => array(
				'type' => 'select',
				'title' => esc_html__( 'Include Bootstrap Framework', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Choose a version of Bootstrap to include, or none at all.', 'simpods-mvc' ),
				'value' => $options['style_include_bootstrap'],
				'choices' => array(
					'bs3' => esc_html__( 'Bootstrap 3', 'simpods-mvc' ),
					'bs4' => esc_html__( 'Bootstrap 4', 'simpods-mvc' ),
					'no'  => esc_html__( 'No', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
			),
			'style_include_fontawesome' => array(
				'type' => 'select',
				'title' => esc_html__( 'Include Font Awesome', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Choose a version of Font Awesome to include, or none at all.', 'simpods-mvc' ),
				'value' => $options['style_include_fontawesome'],
				'choices' => array(
					'fa4'           => esc_html__( 'Font Awesome 4', 'simpods-mvc' ),
					'fa5-css'       => esc_html__( 'Font Awesome 5 - CSS', 'simpods-mvc' ),
					'fa5-css-shims' => esc_html__( 'Font Awesome 5 - CSS & Shims', 'simpods-mvc' ),
					'fa5-js'        => esc_html__( 'Font Awesome 5 - JS', 'simpods-mvc' ),
					'fa5-js-shims'  => esc_html__( 'Font Awesome 5 - JS & Shims', 'simpods-mvc' ),
					'no'            => esc_html__( 'No', 'simpods-mvc' ),
				),
				'options' => array(
					'required' => true,
				),
			),
			'style_include_match_height' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Include Match Height', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'A jQuery plugin to set specified columns to the same height.', 'simpods-mvc' ),
				'value' => $options['style_include_match_height'],
			),
			// 'style_compile_scss' => array(
			// 	'type' => 'toggle',
			// 	'title' => esc_html__( 'Enable SCSS Compiling', 'simpods-mvc' ),
			// 	'subtitle' => sprintf(
			// 		esc_html__( 'Compile all SCSS files within the %1$s and %2$s folders.', 'simpods-mvc' ),
			// 		'<code>mvc/scss</code>',
			// 		'<code>mvc/admin/scss</code>'
			// 	),
			// 	'value' => $options['style_compile_scss'],
			// ),
			// 'style_scss_compile_frequency' => array(
			// 	'type' => 'select',
			// 	'title' => esc_html__( 'SCSS Compiling Frequency', 'simpods-mvc' ),
			// 	'subtitle' => esc_html__( 'How often would you like the SCSS to compile?', 'simpods-mvc' ),
			// 	'value' => $options['style_scss_compile_frequency'],
			// 	'choices' => array(
			// 		'watch'  => esc_html__( 'Watch', 'simpods-mvc' ),
			// 		'always' => esc_html__( 'On Load', 'simpods-mvc' ),
			// 	),
			// 	'options' => array(
			// 		'required' => true,
			// 	),
			// ),
			// 'style_scss_compile_type' => array(
			// 	'type' => 'select',
			// 	'title' => esc_html__( 'SCSS Compile Format', 'simpods-mvc' ),
			// 	'subtitle' => esc_html__( 'How would you like the compiled CSS to be formatted?', 'simpods-mvc' ),
			// 	'value' => $options['style_scss_compile_type'],
			// 	'choices' => array(
			// 		'expanded'   => esc_html__( 'Expanded', 'simpods-mvc' ),
			// 		'compressed' => esc_html__( 'Compressed', 'simpods-mvc' ),
			// 		'compact'    => esc_html__( 'Compact', 'simpods-mvc' ),
			// 		'crunched'   => esc_html__( 'Crunched', 'simpods-mvc' ),
			// 		'nested'     => esc_html__( 'Nested', 'simpods-mvc' ),
			// 	),
			// 	'options' => array(
			// 		'required' => true,
			// 	),
			// ),
			'style_defer_scripts' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Defer Scripts', 'simpods-mvc' ),
				'subtitle' => esc_html__( 'Force JavaScript to only run after the page has loaded. Deferring scripts may cause problems if you have scripts in between the body tags. Use with care.', 'simpods-mvc' ),
				'value' => $options['style_defer_scripts'],
			),
			'style_load_min' => array(
				'type' => 'toggle',
				'title' => esc_html__( 'Load Minified Scripts & Styles', 'simpods-mvc' ),
				'subtitle' => sprintf(
					esc_html__( 'Force enqueued scripts and styles to load their minified versions if available. Must use %1$s, e.g. %2$s.', 'simpods-mvc' ),
					'<code>.min</code>',
					'<code>style.min.css</code>'
				),
				'value' => $options['style_load_min'],
			),
		);
		$this->form_settings['main'] = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => 'simpods-ss-settings-submit',
				'label' => esc_html__( 'Update Settings', 'simpods-mvc' ),
			),
			'form'   => array(
				'id'     => 'simpods-ss-settings-form',
				'class'  => '',
				'method' => 'post',
				'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
				'enctype' => 'multipart/form-data',
			),
			'nonce'  => 'update_settings',
			'action' => 'update_settings',
		);
		// // $this->form_items['add'] = array(
		// // 	'spss_scss_path' => array(
		// // 		'type'     => 'text',
		// // 		'title'    => esc_html__( 'SCSS Folder Path', 'simpods-mvc' ),
		// // 		'subtitle' => sprintf(
		// // 		/* translators: %1$s: Example filepath, %2$s: <strong> tag, %3$s: Closing </strong> tag */
		// // 			esc_html__( 'This is where the SCSS is located. This should be taken from the theme root. e.g. %1$s %2$sNOTE: You cannot re-register an SCSS folder path%3$s', 'simpods-mvc' ),
		// // 			'<code>/mvc/scss</code>',
		// // 			'<br><strong>',
		// // 			'</strong>'
		// // 		),
		// // 		'value'    => '',
		// // 		'options'  => array(
		// // 			'required' => true,
		// // 		),
		// // 		'errors'   => array(
		// // 			'error'   => esc_html__( 'You must enter an SCSS path.', 'simpods-mvc' ),
		// // 			'invalid' => esc_html__( 'You cannot re-register an SCSS folder path.', 'simpods-mvc' ),
		// // 		),
		// // 	),
		// // 	'spss_css_path'  => array(
		// // 		'type'     => 'text',
		// // 		'title'    => esc_html__( 'CSS Folder Path', 'simpods-mvc' ),
		// // 		'subtitle' => sprintf(
		// // 			/* translators: %s: Example filepath */
		// // 			esc_html__( 'This is where the compiled CSS should be saved to. This should be taken from the theme root. e.g. %s', 'simpods-mvc' ),
		// // 			'<code>/mvc/css</code>'
		// // 		),
		// // 		'value'    => '',
		// // 		'options'  => array(
		// // 			'required' => true,
		// // 		),
		// // 		'errors'   => array(
		// // 			'error' => esc_html__( 'You must enter a CSS path.', 'simpods-mvc' ),
		// // 		),
		// // 	),
		// // );
		// // $this->form_settings['add'] = array(
		// // 	'table'  => array(
		// // 		'class' => '',
		// // 		'id'    => '',
		// // 	),
		// // 	'submit' => array(
		// // 		'class' => '',
		// // 		'id'    => 'simpods-ss-add-submit',
		// // 		'label' => esc_html__( 'Register Compile Location', 'simpods-mvc' ),
		// // 	),
		// // 	'form'   => array(
		// // 		'id'     => 'simpods-ss-add-form',
		// // 		'class'  => '',
		// // 		'method' => 'post',
		// // 		'action' => esc_url( admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ) ),
		// // 		'enctype' => 'multipart/form-data',
		// // 	),
		// // 	'nonce'  => 'add_new_entry',
		// // 	'action' => 'add_new_entry',
		// // );
		// $this->edit_form_items = array(
		// 	'spss_scss_path' => array(
		// 		'type'     => 'text',
		// 		'title'    => esc_html__( 'SCSS Folder Path', 'simpods-mvc' ),
		// 		'subtitle' => sprintf(
		// 			/* translators: %1$s: Example filepath, %2$s: <strong> tag, %3$s: Closing </strong> tag */
		// 			esc_html__( 'This is where the SCSS is located. This should be taken from the theme root. e.g. %1$s %2$sNOTE: You cannot re-register an SCSS folder path%3$ss', 'simpods-mvc' ),
		// 			'<code>/mvc/scss</code>',
		// 			'<br><strong>',
		// 			'</strong>'
		// 		),
		// 		'value'    => '',
		// 		'options'  => array(
		// 			'required' => true,
		// 			'id'       => 'edit-spss_scss_path',
		// 		),
		// 		'errors'   => array(
		// 			'error'   => esc_html__( 'You must enter an SCSS path.', 'simpods-mvc' ),
		// 			'invalid' => esc_html__( 'You cannot re-register an SCSS folder path.', 'simpods-mvc' ),
		// 		),
		// 	),
		// 	'spss_css_path'  => array(
		// 		'type'     => 'text',
		// 		'title'    => esc_html__( 'CSS Folder Path', 'simpods-mvc' ),
		// 		'subtitle' => sprintf(
		// 			esc_html__( 'This is where the compiled CSS should be saved to. This should be taken from the theme root. e.g. %s', 'simpods-mvc' ),
		// 			'<code>/mvc/css</code>'
		// 		),
		// 		'value'    => '',
		// 		'options'  => array(
		// 			'required' => true,
		// 			'id'       => 'edit-spss_css_path',
		// 		),
		// 		'errors'   => array(
		// 			'error' => esc_html__( 'You must enter a CSS path.', 'simpods-mvc' ),
		// 		),
		// 	),
		// 	'id'             => array(
		// 		'type'    => 'number',
		// 		'title'   => esc_html__( 'ID', 'simpods-mvc' ),
		// 		'value'   => '',
		// 		'options' => array(
		// 			'required' => true,
		// 			'readonly' => true,
		// 			'id'       => 'edit-id',
		// 		),
		// 		'errors'  => array(
		// 			'error' => esc_html__( 'There should be an ID in here...', 'simpods-mvc' ),
		// 		),
		// 	),
		// );
		// $this->edit_form_settings = array(
		// 	'table'  => array(
		// 		'class' => '',
		// 		'id'    => '',
		// 	),
		// 	'submit' => false,
		// 	'form'   => false,
		// 	'nonce'  => 'edit_entry',
		// 	'action' => 'edit_entry',
		// );
		//$this->add_compile_location();
		//$this->edit_compile_location();
		$this->records = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
	}
	/**
	 * Updates the settings for the style & script system
	 *
	 * @return null
	 */
	private function update_settings() {
		global $funs, $cross_admin;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'update_settings' ) {
			return;
		}
		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'update_settings' ) ) {
			$this->wp_notices .= $cross_admin->notice( esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' ), 'error' );
			return false;
		}
		$post = $_POST;
		unset( $post['action'] );
		unset( $post['_wpnonce'] );
		unset( $post['_wp_http_referer'] );
		$sanitized = array();
		$sanitized = array_map( 'wp_strip_all_tags', $post );
		$sanitized = $funs->escape( $sanitized );
		$sanitized = array_map( 'trim', $sanitized );
		$fields = $funs->simpods_select( array( 'main_tb.`sps_option`' => 'style_%' ), array( 'target_tb' => 'simpods_settings' ) );
		foreach ( $fields as $field_arr ) {
			if ( !array_key_exists( $field_arr['sps_option'], $sanitized ) ) {
				$sanitized[$field_arr['sps_option']] = 0;
			}
		}
		foreach ( $sanitized as $key => $value ) {
			$data = array(
				'sps_value' => $value,
				'sps_modified' => date( 'Y-m-d H:i:s' ),
			);
			$where = array(
				'sps_option' => $key
			);
			$funs->general_update( 'simpods_settings', $data, $where );
		}
		$this->wp_notices .= $cross_admin->notice( esc_html__( 'Settings Updated.', 'simpods-mvc' ), 'success' );
	}
	/**
	 * Adds new compile locations into the system
	 * 
	 * @deprecated
	 * @return null
	 */
	private function add_compile_location() {
		global $funs, $cross_admin, $current_user;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'add_new_entry' ) {
			return;
		}
		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'add_new_entry' ) ) {
			$message_str = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			$this->wp_notices .= $cross_admin->notice( $message_str, $css_class );
			return false;
		}
		$post = $_POST;
		unset( $post['_wpnonce'] );
		unset( $post['_wp_http_referer'] );
		unset( $post['action'] );
		$sanitized = array_map( 'wp_strip_all_tags', $post );
		$sanitized = $funs->escape( $sanitized );
		$sanitized = array_map( 'trim', $sanitized );
		$wp_errors = new WP_Error();
		if ( $sanitized['spss_scss_path'] == '' ) {
			$wp_errors->add( 'spss_scss_path_error', esc_html__( 'You must enter an SCSS folder path', 'simpods-mvc' ), 'spss_scss_path-error' );
		}
		if ( $sanitized['spss_css_path'] == '' ) {
			$wp_errors->add( 'spss_css_path_error', esc_html__( 'You must enter a CSS folder path', 'simpods-mvc' ), 'spss_css_path-error' );
		}
		if ( $wp_errors->has_errors() ) {
			$this->wp_errors = $wp_errors;
			$this->wp_notices .= $cross_admin->notice( esc_html__( 'Unable to register compile location', 'simpods-mvc' ), 'error' );
			return false;
		}
		$fields = array(
			'spss_scss_path' => $sanitized['spss_scss_path'],
		);
		$records = $funs->simpods_select( $fields, array( 'target_tb' => self::DB_TABLE ) );
		if ( !empty( $records ) || $sanitized['spss_scss_path'] == 'mvc/scss' || $sanitized['spss_scss_path'] == 'mvc/admin/scss' ) {
			$this->wp_errors = $wp_errors;
			$this->wp_notices .= $cross_admin->notice( esc_html__( 'You cannot re-register an SCSS folder path', 'simpods-mvc' ), 'error' );
			return false;
		}
					
		$now = date( 'Y-m-d H:i:s' );
		$sanitized['spss_created']         = $now;
		$sanitized['spss_modified']        = $now;
		$sanitized['spss_modified_author'] = $current_user->ID;
		$status = $funs->general_insert( self::DB_TABLE, $sanitized );
		$message = '';
		$class   = '';
		if ( $status ) {
			$message = esc_html__( 'Compile Location registered', 'simpods-mvc' );
			$class   = 'success';
		} else {
			$message = esc_html__( 'Compile Location not registered', 'simpods-mvc' );
			$class   = 'error';
		}
		$this->wp_notices .= $cross_admin->notice( $message, $class );
	}
	/**
	 * Updates compile locations within the system
	 *
	 * @deprecated
	 * @return null
	 */
	private function edit_compile_location() {
		global $funs, $cross_admin, $current_user;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'edit_entry' ) {
			return;
		}
		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'edit_entry' ) ) {
			$this->wp_notices .= $cross_admin->notice( esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' ), 'error' );
			return false;
		}
		$post = $_POST;
		unset( $post['_wpnonce'] );
		unset( $post['_wp_http_referer'] );
		unset( $post['action'] );
		$sanitized = array_map( 'wp_strip_all_tags', $post );
		$sanitized = $funs->escape( $sanitized );
		$sanitized = array_map( 'trim', $sanitized );
		$sanitized['id'] = intval( $sanitized['id'] );
		$wp_errors = new WP_Error();
		if ( $sanitized['spss_scss_path'] == '' ) {
			$wp_errors->add( 'spss_scss_path_error', esc_html__( 'You must enter an SCSS folder path', 'simpods-mvc' ), 'spss_scss_path-error' );
		}
		if ( $sanitized['spss_css_path'] == '' ) {
			$wp_errors->add( 'spss_css_path_error', esc_html__( 'You must enter a CSS folder path', 'simpods-mvc' ), 'spss_css_path-error' );
		}
		if ( $wp_errors->has_errors() ) {
			$this->wp_errors = $wp_errors;
			$this->wp_notices .= $cross_admin->notice( esc_html__( 'Unable to register compile location', 'simpods-mvc' ), 'error' );
			return false;
		}
		$fields = array(
			'spss_scss_path' => $sanitized['spss_scss_path'],
		);
		$records = $funs->simpods_select( $fields, array( 'target_tb' => self::DB_TABLE ) );
		if ( !empty( $records ) || $sanitized['spss_scss_path'] == 'mvc/scss' || $sanitized['spss_scss_path'] == 'mvc/admin/scss' ) {
			if ( count( $records ) != 1 || $records[0]['id'] != $sanitized['id'] ) {
				$this->wp_errors = $wp_errors;
				$this->wp_notices .= $cross_admin->notice( esc_html__( 'You cannot re-register an SCSS folder path', 'simpods-mvc' ), 'error' );
				return false;
			}
		}
		$sanitized['spss_modified']        = date( 'Y-m-d H:i:s' );
		$sanitized['spss_modified_author'] = $current_user->ID;
		$record_id = $sanitized['id'];
		unset( $sanitized['id'] );
		$status = $funs->general_update( self::DB_TABLE, $sanitized, array( 'id' => $record_id ) );
		$message_str = esc_html__( 'Compile Location Edited', 'simpods-mvc' );
		$css_class     = 'success';
		$this->wp_notices .= $cross_admin->notice( $message_str, $css_class );
		$status = $funs->general_update( self::DB_TABLE, $sanitized, array( 'id' => $record_id ) );
		$message = '';
		$class   = '';
		if ( $status ) {
			$message = esc_html__( 'Compile Location edited', 'simpods-mvc' );
			$class   = 'success';
		} else {
			$message = esc_html__( 'Compile Location not edited', 'simpods-mvc' );
			$class   = 'error';
		}
		$this->wp_notices .= $cross_admin->notice( $message, $class );
	}
}
