<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * A collection of functions can be used in course, modules, modules pages and assets
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/05/2014
 */
abstract class MVC_Admin_General {
	
	/**
	 * Gets the class for an item within the simpods admin sidebar
	 * @param  string $page The page to test whether the link is active or not
	 * @return string The class to be added to the link
	 */
	public function get_active_link( $page = '' ) {
		if ( isset( $_GET['page'] ) && $page == $_GET['page'] ) {
			return 'active';
		}
		return '';
	}
	/**
	 * Gets the current post type in the WordPress Admin
	 * @link   	https://simpodsmvc.me/docs/function-reference/functions/get_cur_post_type_fn/
	 * @link 	http://themergency.com/wordpress-tip-get-post-type-in-admin/
	 * @return 	mixed 	The current post type
	 */
	public function get_cur_post_type() {
		global $post, $typenow, $current_screen;
		//we have a post so we can just get the post type from that
		if ( $post && $post->post_type ) {
			return $post->post_type;
		}
		//check the global $typenow - set in admin.php
		else
		if ( $typenow ) {
			return $typenow;
		}
		//check the global $current_screen object - set in sceen.php
		else
		if ( $current_screen && $current_screen->post_type ) {
			return $current_screen->post_type;
		}
		//lastly check the post_type querystring
		else
		if ( isset( $_REQUEST['post_type'] ) ) {
			return sanitize_key( $_REQUEST['post_type'] );
		} else
		if ( isset( $_REQUEST['post'] ) ) {
			return get_post_type( sanitize_key( $_REQUEST['post'] ) );
		} else
		if ( strpos( $_SERVER['PHP_SELF'], '/wp-admin/post-new.php' ) !== false ) {
			return 'post';
		} else
		if ( strpos( $_SERVER['PHP_SELF'], '/wp-admin/edit.php' ) !== false ) {
			return 'post';
		} else
		if ( strpos( $_SERVER['PHP_SELF'], '/wp-admin/users.php' ) !== false ) {
			return 'user';
		} else
		if ( strpos( $_SERVER['PHP_SELF'], '/wp-admin/user-edit.php' ) !== false ) {
			return 'user';
		} else
		if ( strpos( $_SERVER['PHP_SELF'], '/wp-admin/profile.php' ) !== false ) {
			return 'user';
		} else
		if ( isset( $_GET['page'] ) && strpos( $_GET['page'], 'pods-manage-' ) === 0 ) {
			return substr( $_GET['page'], 12 );
		}
		//we do not know the post type!
		return null;
	}
	/**
	 * Get the ID based on the current database table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_the_id_fn/
	 * @param  array $current_table Current Table
	 * @return array
	 */
	public function get_the_ID( $current_table ) {
		//the bind_fn function will remove the one not needed
		$ids = array(
			'id' => -1,
			'ID' => -1,
		);
		if ( $current_table['type'] == 'post_type' ) {
			$ids = array(
				'id' => intval( get_the_ID() ),
				'ID' => intval( get_the_ID() ),
			);
		} else
		if ( $current_table['type'] == 'pod' ) {
			if ( isset( $_GET['id'] ) ) {
				$ids = array(
					'id' => intval( $_GET['id'] ),
					'ID' => intval( $_GET['id'] ),
				);
			}
		} else
		if ( $current_table['type'] == 'user' ) {
			global $user_id;
			if ( is_numeric( $user_id ) ) {
				$ids = array(
					'id' => intval( $user_id ),
					'ID' => intval( $user_id ),
				);
			}
		} else
		if ( $current_table['type'] == 'post' ) {
			if ( isset( $_GET['id'] ) ) {
				$ids = array(
					'id' => intval( get_the_ID() ),
					'ID' => intval( get_the_ID() ),
				);
			}
		}
		return $ids;
	}
	/**
	 * Imports & updates the requested table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/import_database_fn/
	 * @param  string  $table      Table to import or update
	 * @param  array   $columns    Columns to import or update
	 * @param  string  $extra_query     Anything additional to add to the query
	 * @param  boolean $can_add_commons Whether to add common fields to the database or not.
	 * @return boolean
	 */
	public function import_database( $table = '', $columns = array(), $extra_query = '', $can_add_commons = false ) {
		if ( !is_array( $columns ) ) {
			return false;
		}
		if ( empty( $columns ) || $table == '' ) {
			return false;
		}
		global $wpdb;
		$charset 	= $wpdb->get_charset_collate();
		$query 		= 'CREATE TABLE `' . $wpdb->prefix . $table . '` (
			`id` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,' . PHP_EOL;
		foreach ( $columns as $column_name => $column_data ) {
			// if type not available, skip
			if ( !isset( $column_data['type'] ) ) {
				continue;
			}
			$query .= '`' . $column_name . '` ' . $column_data['type'];
			if ( isset( $column_data['settings'] ) && $column_data['settings'] !== '' ) {
				$query .= ' ' . $column_data['settings'];
			}
			if ( isset( $column_data['default'] ) && $column_data['default'] !== '' ) {
				$column_data['default'] = trim( $column_data['default'] );
				//if default is set but the key word "default" is not in place, add it.
				if( strpos( strtolower( $column_data['default'] ), 'default' ) !== 0 ){
					$column_data['default'] = 'DEFAULT ' . $column_data['default'];
				}
				$query .= ' ' . $column_data['default'];
			}
			$query .= ',' . PHP_EOL;
		}
		$query .= 'PRIMARY KEY (`id`)';
		if ( $extra_query !== '' ) {
			$query .= ', ' . PHP_EOL . $extra_query . PHP_EOL;
		}
		$query .= ') ' . $charset;
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		dbDelta( $query );
		$this->update_columns( $table, $columns, $can_add_commons );
		update_option( $table . '_table_created', 1 );
		return true;
	}
	/**
	 * Provide a select field for filtering by pages
	 * @param  string $title   The title of the combo box
	 * @param  string $name    The name of the table
	 * @param  string $element_name The name of the element
	 * @return null   Prints out the select field
	 */
	public function pages_list( $title, $name, $element_name ) {
		global $wpdb, $funs, $current_user;
		$fields = array();
		if ( MVC_Admin_Factory::$act_tables[$name]['type'] == 'post_type' ) {
			$fields['post_tb.post_status'] = array( 'publish', 'draft' );
			// if not an administrator, only list the author's own input
			if ( ! current_user_can( 'manage_options' ) ) {
				$fields['post_tb.post_author'] = $current_user->ID;
			}
		}
		$fields 	= apply_filters( 'simpods_pages_list_fields', $fields, $title, $name, $element_name );
		$attrs  	= array( 'target_tb' => $name );
		$attrs 		= apply_filters( 'simpods_pages_list_attributes', $attrs, $title, $name, $element_name );
		$items  	= $funs->simpods_select( $fields, $attrs );
		$items 		= apply_filters( 'simpods_pages_list_items', $items, $title, $name, $element_name );
		$name_field = MVC_Admin_Factory::$act_tables[$name]['name_field'];
		$values 	= array();
		$count 		= count( $items );
		for ( $i = 0; $i < $count; $i++ ) {
			$values[ $i ] = array(
									'name' 			=> $items[ $i ][ $name_field ],
									'id'   			=> $items[ $i ]['id'],
									'is_category' 	=> false,
								);
			if( isset( $items[ $i ]['is_category'] ) && $items[ $i ]['is_category'] == true ){
				$values[ $i ]['is_category'] 	= true;
			}
		}
		?>
		<select name="<?php echo $element_name; ?>">
			<option value=""><?php echo $title; ?></option>
			<?php
			$current_element_name = isset( $_GET[ $element_name ] ) ? $_GET[ $element_name ] : '';
					foreach ( $values as $value_arr ) {
						printf(
							'<option value="%s" %s %s>%s</option>',
							$value_arr['id'],
							$value_arr['id'] == $current_element_name ? ' selected="selected"' : '',
							$value_arr['is_category'] == true ? ' disabled' : '',
							$value_arr['name']
						);
					}
					?>
		</select>
		<?php
	}
	/**
	 * Updates the columns within a table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/update_columns_fn/
	 * @param  string  $table      The table to update
	 * @param  array   $custom_fields    The fields to update/add
	 * @param  boolean $can_add_commons Whether to add common fields or not
	 * @param  boolean $can_echo_query  If true the query will be echoed out (default false)
	 * @return null
	 */
	public function update_columns( $table, $custom_fields, $can_add_commons = true, $can_echo_query = false ) {
		global $wpdb, $funs;
		$table = esc_sql( $table );
		// special ones from each section
		foreach ( $custom_fields as $key => $field_data ) {
			$is_existant = $funs->column_exist( $table, $key );
			if ( !$is_existant ) {
				$query = 'ALTER TABLE `' . $wpdb->prefix . '' . $table . '` ADD `' . $key . '` ' . implode( ' ', $field_data );
				if ( $can_echo_query ) {
					echo $query;
				}
				$wpdb->query( $query );
			}
		}
		// common ones
		if ( $can_add_commons ) {
			foreach ( MVC_Admin_Factory::$custom_fields as $key => $field_data ) {
				$is_existant = $funs->column_exist( $table, $key );
				if ( !$is_existant ) {
					$query = 'ALTER TABLE `' . $wpdb->prefix . '' . $table . '` ADD `' . $key . '` ' . implode( ' ', $field_data );
					if ( $can_echo_query ) {
						echo $query;
					}
					$wpdb->query( $query );
				}
			}
		}
	}
	/**
	 * Tests if pods is installed/active and whether it meets a minimum version
	 *
	 * @param  string $min_version The minimum version of pods to test against
	 * @return string The notice to display
	 */
	final public function test_for_pods( $min_version = '2.6.1' ) {
		if ( !defined( 'PODS_VERSION' ) ) {
			/* translators: %s: Version number */
			return $this->notice( sprintf( esc_html__( 'Plugin: Pods - Custom Content Types and Fields required, version %s or later.', 'simpods-mvc' ), $min_version ), 'error', false );
		} else {
			//check if Pods version is greater than or equal to minimum supported version for this plugin
			if ( version_compare( $min_version, PODS_VERSION ) > 0 ) {
				/* translators: %s: Version number */
				return $this->notice( sprintf( esc_html__( 'Simpods requires Pods version %s or later.', 'simpods-mvc' ), $min_version ), 'error', false );
			}
		}
		return '';
	}
	/**
	 * Generates am admin notice
	 *
	 * @param  string  $message The message to display in the notice
	 * @param  string  $type    The type of notice to display
	 * @param  boolean $can_dismiss     Whether the notice should be dismissible or not
	 * @param  string  $extra_css_class   Any extra classes to add to the notice
	 * @return string
	 */
	final public function notice( $message = '', $type = 'info', $can_dismiss = true, $extra_css_class = '' ) {
		$css_classes = array(
			'notice',
			'notice-' . $type,
		);
		if ( $can_dismiss ) {
			$css_classes[] = 'is-dismissible';
		}
		if ( $extra_css_class !== '' ) {
			$css_classes[] = $extra_css_class;
		}
		$css_class = implode( ' ', $css_classes );
		return '<div class="' . $css_class . '"><p>' . $message . '</p></div>';
	}
	/**
	 * Returns a table complete with items, edit and delete buttons and an edit modal
	 *
	 * @param array   $headers      The headers to use on the table
	 * @param array   $data         The items to display
	 * @param string  $page         The page parameter for the form action
	 * @param boolean $can_edit         Whether the table should allow editing
	 * @param array   $edit_values     The editable field names
	 * @param array   $edit_settings The settings for the edit form
	 * @return string The table to display
	 */
	final public function simpods_table( $headers, $data, $page, $can_edit = false, $edit_values = array(), $edit_settings = array(), $can_view = false ) {
		$count = count( $data );
		if( $count > 0 ){
			$data	=	array_values( $data );
		}		
		$count_label = esc_html( sprintf(
			/* translators: %d: Number of items */
			_n(
				'%d item',
				'%d items',
				$count,
				'simpods-mvc'
			),
			$count
		) );
		$html = '<form action="' . admin_url( 'admin.php?page=' . $page . '' ) . '" method="post" class="record-form">
			<div class="row align-items-end">
				<div class="col-12 col-sm-6 col-md-4 col-lg-5 col-xl-4 mb-3">
					<input type="hidden" name="_wpnonce" value="' . wp_create_nonce( 'remove_entry' ) . '" />
					<div class="input-group">
						<select class="form-control custom-select" name="action" id="bulk-action" required>
							<option value="" selected>' . esc_html__( 'Bulk Actions', 'simpods-mvc' ) . '</option>
							<option value="remove_entry">' . esc_html__( 'Delete', 'simpods-mvc' ) . '</option>
						</select>
						<div class="input-group-append">
							<button type="submit" class="btn btn-outline-simpods-blue bulk-btn" type="button">' . esc_html__( 'Apply', 'simpods-mvc' ) . '</button>
						</div>
					</div>
				</div>
				<div class="col text-right mb-3">
					<p class="item-count m-0">' . $count_label . '</p>
				</div>
			</div>
			<table class="table table-striped record-table">
				<thead>
					<tr>
						<th class="check-column simpods-select-column">
							<div class="custom-control custom-checkbox">
								<input type="checkbox" class="custom-control-input" id="table-check-all">
								<label class="custom-control-label" for="table-check-all">
									<span class="text-hide">' . esc_html__( 'Select All', 'simpods-mvc' ) . '</span>
								</label>
							</div>
						</th>';
		foreach ( $headers as $name => $label ) {
			$html .= '<th class="' . $name . '" id="' . $name . '-header">' . $label . '</th>';
		}
		$html .= '<th class="values d-none"></th>
						<th class="actions">' . esc_html__( 'Actions', 'simpods-mvc' ) . '</th>
					</tr>
				</thead>
				<tbody>';
		for ( $i = 0; $i < $count; $i++ ) {
			$items    	= $data[$i];
			$all_items 	= array_merge( array_flip( array_keys( $headers ) ), $items );
			$id     	= $all_items['id'];
			$html .= '<tr>
						<td class="check-column simpods-select-column" data-title="' . esc_html__( 'Select', 'simpods-mvc' ) . '">
							<div class="custom-control custom-checkbox">
								<input type="checkbox" class="custom-control-input" name="remove[]" id="table-check-' . $id . '" value="' . $id . '">
								<label class="custom-control-label" for="table-check-' . $id . '">
									<span class="text-hide">' . esc_html__( 'Select Item', 'simpods-mvc' ) . '</span>
								</label>
							</div>
						</td>';
			foreach ( $all_items as $key => $value ) {
				if ( !array_key_exists( $key, $headers ) ) {
					continue;
				}
				$title = $headers[ $key ];
				if ( $value == '' ) {
					$value = '-';
				}
				if( isset( $all_items['alt_displays'] ) ){ // alternative display instead of values
					if( isset( $all_items['alt_displays'][ $key ] ) ){
						$value = esc_html( $all_items['alt_displays'][ $key ] );
					}					
				}
				$html .= '<td class="' . $key . '" id="' . $key . '-' . $id . '" data-title="' . $title . '">' . $value . '</td>';
			}
			$html .= '<td class="values d-none">';
			foreach ( $edit_values as $key ) {
				if ( !array_key_exists( $key, $items ) ) {
					continue;
				}
				$html .= '<span data-key="' . $key . '" data-value="' . $items[$key] . '"></span>';
			}
			$html .= '<td class="actions" id="actions-' . $id . '" data-title="' . esc_html__( 'Actions', 'simpods-mvc' ) . '">
							<div class="btn-group">';
				
			if ( $can_edit ) {
				if( empty( $edit_settings['view_only'] ) ){			
					$html .= '<button type="button" class="edit-btn btn btn-outline-simpods-blue btn-sm" title="' . esc_html__( 'Edit', 'simpods-mvc' ) . '" data-value="' . $id . '">
										<span class="dashicons dashicons-edit"></span>
									</button>';
				
				} else {
				
					$html .= '<button type="button" class="edit-btn btn btn-outline-simpods-blue btn-sm" title="' . esc_html__( 'View', 'simpods-mvc' ) . '" data-value="' . $id . '">
										<span class="dashicons dashicons-welcome-view-site"></span>
									</button>';
				}
			}	
			$html .= '<button type="button" class="trash-btn btn btn-outline-simpods-purple btn-sm" title="' . esc_html__( 'Delete', 'simpods-mvc' ) . '" data-value="' . $id . '">
									<span class="dashicons dashicons-trash"></span>
								</button>
							</div>
							<input type="checkbox" id="trash-' . $id . '" name="remove[]" class="d-none" value="' . $id . '" />
						</td>';
			$html .= '</tr>';
		}
		$html .= '</tbody>
			</table>
		</form>';
		$html .= '<div class="modal fade modal-simpods-purple modal-square delete-modal" id="delete-modal" tabindex="-1" role="dialog" data-backdrop="static">
			<div class="modal-dialog modal-dialog-centered" role="document">
				<div class="modal-content">
					<div class="modal-header">
						<h5 class="modal-title">' . esc_html__( 'Are you sure?', 'simpods-mvc' ) . '</h5>
						<button type="button" class="close" data-dismiss="modal" aria-label="' . esc_html__( 'Close', 'simpods-mvc' ) . '">
							<span aria-hidden="true">&times;</span>
						</button>
					</div>
					<div class="modal-body">
						<h6>' . esc_html__( 'Are you sure you would like to delete this item?', 'simpods-mvc' ) . '</h6>
						<p>' .
							sprintf(
								/* translators: %1$s: <strong> tag, %2$s: </strong> tag */
								esc_html__( '%1$sNOTE:%2$s This cannot be undone.', 'simpods-mvc' ),
								'<strong>',
								'</strong>'
							) .
						'</p>
					</div>
					<div class="modal-footer">
						<div class="row justify-content-between m-0 w-100">
							<div class="col p-0">
								<button type="button" class="btn btn-outline-simpods-blue" id="keep-item">' . esc_html__( 'No, keep this item.', 'simpods-mvc' ) . '</button>
							</div>
							<div class="col p-0 text-right">
								<button type="button" class="btn btn-simpods-purple" id="delete-item">' . esc_html__( 'Yes, delete this item.', 'simpods-mvc' ) . '</button>
							</div>
						</div>
					</div>
				</div>
			</div>
		</div>';
		
		if ( $can_edit && is_array( $edit_settings ) && !empty( $edit_settings ) ) {
			$html .= $this->edit_modal( $edit_settings );
		}
		return $html;
	}
	/**
	 * Returns an edit modal based on the fields set in the page's controller
	 *
	 * @param array   $settings The settings for the modal
	 * @return string The modal to display
	 */
	final public function edit_modal( $settings = array() ) {
		global $ctrl;
		$settings_helper = new MVC_Admin_Settings_Helper();
		$defaults = array(
			'id-prefix' 	=> '',
			'action'    	=> '',
			'title'     	=> '',
			'btn'       	=> '',
			'form_items'	=> array(),	// For the third party controller to pass in form items. The main controller data take the priority
			'form_settings'	=> array(), // For the third party controller to pass in form items. The main controller data take the priority
		);
		$settings = wp_parse_args( $settings, $defaults );
		if ( $settings['btn'] == '' ) {
			$settings['btn'] = $settings['title'];
		}
		
		if( !empty( $ctrl->edit_form_items ) ){ 
			$settings['form_items'] = $ctrl->edit_form_items;
		}
		if( !empty( $ctrl->edit_form_settings ) ){
			$settings['form_settings'] = $ctrl->edit_form_settings;
		}	
		
		$hidden = '';
		if ( isset( $settings['view_only'] ) && $settings['view_only'] == true ) {
			$hidden = 'd-none';
		}
		return '<div class="modal fade modal-simpods-blue modal-square edit-modal" tabindex="-1" role="dialog" aria-labelledby="modal-title" aria-hidden="true" id="' . $settings['id-prefix'] . '-modal">
			<div class="modal-dialog modal-lg modal-dialog-centered" role="document">
				<form class="modal-content" method="post" action="' . $settings['action'] . '" enctype="multipart/form-data" id="' . $settings['id-prefix'] . '-edit" autocomplete="off">
					<div class="modal-header">
						<h5 class="modal-title" id="modal-title">' . $settings['title'] . '</h5>
						<button type="button" class="close" data-dismiss="modal" aria-label="' . esc_html__( 'Close', 'simpods-mvc' ) . '">
							<span aria-hidden="true">&times;</span>
						</button>
					</div>
					<div class="modal-body">
						' . $settings_helper->return_form( $settings['form_items'], $settings['form_settings'] ) . '
					</div>
					<div class="modal-footer">
						<div class="row justify-content-between m-0 w-100">
							<div class="col p-0">
								<button type="button" class="btn btn-outline-simpods-purple" data-dismiss="modal">' . esc_html__( 'Close', 'simpods-mvc' ) . '</button>
							</div>
							<div class="col p-0 text-right ' . $hidden . '">
								<button type="submit" class="btn btn-simpods-blue" id="' . $settings['id-prefix'] . '-edit-submit">' . $settings['btn'] . '</button>
							</div>
						</div>
					</div>
				</form>
			</div>
		</div>';
	}
	/**
	 * Removes items when requested from a POST request
	 *
	 * @param  string $table The table to remove items from
	 * @return string The notice to display
	 */
	final public function simpods_admin_remove_item( $table ) {
		global $funs;
		if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'remove_entry' || !isset( $_POST['remove'] ) ) {
			return;
		}
		if ( !isset( $_POST['_wpnonce'] ) || !wp_verify_nonce( $_POST['_wpnonce'], 'remove_entry' ) ) {
			$message = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			$css_class     = 'error';
			return $this->notice( $message, $css_class );
			return false;
		}
		if ( !is_array( $_POST['remove'] ) || count( $_POST['remove'] ) <= 0 ) {
			$message = esc_html__( 'We were unable to delete your selected items.', 'simpods-mvc' );
			$css_class     = 'error';
			return $this->notice( $message, $css_class );
		}
		$to_remove = $funs->escape_attrs( $_POST['remove'] );
		foreach ( $to_remove as $item_id ) {
			if ( !is_numeric( $item_id ) ) {
				continue;
			}
			$funs->general_delete( $table, 'id', $item_id );
		}
		$message = esc_html__( 'Items Deleted', 'simpods-mvc' );
		$css_class     = 'success';
		return $this->notice( $message, $css_class );
	}
	/**
	 * get_plugin_icons
	 * Returns a set of icons for a plugin
	 * @param $plugin_slug string the slug of the plugin to get the icons for
	 * @author Andrew Palfrey
	 * @return $icons array an array of icons
	 */
	protected function get_plugin_icons( $plugin_slug ) {
		$icons = wp_cache_get( $plugin_slug . '-plugin-icons', 'simpods_mvc' );
		if ( $icons ){
			return $icons;
		}		
		$url = 'https://ps.w.org/' . $plugin_slug . '/assets';
		$response = wp_remote_get( $url );
		if( is_array( $response ) ){
			$content = $response['body'];
			preg_match_all( '/<a href="[a-zA-Z0-9\-\.]*">([a-zA-Z0-9\-\.]*)<\/a>/', $content, $assets );
			$assets = end( $assets );
			$icons  = array();
			foreach ( $assets as $asset ) {
				if ( strpos( $asset, 'icon' ) === false ) {
					continue;
				}
				if ( strpos( $asset, '128x128' ) !== false ) {
					$icons['1x'] = trailingslashit( $url ) . $asset;
				} else if ( strpos( $asset, '256x256' ) !== false ) {
					$icons['2x'] = trailingslashit( $url ) . $asset;
				} else if ( strpos( $asset, 'svg') !== false ) {
					$icons['svg'] = trailingslashit( $url ) . $asset;
				}
			}
			wp_cache_set( $plugin_slug . '-plugin-icons', $icons, 'simpods_mvc' );
		}
		return $icons;
	}
	/**
	 * List the plugins for your component.
	 * @param $plugins array $plugins = array(
	 *							'plugin_name' => array(
	 *								'title'       => esc_html__( 'Plugin Title', 'simpods-cpd' ),
	 *								'slug'        => 'plugin_slug',
	 * 								'subtitle'	  => esc_html__( 'Extra info', 'simpods-cpd' ),
	 *								'active_file' => 'plugin_folder/plugin_active_file.php',
	 *								'required'    => true,
	 * 								'stauts'	  => '', //active or not, needs to call is_plugin_active to update this
	 *							),
	 *							...
	 *						)
	 * @author Andrew Palfrey
	 * @return string the HTML code for the plugins
	 */
	protected function list_plugins( $plugins ){
		global $wp_filesystem, $funs;
	?>
		<div class="row mx-n2">
			<?php
			foreach ( $plugins as $plugin ) {
				$required_class  = 'bg-info text-white';
				$installed_class = 'bg-success text-white';
				$installed = $wp_filesystem->exists( trailingslashit( WP_PLUGIN_DIR ) . $plugin['active_file'] );
				$active = is_plugin_active( $plugin['active_file'] );
				if ( !$active ) {
					if ( $plugin['required'] ) {
						$installed_class = 'bg-danger text-white';
					} else {
						$installed_class = 'bg-warning text-body';
					}
				}
				$required_label   = esc_html( _n( 'Recommended', 'Required', intval( $plugin['required'] ) + 1, 'simpods-mvc' ) );
				$installed_label  = esc_html( _n( 'Not Activated', 'Activated', intval( $active ) + 1, 'simpods-mvc' ) );
				$installed_button = esc_html( _n( 'Activate', 'Active', intval( $active ) + 1, 'simpods-mvc' ) );
				if ( !$installed ) {
					$installed_label  = esc_html__( 'Not Installed', 'simpods-mvc' );
					$installed_button = esc_html__( 'Install', 'simpods-mvc' );
				}
				$thumbnail = array();
				
				// If there is a thumbnail available, use it. For plugins that are not in the plugin directory
				if( isset( $plugin[ 'thumbnail' ] ) ){
					if(  is_array( $plugin[ 'thumbnail' ] ) ) {
						foreach ( $plugin[ 'thumbnail' ] as $key => $value ) {
							if( in_array( $key, array( '1x', '2x', 'svg' ) ) ){
								if( filter_var( $plugin[ 'thumbnail' ][ $key ],  FILTER_VALIDATE_URL ) ){
									$thumbnail[ $key ] = $plugin[ 'thumbnail' ][ $key ];
								
								}
							}
							
						}
					} else {
						if( filter_var( $plugin[ 'thumbnail' ],  FILTER_VALIDATE_URL ) ){
							$thumbnail = array( '1x' => $plugin['thumbnail'] );
						}
					}
				} else {
					$thumbnail = $this->get_plugin_icons( $plugin['slug'] );
				
				}
				$thumbnail_style = '';
				if ( !empty( $thumbnail ) ) {
					
					$thumbnail_style = 'background-image: url(' . end( $thumbnail ) . ');';
				}
				$install_args = array(
					'action' => 'install-plugin',
					'plugin' => $plugin['slug'],
					'_wpnonce' => wp_create_nonce( 'install-plugin_' . $plugin['slug'] ),
				);
				$install_url  = admin_url( 'update.php' ) . '?' . http_build_query( $install_args );
				$activate_args = array(
					'action' => 'activate',
					'plugin' => $plugin['active_file'],
					'_wpnonce' => wp_create_nonce( 'activate-plugin_' . $plugin['active_file'] ),
				);
				$activate_url  = admin_url( 'plugins.php' ) . '?' . http_build_query( $activate_args );
			?>
			<div class="col-12 col-md-6 col-xl-3 px-2 mb-3">
				<div class="card card-square p-0 m-0 h-100"> 
					<!-- <img src="<?php //echo $thumbnail_url; ?>" alt="<?php //echo $plugin['title']; ?>" class="card-img-top" /> -->
					<div class="card-img-top-square simpods-logo-back text-white" style="<?php echo $thumbnail_style ?>" title="<?php echo $plugin['title']; ?>">
						<?php if ( $thumbnail_style == '' ) { ?>
						<img src="<?php echo $funs->get_final_path( 'mvc/admin/images', 'default-plugin.svg', true, false, true ); ?>" alt="<?php echo $plugin['title']; ?>">
						<?php } ?>
					</div>
					<ul class="list-group list-group-flush fs1rem">
						<?php
						if( !isset( $plugin['hide_requirement_info'] ) || $plugin['hide_requirement_info'] == false ){
						?>
						<li class="list-group-item <?php echo $required_class; ?>"><?php echo $required_label; ?></li>
						<?php
						}
						?>
						<li class="list-group-item <?php echo $installed_class; ?>"><?php echo $installed_label; ?></li>
					</ul>
					<div class="card-body p-3">
						<h3 class="card-title m-0"><?php echo $plugin['title']; ?></h3>
						<?php if ( isset( $plugin['subtitle'] ) && $plugin['subtitle'] !== '' ) { ?>
						<p class="m-0"><?php echo $plugin['subtitle']; ?></p>
						<?php } ?>
					</div>
					<div class="card-footer p-3 bg-white">
						<?php 
						if ( !$active && !$installed ) { 
							$zip_link	= '';
							if( isset( $plugin[ 'zip' ] ) ){
								$zip_link = $plugin[ 'zip' ];
							}
						?>
						<a href="<?php echo $install_url; ?>" class="btn btn-simpods-purple btn-block btn-simpods-install" data-plugin="<?php echo $plugin['active_file']; ?>" data-zip="<?php echo $zip_link; ?>" ><?php echo $installed_button; ?></a>
						<?php } else if ( !$active && $installed ) { ?>
						<a href="<?php echo $activate_url; ?>" class="btn btn-simpods-purple btn-block btn-simpods-activate" data-plugin="<?php echo $plugin['active_file']; ?>"><?php echo $installed_button; ?></a>
						<?php } else { ?>
						<button type="button" class="btn btn-outline-simpods-purple btn-block btn-simpods-disabled" data-plugin="<?php echo $plugin['active_file']; ?>" disabled><?php echo $installed_button; ?></button>
						<?php 
						} 
						?>
					</div>
				</div>
			</div>
			<?php
			}
			?>
		</div>
	<?php
	}
	/**
	 * activate pods components
	 * @param $required array required pods components array(
	 *														'components' => array(
	 *																'component-name' => array(),
	 *																...
 	 *																),
	 *														);
	 * @since 2.2.0 26/11/2019
	 */
	protected function activate_pods_components( $required = array() ){
		$components = get_option('pods_component_settings', '' );
		if( $components != '' ){
			$components = (array)json_decode( $components );
			if( is_array( $components ) ){
				if( isset( $components[ 'components' ] ) ){
					$components[ 'components' ] = (array)$components[ 'components' ];
					$components[ 'components' ] = array_replace_recursive( $components[ 'components' ], $required[ 'components' ] ) ;
				} else {
					$components[ 'components' ] = $required[ 'components' ];
				}
			} else {
				$components = $required;
			}
		} else {
			$components = $required;
		}
		update_option('pods_component_settings', json_encode( $components ) );
	}
	/**
	 * Feed capabilities by an array. 
	 * 
	 * @param string $role the role to add the capabilities to
	 * @param array  $caps an array of capabilities
	 */ 
	protected function add_capabilities( $role = '', $caps = array() ){
		global $wp_roles;
		if( $role != '' && is_array( $caps ) && !empty( $caps ) ){
			foreach( $caps as $cap ){
				$wp_roles->add_cap( $role, $cap );				
			}
		}
	}
}
