<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Parent Factory for Simpods
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/05/2014
 */
class MVC_Admin_Parent_Factory {
	// Current post type
	public static $post_type 				= 'post';
	public static $short_controller_slug	= '';
	public static $act_tables    			= array();
	public static $act_pod_tables 			= array();
	public static $styles    				= array();
	public static $scripts   				= array( 'jquery', 'jquery-touch-punch' );
	public static $php_files  				= array();
	public static $simpods_caps   			= array();
	// Additional fields for pods table and functions
	public static $custom_fields 		= array(
		'sp_created'         => array( 'type' => 'datetime', 'settings' => 'NULL', 'default' => 'NULL' ),
		'sp_modified'        => array( 'type' => 'datetime', 'settings' => 'NULL', 'default' => 'NULL' ),
		'sp_start'           => array( 'type' => 'datetime', 'settings' => 'NULL', 'default' => 'NULL' ),
		'sp_end'             => array( 'type' => 'datetime', 'settings' => 'NULL', 'default' => 'NULL' ),
		'sp_modified_author' => array( 'type' => 'int(11)', 'settings' => 'NOT NULL', 'default' => '' ),
		'sp_author'          => array( 'type' => 'int(11)', 'settings' => 'NOT NULL', 'default' => '' ),
		'sp_status'          => array( 'type' => 'varchar(25)', 'settings' => 'NOT NULL', 'default' => 'DEFAULT "publish"' ),
	);
	// Save all files included by this theme
	public static $included 	= array();
	// Save all files included by this theme
	public static $theme_slug 	= 'simpods-mvc';
	// Include all main classes for child classes
	private static $main_classes = array( 'MVC_Admin_General', 'MVC_Admin_Parent_Cross_Admin', 'MVC_Admin_Settings_Helper' );
	private static $models = array();
	// Include all classes
	public static $classes = array();
	// Classes need to initialise
	private static $core_init = array(
		'mvc_admin_parent_cross_admin', // stay at the top
		'mvc_admin_simpods_user_code',
		'mvc_admin_simpods_settings',
		'mvc_admin_simpods_home',
		'mvc_admin_pods_files',
		'mvc_admin_pods',
		'mvc_admin_simpods_linked_areas',
		'mvc_admin_pods_header_field',
		'mvc_admin_pods_button_field',
		'mvc_admin_simpods_guest_accounts',
		'mvc_admin_pods_area_field',
		'mvc_admin_simpods_hierarchy',
//		'mvc_admin_simpods_install_update',
		'mvc_admin_simpods_style_script',
		'mvc_admin_simpods_custom_controllers',
		'mvc_admin_simpods_email',
	);
	public static $init_classes = array();
	public static $menu_items = array();
	private static $submenu_items = array();
	final public static function build( $class_to_build = '' ) {
		global $funs, $wp_filesystem, $wp_roles;
		$tables          		= unserialize( SIMPODS_ALL_TABLES );
		self::$act_tables    	= $tables['act_tables']; //array_merge( $tables['act_tables'], $tables['act_pod_tables'] ) ;
		self::$act_pod_tables 	= $tables['act_pod_tables'];
		$main_classes = self::$main_classes;
		$main_classes = apply_filters( 'admin_parent_factory_main_classes', $main_classes );
		foreach ( $main_classes as $name ) {
			// Include main classes first, before loading the child classes
			$file = $funs->get_final_path( 'mvc/admin/models/', strtolower( $name ) . '.class.php', 0, 0, 1 );
			if ( $wp_filesystem->exists( $file ) ) {
				include_once $file;
			}
		}
		self::$classes = Simpods_Autoload::$classes;
		self::$classes = array_unique( self::$classes );
		self::$classes = array_filter( self::$classes, function( $class ) { return class_exists( $class ); } );
		$class_built          = '';
		if ( class_exists( $class_to_build ) ) {
			$class_built = new $class_to_build();
		}
		self::$init_classes = apply_filters( 'simpods_mvc_admin_init_classes', self::$init_classes );
		
		self::$init_classes = array_merge( self::$core_init, self::$init_classes );
		self::$init_classes = array_unique( self::$init_classes );
		$submenu_items 	= array();
		$menu_items    	= array();
		//$role_cla		= get_role( 'administrator' );
		
		foreach ( self::$classes as $class ) {
			$reflection  = new ReflectionClass( $class );
			$valid_class = !$reflection->isAbstract() && !$reflection->isInterface() && $reflection->isInstantiable();
			if ( $valid_class ) {
				$page_slug = self::fallback_const( $class, 'PAGE_SLUG', 'page_str' );
				$page_cap = self::fallback_const( $class, 'PAGE_CAP', 'cap_str' );
				if ( $page_slug && $page_cap ) {
					if ( $page_cap !== '' && $page_slug !== '' ) {
						if ( !in_array( $page_cap, self::$simpods_caps ) ) {
							self::$simpods_caps[] = array( 'cap' => $page_cap, 'slug' => $page_slug );
							ksort( self::$simpods_caps );
						}
						
						$wp_roles->add_cap( 'administrator', $page_cap );
					}
					$page_title = self::fallback_static( $class, 'page_title', 'title_str' );
					$page_icon  = defined( $class . '::PAGE_ICON' ) ? $class::PAGE_ICON : false;
					if ( $page_title ) {
						$submenu_items[$class] = array(
							'slug' => $page_slug,
							'cap'  => $page_cap,
							'title' => $page_title,
							'interface' => self::fallback_method( $class, 'interface', 'interface' ),
						);
						if ( current_user_can( $page_cap ) ) {
							$menu_items[$class] = array(
								'slug' => $page_slug,
								'title' => $page_title,
								'icon' => $page_icon,
							);
						}
					}
				}
				$create_db = self::fallback_method( $class, 'create_db', 'create_db' );
				if ( $create_db ) {
					$instance = $reflection->newInstanceWithoutConstructor();
					$instance->$create_db();
				}
				$init_class = in_array( $class, self::$init_classes ) || ( defined( $class . '::INIT_CLASS' ) && $class::INIT_CLASS == true );
				if ( $reflection->isSubclassOf( 'MVC_Admin_General' ) && $class_to_build != $class && $init_class ) {
					// if extended by the general class and not the main one, run the class with init
					$GLOBALS[ $class . '' ]	= new $class(); // Give auto instantiated classes names to reduce potential re-instantiations
					
				}
			}
		}
		self::$submenu_items 	= self::order_menu( $submenu_items );
		self::$menu_items 		= self::order_menu( $menu_items );
		add_action( 'admin_menu', 'MVC_Admin_Factory::simpods_admin_menu' );
		return $class_built;
	}
	private static function order_menu( $items ) {
		$core_models = array(
			'mvc_admin_simpods_home',
			'mvc_admin_simpods_settings',
			'mvc_admin_simpods_style_script',
			'mvc_admin_simpods_custom_controllers',
			'mvc_admin_simpods_hierarchy',
			'mvc_admin_simpods_linked_areas',
			'mvc_admin_simpods_guest_accounts',
			'mvc_admin_simpods_user_code',
			'mvc_admin_simpods_install_update',
		);
		ksort( $items );
		$items = array_merge( array_flip( $core_models ), $items );
		$items = array_filter( $items, function( $val ) {
			return is_array( $val );
		} );
		return $items;
	}
	public static function simpods_admin_menu() {
		foreach ( self::$submenu_items as $class => $details ) {
			$reflection = new ReflectionClass( $class );
			$instance = $reflection->newInstanceWithoutConstructor();
			if ( $details['slug'] == self::$theme_slug ) { // Create a menu item Simpods MVC in the menu, only one.
				add_menu_page( $details['title'], $details['title'], $details['cap'], $details['slug'], array( $instance, $details['interface'] ), 'dashicons-sim-logo' );
			} else { // Others as sub menu items under Simpods MVC
				add_submenu_page( self::$theme_slug, $details['title'], $details['title'], $details['cap'], $details['slug'], array( $instance, $details['interface'] ) );
			}
		}
	}
	private static function fallback_const( $class, $new_const, $old_const ) {
		if ( defined( $class . '::' . $new_const ) ) {
			return constant( $class . '::' . $new_const );
		} else if ( defined( $class . '::' . $old_const ) ) {
			// _simpods_deprecated_message();
			return constant( $class . '::' . $old_const );
		}
		return false;
	}
	private static function fallback_method( $class, $new_method, $old_method ) {
		if ( method_exists( $class, $new_method ) ) {
			return $new_method;
		} else if ( method_exists( $class, $old_method ) ) {
			// _simpods_deprecated_message();
			return $old_method;
		}
		return false;
	}
	private static function fallback_static( $class, $new_static, $old_static ) {
		if ( property_exists( $class, $new_static ) ) {
			return $class::$$new_static;
		} else if ( property_exists( $class, $old_static ) ) {
			// _simpods_deprecated_message();
			return $class::$$old_static;
		}
		return false;
	}
}
