<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Pods Files Field class
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 01/03/2016
 */
class MVC_Admin_Pods_Files extends MVC_Admin_General {

	const PAGE_SLUG  = 'simpods-pods-files';
	const FIELD = 'simpodspodsfile';

	public static $page_title = 'Pods Files Field';

	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor
	 */
	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Pods Files Field', 'simpods-mvc' );

		// carry on if it is on the right section
		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_PODS_FILES' ) ) {
				define( 'SIMPODS_PODS_FILES', true );
			}

			$this->actions();
			$this->filters();
		}

	}

	public function actions() {
		add_action( 'admin_init', array( $this, 'register' ) );

	}

	public function filters() {}

	public function enqueue() {}

	/**
	 * to_submenu_fn: add this to the submenu of Simpods
	 */
	public function to_submenu() {}

	/**
	 * interface_fn: add an interface of this
	 */
	public function interface() {
		$is_allowed = true;

		if ( !current_user_can( 'edit_posts' ) ) {
			// action before the iframe
			$is_allowed = false;
		}

		// allow coder to set permission
		$is_allowed = apply_filters( self::PAGE_SLUG . '_allow', $is_allowed, $_GET );

		if ( !$is_allowed ) {
			echo apply_filters( self::PAGE_SLUG . '_allow_msg', esc_html__( 'You do not have permission to edit this item.', 'simpods-mvc' ) );
			return '';
		}		

		?>
		<div class="wrap">
			<h1><?php echo self::$page_title ?></h1>
			<div id="welcome-panel" class="welcome-panel">
				<h2><?php esc_html_e( 'Pods Tables', 'simpods-mvc' ); ?></h2>
				<br/>
				Alternative way to save files to Pods
			</div>

		</div>
<?php

	}

	/**
	 * Initializes the MVC_Admin_Pods_Files() class
	 *
	 * Checks for an existing MVC_Admin_Pods_Files() instance
	 * and if it doesn't find one, creates it.
	 *
	 * @since 1.0.0
	 * @deprecated
	 */
	public static function init() {
		static $this_cla = false;

		if ( !$this_cla ) {
			$this_cla = new self( false );
		}

		return $this_cla;

	}

	/**
	 * Initialize class, if Pods is active.
	 *
	 * @since 1.0.0
	 */
	public function register() {
		// if ( defined( 'PODS_VERSION' ) ) {
		// 	$GLOBALS['MVC_Admin_Pods_Files'] = self::init();
		// }

		if ( function_exists( 'pods_register_field_type' ) ) {
			pods_register_field_type( self::FIELD, dirname( __FILE__ ) . '/pods-fields/podsfield-' . self::FIELD . '.php' );
		}
	}

	/**
	 * Creates files/directories and protect them
	 *
	 * @param string  $folder     The folder to change place items
	 * @param string  $protection  How to protect the items
	 * @param boolean $is_local_allowed deprecated Whether to allow local or not
	 * @return null
	 */
	public function create_files( $folder, $protection = '' ) {
		global $pagenow, $funs, $simpods_global;
		// Install files and folders for uploading files and prevent hotlinking
		$upload_dir = wp_upload_dir();

		$files  = array(

			array(
				'base'    => $upload_dir['basedir'] . '/' . $folder,
				'file'    => '.htaccess',
				'content' => '',
			)		
		);

		$folders = explode( '/', $folder );
		// may not be array any more? Check.

		if ( 'no_external_access' == $protection ) { 

			$doamin 	= $funs->get_domain();
			$ip_server 	= $_SERVER['SERVER_ADDR']; 
			$files[0]['content'] = '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://www.' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://www.' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://www.' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://www.' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteRule ^ /index.php [NC,L]' . PHP_EOL . '</IfModule>';
		}
		if( 'patterned_code_in_url' == $protection ){
			$pattern_params = $simpods_global->get_simpods_patterned_code_for_url();
			if( ! empty( $pattern_params['pattern_data'] ) ){
				$pattern_option = $pattern_params['pattern_data'];
				if( ! empty( $pattern_option['pattern'] ) ){
					$files[0]['content'] = '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL . 'RewriteCond %{QUERY_STRING} !^code=' . $pattern_option['pattern'] . '$' .PHP_EOL . 'RewriteRule ^ /index.php [NC,L]' . PHP_EOL . '</IfModule>';
				}
			}
		}

		global $wp_filesystem;

		if ( ! $wp_filesystem->is_dir( $upload_dir['basedir'] . '/' . $folder ) ){
			$wp_filesystem->mkdir( $upload_dir['basedir'] . '/' . $folder );
		}

		foreach ( $files as $file ) {
			if( $file['content'] != '' ){
				$base 		= str_replace( ABSPATH, $wp_filesystem->abspath(), $file['base'] );
				$file_path 	= trailingslashit( $base ) . $file['file'];

				if ( $wp_filesystem->exists( $base ) ){
					if ( $wp_filesystem->exists( $file_path ) ){
						$wp_filesystem->delete( $file_path );
					}
				} else {
					$wp_filesystem->mkdir( $base );
				}

				$wp_filesystem->put_contents( $file_path, $file['content'], FS_CHMOD_FILE );
			}
		}

	}

	/**
	 * Protect downloads from ms-files.php in multisite. understand it later
	 *
	 * @param mixed   $rewrite
	 * @return string
	 */
	/*public function simpods_files_rewite_rules( $rewrite ) {

		$rule = "\n# WooCommerce Rules - Protect Files from ms-files.php\n\n";
		$rule .= "<IfModule mod_rewrite.c>\n";
		$rule .= "RewriteEngine On\n";
		$rule .= "RewriteCond %{QUERY_STRING} file=woocommerce_uploads/ [NC]\n";
		$rule .= "RewriteRule /ms-files.php$ - [F]\n";
		$rule .= "</IfModule>\n\n";

		return $rule . $rewrite;
	}*/

}
