<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Pods Header field class
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/07/2016
 */
class MVC_Admin_Pods_Header_Field extends MVC_Admin_General {
	const PAGE_SLUG  = 'simpods-header-field';
	const FIELD = 'simpodsheaderfield';
	public static $page_title = 'Pods Header Field';
	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor
	 */
	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Pods Header Field', 'simpods-mvc' );
		// carry on if it is on the right section
		if ( $can_init ) {
			// for checking if this class is activated
			if ( !defined( 'SIMPODS_HEADER_FIELD' ) ) {
				define( 'SIMPODS_HEADER_FIELD', true );
			}
			$this->actions();
			$this->filters();
		}
	}
	public function actions() {
		add_action( 'admin_init', array( $this, 'register' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );
	}
	public function filters() {}
	public function enqueue() {
		global $funs;
		$files = array(
			'pods-header-field-css' => '/mvc/admin/css/pods-header-field.css',
		);
		$funs->include( $files, false );
	}
	public function panel(){}
	/**
	 * to_submenu_fn: add this to the submenu of Simpods
	 */
	public function to_submenu() {}
	/**
	 * interface_fn: add an interface of this
	 */
	public function interface() {
		$is_allowed = true;
		if ( !current_user_can( 'edit_posts' ) ) {
			// action before the iframe
			$is_allowed = false;
		}
		// allow coder to set permission
		$is_allowed = apply_filters( self::PAGE_SLUG . '_allow', $is_allowed, $_GET );
		if ( !$is_allowed ) {
			echo apply_filters( self::PAGE_SLUG . '_allow_msg', esc_html__( 'You do not have permission to edit this item.', 'simpods-mvc' ) );
			return '';
		}
		$tables = MVC_Admin_Factory::$act_tables;
		?>
		<div class="wrap">
			<h1><?php echo self::$page_title ?></h1>
			<div id="welcome-panel" class="welcome-panel">
				<h2><?php esc_html_e( 'Pods Tables', 'simpods-mvc' ); ?></h2>
				<br/>
				<table class="w100">
					<?php
					foreach ( $tables as $table_data ) {
						echo '<tr>';
						foreach ( $table_data as $key => $value ) {
							echo '<th><span  class="alignleft">' . str_replace( '_', ' ', strtoupper( $key ) ) . '</span></th>';
						}
						echo '</tr>';
						break;
					}
					foreach ( $tables as $table_data ) {
						echo '<tr>';
						foreach ( $table_data as $key => $value ) {
							echo '<td>' . $value . '</td>';
						}
						echo '</tr>';
					}
					?>
				</table>
			</div>
		</div>
<?php
	}
	/**
	 * Initializes the MVC_Admin_Pods_Header_Field() class
	 *
	 * Checks for an existing MVC_Admin_Pods_Header_Field() instance
	 * and if it doesn't find one, creates it.
	 *
	 * @since 1.0.0
	 * @deprecated
	 */
	public static function init() {
		static $this_cla = false;
		if ( !$this_cla ) {
			$this_cla = new self( false );
		}
		return $this_cla;
	}
	/**
	 * Registers class, if Pods is active.
	 *
	 * @since 1.0.0
	 */
	public function register() {
		// if ( defined( 'PODS_VERSION' ) ) {
		// 	$GLOBALS['MVC_Admin_Pods_Header_Field'] = self::init();
		// }
		if ( function_exists( 'pods_register_field_type' ) ) {
			pods_register_field_type( self::FIELD, dirname( __FILE__ ) . '/pods-fields/podsfield-' . self::FIELD . '.php' );
		}
	}
}
