<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Pods class
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/05/2015
 */
class MVC_Admin_Pods extends MVC_Admin_General {

	const PAGE_SLUG  = '';
	const DB_TABLE    = '';	
	public $field_types = array('avatar','boolean','code','color', 'currency','date','datetime','email','file','html','number','paragraph','password','phone','pick','slug','taxonomy','text','time','website','wysiwyg','simpodsareafield', 'simpodsheaderfield', 'simpodspodsfile');


	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor
	 */
	public function __construct( $can_init = true ) {

		// carry on if it is on the right section
		if ( $can_init ) {

			$this->actions();
			$this->filters();

		}

	}

	public function actions() {}

	public function filters() {
		add_filter( 'pods_admin_setup_edit_field_options', array( $this, 'extra_options' ), 10, 2 );
		add_filter( 'pods_meta_post_fields', array( $this, 'filter_fields' ), 10, 5 );
		add_filter( 'pods_meta_meta_post_fields', array( $this, 'filter_fields' ), 10, 2 );
		add_filter( 'pods_form_ui_field_pandarepeaterfield', array( $this, 'toggle_fields' ), 10, 6 );
		add_filter( 'pods_form_ui_field_simpodspodsfile', array( $this, 'toggle_fields' ), 10, 6 );

		for ( $i = 0; $i < count( $this->field_types ); $i++ ) {
			add_filter( 'pods_form_ui_field_' . $this->field_types[$i], array( $this, 'toggle_fields' ), 10, 6 );
		}

	}

	public function enqueue() {}

	/**
	 * to_submenu_fn: add this to the submenu of Simpods
	 */
	public function to_submenu() {}

	/**
	 * interface_fn: add an interface of this
	 */
	public function interface() {}

	/**
	 * Check if the pods table contains the keys, if missing add them in
	 *
	 * @return null
	 */
	/*public function add_keys_to_db() {
		global $funs;
		if ( isset( $_GET['page'] ) && $_GET['page'] == 'pods' && isset( $_GET['id'] ) && is_numeric( $_GET['id'] ) ) {
			// check if it is a pods table
			$posts_arr = $funs->simpods_select( array( 'ID' => $_GET['id'] ), array( 'limit' => 1 ) );
		}

	}*/

	/**
	 * Adds simpods_post_ids and simpods_post_condition fields to pods fields for filter_fields_fn
	 *
	 * @param array  $options The options to alter
	 * @param object $pods    The Pods object
	 * @return array
	 */
	public function extra_options( $options, $pods ) {

		foreach ( $options as $key => $opts ) {

			if ( $key == 'advanced' ) {

				$options[$key]['Simpods'] = array(
					'simpods_post_ids'  => array(
						'name'    => 'simpods_post_ids',
						'label'   => esc_html__( 'With These Post IDs Only', 'simpods-mvc' ),
						'help'    => esc_html__( 'Enter post ids with commas: e.g. 1,2,3,4,5', 'simpods-mvc' ),
						'type'    => 'text',
						'default' => '',
					),
					'simpods_condition' => array(
						'name'    => 'simpods_post_condition',
						'label'   => esc_html__( 'Condition', 'simpods-mvc' ),
						'help'    => esc_html__( 'Display this field based on the value from other fields. pod_field_name|value(|value|...). One condition per line', 'simpods-mvc' ),
						'type'    => 'paragraph',
						'default' => '',
					),
					'simpods_condition_labels' => array(
						'name'    => 'simpods_condition_labels',
						'label'   => esc_html__( 'Dynamic labels based on the condition.', 'simpods-mvc' ),
						'help'    => esc_html__( 'Display this field\'s label based on the value from other fields. pod_field_name|value|label. One condition per line', 'simpods-mvc' ),
						'type'    => 'paragraph',
						'default' => '',
					),		
					'simpods_condition_descriptions' => array(
						'name'    => 'simpods_condition_descriptions',
						'label'   => esc_html__( 'Dynamic description based on the condition.', 'simpods-mvc' ),
						'help'    => esc_html__( 'Display this field\'s description based on the value from other fields. pod_field_name|value|description. One condition per line', 'simpods-mvc' ),
						'type'    => 'paragraph',
						'default' => '',
					),									
				);
			}

		}

		return $options;
	}

	/**
	 * Filter fields for certain post ids. Required Pods to add
	 * $metabox[ 'args' ][ 'group' ][ 'fields' ] = apply_filters( 'pods_meta_' . __FUNCTION__ . '_fields', $metabox[ 'args' ][ 'group' ][ 'fields' ], $id );
	 * to classes/PodsMeta.php abvoe foreach ( $metabox[ 'args' ][ 'group' ][ 'fields' ] as $field ) in public function meta_post ( $post, $metabox );
	 *
	 * $group[ 'fields' ] = apply_filters( 'pods_meta_meta_post_fields', $group[ 'fields' ], $pod->id() );
	 * to ui/admin/form.php below
	 * if ( false === apply_filters( 'pods_meta_box_override', false, $pod, $group, $obj ) ) {
	 *
	 * Simpods adds a simpods_post_ids field to filter the fields for certain posts
	 *
	 * @param array   $fields  Fields from the current Pod metabox group
	 * @param integer $id      The post ID
	 * @param object  $post    The post object
	 * @param object  $metabox Metabox args from the current Pod metabox group
	 * @param object  $pod     The pods object
	 * @return array
	 */
	public function filter_fields( $fields, $id, $post = null, $metabox = null, $pod = null ) {

		foreach ( $fields as $key => $field_data ) {

			if ( is_array( $field_data ) ) {
				foreach ( $field_data as $sub_key => $field_details ) {
					if ( $sub_key == 'options' && isset( $field_details['simpods_post_ids'] ) && trim( $field_details['simpods_post_ids'] ) != '' ) {
						$ids_arr = explode( ',', str_replace( ' ', '', $field_details['simpods_post_ids'] ) );
						if ( !in_array( $id, $ids_arr ) ) {
							unset( $fields[ $key ] );
						}

					}

				}

			}

		}

		return $fields;
	}

	/**
	 * Toggle the field based on the Simpods Conditions set for the field
	 *
	 * @param string  $output  The HTML output
	 * @param string  $name    The name of the pods field
	 * @param string  $value   The value of the pods field
	 * @param array   $options Options for the pods field
	 * @param object  $pod     The pods object
	 * @param integer $id      The post ID
	 * @return void
	 */
	public function toggle_fields( $output, $name, $value, $options, $pod, $id ) {

		if( strpos( $name, 'pods_meta_' ) === 0 ){
			$field_name 	= substr( $name, strlen( 'pods_meta_' ) );
		} 
		if( strpos( $name, 'pods_field_' ) === 0 ){
			$field_name 	= substr( $name, strlen( 'pods_field_' ) );
		}

		$field_options = array();
		if ( isset( $pod->fields ) && is_array( $pod->fields ) ) {
			foreach ( $pod->fields as $key => $value_data ) {
				if ( $key == $field_name ) {
					// get the conditions
					$field_options = $value_data['options'] ;
					break;
				}
			}
		} else {
			return $output;
		}
		
		$custom 		= '';
		$labels 		= array();
		$descriptions 	= array();

		if ( ! empty( $field_options['simpods_condition_labels'] ) ) {
							
			$lines_to_values    	= explode( PHP_EOL, $field_options['simpods_condition_labels'] );

			foreach ( $lines_to_values as $line_to_value ) {
				$targets = explode( '|', $line_to_value );
				//field_name|condition|label
				if ( count( $targets ) >= 3 ) { // ignore anything less than 3 as they don't follow the format
					$labels[ $targets[0] ][ $targets[1] ] = esc_html( substr( $line_to_value, ( strlen( $targets[0] . $targets[1] ) + 2 ) ) );
				}
			}				
			
		}
		
		if ( ! empty(  $field_options['simpods_condition_descriptions'] ) ) {
					
			$lines_to_values    	= explode( PHP_EOL, $field_options['simpods_condition_descriptions'] );

			foreach ( $lines_to_values as $line_to_value ) {
				$targets = explode( '|', $line_to_value );
				//field_name|condition|description
				if ( count( $targets ) >= 3 ) { // ignore anything less than 3 as they don't follow the format
					$descriptions[ $targets[0] ][ $targets[1] ] = esc_html( substr( $line_to_value, ( strlen( $targets[0] . $targets[1] ) + 2 ) ) );
				}
			}
										
		}
		//foreach ( $field_options as $key => $v_ukn ) {
			if ( ! empty(  $field_options['simpods_condition'] ) ) {
				$v_ukn =  $field_options['simpods_condition'];
			//if ( 'simpods_condition' == $key ) {

				if ( ! empty( trim( $v_ukn ) ) ) {
									
					// sometimes it is pods_field sometimes it pods_meta
					$name_str 	= str_replace( '_', '-', str_replace( 'pods_field_', 'pods-form-ui-row-name-', $name ) );
					$tr_meta 	= str_replace( '_', '-', str_replace( 'pods_meta_', 'pods-form-ui-row-name-', $name ) );
					$lines_to_values    	= explode( PHP_EOL, $v_ukn );
					// merge all lines with the same target field tgothers
					$conditions = array();
					foreach ( $lines_to_values as $line_to_value ) {
						$targets 		= explode( '|', $line_to_value );
						$target_count 	= count( $targets );
						if ( $target_count >= 2 ) {
							for ( $i = 1; $i < $target_count; $i++ ) {
								$conditions[ $targets[0] ][] = $targets[ $i ];
							}						
						}
					}
					$new_conditions = array();
					foreach ( $conditions as $target => $values ) {
						// remove duplicated
						$conditions[ $target ] 	= array_unique ( $conditions[ $target ] );
						$new_conditions[] 		= $target . '|' . implode( '|',  $conditions[ $target ] );
						//$descriptions[ $target ][ $field_name ] = $options['description'];
					}

					$output .= '<script>
								jQuery(document).ready( function($) {
									 var simpods_condition_labels 		= JSON.parse(\'' . json_encode( $labels ) . '\');
									 var simpods_condition_descriptions = JSON.parse(\'' . json_encode( $descriptions ) . '\');
								
					';					
					$output     .= 'var original_label_field = $( ".' . $name_str . ' th .pods-form-ui-label" ).html();
									var original_label_meta = $( ".' . $tr_meta . ' th .pods-form-ui-label" ).html();
									 ';
					$output     .= 'var original_desc_field = $( ".' . $name_str . ' .pods-form-ui-comment" ).html();
									var original_desc_meta = $( ".' . $tr_meta . ' .pods-form-ui-comment" ).html();
									if( $( ".' . $name_str . '" ).length != 0 ) {
										if( $( ".' . $name_str . ' .pods-form-ui-comment" ).length == 0 ){ // add missing description
											$( ".' . $name_str . '" ).append("<p class=\"description pods-form-ui-comment pods-form-ui-comment-pods-field-' . $field_name . '\"></p>");
										}
									}	
									if( $( ".' . $tr_meta . '" ).length != 0 ){
										if( $( ".' . $tr_meta . ' .pods-form-ui-comment" ).length == 0 ){ // add missing description
											$( ".' . $tr_meta . '" ).append("<p class=\"description pods-form-ui-comment pods-form-ui-comment-pods-field-' . $field_name . '\"></p>");
										}
									}	
									var desc_found 	= false;		
									var label_found = false;								
									 ';		
					$condition  = '';
					$label_js  	= '';
					$desc_js	= '';
										 							 
					foreach ( $new_conditions as $line_to_value ) { //Merge all conditions with OR operators
						// add the original label and description to the simpods labels and descriptions so they can appear when it is not available
						$targets 	= explode( '|', $line_to_value );
						$target_count 	= count( $targets );
						if ( $target_count >= 2 ) {

							$picked = pods_field( $pod->pod, $id, $targets[0] );
							if( '' == $picked && isset( $pod->fields['test_user_only']['options']['default_value'] ) ){ // use default value if no values saved
								$picked	=	$pod->fields['test_user_only']['options']['default_value'];
							}
							$refined_target = str_replace( '_', '-', esc_attr( $targets[0] ) );
							$target_field 	= '#pods-form-ui-pods-field-' . $refined_target;
							$target_meta 	= '#pods-form-ui-pods-meta-' . $refined_target;						

							for ( $i = 1; $i < $target_count; $i++ ) {
								$condition .= '( $("' . $target_field . '").val() == "' . $targets[ $i ] . '" || $("' . $target_meta . '").val() == "' . $targets[ $i ] . '" )';
								
								//if ( $i < $target_count - 1 ) {
									$condition .= ' || ';									
								//}
								if( ! empty( $labels ) ){
									$label_js  .= '
									if( typeof simpods_condition_labels["' . $targets[0] . '"] != "undefined" ){
										if( typeof simpods_condition_labels["' . $targets[0] . '"]["' . $targets[ $i ] . '"] != "undefined" && ( $("' . $target_field . '").val() == "' . $targets[ $i ] . '" || $("' . $target_meta . '").val() == "' . $targets[ $i ] . '" ) ){
											$( ".' . $name_str . ' .pods-form-ui-label:first-child" ).text( simpods_condition_labels["' . $targets[0] . '"]["' . $targets[ $i ] . '"] );
											$( ".' . $tr_meta . ' .pods-form-ui-label:first-child" ).text( simpods_condition_labels["' . $targets[0] . '"]["' . $targets[ $i ] . '"] );
											label_found = true; 
										}
									};';
								}
								if( ! empty( $descriptions ) ){
									$desc_js  .= '
									if( typeof simpods_condition_descriptions["' . $targets[0] . '"] != "undefined" ){
										if( typeof simpods_condition_descriptions["' . $targets[0] . '"]["' . $targets[ $i ] . '"] != "undefined" && ( $("' . $target_field . '").val() == "' . $targets[ $i ] . '" || $("' . $target_meta . '").val() == "' . $targets[ $i ] . '" ) ){
											$( ".' . $name_str . ' .pods-form-ui-comment" ).text( simpods_condition_descriptions["' . $targets[0] . '"]["' . $targets[ $i ] . '"] );
											$( ".' . $tr_meta . ' .pods-form-ui-comment" ).text( simpods_condition_descriptions["' . $targets[0] . '"]["' . $targets[ $i ] . '"] );
											desc_found = true;
										}
									};';	
								}	
																						
							}																	
						}
					}

					$condition  = rtrim( $condition, ' || ' );
					$toggle_js	=	'											
									if( ' . $condition . ' ){
										desc_found 	= false;		
										label_found = false;					
										$( ".' . $name_str . '" ).removeClass("hidden");
										$( ".' . $tr_meta . '" ).removeClass("hidden");
										' . $label_js . '
										' . $desc_js . '
										if( label_found == false ){ // No label based on the simpods condition, restore origianl text
											$( ".' . $name_str . ' .pods-form-ui-label:first-child" ).text( original_label_field );
											$( ".' . $tr_meta . ' .pods-form-ui-label:first-child" ).text( original_label_meta );
										}
										if( desc_found == false ){ // No description based on the simpods condition, restore origianl text										
											$( ".' . $name_str . ' .pods-form-ui-comment" ).text( original_desc_field );
											$( ".' . $tr_meta . ' .pods-form-ui-comment" ).text( original_desc_meta );
										}
										
									} else {
										$( ".' . $name_str . '" ).addClass("hidden");
										$( ".' . $tr_meta . '" ).addClass("hidden");
										$( ".' . $name_str . ' .pods-form-ui-label:first-child" ).text( original_label_field );
										$( ".' . $tr_meta . ' .pods-form-ui-label:first-child" ).text( original_label_meta );
										$( ".' . $name_str . ' .pods-form-ui-comment" ).text( original_desc_field );
										$( ".' . $tr_meta . ' .pods-form-ui-comment" ).text( original_desc_meta );
									};';

					$output .= '
								var toggled_bln = false;
								$( window ).load(function() {
									' . $toggle_js . '
								} );
								
								$(document.body).on("change click", "' . $target_field . ', ' . $target_meta . '" , function(){	
															
									' . $toggle_js . '
									//toggled_bln = true;
								});	

								';
				

					$output .= '});
								</script>';
				}
			}

		//}

		return $output;
	}

	/**
	 * Delete fields from a pod. Pods doesn't delete a newly created field, 
	 * so don't run deleting field right after creating a new field
	 * @param  string  $pod_name  the pod slug
	 * @param  int 	   $pod_id    the pod id
	 * @param  array   $remove    the array of the field names
	 * @param  boolean $table_operation whether to delete the field from the database table 
	 * @since  2.2.0   04/12/2019
	 * @return array   $deleted   the details of the delete: array( 
	 * 					 		  'all_done' => true, // all deleted successfully or not
	 * 							  'deleted' => array(), // successfull or not for each field
	 * 							  )
	 */
	public function delete_fields( $pod_name, $pod_id, $remove, $table_operation = true ){

		if ( ! class_exists( 'PodsAPI' ) ) {
			return false;
		}
		if( ! is_string( $pod_name ) || ! is_numeric( $pod_id ) || ! is_array( $remove ) ){
			return false;
		}

		$pods_API 	= new PodsAPI();
		$deleted  	= array( 'all_done' => true, 'deleted' => array() );
		$pod 		= pods( $pod_name );
		$fields 	= $pod->fields();
		
		foreach ( $remove as $field_name ) {	
			if( is_string( $field_name ) && isset( $fields[ $field_name ] ) ){		
				$done = $pods_API->delete_field( array(
								'pod'    => $pod_name,
								'pod_id' => $pod_id,
								'name'   => $field_name,
							), 
							$table_operation
						);
				$deleted[ 'deleted' ][ $field_name ] = $done;
				if( ! $done ){
					$deleted[ 'all_done' ] = false;
				}
			}
		}		
		return $deleted;

	}

	/**
	 * This function is created to work with create_pods_fn in simpods_functions.class.php 
	 * @param array $pod_ids e.g. array( 'pod_name' => pod_id );
	 * @param array $pod_ids e.g. array( 'pod_name' => array( 
	 * 													'options' => array( 'option_name' => 'option_value', ) ),
	 * 													'delete_options' => array( 'option_name', ) ) 
	 * 								);
	 * @param boolean $flush_cache 	Flush pods cache to see the updates. Keep it false if you have 
	 * 								a lot of pods to update and run cache_flush_pods() at the end.
	 * @return array $pod_settings with feedback of the updates
	 */ 
	public function update_pod_options( $pod_ids = array(), $pod_settings = array(), $flush_cache = false ){

		foreach( $pod_ids as $name => $id ){
		
			if( isset( $pod_settings[ $name ] ) ){
		
				$pod_settings[ $name ][ 'pod_id' ]	= $id;	
		
				if( isset( $pod_settings[ $name ][ 'options' ] ) && is_array( $pod_settings[ $name ][ 'options' ] ) ){
					$pod_settings[ $name ][ 'options_updated' ]	=	array();
					foreach ( $pod_settings[ $name ][ 'options' ] as $option => $val ) {
						$success = update_post_meta( $id, $option, $val );
						$pod_settings[ $name ][ 'options_updated' ][ $option ] = $success;
					}
				}
				if( isset( $pod_settings[ $name ][ 'delete_options' ] ) && is_array( $pod_settings[ $name ][ 'delete_options' ] ) ){
					$pod_settings[ $name ][ 'options_deleted' ]	=	array();
					foreach ( $pod_settings[ $name ][ 'delete_options' ] as $option ) {
						$success = delete_post_meta( $id, $option );
						$pod_settings[ $name ][ 'options_deleted' ][ $option ] = $success;
					}
				}		
			}	
		}
		
		if( true === $flush_cache ){
			$pods_API 	= new PodsAPI();
			$pods_API->cache_flush_pods();
		}

		return $pod_settings;
	}
}
