<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Settings Helper
 *
 * For managing Simpods Settings tables and generating fields using Pods Framework
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Mark Wane
 * @since: 05/12/2017
 */
class MVC_Admin_Settings_Helper extends MVC_Admin_General {

	public function __construct( $can_init = true ) {

		if ( $can_init ) {
			$this->actions();
			$this->filters();
		}

	}

	public function actions() {}

	public function filters() {
		add_filter( 'the_editor', array( $this, 'wp_editor_required' ), 10, 1 );
		add_filter( 'wp_dropdown_pages', array( $this, 'wp_pages_required' ), 10, 1 );
	}

	public function enqueue() {}

	public function to_submenu() {}

	public function interface() {}

	/**
	 * Alters the output of a WYSIWYG to add a required attribute to the textarea if necessary.
	 * Hooked to "the_editor"
	 *
	 * @param string $editor The HTML for the WYSIWYG
	 * @return string The altered HTML
	 */
	public function wp_editor_required( $editor ) {

		if ( stripos( $editor, 'required-editor' ) !== false ) {
			$editor = str_replace( '<textarea', '<textarea required', $editor );
		}

		return $editor;
	}

	/**
	* Alters the output of a pages dropdown to add a required attribute to the select field if necessary.
	* Hooked to "wp_dropdown_pages"
	*
	* @param string $select The HTML for the WYSIWYG
	* @return string The altered HTML
	*/
	public function wp_pages_required( $select ) {

		if ( stripos( $select, 'required-dropdown' ) !== false ) {
			$select = str_replace( '<select', '<select required', $select );
		}

		return $select;
	}

	/**
	 * Sets up the defaults array for parsing arguments. Passes through to array_merge_recursive_distinct
	 *
	 * @param array $options The arguments to parse
	 * @return array The parsed arguments
	 */
	final private function parse_options( $options ) {
		$defaults = array(
			'type'     => 'text',
			'title'    => '',
			'subtitle' => '',
			'value'    => '',
			'options'  => array(),
			'choices'  => array(),
			'repeater' => array(),
		);

		return $this->array_merge_recursive_distinct( $defaults, $options );
	}

	private function array_merge_recursive_distinct ( array &$array1, array &$array2 ) {
		$merged = $array1;

		foreach ( $array2 as $key => &$value ) {
			if ( is_array ( $value ) && isset ( $merged [$key] ) && is_array ( $merged [$key] ) ) {
				$merged [$key] = $this->array_merge_recursive_distinct ( $merged [$key], $value );
			} else {
				$merged [$key] = $value;
			}
		}

		return $merged;
	}

	/**
	 * Creates the requested table and returns the results
	 *
	 * @param array $form_items The items to loop through
	 * @param array $settings  Settings for the table
	 * @return string The completed form
	 */
	final public function return_form( $form_items = array(), $settings = array() ) {
		$settings_defaults = array(
			'table'  => array(
				'class' => '',
				'id'    => '',
			),
			'submit' => array(
				'class' => '',
				'id'    => '',
				'label' => '',
			),
			'form'   	=> false,
			'tabs'   	=> false,
			'nonce'  	=> '',
			'action' 	=> '',			
		);

		if ( isset( $settings['form'] ) && is_array( $settings['form'] ) ) {
			$settings_defaults['form'] = array(
				'id'     => '',
				'class'  => '',
				'method' => 'post',
				'action' => '',
				'enctype' => 'multipart/form-data',
			);
		}

		$settings = $this->array_merge_recursive_distinct( $settings_defaults, $settings );

		ob_start();

		if ( isset( $settings['form'] ) && is_array( $settings['form'] ) ) {
			$form_attrs = ' action="' . $settings['form']['action'] . '" method="' . $settings['form']['method'] . '"  class="simpods-settings ' . $settings['form']['class'] . '" id="' . $settings['form']['id'] . '" enctype="' . $settings['form']['enctype'] . '"';
		?>
		<form <?php echo $form_attrs; ?>>
		<?php 
		}
		if ( !$settings['tabs'] ) {
			$this->generate_table( $form_items, $settings );
		} else {
			$this->generate_tabs( $settings['tabs']['items'], $settings['tabs']['id'] );
			?>
			<div class="tab-content">
				<?php
				foreach ( $settings['tabs']['items'] as $tab_name => $tab_label ) {
					$tab_sanitized = str_replace( '_', '-', $tab_name );
					$tab_settings = $settings;
					$tab_settings['submit'] = false;
					$tab_settings['table']['id'] = str_replace( '{tab}', $tab_sanitized, $tab_settings['table']['id'] );
					$tab_id = str_replace( '{tab}', $tab_sanitized, $settings['tabs']['id'] );
					$active = '';

					if ( isset( $_GET['tab'] ) ) {
						if ( $_GET['tab'] == $tab_id ) {
							$active = ' show active';
						}
					} else {
						if ( $tab_name == array_key_first( $settings['tabs']['items'] ) ) {
							$active = ' show active';
						}
					}
					?>
					<div class="tab-pane fade<?php echo $active; ?>" id="<?php echo $tab_id; ?>" role="tabpanel" aria-labelledby="nav-<?php echo $tab_id; ?>">
						<h2><?php echo $tab_label; ?></h2>
						<?php
						$this->generate_table( $form_items[$tab_name], $tab_settings );
						?>
					</div>
					<?php
				}
				if ( $settings['submit'] !== false ) {
				?>
				<button type="submit" class="btn btn-simpods-blue float-right <?php echo $settings['submit']['class']; ?>" id="<?php echo $settings['submit']['id']; ?>"><?php echo $settings['submit']['label']; ?></button>
				<?php 
				}
				?>				
				<?php wp_nonce_field( $settings['nonce'] ); ?>
				<input type="hidden" value="<?php echo $settings['action']; ?>" name="action" />

			</div>
			<?php

		}
		if ( isset( $settings['form'] ) && is_array( $settings['form'] ) ) {
		?>
		</form>
		<?php
		}
		$return = ob_get_clean();
		return $return;
	}

	/**
	 * Prints out the form. Wrapper for return_form_fn
	 *
	 * @param array $form_items The items to loop through
	 * @param array $settings  Settings for the table
	 * @return string The completed form
	 */
	final public function render_form( $form_items = array(), $settings = array() ) {
		echo $this->return_form( $form_items, $settings );
	}

	final public function generate_table( $form_items, $form_settings ) { 
		?>
		<table class="table table-equal w-100 <?php echo $form_settings['table']['class']; ?>" id="<?php echo $form_settings['table']['id']; ?>">
			<?php
			$count = 0;
			foreach ( $form_items as $name => $options ) {
				$options = $this->parse_options( $options );
				$row_css_class = '';

				if ( $count % 2 == 0 ) {
					$row_css_class .= ' striped';
				}

				$count++;

				$type  = $options['type'];

				if ( $type == 'header' ) {
					$row_css_class .= ' header-row';
				}

			?>
			<tr id="<?php echo $name; ?>-row" class="<?php echo $row_css_class; ?>">
				<?php if ( $type == 'header' ) { ?>
				<th class="px-2 py-3 bg-white" colspan="2">
					<?php $this->header_field( $options, true ); ?>
				</th>
				<?php } else { ?>
				<td class="px-2 py-3">
					<?php $this->header_field( $options ); ?>
				</td>
				<td class="px-2 py-3">
					<div class="form-group mb-0">
						<?php
						switch ( $type ) {
							case 'select':
								$this->select_field( $name, $options );
								break;
							case 'toggle':
								$this->toggle_field( $name, $options );
								break;
							case 'check':
							case 'checkbox':
							case 'radio':
								if ( $type == 'check' ) {
									$this->check_field( $name, $options, 'checkbox' );
								} else {
									$this->check_field( $name, $options, $type );
								}
								break;
							case 'file':
								$this->file_field( $name, $options );
								break;
							case 'wysiwyg':
								$this->wysiwyg_field( $name, $options );
								break;
							case 'page':
								$this->page_field( $name, $options );
								break;
							case 'role':
								$this->role_field( $name, $options );
								break;
							case 'textarea':
								$this->textarea_field( $name, $options );
								break;
							case 'button':
							case 'submit':
								$this->button_field( $name, $options, $type );
								break;
							case 'date':
							case 'time':
							case 'date-time':
								$this->date_field( $name, $options, $type );
								break;
							case 'color':
								$this->color_field( $name, $options );
								break;
							case 'code':
								$this->code_field( $name, $options );
								break;
							case 'repeater':
								$this->repeater_field( $name, $options );
								break;
							case 'repeater':
								$this->number_field( $name, $options );
								break;																
							default:
								$this->default_field( $name, $options, $type );
								break;
						}

						if ( isset( $options['errors'] ) && is_array( $options['errors'] ) ) {
							$id = $name;

							if ( isset( $options['options']['id'] ) ) {
								$id = $options['options']['id'];
							}

							foreach ( $options['errors'] as $error_id => $message_str ) {
								$error_id = $id . '-' . $error_id;
						?>
						<span id="<?php echo $error_id ?>" class="form-feedback d-none"><?php echo $message_str; ?></span>
						<?php
							}
						}
						?>
					</div>
				</td>
				<?php } ?>
			</tr>
			<?php } ?>

			<?php if ( $form_settings['submit'] !== false ) { ?>
			<tr class="bg-white">
				<td colspan="2" class="px-2 py-3">
					<button type="submit" class="btn btn-simpods-blue float-right <?php echo $form_settings['submit']['class']; ?>" id="<?php echo $form_settings['submit']['id']; ?>"><?php echo $form_settings['submit']['label']; ?></button>
				</td>
			</tr>
			<?php } ?>			
			<tr class="d-none">
				<td colspan="2" class="px-2 py-3">
					<?php 
					wp_nonce_field( $form_settings['nonce'] );
					?>
					<input type="hidden" value="<?php echo $form_settings['action']; ?>" name="action" />
				</td>
			</tr>

		</table>
		<?php
	}

	final public function generate_tabs( $tabs, $id ) {
		?>
		<nav>
			<div class="nav nav-tabs" role="tablist">
				<?php 
				foreach ( $tabs as $tab_name => $tab_label ) {
					$tab_sanitized = str_replace( '_', '-', $tab_name );
					$tab_id        = str_replace( '{tab}', $tab_sanitized, $id );

					$active = '';

					if ( isset( $_GET['tab'] ) ) {
						if ( $_GET['tab'] == $tab_id ) {
							$active = ' active';
						}
					} else {
						if ( $tab_name == array_key_first( $tabs ) ) {
							$active = ' active';
						}
					}
				?>
				<a class="nav-item nav-link<?php echo $active; ?>" id="nav-<?php echo $tab_id; ?>"
					href="#<?php echo $tab_id; ?>" data-toggle="tab" role="tab"
					aria-controls="<?php echo $tab_id; ?>" aria-selected="true">
					<?php echo $tab_label; ?>
				</a>
				<?php
				}
				?>
			</div>
		</nav>
		<?php
	}

	/**
	 * Creates an attribute string for fields
	 *
	 * @param string $name   The name of the field
	 * @param array $options The options for the field
	 * @return string The field attributes
	 */
	final private function atts( $name = '', $options = array() ) {
		$attrs    = '';
		$css_class   = '';
		$id      = '';
		$options = $this->parse_options( $options );

		switch ( $options['type'] ) {
			case 'select':
				$css_class .= 'form-control custom-select';
				$id = $name;
				break;
			case 'check':
			case 'checkbox':
			case 'radio':
			case 'role':
				$css_class .= 'custom-control-input';
				$id    = '';
				break;
			case 'toggle':
			case 'wysiwyg':
			case 'page':
				$css_class .= '';
				$id    = $name;
				break;
			case 'submit':
			case 'button':
				$css_class .= 'btn';
				$id    = $name;
				break;
			case 'code':
				$css_class .= 'form-control code-editor';
				$id    = $name;
				break;
			case 'range':
				$css_class .= 'custom-range';
				$id		= $name;
				break;
			default:
				$css_class .= 'form-control';
				$id    = $name;
				break;
		}

		if ( isset( $options['options'] ) ) {
			if ( isset( $options['options']['class'] ) ) {
				$css_class .= ' ' . $options['options']['class'];
			}

			if ( isset( $options['options']['id'] ) ) {
				$id = $options['options']['id'];
			}

			if ( isset( $options['options']['min'] ) && is_numeric( $options['options']['min'] ) ) {
				$attrs .= ' min="' . $options['options']['min'] . '"';
			}

			if ( isset( $options['options']['max'] ) && is_numeric( $options['options']['max'] ) ) {
				$attrs .= ' max="' . $options['options']['max'] . '"';
			}

			if ( isset( $options['options']['step'] ) && is_numeric( $options['options']['step'] ) ) {
				$attrs .= ' step="' . $options['options']['step'] . '"';
			}

			if ( isset( $options['options']['placeholder'] ) ) {
				$attrs .= ' placeholder="' . $options['options']['placeholder'] . '"';
			}

			$date_types = array( 'date', 'time', 'date-time' );

			if ( in_array( $options['type'], $date_types ) ) {
				$css_class .= '  datetimepicker-input';
				$attrs .= ' data-type="' . $options['type'] . '"';

				if ( isset( $options['options']['format'] ) ) {
					$attrs .= ' data-value="' . $this->php_to_moment_format( $options['options']['format'] ) . '"';
				}

				$attrs .= ' data-toggle="datetimepicker" data-target="#' . $id . '-picker"';
			}

			$format_types = array( 'color', 'code' );

			if ( in_array( $options['type'], $format_types ) ) {
				if ( isset( $options['options']['format'] ) ) {
					$attrs .= ' data-format="' . $options['options']['format'] . '"';
				}
			}

			if ( isset( $options['options']['required'] ) && $options['options']['required'] ) {
				$attrs .= ' required';
			}

			if ( isset( $options['options']['disabled'] ) && $options['options']['disabled'] ) {
				$attrs .= ' disabled';
			}

			if ( isset( $options['options']['readonly'] ) && $options['options']['readonly'] ) {
				$attrs .= ' readonly';
			}
		}

		$attrs = ' class="' . $css_class . '"' . $attrs;

		if ( $options['type'] !== 'check' && $options['type'] !== 'checkbox' && $options['type'] !== 'radio' && $options['type'] !== 'role' ) {
			$attrs = ' id="' . $id . '"' . $attrs;
		}

		$attrs .= ' autocomplete="off"';

		return $attrs;
	}

	final private function nonce_field( $action, $id ) {

	}

	/**
	 * Prints out a header
	 *
	 * @param array   $options Options for the header
	 * @param boolean $strong_bln  Whether to make the header bold or not
	 * @return null
	 */
	final public function header_field( $options = array(), $strong_bln = false ) {
		$options     = $this->parse_options( $options );
		$header_css_class = 'mt-0 form-field-title';
		$sub_css_class    = 'mt-0 mb-0 form-field-subtitle';

		if ( $strong_bln ) {
			$header_css_class .= ' font-weight-bold';
			$sub_css_class    .= ' font-weight-bold';
		}

		if ( isset( $options['subtitle'] ) && $options['subtitle'] !== '' ) {
			$header_css_class .= ' mb-2';
		} else {
			$header_css_class .= ' mb-0';
		}

		?>
		<h3 class="<?php echo $header_css_class; ?>">
			<?php echo $options['title']; ?>
			<?php if ( isset( $options['reference'] ) && $options['reference'] !== '' ) { ?>
			<a href="<?php echo $options['reference']; ?>" target="_blank" title="<?php esc_html_e( 'Reference', 'simpods-mvc' ); ?>">
				<i class="far fa-question-circle"></i>
				<span class="sr-only"><?php printf(
					esc_html__( 'Reference: %1$s', 'simpods-mvc' ),
					$options['reference']
				); ?></span>
			</a>
			<?php } ?>
		</h3>

		<?php if ( isset( $options['subtitle'] ) && $options['subtitle'] !== '' ) { ?>
		<p class="<?php echo $sub_css_class; ?>"><?php echo $options['subtitle']; ?></p>
		<?php } ?>
		<?php
	}

	/**
	 * Prints out a select field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function select_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		?>
		<select name="<?php echo $name; ?>"<?php echo $attrs; ?>>
			<?php foreach ( $options['choices'] as $key => $label ) { ?>
				<option value="<?php echo $key; ?>"<?php echo ( $options['value'] == $key ? ' selected' : '' ); ?>><?php echo $label; ?></option>
			<?php } ?>
		</select>
		<?php
	}

	/**
	 * Prints out a toggle field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function toggle_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		$checked_html = $options['value'] == '1' ? ' checked' : '';
		$id      = $name;

		if ( isset( $options['options'] ) && isset( $options['options']['id'] ) ) {
			$id = $options['options']['id'];
		}

		?>
		<label for="<?php echo $id; ?>" class="toggle-container">
			<input type="checkbox" value="1" name="<?php echo $name; ?>"<?php echo $checked_html . $attrs; ?> />
			<span class="toggle<?php echo $checked_html; ?>"></span>
		</label>
		<div class="validation-icon" id="<?php echo $name; ?>-icon"></div>
		<?php
	}

	/**
	 * Prints out a checkbox/radio buton field group
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function check_field( $name = '', $options = array(), $type = 'checkbox' ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );

		if ( $type == 'checkbox' ) {
			$name .= '[]';
		}

		if ( !is_array( $options['value'] ) ) {
			$val_arr = explode( ',', $options['value'] );
			$options['value'] = array_map( 'trim', $val_arr );
		}
		$group_id = '';
		if ( ! empty( $options['options']['id'] ) ) {
			$group_id = $options['options']['id'];
		}
		foreach ( $options['choices'] as $key => $label ) {
			$checked_html = in_array( $key, $options['value'] ) ? ' checked' : '';
			$id = $name . '-' . $key;

			if ( ! empty( $group_id ) ) {
				$id = $group_id . '-' . $key;
			}
		?>
		<div class="custom-control custom-<?php echo $type; ?> checkboxes" id="<?php echo  $group_id; ?>">
			<input type="<?php echo $type; ?>" name="<?php echo $name; ?>" id="<?php echo $id; ?>" value="<?php echo $key; ?>"<?php echo $checked_html . $attrs; ?> />
			<label class="custom-control-label" for="<?php echo $id; ?>"><?php echo $label; ?></label>
		</div>
		<?php
		}
	}

	/**
	 * Prints out a file field utilising the WP Media Library
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function file_field( $name = '', $options = array() ) {
		$options 	= $this->parse_options( $options );
		$attrs    	= $this->atts( $name, $options );

		$data_type 		= $options['file_options']['type'];
		$is_multiple 	= ( isset( $options['file_options']['multiple'] ) ) ? $options['file_options']['multiple'] : false;

		if ( !is_array( $options['value'] ) ) {
			$val_arr = explode( ',', $options['value'] );
			$options['value'] = array_map( 'trim', $val_arr );
			$options['value'] = array_map( 'intval', $options['value'] );
		}
		?>
		<div class="simpods-file-field">
		<?php
		if ( !empty( $options['value'] ) ) {
			?>
			<div id="<?php echo $name; ?>-display">
			<?php
			foreach ( $options['value'] as $id_int ) {
				$images = wp_get_attachment_image_src( $id_int, 'thumbnail', false );

				if ( is_array( $images ) ) {
					$img_str = reset( $images );
					?>
					<img src="<?php echo $img_str; ?>" class="simpods_media_set_image" width="82" height="82" />
					<?php
				}
			}
			?>
			</div>
		<?php } else { ?>
			<div id="<?php echo $name; ?>-display"></div>
		<?php
		}

		$upload_text = _n( 'Upload File', 'Upload Files', intval( $is_multiple ) + 1, 'simpods-mvc' );
		$clear_text  = _n( 'Clear File', 'Clear Files', intval( $is_multiple ) + 1, 'simpods-mvc' );
		?>
			<button type="button" class="btn btn-outline-simpods-blue sim-file-upload" data-type="<?php echo $data_type; ?>" data-field="<?php echo $name; ?>" data-display="<?php echo $name; ?>-display" data-multiple="<?php echo strval( $is_multiple ); ?>"><?php echo $upload_text; ?></button>
			<button type="button" class="btn btn-outline-danger sim-file-clear" data-field="<?php echo $name; ?>" data-display="<?php echo $name; ?>-display"><?php echo $clear_text; ?></button>
			<input type="hidden" name="<?php echo $name; ?>" id="<?php echo $name; ?>" value="<?php echo implode( ',', $options['value'] ); ?>"<?php echo $attrs; ?> />
			<div class="validation-icon" id="<?php echo $name; ?>-icon"></div>
		</div>
		<?php
	}

	/**
	 * Prints out a WYSIWYG field utilising the WP TinyMCE editor
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function wysiwyg_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );

		$editor_settings = array(
			'textarea_name' => $name,
			'editor_class'  => 'form-control',
			'textarea_rows' => '10',
			'media_buttons' => ( isset( $options['options']['media_buttons'] ) && is_bool( $options['options']['media_buttons'] ) ) ? $options['options']['media_buttons'] : false,
			'teeny'         => ( isset( $options['options']['teeny'] ) && is_bool( $options['options']['teeny'] ) ) ? $options['options']['teeny'] : false,
			'quicktags'     => ( isset( $options['options']['quicktags'] ) && is_bool( $options['options']['quicktags'] ) ) ? $options['options']['quicktags'] : true,
		);

		if ( isset( $options['options'] ) ) {
			if ( isset( $options['options']['required'] ) && $options['options']['required'] ) {
				$editor_settings['editor_class'] .= ' required-editor';
			}

			if ( isset( $options['options']['class'] ) ) {
				$editor_settings['editor_class'] .= ' ' . $options['options']['class'];
			}
		}

		wp_editor( $options['value'], $name, $editor_settings );
	}

	/**
	 * Prints out a WP pages select field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function page_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );

		$id = $name;

		if ( isset( $options['options']['id'] ) ) {
			$id = $options['options']['id'];
		}

		$field_settings = array(
			'class'             => 'form-control custom-select',
			'show_option_none'  => esc_html__( 'Select One', 'simpods-mvc' ),
			'option_none_value' => '',
			'name'              => $name,
			'id'                => $id,
			'selected'          => $options['value'],
		);

		if ( isset( $options['options'] ) ) {
			if ( isset( $options['options']['required'] ) && $options['options']['required'] ) {
				$field_settings['class'] .= ' required-dropdown';
			}

			if ( isset( $options['options']['class'] ) ) {
				$field_settings['class'] .= ' ' . $options['options']['class'];
			}

			if ( isset( $options['options']['child_of'] ) ) {
				$field_settings['child_of'] = $options['options']['child_of'];
			}

			if ( isset( $options['options']['depth'] ) ) {
				$field_settings['depth'] = $options['options']['depth'];
			}

			if ( isset( $options['options']['show_option_none'] ) ) {
				$field_settings['show_option_none'] = $options['options']['show_option_none'];
			}

			if ( isset( $options['options']['option_none_value'] ) ) {
				$field_settings['option_none_value'] = $options['options']['option_none_value'];
			}
		}

		wp_dropdown_pages( $field_settings );
	}

	/**
	 * Prints out a WP roles select/checkbox field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function role_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );

		$options['type'] = 'role';

		global $wp_roles;
		$roles    	= array();
		$all_roles 	= $wp_roles->roles;

		foreach ( $all_roles as $key => $role_data ) {
			if ( $key == 'administrator' ) {
				continue;
			}
			$roles[$key] = $role_data['name'];
		}

		$options['choices'] = $roles;

		ksort( $options['choices'] );

		if ( isset( $options['options'] ) ) {

			if ( isset( $options['options']['style'] ) ) {

				if ( $options['options']['style'] == 'check' ) {
					$options['options']['style'] = 'checkbox';
				}

				if ( $options['options']['style'] == 'select' ) {
					$options['type'] = 'select';
					$this->select_field( $name, $options );
				} else {
					$this->check_field( $name, $options, $options['options']['style'] );
				}
			} else {
				$this->check_field( $name, $options, 'checkbox' );
			}

		} else {
			$this->check_field( $name, $options, 'checkbox' );
		}

	}

	/**
	 * Prints out a textarea field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function textarea_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		?>
		<textarea name="<?php echo $name; ?>"<?php echo $attrs; ?>><?php echo $options['value']; ?></textarea>
		<?php
	}

	/**
	 * Prints out a button/submit/reset element
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @param string $type    The type of button to display
	 * @return null
	 */
	final public function button_field( $name = '', $options = array(), $type = 'button' ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );

		if ( isset( $options['options']['add_name'] ) && isset( $options['options']['add_name'] ) === true ) {
			$attrs = ' name="' . $name . '"' . $attrs;
		}
		?>
		<button type="<?php echo $type; ?>"<?php echo $attrs; ?>><?php echo $options['value']; ?></button>
		<?php
	}

	/**
	 * Prints out a date/time picker field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @param string $type    The type of picker to display
	 * @return null
	 */
	final public function date_field( $name = '', $options = array(), $type = 'date' ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		$format = '';

		if ( isset( $options['options'] ) ) {

			if ( isset( $options['options']['format'] ) ) {
				$format = $options['options']['format'];
			}

		}

		if ( $format == '' ) {

			switch ( $type ) {
				case 'date':
					$format = 'Y-m-d';
					break;
				case 'time':
					$format = 'H:i:s';
					break;
				case 'date-time':
					$format = 'Y-m-d H:i:s';
					break;
			}

		}

		if ( $options['value'] !== '' ) {
			$options['value'] = date( $format, strtotime( $options['value'] ) );
		}

		$id = $name;

		if ( isset( $options['options']['id'] ) ) {
			$id = $options['options']['id'];
		}

		$id .= '-picker';
		?>
		<div class="input-group date" id="<?php echo $id; ?>" data-target-input="nearest">
			<input type="text" name="<?php echo $name; ?>" value="<?php echo $options['value']; ?>"<?php echo $attrs; ?> />
			<div class="input-group-append" data-target="#<?php echo $id; ?>" data-toggle="datetimepicker">
				<button class="btn btn-simpods-blue" type="button">
					<?php
					$icon_css_class = 'far fa-calendar-alt';

					switch ( $type ) {
						case 'time':
							$icon_css_class = 'fas fa-clock';
							break;
						default:
							$icon_css_class = 'far fa-calendar-alt';
							break;
					}

					$icon_css_class .= ' fa-fw';
					?>
					<i class="<?php echo $icon_css_class; ?>"></i>
				</button>
			</div>
		</div>
		<?php
	}

	/**
	 * Prints out a colorpicker field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function color_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );

		$id = $name;

		if ( isset( $options['options']['id'] ) ) {
			$id = $options['options']['id'];
		}
		?>
		<div class="colorpicker-container" id="<?php echo $id; ?>-cp-container">
			<div class="input-group">
				<input type="text" name="<?php echo $name; ?>" value="<?php echo $options['value'] ?>"<?php echo $attrs; ?> />
				<span class="input-group-append">
					<button type="button" class="btn btn-outline-simpods-blue colorpicker-input-addon"><i></i></button>
				</span>
			</div>
		</div>
		<?php
	}

	/**
	 * Prints out a code(syntax highlighting) field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @return null
	 */
	final public function code_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		?>
		<textarea name="<?php echo $name; ?>"<?php echo $attrs; ?>><?php echo $options['value']; ?></textarea>
		<?php
	}

	final public function repeater_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$template = $this->repeater_template( $name, $options );

		if ( !isset( $options['repeater']['single'] ) || !$options['repeater']['single'] ) {
			foreach ( $options['value'] as $key => $val ) {
				$search = array_map( function( $a ) {
					return '{value_' . $a . '}';
				}, array_keys( $val ) );

				$replace = array_map( function( $a ) {
					return $a;
				}, array_values( $val ) );

				$search = array_merge( $search, array_map( function( $a ) {
					return 'option value="' . $a . '"';
				}, array_values( $val ) ) );

				$replace = array_merge( $replace, array_map( function( $a ) {
					return 'option value="' . $a . '" selected';
				}, array_values( $val ) ) );

				$search[] = '{index}';
				$replace[] = $key;

				echo str_replace( $search, $replace, $template );
			}
		} else {
			foreach ( $options['value'] as $key => $val ) {
				echo str_replace( array( '{index}', '{value}' ), array( $key, $val ), $template );
			}
		}
		?>
		<!-- <?php echo $template; ?> -->
		<div class="w-100 text-right">
			<button class="btn btn-outline-simpods-blue add-repeater" type="button">
				<span class="dashicons dashicons-plus"></span>
				<span> Add</span>
			</button>
		</div>
		<?php
	}

	final private function repeater_template( $name = '', $options = array() ) {
		ob_start();
		?>
		<div class="input-group mb-3" data-index="{index}">
		<?php
		$field_name = $name . '[{index}]';
		
		if ( !isset( $options['repeater']['single'] ) || !$options['repeater']['single'] ) {
			foreach ( $options['repeater']['fields'] as $field => $field_options ) {
				$new_name = $field_name .  '[' . $field . ']';
				$field_options['value'] = '{value_' . $field . '}';
				$field_options['options']['id'] = str_replace( array( '[]', ']', '[' ), array( '', '', '-' ), $new_name );
				$this->repeater_switch( $new_name, $field_options );
			}
		} else {
			$field_options = $options['repeater']['fields'];
			$field_options['value'] = '{value}';
			$field_options['options']['id'] = str_replace( array( '[]', ']', '[' ), array( '', '', '-' ), $field_name );
			$this->repeater_switch( $field_name, $field_options );
		}
		?>
			<div class="input-group-append">
				<button class="btn btn-outline-danger remove-repeater" type="button">
					<span class="dashicons dashicons-trash" title="Remove"></span>
					<span class="sr-only">Remove</span>
				</button>
			</div>
		</div>
		<?php

		return ob_get_clean();
	}

	final private function repeater_switch( $field_name = '', $field_options = array() ) {
		$field_options = $this->parse_options( $field_options );
		$field_type = $field_options['type'];

		switch ( $field_options['type'] ) {
			case 'select':
				$this->select_field( $field_name, $field_options );
				break;
			// case 'page':
			// 	$this->page_field( $field_name, $field_options );
			// 	break;
			// case 'role':
			// 	$this->role_field( $field_name, $field_options );
			// 	break;
			// case 'button':
			// 	$this->button_field( $field_name, $field_options, $field_type );
			// 	break;
			default:
				$this->default_field( $field_name, $field_options, $field_type );
				break;
		}
			
	}

	/**
	 * Prints out a basic input field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @param string $type    The type of input to display
	 * @return null
	 */
	final public function default_field( $name = '', $options = array(), $type = 'text' ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		?>
		<input type="<?php echo $type; ?>" name="<?php echo $name; ?>" value="<?php echo $options['value']; ?>"<?php echo $attrs; ?> />
		<?php
	}
	/**
	 * Prints out a basic input field
	 *
	 * @param string $name    The name of the field
	 * @param array  $options Options for the field
	 * @param string $type    The type of input to display
	 * @author Dongjie Xu
	 * @return null
	 */
	final public function number_field( $name = '', $options = array() ) {
		$options = $this->parse_options( $options );
		$attrs    = $this->atts( $name, $options );
		?>
		<input type="number" name="<?php echo $name; ?>" value="<?php echo $options['value']; ?>" <?php echo $attrs; ?> />
		<?php
	}
	/**
	 * Convert PHP date format to Moment.js format
	 * @link https://stackoverflow.com/a/30192680
	 * @param  string $format The format to convert
	 * @return string The date format for use in Moment.js
	 */
	final public function php_to_moment_format( $format = '' ) {
		$replacements = [
			'd' => 'DD',
			'D' => 'ddd',
			'j' => 'D',
			'l' => 'dddd',
			'N' => 'E',
			'S' => 'o',
			'w' => 'e',
			'z' => 'DDD',
			'W' => 'W',
			'F' => 'MMMM',
			'm' => 'MM',
			'M' => 'MMM',
			'n' => 'M',
			't' => '', // no equivalent
			'L' => '', // no equivalent
			'o' => 'YYYY',
			'Y' => 'YYYY',
			'y' => 'YY',
			'a' => 'a',
			'A' => 'A',
			'B' => '', // no equivalent
			'g' => 'h',
			'G' => 'H',
			'h' => 'hh',
			'H' => 'HH',
			'i' => 'mm',
			's' => 'ss',
			'u' => 'SSS',
			'e' => 'zz', // deprecated since version 1.6.0 of moment.js
			'I' => '', // no equivalent
			'O' => '', // no equivalent
			'P' => '', // no equivalent
			'T' => '', // no equivalent
			'Z' => '', // no equivalent
			'c' => '', // no equivalent
			'r' => '', // no equivalent
			'U' => 'X',
		];
		$moment_format = strtr($format, $replacements);
		return $moment_format;
	}

	/**
	 * Outputs table rows using Pods Framework for a Simpods MVC settings table
	 * @link  https://simpodsmvc.me/docs/function-reference/functions/render_form_rows/
	 * @param   array  $fields 	Array of field object
	 * @param   string $prefix 	Prefix for id/name of form elements
	 * @param   Pods   $pod 		Corresponding pods object
	 * @return  null
	 */
	public function render_form_rows( $fields, $prefix, Pods $pod ) {
		$pod_fields = $pod->fields();
		$count 		= 0;
		foreach ( $pod_fields as $pod_field_data ) {
			$setting_obj = $fields[ $pod_field_data['name'] ];
			$row_css_class = '';

			if ( $count % 2 == 0 ) {
				$row_css_class .= ' striped';
			}

			$count++;			
			?>
			<tr class="pdt10 pdb10 simpods-setting-row pods-field-option pods-field pods-form-ui-row-type-<?php echo esc_attr( $setting_obj->type ); echo $row_css_class; ?> pods-form-ui-row-name-<?php echo esc_attr( $prefix . '-' . $setting_obj->name ); ?>">
				<td>
					<h3><?php echo PodsForm::label( $prefix . '-' . $setting_obj->name, $pod_field_data['label'] ); ?></h3>
					<p class="mgt0"><?php echo $pod_field_data['description'] ?></p>
				</td>
				<td>
				<?php
				if ( 'boolean' == $setting_obj->type ) {
					$tog_cla_str = 'toggle';

					if ( $setting_obj->value ) {
						$tog_cla_str .= ' checked';
					}

					echo '<span class="' . $tog_cla_str . '" data-checkbox="' . 'pods-form-ui-' . $prefix . '-' . str_replace( '_', '-', $setting_obj->name ) . '"></span>';
				}

				echo PodsForm::field( $prefix . '-' . $setting_obj->name, $setting_obj->value, $setting_obj->type, $pod_field_data, $pod->pod_data, $pod->pod_id );
				?>
				</td>
			</tr>
			<?php
		}

	}

	/**
	 * Get non-hidden rows from a Simpods MVC settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_settings_for_form_fn/
	 * @param  string $table 	Name of table (without table_prefix)
	 * @return array 				Associate array of row objects keyed by name
	 */
	public function get_settings_for_form( $table ) {
		global $funs;
		$results = $funs->simpods_select(
			array(),
			array(
				'target_tb' => $table,
				'search'    => 'AND `type` NOT LIKE "hidden"',
			)			
		);

		$returns = array();

		foreach ( $results as $row ) {
			$row        = (object) $row;
			$row->value = maybe_unserialize( $row->value );

			$returns[$row->name] = $row;

			wp_cache_set( $row->name, $row->value, $table );
		}

		return $returns;
	}

	/**
	 * Get rows from a Simpods MVC settings table by prefix
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_settings_by_prefix/
	 * @param  string 	$table 		Name of table (without table_prefix)
	 * @param  string 	$prefix 	Prefix of setting name
	 * @param  int|null	$user_id 		Not used (for user settings only)
	 * @return array 					Associate array of row objects keyed by name
	 */
	public function get_settings_by_prefix( $table, $prefix, $user_id = null ) {
		global $funs;

		$results = $funs->simpods_select(
			array( 'name' => $prefix . '%' ),
			array( 'target_tb' => $table )
		);

		$returns = array();

		foreach ( $results as $row ) {
			$row        = (object) $row;
			$row->value = maybe_unserialize( $row->value );

			$returns[$row->name] = $row;

			wp_cache_set( $row->name, $row->value, $table );
		}

		return $returns;
	}

	/**
	 * Insert a hidden setting row in a Simpods MVC settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/update/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  string 	$name 	Name of option to insert
	 * @param  mixed 	$value 		Value of option
	 * @param  int|null	$user_id 	Not used (for user settings only)
	 * @return null
	 */
	public function update( $table, $name, $value, $user_id = null ) {
		global $funs;
		$funs->general_update(
			$table,
			array( 'value' => maybe_serialize( $value ) ),
			array( 'name' => $name )
		);

		wp_cache_set( $name, $value, $table );
	}

	/**
	 * Update a row in a Simpods MVC settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/insert/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  string 	$name 	Name of option to insert
	 * @param  mixed 	$value 		Value of option
	 * @param  int|null	$user_id 	Not used (for user settings only)
	 * @return null
	 */
	public function insert( $table, $name, $value, $user_id = null ) {
		global $funs;
		$funs->general_insert(
			$table,
			array(
				'name'  => $name,
				'type'  => 'hidden',
				'value' => maybe_serialize( $value ),
			)
		);

		wp_cache_set( $name, $value, $table );
	}

	/**
	 * Remove a setting row from a Simpods MVC settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/delete/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  string 	$name 	Name of option to insert
	 * @param  int|null $user_id 	Not used (for user settings only)
	 * @return null
	 */
	public function delete( $table, $name, $user_id = null ) {
		global $funs;
		$funs->simpods_delete(
			$table,
			array( 'name' => $name )
		);

		wp_cache_delete( $name, $table );
	}

}
