<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Custom Controllers class, sometimes, the template path go overwriten by plugins or other code, log them here in order to load the proper controller
 *
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 1.1.4 31/10/2017
 */
class MVC_Admin_Simpods_Custom_Controllers extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-custom-controllers';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_custom_controllers';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_custom_controllers';
	const PAGE_ICON = 'dashicons dashicons-admin-tools';
	// The page's title to display in-page
	public static $page_title = 'Custom Controllers';
	/* End Simpods MVC page variables */

	// databse table version
	private $database_version = 1.0;

	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Custom Controllers', 'simpods-mvc' );

		// carry on if it is on the right section
		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_CUSTOM_CONTROLLERS' ) ) {
				define( 'SIMPODS_CUSTOM_CONTROLLERS', true );
			}

			$this->create_db();
			$this->actions();
			$this->filters();

		}

	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}

	/**
	 * Scripts & styles to register & enqueue when the admin controller is loaded
	 * @return null
	 */
	public function enqueue() {}

	/**
	 * Create the database for this area
	 * @return null
	 */
	public function create_db() {

		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->database_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}
		$columns = array(
			'spcc_where' => array(
				'type'     => 'varchar(20)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spcc_look_up' => array(
				'type'     => 'varchar(500)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spcc_controller_path' => array(
				'type'     => 'varchar(500)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spcc_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spcc_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spcc_modified_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spcc_status' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "publish"',
			),
			'spcc_note' => array(
				'type'     => 'text',
				'settings' => '',
				'default'  => '',
			),
		);
		$this->import_database( self::DB_TABLE, $columns );

		update_option( self::DB_TABLE . '_version', $this->database_version );
	}

	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $ctrl, $cross_admin, $funs;

		$settings_helper = new MVC_Admin_Settings_Helper();

		$ctrl->forms();
		$ctrl->notice .= $this->simpods_admin_remove_item( self::DB_TABLE );

		$records = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
		//$count_data   = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE, 'count_only' => true ) );
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
				<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1><?php echo self::$page_title ?></h1>
					<?php echo $ctrl->notice; ?>
					<div id="simpods-cc-settings">
						<h2><?php esc_html_e( 'Settings', 'simpods-mvc'); ?></h2>
						<?php $settings_helper->render_form( $ctrl->main_form_items, $ctrl->main_form_settings ); ?>
					</div>
					<hr>
					<div id="simpods-cc-add">
						<h2><?php esc_html_e( 'Add Controllers', 'simpods-mvc'); ?></h2>
						<?php $settings_helper->render_form( $ctrl->add_form_items, $ctrl->add_form_settings ); ?>
					</div>

					<?php if ( is_array( $records ) && !empty( $records ) ) { ?>
					<hr>
					<div id="simpods-cc-table">
						<?php
						foreach ( $records as &$record_data ) {
							$record_data = $funs->escape_attrs( $record_data );

							$record_data['spcc_where-formatted'] = ucfirst( $record_data['spcc_where'] );
							$record_data['spcc_note-formatted']  = str_replace( '\r\n', ' ', $record_data['spcc_note'] );
						}

						$headers = array(
							'spcc_where-formatted' => esc_html__( 'Area', 'simpods-mvc' ),
							'spcc_look_up'         => esc_html__( 'URL Path', 'simpods-mvc' ),
							'spcc_controller_path' => esc_html__( 'Controller Path', 'simpods-mvc' ),
							'spcc_note-formatted'  => esc_html__( 'Note', 'simpods-mvc' ),
						);

						$edit_values = array(
							'spcc_where',
							'spcc_look_up',
							'spcc_controller_path',
							'spcc_note',
							'id',
						);

						$edit_settings = array(
							'id-prefix' => 'simpods-cc',
							'action'    => admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ),
							'title'     => esc_html__( 'Edit Custom Controller', 'simpods-mvc' )
						);
						echo $this->simpods_table( $headers, $records, self::PAGE_SLUG, true, $edit_values, $edit_settings );
						?>
					</div>
					<?php } ?>
					<script>var wp_errors = <?php echo is_wp_error( $ctrl->wp_errors ) ? wp_json_encode( $ctrl->wp_errors ) : "''"; ?>;</script>
				</div>
			</div>
		</div>
		<?php

	}

}
