<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Guest Accounts Class
 * This class will allow an admin to be able to create multiple guest accounts at a time.
 *
 * @package Simpods MVC
 * @author Andrew Palfrey
 * @since 1.1.4 23/05/2017
 */
class MVC_Admin_Simpods_Guest_Accounts extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-guest-accounts';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_guest_accounts';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_guest_accounts';
	const PAGE_ICON = 'dashicons dashicons-admin-users';
	// The page's title to display in-page
	public static $page_title = 'Guest Accounts';
	/* End Simpods MVC page variables */

	// database table version
	private $database_version = 1.0;

	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Guest Accounts', 'simpods-mvc' );

		// carry on if it is on the right section
		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_GUEST_ACCOUNTS' ) ) {
				define( 'SIMPODS_GUEST_ACCOUNTS', true );
			}

			$this->create_db();
			$this->actions();
			$this->filters();
			$this->guest_user_check();
		}

	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}

	/**
	 * Scripts & styles to register & enqueue when the admin controller is loaded
	 * @return null
	 */
	public function enqueue() {}

	/**
	 * Create the database for this area
	 * @return null
	 */
	public function create_db() {

		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->database_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}

		$columns = array(
			'spga_user_id' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spga_user_name' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spga_expiry_date' => array(
				'type'     => 'date',
				'settings' => '',
				'default'  => 'DEFAULT "0000-00-00"',
			),
			'spga_last_login' => array(
				'type'     => 'date',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "0000-00-00"',
			),
			'spga_role' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spga_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spga_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spga_modified_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spga_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spga_status' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "publish"',
			),
		);
		$this->import_database( self::DB_TABLE, $columns );

		update_option( self::DB_TABLE . '_version', $this->database_version );
	}

	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $ctrl, $cross_admin, $funs;

					
		
		$settings_helper = new MVC_Admin_Settings_Helper();

		$ctrl->forms();
		$ctrl->notice .= $this->simpods_admin_remove_item( self::DB_TABLE );

		$records 	= $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
		//$count_data = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE, 'count_only' => true ) );
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
				<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1><?php echo self::$page_title; ?></h1>
					<?php echo $ctrl->notice; ?>
					<div id="simpods-ga-add">
						<h2><?php esc_html_e( 'Generate Users', 'simpods-mvc' ); ?></h2>
						<?php $settings_helper->render_form( $ctrl->add_form_items, $ctrl->add_form_settings ); ?>
					</div>

					<?php if ( is_array( $records ) && !empty( $records ) ) { ?>
					<hr>
					<div id="simpods-ga-table">
						<?php
						global $wp_roles;
						$roles    	= array();
						$all_roles 	= $wp_roles->roles;

						foreach ( $all_roles as $key => $role_data ) {
							$roles[ $key ] = $role_data['name'];
						}

						foreach ( $records as &$record_data ) {
							$record_data = $funs->escape_attrs( $record_data );

							$last_login = $record_data['spga_last_login'];

							if ( $last_login !== '' && $last_login !== '0000-00-00' ) {
								$last_login = strtotime( $last_login );
								$last_login = date( 'd/m/Y', $last_login );
							} else {
								$last_login = 'Never';
							}

							$date = 'N/A';
							$raw_date = '';

							$record_data['spga_expiry_date-formatted'] = $record_data['spga_expiry_date'];

							if ( $record_data['spga_expiry_date'] !== '0000-00-00' ) {
								$raw_date = $record_data['spga_expiry_date'];
								$date = date( get_option( 'date_format' ), strtotime( $record_data['spga_expiry_date'] ) );
								$record_data['spga_expiry_date'] = date( get_option( 'date_format' ), strtotime( $record_data['spga_expiry_date'] ) );
							} else {
								$record_data['spga_expiry_date'] = '';
							}

							$record_data['spga_last_login']  = $last_login;
							$record_data['spga_expiry_date-formatted'] = $date;
							$record_data['spga_role-formatted'] = $roles[$record_data['spga_role']];
						}

						$headers = array(
							'spga_user_id'               => esc_html__( 'User ID', 'simpods-mvc' ),
							'spga_user_name'             => esc_html__( 'Username', 'simpods-mvc' ),
							'spga_role-formatted'        => esc_html__( 'Role', 'simpods-mvc' ),
							'spga_last_login'            => esc_html__( 'Last Logged In', 'simpods-mvc' ),
							'spga_expiry_date-formatted' => esc_html__( 'Expiry Date', 'simpods-mvc' ),
						);

						$edit_fields = array(
							'spga_user_id',
							'spga_role',
							'spga_expiry_date',
							'id',
						);

						$edit_settings = array(
							'id-prefix' => 'simpods-ga',
							'action'    => admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ),
							'title'     => esc_html__( 'Edit Guest Account', 'simpods-mvc' )
						);

						echo $this->simpods_table( $headers, $records, self::PAGE_SLUG, true, $edit_fields, $edit_settings );
						?>
					</div>
					<?php } ?>
					<script>var wp_errors = <?php echo is_wp_error( $ctrl->wp_errors ) ? wp_json_encode( $ctrl->wp_errors ) : "''"; ?>;</script>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Checks whether an account is a guest account, is active and not expired. Logs the login for the user in the database
	 *
	 * @return null
	 */
	private function guest_user_check() {
		global $current_user, $funs;

		if ( is_user_logged_in() && get_option( self::DB_TABLE . '_version', 0 ) ) {

			$fields = array(
				'spga_user_id' => $current_user->ID,
			);
			$attributes = array(
				'target_tb' => 'simpods_guest_accounts',
			);

			$guest_info = $funs->simpods_select( $fields, $attributes );

			if ( !empty( $guest_info ) ) {
				$guest_info = $guest_info[0];

				$date = strtotime( $guest_info['spga_expiry_date'] );
				$now  = strtotime( date( 'Y-m-d' ) );

				if ( $date < $now ) {
					wp_update_user( array( 'ID' => $current_user->ID, 'role' => '' ) );
					wp_logout();
					$message = '<h1 style="text-align:center;font-family:sans-serif;">' . esc_html__( 'Your account has expired', 'simpods-mvc' ) . '</h1>';
					exit( print_r( $message, true ) );
				} else {
					$update_info = array(
						'spga_last_login' => date( 'Y-m-d' ),
					);
					$where = array(
						'spga_user_id' => $current_user->ID,
					);

					$funs->general_update( self::DB_TABLE, $update_info, $where );

					update_user_meta( $current_user->ID, 'last_login', date( 'Y-m-d' ) );
				}

			} else {
				update_user_meta( $current_user->ID, 'last_login', date( 'Y-m-d' ) );
			}

		}

	}

}
