<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Hierarchy class
 *
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 1.0.0 20/05/2015
 */
class MVC_Admin_Simpods_Hierarchy extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-hierarchy';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_area_categories';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_hierarchy';
	const PAGE_ICON = 'dashicons dashicons-networking';
	// The page's title to display in-page
	public static $page_title = 'Hierarchy';
	/* End Simpods MVC page variables */

	const FIELD_PREFIX    = 'sp_';
	const URL_VAR = 'simpods_hierarchy';

	// Database table version
	private $database_version = 1.01;

	// Additional fields for pods table and functions
	public $custom_fields = array(
		'sp_categories' => array( 'type' => 'text', 'settings' => '', 'default' => '' ),
		'sp_order'      => array( 'type' => 'text', 'settings' => '', 'default' => '' ),
		'sp_granditem'  => array( 'type' => 'text', 'settings' => '', 'default' => '' ), // overwrite granditem from the settings
	);

	private $capabilities = array(
								'simpods_assign_hierarchy',
								'simpods_assign_private_hierarchy',
							);

	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor
	 */
	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Hierarchy', 'simpods-mvc' );

		// carry on if it is on the right section
		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_HIERARCHY' ) ) {
				define( 'SIMPODS_HIERARCHY', true );
			}

			$this->create_db();
			$this->actions();
			$this->filters();
			
		}

	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {

		foreach ( MVC_Admin_Factory::$act_pod_tables as $table => $table_data ) {

			// after pod saved
			if ( $table_data['type'] == 'pod' ) {

				add_action( 'pods_api_post_save_pod_item_' . $table_data['pod'], array( $this, 'simpods_pods_post_save' ), 10, 3 );
			}

		}

		// check table fields when update pod editor
		add_action( 'save_post', array( $this, 'simpods_post_save' ), 10, 3 );

		add_action( 'pods_act_editor_before_sidebar', array( $this, 'hierarchy_meta_box' ) );
		add_action( 'add_meta_boxes', array( $this, 'hierarchy_meta_box' ) );

		// Users
		add_action( 'user_new_form', array( $this, 'simpods_users_cats' ), 10, 1 );
		add_action( 'show_user_profile', array( $this, 'simpods_users_cats' ), 10, 1 );
		add_action( 'edit_user_profile', array( $this, 'simpods_users_cats' ), 10, 1 );
		add_action( 'user_register', array( $this, 'save_simpods_users_cats' ), 10, 1 );
		add_action( 'personal_options_update', array( $this, 'save_simpods_users_cats' ), 10, 1 );
		add_action( 'edit_user_profile_update', array( $this, 'save_simpods_users_cats' ), 10, 1 );

		add_action( 'restrict_manage_posts', array( $this, 'hierarchy_list' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );
	}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {
		add_filter( 'parse_query', array( $this, 'hierarchy_filter' ), 0 );
		add_filter( 'admin_url', array( $this, 'url' ), 10, 2 );
		add_filter( 'simpods_pages_list_items', array( $this, 'simpods_pages_list_items' ), 10, 4 );
	}

	/**
	 * Scripts & styles to register & enqueue when the admin controller is loaded
	 * @return null
	 */
	public function enqueue() {}

	/**
	 * Create the database for this area
	 * @return null
	 */
	public function create_db() {

		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->database_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}

		$columns = array(
			'sp_parent' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'sp_grandparent' => array(
				'type'     => 'varchar(1000)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'sp_granditem' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'sp_child' => array(
				'type'     => 'text',
				'settings' => '',
				'default'  => '',
			),
			'sp_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'sp_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'sp_modified_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'sp_status' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "publish"',
			),
			'sp_input' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "checkbox"',
			),
		);

		$this->import_database( self::DB_TABLE, $columns );
		
		$this->add_capabilities( 'administrator', $this->capabilities );

		update_option( self::DB_TABLE . '_version', $this->database_version );
	}

	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $ctrl, $cross_admin, $funs;

		if( ! current_user_can( 'edit_posts' ) ){
			return false;
		}

		$settings_helper = new MVC_Admin_Settings_Helper();

		$tables = MVC_Admin_Factory::$act_pod_tables;

		$ctrl->notice .= $this->test_for_pods();
		$ctrl->forms();
		$ctrl->notice .= $this->simpods_admin_remove_item( self::DB_TABLE );

		$records = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
		//$count_data   = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE, 'count_only' => true ) );
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
			<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1><?php echo self::$page_title; ?></h1>
					<?php echo $ctrl->notice; ?>
					<div id="simpods-sph-add">
						<h2><?php esc_html_e( 'Create a Link', 'simpods-mvc') ?></h2>
						<p><?php esc_html_e( 'If you have set Grandparent Item to All, other items from the same grandparent for the same parent will be ignored.', 'simpods-mvc' ); ?></p>

						<?php $settings_helper->render_form( $ctrl->add_form_items, $ctrl->add_form_settings ); ?>
					</div>

					<?php if ( is_array( $records ) && !empty( $records ) ) { ?>
					<hr>
					<div id="simpods-sph-table">
						<?php
						foreach ( $records as &$record_data ) {
							$record_data = $funs->escape_attrs( $record_data );

							if ( !isset( $tables[ $record_data['sp_child'] ] ) ) {
								unset( $record_data );
								continue;
							}

							$child_tables 	= $tables[ $record_data['sp_child'] ];
							$parent_tables 	= $tables[ $record_data['sp_parent'] ];
							$grandpa_tables = array();
							if( trim( $record_data['sp_grandparent'] ) != '' ){
								$grandpa_tables = $tables[ $record_data['sp_grandparent'] ];
							}

							$child_tables['type'] = str_replace( '_', ' ', $child_tables['type'] );
							$parent_tables['type'] = str_replace( '_', ' ', $parent_tables['type'] );

							$record_data['sp_child-formatted']     = ucfirst( $child_tables['type'] ) . ': ' . $child_tables['title'];
							$record_data['sp_parent-formatted'] 	  = ucfirst( $parent_tables['type'] ) . ': ' . $parent_tables['title'];
							$record_data['sp_grandparent-formatted']	= '';
							if( isset( $grandpa_tables['type'] ) ){
								$record_data['sp_grandparent-formatted']	= ucfirst( $grandpa_tables['type'] ) . ': ' . $grandpa_tables['title'];
							}
							$record_data['sp_granditem-formatted'] = esc_html__( 'All', 'simpods-mvc' );

							$heading = esc_html__( 'All', 'simpods-mvc' );
							$grandpa_name  = !empty( $record_data['sp_grandparent'] ) ? $tables[$record_data['sp_grandparent']]['name'] : '';

							if ( $record_data['sp_granditem'] != 0 ) {
								$id_name = 'id';

								$pod_index = '';

								if ( $tables[$record_data['sp_grandparent']]['name'] == 'posts' ) {
									$id_name = 'ID';
								} else if ( $tables[$record_data['sp_grandparent']]['name'] == 'users' ) {
									$id_name = 'ID';
								} else {
									$pod_index = get_post_meta( $tables[$record_data['sp_grandparent']]['id'], 'pod_index', true );
								}

								$grandpa_items = $funs->simpods_select( array( $id_name => $record_data['sp_granditem'] ), array( 'target_tb' => $grandpa_name, 'limit' => 1 ) );

								if ( count( $grandpa_items ) ) {
									if ( !isset( $grandpa_items[0]['sp_title'] ) || !$grandpa_items[0]['sp_title'] ) {
										$grandpa_items[0]['sp_title'] = '(' . esc_html__( 'No Title', 'simpods-mvc' ) . ')';
										if ( !empty( $pod_index ) ) {
											$grandpa_items[0]['sp_title'] = $grandpa_items[0][$pod_index];
										}

									}

									$heading = $grandpa_items[0]['sp_title'] . ' (ID: ' . $record_data['sp_granditem'] . ') - ' . $tables[$record_data['sp_grandparent']]['name'];

									$record_data['sp_granditem-formatted'] = $heading;
								}


							}

							$record_data['sp_granditem'] = $heading;
						}

						$headers = array(
							'sp_child-formatted'     	=> esc_html__( 'Child Table', 'simpods-mvc' ),
							'sp_parent-formatted'    	=> esc_html__( 'Parent Table', 'simpods-mvc' ),
							'sp_grandparent-formatted'	 => esc_html__( 'Grandparent', 'simpods-mvc' ),
							'sp_granditem-formatted' 	=> esc_html__( 'Grandparent Item', 'simpods-mvc' ),
						);

						$edit_fields = array(
							'sp_child',
							'sp_parent',
							'sp_granditem',
							'id',
						);

						$edit_settings = array(
							'id-prefix' => 'simpods-sph',
							'action'    => admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ),
							'title'     => esc_html__( 'Edit Hierarchy', 'simpods-mvc' )
						);

						echo $this->simpods_table( $headers, $records, self::PAGE_SLUG, true, $edit_fields, $edit_settings );

						?>
					</div>
					<?php } ?>
					<script>var wp_errors = <?php echo is_wp_error( $ctrl->wp_errors ) ? wp_json_encode( $ctrl->wp_errors ) : "''"; ?>;</script>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Get all the ancestor tables for a specific table
	 *
	 * @param array $attributes The attributes of what to search for
	 * @return array The ancestor tables
	 */
	public function ancestors( $attributes ) {

		$attributes =  wp_parse_args( $attributes,  array(
			'table'     => '',
			'ancestors' => array(),
		) );

		$ancestors = $attributes['ancestors'];

		// find the parent
		$children = $funs->simpods_select( array( 'sp_child' => array( $attributes['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		if ( $children != NULL && count( $children ) > 0 ) {
			array_push( $ancestors, $children[0]['sp_parent'] );

			// loop the function itself to get all ancestors
			$ancestors = $this->ancestors( array( 'table' => $children[0]['sp_parent'], 'ancestors' => $ancestors ) );
		}

		return $ancestors;
	}

	/**
	 * Gets items from a category
	 *
	 * @param string  $table        The category table to pull from
	 * @param unknown $category_table_id     The category table id. It could be pod id or posts, users
	 * @param integer $category_id     The category ID
	 * @param array   $attributes      Some attributes of what to pull
	 * @param boolean $can_echo_query Echo the query on screen
	 * @return array
	 */
	public function cat_items( $table, $category_table_id, $category_id, $attributes = array(), $can_echo_query = false ) {

		global $wpdb;

		$attributes =  wp_parse_args(  $attributes, array(
			'order'    => 'ASC',
			'order_by' => '',
			'group_by' => '',
			'start'    => 0,
			'limit'    => 0,

		) );

		$params = array();

		$where = ' AND `sp_categories` REGEXP "(:\"' . esc_sql( $category_table_id ) . '.' . intval( $category_id ) . '\";{1,})"';

		if ( count( $params ) > 0 ) {
			$query = $wpdb->prepare( 'SELECT
										 *
										 FROM `' . $wpdb->prefix . $table . '`

										 WHERE 1 = 1 ' . $where . ' ', $params );
		} else {
			$query = 'SELECT
										 *
										 FROM `' . $wpdb->prefix . $table . '`

										 WHERE 1 = 1 ' . $where . ' ';
		}

		if ( $can_echo_query ) {
			echo $query;
		}

		$items = $wpdb->get_results( $query, ARRAY_A );

		return $items;
	}

	/**
	 * Get all the decendant tables for a specific table
	 *
	 * @param array $attributes The attributes of what to search for
	 * @return array The decendant tables
	 */
	public function decendants( $attributes ) {
		global $funs;

		$attributes =  wp_parse_args(  $attributes, array(
			'table'      => '',
			'decendants' => array(),
		) );

		$decendants = $attributes['decendants'];

		// find the child
		$children = $funs->simpods_select( array( 'sp_parent' => array( $attributes['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		if ( $children != NULL && count( $children ) > 0 ) {
			array_push( $decendants, $children[0]['sp_child'] );

			// loop the function itself to get all ancestors
			$decendants = $this->decendants( array( 'table' => $children[0]['sp_child'], 'decendants' => $decendants ) );
		}

		return $decendants;
	}

	/**
	 * Filters the main tables when a child table is selected
	 *
	 * @param array $attributes An array containing the child table name
	 * @return array
	 */
	public function filter_child_table( $attributes ) {
		global $funs;

		$filters =  wp_parse_args( $attributes,  array(
			'table' => '',
		) );

		$parent_tables = MVC_Admin_Factory::$act_pod_tables;

		// if already has a parent, use it
		$items = $funs->simpods_select( array( 'sp_child' => array( $filters['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		if ( is_array( $items ) && count( $items ) > 0 && isset( $parent_tables[ $items[0]['sp_parent'] ] ) ) {
			$parent_tables = array( $items[0]['sp_parent'] => $parent_tables[ $items[0]['sp_parent'] ] );
		} else {

			unset( $parent_tables[ $filters['table'] ] );

			$filters['table'] = esc_sql( $filters['table'] );

			// the decendants cannot be the parents
			$decendants = $this->decendants( array( 'table' => $filters['table'], 'decendants' => array() ) );

			foreach ( $parent_tables as $table => $table_data ) {

				// if it is an ancestor of the parent table, unset
				if ( in_array( $table, $decendants ) ) {
					unset( $parent_tables[ $table ] );
				}

			}

		}

		return $parent_tables;

	}

	/**
	 * Filters the categories when a parent table is selected
	 *
	 * @param array $attributes An array containing the parent table name
	 * @return array
	 */
	public function filter_parent_table( $attributes ) {
		global $funs, $current_user;

		$filters =  wp_parse_args( $attributes, array(
			'table' => '',
		) );

		$tables = MVC_Admin_Factory::$act_tables;

		// if the table name is not from the database, return
		if ( ! isset( $tables[ $filters['table'] ] ) || ! isset( $tables[ 'pod_' . $tables[ $filters['table'] ]['id'] ] ) ) {
			return false;
		}
		// convert the array with slug into the array with pod_num
		$filters['table'] = 'pod_' . $tables[ $filters['table'] ]['id'];

		$return_data = array( 'categories' => array() );


		// get grandparent table, then get its items as categories
		$grandpa_records = $funs->simpods_select( array( 'sp_child' => array( $filters['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ), 0 );

		if ( $grandpa_records != NULL && count( $grandpa_records ) > 0 ) {
			$fields 	= array();
			if ( $tables[ $grandpa_records[0]['sp_parent']]['type'] == 'post_type' ) {

				$fields['post_tb.post_status'] = array( 'publish', 'draft' );
				// if not an administrator, only list the author's own input
				if ( ! current_user_can( 'simpods_assign_private_hierarchy' ) ) {
					$fields['post_tb.post_author'] = $current_user->ID; 
				} 
			}
			$grandpas = $funs->simpods_select( $fields, array( 'target_tb' => $tables[ $grandpa_records[0]['sp_parent']]['name'] ) );

			$pod_index = '';

			if ( $tables[ $grandpa_records[0]['sp_parent'] ]['name'] == 'posts' ) {
				//$grandpas = array_column( $grandpas , 'sp_title' );
			} else

			if ( $tables[ $grandpa_records[0]['sp_parent'] ]['name'] == 'users' ) {
				//$grandpas = array_column( $grandpas , 'sp_title' );
			} else {

				$pod_index = get_post_meta( $tables[ $grandpa_records[0]['sp_parent'] ]['id'], 'pod_index', true );
				//$grandpas = array_column( $grandpas , $pod_index );
			}

			foreach ( $grandpas as $grandpa_data ) {

				if ( !isset( $grandpa_data['sp_title'] ) || !$grandpa_data['sp_title'] ) {
					$grandpa_data['sp_title'] = '(' . esc_html__( 'No Title', 'simpods-mvc' ) . ')';

					if ( !empty( $pod_index ) ) {
						$grandpa_data['sp_title'] = $grandpa_data[ $pod_index ];
					}

				}

				$grandpa_data['sp_title'] = esc_attr( $grandpa_data['sp_title'] );

				$return_data['categories'][ $grandpa_data['id'] ] = $grandpa_data['sp_title'];
			}

			$return_data['grandtable'] = $tables[ $grandpa_records[0]['sp_parent'] ]['name'];
		}

		return $return_data;

	}

	/**
	 * Filters search results
	 *
	 * @param  object $query_obj The query object that parsed the query.
	 * @return null
	 */
	public function hierarchy_filter( $query_obj ) {
		global $funs, $current_user;

		// find the pod table assets
		if ( isset( $_GET[self::URL_VAR] ) && is_numeric( $_GET[self::URL_VAR] ) ) {
			$pod_tables = $funs->find_a_pod_tb( 'pod', MVC_Admin_Factory::$post_type );

			if ( isset( $pod_tables['id'] ) ) {

				$linked_areas = $funs->simpods_select( array( 'sp_child' => 'pod_' . $pod_tables['id'] ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'sp_parent' ) );

				if ( isset( $linked_areas[0] ) && isset( MVC_Admin_Factory::$act_pod_tables[ $linked_areas[0]['sp_parent'] ] ) ) {
					$pod_parent_tables = MVC_Admin_Factory::$act_pod_tables[ $linked_areas[0]['sp_parent'] ];

					$assigned_data = $this->cat_items( $pod_tables['name'], $pod_parent_tables['id'], $_GET[self::URL_VAR], array(), 0 );

					if ( $assigned_data ) {
						$post_ids = Arrays::array_column( $assigned_data, 'id' );
					} else {
						$post_ids = array( -1 );
					}

					if ( count( $query_obj->query_vars['post__in'] ) > 0 ) {
						$post_ids = array_intersect( $query_obj->query_vars['post__in'], $post_ids );
					}

					if ( count( $post_ids ) == 0 ) {
						$post_ids = array( -1 );
					}

					$query_obj->query_vars['post__in'] = $post_ids;

					if ( isset( $_GET['orderby'] ) ) {
						$query_obj->set( 'orderby', $_GET['orderby'] );
					}

				}

			}

		}

	}

	/**
	 * Provide a combo box for filtering 
	 *
	 * @return null
	 */
	public function hierarchy_list() {
		global $funs;
		// find the pod table assets
		$pod_tables = $funs->find_a_pod_tb( 'pod', MVC_Admin_Factory::$post_type );

		if ( isset( $pod_tables['id'] ) ) {

			$fields = array( 'sp_child' => 'pod_' . $pod_tables['id'] );
			$attrs  = array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'sp_parent' );

			$linked_areas = $funs->simpods_select( $fields, $attrs );

			if ( isset( $linked_areas[0] ) && isset( MVC_Admin_Factory::$act_pod_tables[ $linked_areas[0]['sp_parent'] ] ) ) {
				$pod_parent_table = MVC_Admin_Factory::$act_pod_tables[ $linked_areas[0]['sp_parent'] ]['name'];

				$this->pages_list( esc_html__( 'Simpods Hierarchy', 'simpods-mvc' ), $pod_parent_table, self::URL_VAR );
			}

		}

	}

	/**
	 * Add a Simpods Hierarchy meta box to relevant post types
	 *
	 * @return null
	 */
	public function hierarchy_meta_box() {

		if ( !current_user_can( 'simpods_assign_hierarchy' ) ) {
			return '';
		}		
		global $funs;
		$is_allowed = true;

		if ( isset( $_GET['post'] ) ){
			$post_id = (int) $_GET['post'];
		} elseif ( isset( $_GET['id'] ) ){
			$post_id = (int) $_GET['id'];
		} else {
			$post_id = 0;
		}

		$tables = MVC_Admin_Factory::$act_pod_tables;

		if ( isset( $_GET['page'] ) && strpos( $_GET['page'], 'pods-manage-' ) === 0 ) {
			$table = substr( $_GET['page'], 12 );
		} else {
			$table = MVC_Admin_Factory::$post_type;
		}

		$idx = '';

		// find out the index in the table array
		foreach ( $tables as $key => $table_data ) {
			if ( $table == $table_data['pod'] ) {
				$idx = $key;
				break;
			}

		}

		// if no table found, return
		if ( $idx == '' ) {
			return;
		}

		$linked_areas = $funs->simpods_select( array( 'sp_child' => $idx, 'sp_status' => 'publish' ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'sp_parent' ) );

		// if parent table exist
		if ( is_array( $linked_areas ) && count( $linked_areas ) > 0 ) {
			foreach ( $linked_areas as $area_idx => $area_data ) {

				$area_data['child_tb_idx'] = $idx;
				if ( ( isset( $_GET['page'] ) && strpos( $_GET['page'], 'pods-manage-' ) === 0 ) ) {

					// build an array like the one for add_meta_box call back
					$args = array(
						'id'    => 'simpods_parent_area_cats',
						'title' => esc_html__( 'Categories', 'simpods-mvc' ) . '(' . $tables[$area_data['sp_parent']]['name'] . ')',
						'args'  => $area_data,
					);
					echo $this->simpods_cats_box( $args );

				} else {

					// get the table info to check if it a pods table
					if ( $tables[$idx]['type'] != 'pod' ) {

						$screens = array( MVC_Admin_Factory::$post_type );

						add_meta_box(
							'simpods_parent_area_cats',
							'' . self::$page_title . ' (' . $tables[ $area_data['sp_parent'] ]['title'] . ')',
							array( $this, 'simpods_post_type_cats' ),
							$screens,
							'side',
							'high',
							$area_data
						);

					}

				}

			}

		}

	}

	/**
	 * Save data when Hierarchy is used in the Users section
	 *
	 * @param integer $user_id The ID of the user
	 * @return null
	 */
	public function save_simpods_users_cats( $user_id ) {

		if ( !isset( $_POST['simpods_category'] ) ) {
			return;
		}

		$index = '';
		$table    = '';

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $table_data ) {

			if ( $table_data['pod'] == 'user' ) {
				$index = $key;
				break;
			}

		}

		// pods user not set , return
		if ( $index == '' ) {
			return;
		} else {
			$table = MVC_Admin_Factory::$act_pod_tables[$index]['name'];
		}

		$this->simpods_cats_save( $user_id, $table );

	}

	/**
	 * Prints out the metabox for Hierarchy
	 *
	 * @param array $args Data saved for the current item
	 * @return string The HTML for the box
	 */
	public function simpods_cats_box( $args ) {
		if( ! current_user_can( 'simpods_assign_hierarchy' ) ){
			return false;
		}		
		global $funs, $current_user;

		$html = '';

		$tables = MVC_Admin_Factory::$act_pod_tables;

		$area_data = $args['args'];
		$table   = $area_data['child_tb_idx'];

		$linked_areas = $funs->simpods_select( array( 'sp_granditem' => 0, 'sp_parent' => $area_data['sp_parent'], 'sp_child' => $table, 'sp_status' => 'publish' ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		// if the All category not found, find other categories, otherwise ignore then

		if ( !is_array( $linked_areas ) || count( $linked_areas ) == 0 ) {
			$linked_areas = $funs->simpods_select( array( 'sp_parent' => $area_data['sp_parent'], 'sp_child' => $table, 'sp_status' => 'publish' ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC' ) );
		}

		if ( !is_array( $linked_areas ) || count( $linked_areas ) == 0 ) {
			return '';
		}


		$wheres = $this->get_the_ID( $tables[ $table ] );

		$this->update_columns( $tables[ $table ]['name'], $this->custom_fields );

		$items 		= $funs->simpods_select( $wheres, array( 'target_tb' => $tables[ $table ]['name'], 'limit' => 1 ) );

		$parent 	= $tables[ $linked_areas[0]['sp_parent'] ]['name'];
		$css_class    	= $tables[ $table ]['pod'] == 'user' ? 'mgl10 pdb10' : '';

		$grandparents 	=  $this->filter_parent_table( array( 'table' => $parent ) ) ;


		if ( ( $tables[ $table ]['type'] == 'pod' || isset( $_GET['page'] ) ) || $tables[ $table ]['pod'] == 'user' ) {
			$html .= '
			<div class="postbox simpods_parent_area_cats" >
				<button aria-expanded="true" class="handlediv button-link" type="button">
					<span class="screen-reader-text">' . esc_html__( 'Toggle panel', 'simpods-mvc' ) . ': ' . esc_html__( 'Categories', 'simpods-mvc' ) . '</span><span aria-hidden="true" class="toggle-indicator"></span>
				</button>
				<h2 class="hndle ui-sortable-handle ' . $css_class . '"><span> ' . esc_html__( 'Hierarchy', 'simpods-mvc' ) . ' (' . $parent . ')</span></h2>
				<div class="inside">';
		}

		$html .= '
				<div class="categorydiv" id="simpods_categories">

					<div class="tabs-panel" style="display: block;">

						<ul class="categorychecklist form-no-clear" data-wp-lists="list:category">
						';						

		foreach ( $linked_areas as $k => $values ) {

			$heading = 'All';
			$grandpa_name  = !empty( $values['sp_grandparent'] ) ? esc_attr( $tables[$values['sp_grandparent']]['name'] ) : '';

			if ( !empty( $values['sp_granditem'] ) ) {
				$id_name = 'id';

				if ( $grandpa_name == 'posts' || $grandpa_name == 'users' ) {
					$id_name = 'ID';
				}

				$grandpa_items = $funs->simpods_select( array( $id_name => $values['sp_granditem'] ), array( 'target_tb' => $grandpa_name, 'limit' => 1 ) );

				if ( count( $grandpa_items ) ) {
					$heading = esc_attr( $grandpa_items[0][$tables[$values['sp_grandparent']]['name_field']] ) . '';
				}
				$category_areas		=	array(
											array(
													'join' 	=> 'AND',
													'cattb' => ltrim( $values['sp_grandparent'], 'pod_' ),
													'catid' => $values['sp_granditem'],
												)
										);
				$parent_items 	= 	$funs->simpods_select( array(), array( 'target_tb' => $parent, 'cat_areas' => $category_areas ) );

			} else {
				$parent_table    = $tables[ $linked_areas[0]['sp_parent'] ]['name'];
				$attrs  = array( 'target_tb' => $parent_table );
				$fields = array();

				// if post type table rather than advance pods table
				if ( $tables[$linked_areas[0]['sp_parent']]['type'] == 'post_type' ) {
					$fields['post_tb.post_status'] = array( 'publish', 'draft' );
					if ( ! current_user_can( 'simpods_assign_private_hierarchy' ) ) {
						$fields['post_tb.post_author'] = $current_user->ID; 
					} 					
				}

				$parent_items = $funs->simpods_select( $fields, $attrs );

			}

			if( isset( $grandparents['grandtable'] ) ){
				
				if( isset( MVC_Admin_Parent_Factory::$act_tables[ $grandparents['grandtable'] ] ) ) {
					$grandparent_id = MVC_Admin_Parent_Factory::$act_tables[ $grandparents['grandtable'] ]['id'];
					// reorder the items based on the grandparents
					$new_items	= array();
					foreach ( $grandparents['categories'] as $categroy_id => $name ) {
						$new_items[]	= array( 
											'sp_title' 		=> MVC_Admin_Factory::$act_tables[ $grandparents['grandtable'] ]['title'] . ': ' . $name,
											'id'			=> $categroy_id, 
											'is_category'	=> true,
											);
						
						foreach ( $parent_items as $i => $item ) {
							if( strpos( $item['sp_categories'], ':"'. $grandparent_id . '.' . $categroy_id . '";' ) !== false ){
								$new_items[]	= $item;
							}
						}
					}
					$parent_items 	= $new_items;
				}
			}
			$html .= '
							<li class="popular-category" id="simpodscategory-' . $k . '">

								<strong>' . $heading . '</strong>

								<ul class="children">
							';
			// make sure it is uploaded
			$html .= '<input type="hidden" name="simpods_category[' . $tables[$linked_areas[0]['sp_parent']]['id'] . '_unset]" value="0" autocomplete="off">';

			$i = 0;
			if( count( $parent_items ) ){
				$pod_index = get_post_meta( $tables[ $linked_areas[0]['sp_parent']]['id'], 'pod_index', true );
				foreach ( $parent_items as $parent_key => $parent_item_data ) {

					$parent_item_data['id'] = $parent_item_data['id'];					

					if ( ! current_user_can( 'edit_post', $parent_item_data['id'] ) && !current_user_can( 'simpods_assign_private_hierarchy' ) ){
						continue;
					}

					if ( !empty( $pod_index ) ) {
						$parent_item_data['sp_title'] = $parent_item_data[ $pod_index ];
					}

					$parent_categories = array();

					if ( isset( $items[0] ) ) {
						$parent_categories = maybe_unserialize( $items[0]['sp_categories'] );
					}

					if ( !is_array( $parent_categories ) ) {
						$parent_categories = array();
					}

					if ( $values['sp_grandparent'] == '' ) {
						$values['sp_grandparent'] = 0;
					}

					// check the boxes, with types
					$checked_style = in_array( $tables[ $linked_areas[0]['sp_parent']]['id'] . '.' . $parent_item_data['id'], $parent_categories, true ) == true ? 'checked="checked"' : '';

					// add new will pass the filter - simpods_hierarchy
					if ( isset( $_GET[self::URL_VAR] ) && is_numeric( $_GET[self::URL_VAR] ) && $parent_item_data['id'] == $_GET[self::URL_VAR] ) {
						$checked_style = 'checked="checked"';
					}

					//input type
					$input_type = 'checkbox';
					$group     = '';
					$input_name = 'simpods_category[' . $tables[$linked_areas[0]['sp_parent']]['id'] . '_' . $i . ']';

					if ( $linked_areas[0]['sp_input'] == 'radio' ) {
						$input_type = $linked_areas[0]['sp_input'];
						$group     = 'group="simpods_category"';
						$input_name = 'simpods_category[' . $tables[$linked_areas[0]['sp_parent']]['id'] . '_0]';
					}
					if( isset( $parent_item_data['is_category'] ) ){
						$html .= '
									<li >										
										<strong>' . $parent_item_data['sp_title'] . '</strong>										
									</li>
									';
					} else {
						$html .= '
									<li >
										<label class="selectit">
											<input type="' . $input_type . '" ' . $checked_style . ' ' . $group . ' name="' . $input_name . '" value="' . $parent_item_data['id'] . '" autocomplete="off">
											' . $parent_item_data['sp_title'] . '
										</label>
									</li>
									';
					}

					$i++;
				}
			}

			if ( $i == 0 ) {
				$html .= '
										<li >
											' . esc_html__( 'No items', 'simpods-mvc' ) . '
										</li>
										';
			}

			$html .= '
								</ul>
							</li>
							';

		}

		$html .= '
						</ul>
					</div>

				</div>';

		if (  ( $tables[ $table ]['type'] == 'pod' || isset( $_GET['page'] ) ) || $tables[ $table ]['pod'] == 'user' ) {
			$html .= '
				</div>
			</div>';
		}

		return $html;
	}

	/**
	 * Save or update the Hierarchy data for normal post type
	 *
	 * @param integer $post_id Current post ID
	 * @param string  $table     Target table
	 * @return void
	 */
	public function simpods_cats_save( $post_id, $table ) {
		global $wpdb, $current_user, $funs;
					
		$wheres                      = array( 'id' => $post_id );
		$parent_categories                      = array();
		$now                        = date( 'Y-m-d H:i:s' );
		$data                       = array();
		$data['sp_modified']        = $now;
		$data['sp_modified_author'] = $current_user->ID;

		$items = $funs->simpods_select( $wheres, array( 'target_tb' => $table, 'limit' => 1 ) );

		if ( isset( $items[0] ) ) {

			$parent_categories = array();

			// hierachy is one to many so only one one option is picked.
			foreach ( $_POST['simpods_category'] as $key => $category_id ) {
				$key_exploded = explode( '_', $key );

				if ( $category_id != 0 ) {
					array_push( $parent_categories, $key_exploded[0] . '.' . $category_id );
				}

			}

			$parent_categories = array_values( array_unique( $parent_categories ) );

			$data['sp_categories'] = maybe_serialize( $parent_categories );
			if ( empty( $items[0]['sp_author'] ) ) {
				$data['sp_author'] = $current_user->ID;
			}

			if ( $items[0]['sp_created'] == '0000-00-00 00:00:00' ) {
				$data['sp_created'] = $now;
			}

			$funs->general_update( $wpdb->prefix . $table, $data, $wheres );

		} else {

			$data['sp_author']  = $current_user->ID;
			$data['sp_created'] = $now;

			foreach ( $_POST['simpods_category'] as $key => $category_id ) {
				$key_exploded = explode( '_', $key );

				if ( $category_id != 0 ) {
					array_push( $parent_categories, $key_exploded[0] . '.' . $category_id );
				}

			}

			$data['sp_categories'] = maybe_serialize( $parent_categories );
			$data['id']            = $post_id;
			$funs->general_insert( $wpdb->prefix . $table, $data );


		}

	}

	/**
	 * Save or update the Hierarchy data for advanced content type
	 *
	 * @param array   $pieces Current Pod details
	 * @param boolean $is_new  Is a new item or not
	 * @param integer $id     ID of item being updated
	 * @return null
	 */
	public function simpods_pods_post_save( $pieces, $is_new, $id ) {

		if ( isset( $_POST['_pods_pod'] ) && isset( $_POST['simpods_category'] ) ) {

			$this->simpods_cats_save( $id, 'pods_' . $_POST['_pods_pod'] );
		}

	}

	/**
	 * Passes data to simpods_cats_box_fn to create the Hierarchy metabox for normal post type
	 *
	 * @param object $post     Current post
	 * @param array  $area_data Data saved for the current item
	 * @return null
	 */
	public function simpods_post_type_cats( $post, $area_data ) {
		echo $this->simpods_cats_box( $area_data );
	}

	/**
	 * Updates a child post when a post is saved
	 *
	 * @param integer $post_id The post ID
	 * @param object  $post_obj   The post object
	 * @param boolean $update_bln Whether the post is being updated
	 * @return null
	 */
	public function simpods_post_save( $post_id, $post_obj, $update_bln ) {
		if ( !isset( $_POST['simpods_category'] ) ) {
			return;
		}

		// if the post type is post or user, and s to the end to distinguish it from the pods post or user entension
		$post_type = MVC_Admin_Factory::$post_type;
		$table    = '';

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $table_data ) {

			if ( $table_data['pod'] == $post_type ) {
				$table = $table_data['name'];
				break;

			}

		}

		$this->simpods_cats_save( $post_id, $table );
	}

	/**
	 * Passes data to simpods_cats_box_fn to create the Hierarchy metabox for the Users section
	 *
	 * @param object $user The user object
	 * @return null
	 */
	public function simpods_users_cats( $user ) {
		global $funs;
		$tables = MVC_Admin_Factory::$act_pod_tables;
		$index  = '';

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $table_data ) {

			if ( $table_data['pod'] == 'user' ) {
				$index = $key;
				break;
			}

		}

		$linked_areas = $funs->simpods_select( array( 'sp_child' => $index, 'sp_status' => 'publish' ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'sp_parent' ) );

		if ( is_array( $linked_areas ) && count( $linked_areas ) > 0 ) {
			$user_id               = isset( $user->ID ) ? $user->ID : -1;
			$area_data                 = $linked_areas[0];
			$area_data['child_tb_idx'] = $key;
			$area_data['user_id']      = $user_id;
			$args                 = array(
				'id'    => 'simpods_parent_area_cats_pod_' . $area_data['sp_parent'],
				'title' => $tables[$area_data['sp_parent']]['name'] . '',
				'args'  => $area_data,
			);
			echo $this->simpods_cats_box( $args );
		}

	}

	/**
	 * Append parameters to a provided URL. Limit to edit.php so it won't cause problems to the details
	 * @param  string $url The URL to append parameters to
	 * @return string The appended URL
	 */
	public function url( $url ) {

		if ( isset( $_GET[self::URL_VAR] ) && is_numeric( $_GET[self::URL_VAR] ) && strpos( $url, 'edit.php' ) !== false ) {

			if ( strpos( $url, '?' ) !== false ) {
				$url .= '&';
			}

			$url .= self::URL_VAR . '=' . $_GET[ self::URL_VAR ];

		}

		return $url;
	}

	/**
	 * Display the filter list with grandparent
	 */ 
	public function simpods_pages_list_items( $items, $title, $table_name, $element_name ){
		$grandparents 	=  $this->filter_parent_table( array( 'table' => $table_name ) ) ;
		if( isset( $grandparents['grandtable'] ) ){
			if( 'simpods_hierarchy' == $element_name && isset( MVC_Admin_Parent_Factory::$act_tables[ $grandparents['grandtable'] ] ) ) {
				if( ! empty( MVC_Admin_Parent_Factory::$act_tables[ $grandparents['grandtable'] ] ) ){
					$grandparent_id = MVC_Admin_Parent_Factory::$act_tables[ $grandparents['grandtable'] ]['id'];
					// reorder the items based on the grandparents
					$new_items	= array();
					foreach ( $grandparents['categories'] as $categroy_id => $name ) {
						$new_items[]	= array( 
											MVC_Admin_Factory::$act_tables[ $table_name ]['name_field'] => MVC_Admin_Factory::$act_tables[ $grandparents['grandtable'] ]['title'] . ': ' . $name,
											'id'			=> $categroy_id, 
											'is_category'	=> true,
											);
						
						foreach ( $items as $i => $item ) {
							if( strpos( $item['sp_categories'], ':"'. $grandparent_id . '.' . $categroy_id . '";' ) !== false ){
								$new_items[]	= $item;
							}
						}
					}
					$items 	= $new_items;
				}

			}
		}
		
		
		return $items;
	}
}
