<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods Home Admin class
 *
 * @package Simpods MVC
 * @author Andrew Palfrey
 * @since 24/01/2019
 */
class MVC_Admin_Simpods_Home extends MVC_Admin_General {
	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG  = 'simpods-mvc';
	// The table name to be created to hold the settings
	const DB_TABLE    = '';
	// The page's title to display in-page
	public static $page_title = 'Simpods MVC';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP = 'edit_theme_options';
	const PAGE_ICON = 'dashicons dashicons-sim-logo align-bottom';
	/* End Simpods MVC page variables */
	/**
	 * Constructer
	 * @param Boolean $can_init toggle to run the functions in the constructer,
	 *  			  $can_init set to true will run automatically
	 */
	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Simpods MVC', 'simpods-mvc' );
		if ( $can_init ) {
			$this->actions();
			$this->filters();
		}
	}
	/**
	 * actions_fn: WP actions to globally hook when an instance of the class is created
	 *  		   Abstract method in the parent class, must be included
	 */
	public function actions() {}
	/**
	 * filters_fn: WP filters to globally hook when an instance of the class is created
	 *  		   Abstract method in the parent class, must be included
	 */
	public function filters() {}
	/**
	 * enqueue_fn: Scripts and styles to globally enqueue when an instance of the class is created
	 *  		   Abstract method in the parent class, must be included
	 */
	public function enqueue() {}
	/**
	 * interface_fn: The interface of the model when added to the Simpods MVC settings
	 *  			 Abstract method in the parent class, must be included
	 */
	public function interface() {
		global $ctrl, $funs, $cross_admin, $wp_filesystem;
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
				<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<div id="simpods-mvc-opening" class="text-center">
						<h1 class="display-4"><?php esc_html_e( 'Welcome to Simpods MVC', 'simpods-mvc' ); ?></h1>
						<p class="lead"><?php esc_html_e( 'A developer-friendly, flexible framework which brings the benefits of an MVC environment to WordPress. Bundled with developer\'s favourite tools including; Bootstrap, Font Awesome and a Gulp.js driven SCSS & JS compiler. Integrated with custom features and components to speed up and ease application building.', 'simpods-mvc' ); ?></p>
						<hr class="my-5">
					</div>
					<div id="simpods-mvc-recommended">
						<h2><?php esc_html_e( 'Plugins', 'simpods-mvc' ); ?></h2>
						<?php 
							$this->list_plugins(  $ctrl->plugins );
						?>
						<div class="row mx-n2">
						<?php 
							
							/*
							foreach ( $ctrl->plugins as $plugin ) {
								$required_class  = 'bg-info text-white';
								$installed_class = 'bg-success text-white';
								$installed = $wp_filesystem->exists( trailingslashit( WP_PLUGIN_DIR ) . $plugin['active_file'] );
								$active = is_plugin_active( $plugin['active_file'] );
								
								if ( !$active ) {
									if ( $plugin['required'] ) {
										$installed_class = 'bg-danger text-white';
									} else {
										$installed_class = 'bg-warning text-body';
									}
								}
								$required_label   = esc_html( _n( 'Recommended', 'Required', intval( $plugin['required'] ) + 1, 'simpods-mvc' ) );
								$installed_label  = esc_html( _n( 'Not Activated', 'Activated', intval( $active ) + 1, 'simpods-mvc' ) );
								$installed_button = esc_html( _n( 'Activate', 'Active', intval( $active ) + 1, 'simpods-mvc' ) );
								
								if ( !$installed ) {
									$installed_label  = esc_html__( 'Not Installed', 'simpods-mvc' );
									$installed_button = esc_html__( 'Install', 'simpods-mvc' );
								}
								$thumbnail = $this->get_plugin_icons( $plugin['slug'] );
								$thumbnail_style = '';
								if ( !empty( $thumbnail ) ) {
									$thumbnail_style = 'background-image: url(' . end( $thumbnail ) . ');';
								}
								$install_args = array(
									'action' => 'install-plugin',
									'plugin' => $plugin['slug'],
									'_wpnonce' => wp_create_nonce( 'install-plugin_' . $plugin['slug'] ),
								);
								$install_url  = admin_url( 'update.php' ) . '?' . http_build_query( $install_args );
								$activate_args = array(
									'action' => 'activate',
									'plugin' => $plugin['active_file'],
									'_wpnonce' => wp_create_nonce( 'activate-plugin_' . $plugin['active_file'] ),
								);
								$activate_url  = admin_url( 'plugins.php' ) . '?' . http_build_query( $activate_args );
							
							<div class="col-12 col-md-6 col-xl-3 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100">
									<div class="card-img-top-square simpods-logo-back text-white" style="<?php echo $thumbnail_style ?>" title="<?php echo $plugin['title']; ?>">
										<?php if ( $thumbnail_style == '' ) { ?>
										<img src="<?php echo $funs->get_final_path( 'mvc/admin/images', 'default-plugin.svg', true, false, true ); ?>" alt="<?php echo $plugin['title']; ?>">
										<?php } ?>
									</div>
									<ul class="list-group list-group-flush fs1rem">
										<li class="list-group-item <?php echo $required_class; ?>"><?php echo $required_label; ?></li>
										<li class="list-group-item <?php echo $installed_class; ?>"><?php echo $installed_label; ?></li>
									</ul>
									<div class="card-body p-3">
										<h3 class="card-title m-0"><?php echo $plugin['title']; ?></h3>
										<?php if ( isset( $plugin['subtitle'] ) && $plugin['subtitle'] !== '' ) { ?>
										<p class="m-0"><?php echo $plugin['subtitle']; ?></p>
										<?php } ?>
									</div>
									<div class="card-footer p-3 bg-white">
										<?php if ( !$active && !$installed ) { ?>
										<a href="<?php echo $install_url; ?>" class="btn btn-simpods-purple btn-block"><?php echo $installed_button; ?></a>
										<?php } else if ( !$active && $installed ) { ?>
										<a href="<?php echo $activate_url; ?>" class="btn btn-simpods-purple btn-block"><?php echo $installed_button; ?></a>
										<?php } else { ?>
										<button type="button" class="btn btn-outline-simpods-purple btn-block" disabled><?php echo $installed_button; ?></button>
										<?php } ?>
									</div>
								</div>
							</div>
							
							}*/
							?>
						</div>
						<hr class="my-5">
					</div>
					<div id="simpods-mvc-links">
						<h2><?php esc_html_e( 'Useful Links', 'simpods-mvc' ); ?></h2>
						<div class="row mx-n2">
							<div class="col-12 col-md-6 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100 border-0 shadow-none">
									<div class="card-header px-3 py-0 bg-white">
										<h3><i class="fas fa-book mr-3"></i><?php esc_html_e( 'Documentation', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-body p-3">
										<p class="m-0"><?php esc_html_e( 'The Simpods MVC documentation includes everything you need to know to get developing with Simpods MVC. The function reference includes detailed information about each function which should help you take full control of the theme.', 'simpods-mvc' ); ?></p>
									</div>
									<div class="card-footer p-3 bg-white">
										<a href="https://simpodsmvc.me/docs/" class="btn btn-outline-simpods-blue btn-block" target="_blank"><?php esc_html_e( 'Documentation', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
							<div class="col-12 col-md-6 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100 border-0 shadow-none">
									<div class="card-header px-3 py-0 bg-white">
										<h3><i class="fas fa-video mr-3"></i><?php esc_html_e( 'Video Tutorials', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-body p-3">
										<p class="m-0"><?php esc_html_e( 'We are constantly looking to implement a fun and interactive learning environment for our users. Whether you are a beginner or an advanced user, our library of comprehensive video tutorials includes everything you need to get started with the theme. Learning to use Simpods MVC could not get any easier!', 'simpods-mvc' ); ?></p>
									</div>
									<div class="card-footer p-3 bg-white">
										<a href="https://simpodsmvc.me/docs/simpods/tutorials/lessons/simpods-mvc-mvc-quick-demo/" class="btn btn-outline-simpods-blue btn-block" target="_blank"><?php esc_html_e( 'Video Tutorials', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
							<!-- <div class="col-12 col-md-6 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100 border-0 shadow-none">
									<div class="card-header px-3 py-0 bg-white">
										<h3><div class="fas fa-bug mr-3"></div><?php esc_html_e( 'Issue Tracker', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-body p-3">
										<p class="m-0"><?php esc_html_e( 'We are always looking for ways to improve Simpods MVC and are actively working on it. If you would like to report any bugs or submit a feature request then please visit our Issue Tracker. To learn more about the Issue tracker, please visit our documentation which provides more information on this.', 'simpods-mvc' ); ?></p>
									</div>
									<div class="card-footer p-3 bg-white">
										<a href="https://bitbucket.org/ilabdev/simpods-mvc/issues?status=new&status=open" class="btn btn-outline-simpods-blue btn-block" target="_blank"><?php esc_html_e( 'Issue Tracker', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div> -->
							<div class="col-12 col-md-6 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100 border-0 shadow-none">
									<div class="card-header px-3 py-0 bg-white">
										<h3><i class="fas fa-envelope mr-3"></i><?php esc_html_e( 'Contact Us', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-body p-3">
										<p class="m-0"><?php esc_html_e( 'If you have any general questions or concerns then please get in touch using the contact details provided on the website or by filling out the contact form. Our friendly team will be happy to assist with all your queries', 'simpods-mvc' ); ?></p>
									</div>
									<div class="card-footer p-3 bg-white">
										<a href="https://simpodsmvc.me/contact-us/" class="btn btn-outline-simpods-blue btn-block" target="_blank"><?php esc_html_e( 'Contact Us', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
						</div>
						<hr class="my-5">
					</div>
					<?php if ( false ) { ?>
					<div id="simpods-mvc-components" class="d-none">
						<h2><?php esc_html_e( 'Simpods Components - Coming Soon!', 'simpods-mvc' ); ?></h2>
						<div class="row mx-n2">
							<div class="col-12 col-md-6 col-lg-3 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100">
									<?php
									$url = 'https://via.placeholder.com/300';
									?>
									<img src="<?php echo $url; ?>" alt="<?php esc_html_e( 'Simpods CPD Component', 'simpods-mvc' ); ?>" class="card-img-top" />
									<div class="card-body p-3">
										<h3 class="card-title m-0"><?php esc_html_e( 'Simpods CPD', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-footer p-3 bg-white">
										<a class="btn btn-outline-simpods-purple btn-block" target="_blank" disabled><?php esc_html_e( 'Find Out More', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
							<div class="col-12 col-md-6 col-lg-3 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100">
									<?php
									// $url = $funs->get_final_path( 'mvc/admin/images/plugins', 'pods-framework.png', true );
									?>
									<img src="<?php echo $url; ?>" alt="<?php esc_html_e( 'Simpods Portfolio Component', 'simpods-mvc' ); ?>" class="card-img-top" />
									<div class="card-body p-3">
										<h3 class="card-title m-0"><?php esc_html_e( 'Simpods Portfolios', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-footer p-3 bg-white">
										<a class="btn btn-outline-simpods-purple btn-block" target="_blank" disabled><?php esc_html_e( 'Find Out More', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
							<div class="col-12 col-md-6 col-lg-3 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100">
									<?php
									// $url = $funs->get_final_path( 'mvc/admin/images/plugins', 'pods-framework.png', true );
									?>
									<img src="<?php echo $url; ?>" alt="<?php esc_html_e( 'Simpods Account Component', 'simpods-mvc' ); ?>" class="card-img-top" />
									<div class="card-body p-3">
										<h3 class="card-title m-0"><?php esc_html_e( 'Simpods Accounts', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-footer p-3 bg-white">
										<a class="btn btn-outline-simpods-purple btn-block" target="_blank" disabled><?php esc_html_e( 'Find Out More', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
							<div class="col-12 col-md-6 col-lg-3 px-2 mb-3">
								<div class="card card-square p-0 m-0 h-100">
									<?php
									$url = $funs->get_final_path( 'mvc/admin/images/simpods-logos', 'simpods-logo-cutout-white-text.svg', true );
									?>
									<div class="card-img-top bg-simpods aspect-square">
										<div class="content p-3">
											<img src="<?php echo $url; ?>" alt="<?php esc_html_e( 'View More Components', 'simpods-mvc' ); ?>" class="" />
										</div>
									</div>
									<div class="card-body p-3">
										<h3 class="card-title m-0"><?php esc_html_e( 'And Many More!', 'simpods-mvc' ); ?></h3>
									</div>
									<div class="card-footer p-3 bg-white">
										<a class="btn btn-outline-simpods-purple btn-block" target="_blank" disabled><?php esc_html_e( 'View More', 'simpods-mvc' ); ?></a>
									</div>
								</div>
							</div>
						</div>
					</div>
					<?php } ?>
				</div>
			</div>
		</div>
		<?php
	}
}