<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods install and update
 *
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since 1.1.4 12/10/2018
 */
class MVC_Admin_Simpods_Install_Update extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-install-update';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_install_update';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_install_update';
	const PAGE_ICON = 'dashicons dashicons-update-alt';
	// Where to check the licence key
	const KEY_URL  = 'https://simpodsmvc.me/wp-admin/admin-ajax.php';
	// remote link
	private $simpods_url	=	'';
	// The page's title to display in-page 
	public static $page_title = ''; // Install & Update, leave it empty for now so it can't be accessed
	/* End Simpods MVC page variables */

	private $message 		= '';
	private $message_css_class	= '';
	private $notice = '';

	private $wp_errors = '';

	// database table version
	private $database_version = 2.0;
	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor,
	 *							$can_init set to true will run automatically
	 */
	private $own_cla	=	null;
	private $key	=	'';

	public $key_status;
	public $is_active;
	// same as the main site
	public $products	=	array(									
									'simpods-components'	=> array(),																		
								);

	public function __construct( $can_init = true ) {

		self::$page_title = esc_html__( 'Install & Update', 'simpods-mvc' );
		if ( defined( 'SIMPODS_INSTALLER_URL' ) && wp_http_validate_url( SIMPODS_INSTALLER_URL ) ) {
			$this->simpods_url = SIMPODS_INSTALLER_URL;
		} else if ( wp_http_validate_url( get_option('global_simpods_installer_staging', '' ) ) ){
			$this->simpods_url	=	get_option('global_simpods_installer_staging');
		} else {
			$this->simpods_url	=	self::KEY_URL;
		}
		
		// carry on if it is on the right section
		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_INSTALL_UPDATE' ) ) {
				define( 'SIMPODS_INSTALL_UPDATE', true );
			}

			$this->create_db();
			$this->actions();
			$this->filters();
		}
	}

	/**
	 * actions_fn wp actions to hook automatically. Global effect.
	 */
	public function actions() {
		//add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ), 11 );
	}

	/**
	 * filters_fn wp filters to hook automatically. Global effect.
	 */
	public function filters() {}

	/**
	 * enqueue_fn scripts and styles to enqueue automatically. Global effect.
	 */
	public function enqueue() {

	}

	/**
	 * create the areas relationship database, important logic: each main can have more than one friend
	 */
	public function create_db() {

		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->database_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}
		$columns = array(
			'spiu_title' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_slug' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_category' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_licence' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_licence_required' => array(
				'type'     => 'tinyint(1)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "1"',
			),
			'spiu_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spiu_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spiu_modified_user' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_user' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_status' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "active"',
			),
			'spiu_installed_version' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_latest_version' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_image' => array(
				'type'     => 'varchar(500)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_target_theme' => array(
				'type'     => 'varchar(50)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_allow_installations' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_price' => array(
				'type'     => 'decimal(10,2)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_link' => array(
				'type'     => 'varchar(500)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_expiry_date' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spiu_admin_page_slug' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spiu_min_simpods_version' => array(
				'type'     => 'varchar(20)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),	
			'spiu_invalid' => array(
				'type'     => 'tinyint(1)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "0"',
			),										
		); 
		$this->import_database( self::DB_TABLE, $columns );

		update_option( self::DB_TABLE . '_version', $this->database_version );
	}

	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $current_user, $cross_admin, $funs, $ctrl;

					

		$is_allowed = true;

		if ( !current_user_can( 'edit_posts' ) ) {
			// action before the iframe
			$is_allowed = false;

		}

		
		// allow coder to set permission
		$is_allowed = apply_filters( self::PAGE_SLUG . '_allow', $is_allowed, $_GET );

		if ( !$is_allowed ) {
			echo apply_filters( self::PAGE_SLUG . '_allow_msg', esc_html__( 'You do not have permission to edit this item.', 'simpods-mvc' ) );
			return '';
		}

		// submit an entry

		$this->notice .= $this->simpods_admin_remove_item( self::DB_TABLE );
		$nounc_field		= wp_nonce_field( 'simpods_install_update', 'spiu_install_update_nonce', true, false );

		$now 			= date( 'Y-m-d H:i:s' );
		$last_update		= get_option( 'spiu_last_update', '0000-00-00 00:00:00' );

		$install_feedback		= $this->install_update();
		// redirect if required
		$js_redirect		= '';
		if( isset( $_POST['spiu_admin_page_slug'] ) && $_POST['spiu_admin_page_slug'] != '' && isset( $_POST['spiu_redirect_to_its_area'] ) && $_POST['spiu_redirect_to_its_area'] == 1 ){
			
			//wp_safe_redirect( esc_url( admin_url( 'admin.php?page=' . $_POST['spiu_admin_page_slug'] ) ) );
			// $plugin_activated		 = activate_plugin( $_POST['spiu_product'] . '/' . $_POST['spiu_product'] . '.php' );
			// if ( is_wp_error( $plugin_activated ) ) {				
			// 	$return_data['message'] = esc_html__( 'Failed to active the plugin.', 'simpods-mvc' );
			// 	$return_data['success']	= false;									
			// }	
			$plugin_file	= $_POST['spiu_product'] . '/' . $_POST['spiu_product'] . '.php';
			$redirect_to 	= admin_url( 'admin.php?page=' . $_POST['spiu_admin_page_slug'] );
			$js_redirect	= ' activate_plugin ( "' . $plugin_file . '", "' . $redirect_to . '" );
								
							  ';
								//$("#spiu-check-update-spinner").removeClass( "d-none" ); 
								//window.location ="' . admin_url( 'admin.php?page=' . $_POST['spiu_admin_page_slug'] ) . '"
		}
		$css_class			= '';

		$date_format     	= get_option( 'date_format' );
		$time_format     	= get_option( 'time_format' );
		$date_time_format 	= $date_format . ' ' . $time_format;
		
		?>
		<script>
		jQuery( document ).ready( function( $ ) {			
			<?php
			echo $js_redirect;
			?>
		});
		</script>		
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0" id="spiu-wrap">
			<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1 class=""><?php echo self::$page_title; ?>
						<?php echo $ctrl->wp_notices; ?>
						<button id="spiu-check-update-top-btn" class="btn btn-simpods-blue spiu-check-update-btn float-right"><?php esc_html_e( 'Check Updates', 'simpods-mvc' ) ?></button>
						<span class="float-right" id="spiu-last-check-info">
						<?php
						if( $last_update != '0000-00-00 00:00:00' && $last_update != '' ){
							$update_display = date( get_option( 'date_format' ), strtotime( $last_update ) );
							/* translators: %s: Date */
							echo esc_html( sprintf( __( 'Last check: %s', 'simpods-mvc'), $update_display ) );
						}
						?>
						</span>
					</h1>
					<div id="spiu-notice" class=" is-dismissible">
						<?php echo esc_html( $this->notice ); ?>
					</div>

					<div id="simpods-mvc-install-update">
						<?php
						if( $install_feedback['active'] == true ){
							$css_class	= 'notice ';
							if( $install_feedback['success'] ){
								$css_class	.=	'notice-success';
							} else {
								$css_class	.=	'notice-error';
							}

							echo '
							<div class="' . $css_class . ' is-dismissible" id="spiu_info">
							' .	esc_html( $install_feedback['message'] ) . '
							</div>
							';
						}
						// get the products from the database
						foreach( $this->products as $k => $products ){
							$products 	= $funs->simpods_select( array( 'main_tb.`spiu_category`' => $k ), array( 'target_tb' => self::DB_TABLE ) );
							if( count( $products ) > 0 ){
								$this->products[ $k ] = $products;
							}							
						}
						?>
						<!-- Nav tabs -->
						<ul class="nav nav-tabs">
						<?php
						// get the transparent image
						//$transparent_img	=	$this->get_final_path( '', '/mvc/admin/images/transparent.png', 1, 0, 1, 0 );
						$transparent_img	=	SIMPODS_MVC_URL . '/admin/images/transparent.png';
						$is_active 		=	false;
						if( isset( $_POST['spiu_cat'] ) && isset( $this->products[ $_POST['spiu_cat'] ] ) ){
							$is_active 	=	true;
						}
						//print( $this->products, 1 );
						foreach( $this->products as $k => $products ){
							if( count( $products ) > 0 ){
								$active	=	'';
								if( isset( $_POST['spiu_cat'] ) && $_POST['spiu_cat'] == $k ){									
									$active	=	' active';									
								}
								if( ! $is_active ){ // run when it is not submitted.
									$is_active	=	true;
									$active	=	' active';
								}
								echo '
								<li class="nav-item">
									<a class="nav-link ' . $active . '" data-toggle="tab" href="#' . esc_attr( $k ) . '">' . esc_html( strtoupper( str_replace( '-', ' ', $k ) ) ) . '</a>
								</li>
							  	';
						  	}
						}
						?>

						</ul>

						<!-- Tab panes -->
						<div class="tab-content">
							<?php
							$is_active =	false;
							if( isset( $_POST['spiu_cat'] ) && isset( $this->products[ $_POST['spiu_cat'] ] ) ){
								$is_active 	=	true;
							}							
							foreach( $this->products as $k => $products ){
								if( count( $products ) > 0 ){
									$active	=	'';
									if( isset( $_POST['spiu_cat'] ) && $_POST['spiu_cat'] == $k ){									
										$active	=	' active';									
									}									
									if( ! $is_active ){
										$is_active	=	true;
										$active	=	' active';
									}
							?>
						  	<div class="tab-pane container <?php echo esc_attr( $active ) ?>" id="<?php echo esc_attr( $k ) ?>">
							<!-- <form class="simpods-settings" method="post" action="<?php //echo admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ); ?>" enctype="multipart/form-data" > -->
								<div class="row">
									<?php
									foreach( $products as $simpods_product ){
									//$simpods_product 	= $funs->simpods_select( array( 'main_tb.`spiu_slug`' => 'simpods-mvc' ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );
										if( ! empty( $simpods_product['spiu_target_theme'] ) ){ // no longer need the target theme, all save into the plugin folder
											continue;
										}
										$plugin_active  = is_plugin_active( $simpods_product['spiu_slug'] . '/' . $simpods_product['spiu_slug'] . '.php' );
										$licence 		= '';
										$info			= '<h5><a href="' . esc_url( $simpods_product['spiu_link'] ) . '" target="_blank" title="' . esc_attr__('Go to Simpods site to find out more.', 'simpods-mvc') . '">' . esc_html(  $simpods_product['spiu_title'] ) . '</a></h5>';
										$latest_version		=	'';
										$licence 			=	$simpods_product['spiu_licence'];
										$admin_page_slug	=	'';
										$admin_url_full		=	'';
										if( isset( $simpods_product['spiu_admin_page_slug'] ) ){
											$admin_url_full = 	$simpods_product['spiu_admin_page_slug'];
											$slug_parts 	=	explode('&', $simpods_product['spiu_admin_page_slug']);
											$admin_page_slug 	=	$slug_parts[ 0 ];
										}
										if( ! empty( $simpods_product['spiu_installed_version'] ) && is_dir( WP_PLUGIN_DIR . '/' . $simpods_product['spiu_slug'] ) ){ // not installed
											if( $simpods_product['spiu_licence_required'] == 1 ){
												if( $licence != '' ){
													$info		.=	'<strong>' . esc_html__('Licence:', 'simpods-mvc') . '</strong> ' . $licence . '<br/>';

												} else {

													$info	.=	esc_html__('Don\'t have a licence?', 'simpods-mvc');
													$info	.=	' <a href="https://simpodsmvc.me/my-account/simpods-licencing/" target="_blank" class="text-white">';
													if( empty( $simpods_product['spiu_price'] )  ){

														if(  $simpods_product['spiu_modified'] != '0000-00-00 00:00:00' ){
															$info	.=	esc_html__('Get a free licence.', 'simpods-mvc');
														} else {
															$info	.=	esc_html__('Get a licence.', 'simpods-mvc') . ' ';
														}

													} else {
														$info	.=	esc_html__('Get a licence.', 'simpods-mvc') . ' ';
													}
													$info	.=	'</a>';

												}
											}
											if( $plugin_active ){
												$info	.=	'<br/><strong>' . esc_html__('Installed Version:', 'simpods-mvc')  . '</strong> ' . $simpods_product['spiu_installed_version'] ;
											} else {
												$info	.=	'<br/><strong>' . esc_html__('Plugin not activated:', 'simpods-mvc')  . '</strong> ' ;
											}
										} else {
											$info		.=	'<strong>' . esc_html__('Not installed.', 'simpods-mvc')  . '</strong><br/>';
										}
										$latest_version	= '<strong>' . esc_html__('Latest Version:', 'simpods-mvc') . '</strong> ' . $simpods_product['spiu_latest_version'];
										$simpods_version 	= '';

										$css_class 		= '';
										if( isset( $simpods_product['spiu_min_simpods_version'] ) && $simpods_product['spiu_min_simpods_version'] != '' ){
											$simpods_version	=	'<strong>' . esc_html__('Simpods Min Version Required:', 'simpods-mvc') . '</strong> ' . $simpods_product['spiu_min_simpods_version'];
											// Disable the pop up if the the required Simpods version is greater than the installed simpods version											
											if( version_compare( $simpods_product['spiu_min_simpods_version'], SIMPODS_VERSION, '>' ) ){
												$css_class 	.= ' disabled';
											}											
										}
										//$count	=	count( $products );
									
									?>
									<div class="col-12 col-xs-12 col-sm-6 col-md-4 ">
										<div class="btn spiu-call-modal w100 mgt8 <?php echo $css_class; ?>"
											data-title="<?php echo esc_attr( $simpods_product['spiu_title'] ); ?>"
											data-product="<?php echo esc_attr( $simpods_product['spiu_slug'] ); ?>"
											data-licence="<?php echo esc_attr( $licence ); ?>"
											data-admin-page-slug="<?php echo esc_attr( $admin_url_full ); ?>"
											data-cat="<?php echo esc_attr( $k ); ?>"
											data-licence-required="<?php echo esc_attr( $simpods_product['spiu_licence_required'] ); ?>"
											style="background-image: url(<?php echo esc_url( $simpods_product['spiu_image'] ); ?>);"
											id="<?php echo esc_attr( $simpods_product['spiu_slug'] ); ?>-product-btn"
										>
											<?php
											
											if( $admin_page_slug != '' && ! empty( $simpods_product['spiu_installed_version'] ) && is_dir( WP_PLUGIN_DIR . '/' . $simpods_product['spiu_slug'] )  && $plugin_active ){
											?>	
											<a href="<?php echo esc_url( get_admin_url( null, 'admin.php?page=' . $admin_page_slug ) ); ?>" class="spiu-product-view">
												<span class="dashicons dashicons-welcome-view-site"></span>						 
											</a>
											<?php
											}
											?>									
											<div class="content spiu-product-info pd15">
												<div class="mgt5 aligncenter" id="<?php echo esc_attr( $simpods_product['spiu_slug'] ) ?>-info">
													<?php echo $info; ?>
												</div>
												<div class="simpods-white-box mgt5 pd5">
													<div class="mgt5 aligncenter" id="<?php echo esc_attr( $simpods_product['spiu_slug'] ) ?>-latest-version">	
														<?php echo $latest_version; ?>
													</div>
													<div class="mgt5 aligncenter" id="<?php echo esc_attr( $simpods_product['spiu_slug'] ) ?>-latest-version">	
														<?php echo $simpods_version; ?>
													</div>
												</div>
											</div>
										</div>
									</div>
									<?php
									}
									?>
								</div>
							<!-- </form>	 -->
						  	</div>
						  	<?php
						  		}
						  	}
						  	?>

						</div>
					</div>
					<div class="aligncenter mgt20">
					<?php
					/* translators: %1$s: <a> tag for update link, %2$s closing </a> tag */
					echo sprintf( esc_html__('To get the latest information about Simpods products, please %1$sCheck Updates.%2$s', 'simpods-mvc'), '<a href="#" class="spiu-check-update-btn">', '</a>' );
					echo '<br/>';
					if( $last_update != '0000-00-00 00:00:00' && $last_update != '' ){
						$update_display = date( $date_time_format, strtotime( $last_update ) );
						/* translators: %s: Date and time */
						echo esc_html( sprintf( esc_html__( 'Last check: %s', 'simpods-mvc'), $update_display ) );
					}
					?>
					</div>
				</div>
			</div>
			<div class="modal fade modal-simpods-blue modal-square" tabindex="-1" role="dialog" data-backdrop="static" class="edit-modal" id="spiu-modal">
				<div class="modal-dialog modal-lg modal-dialog-centered modal-simpods-blue" role="document">
					<form class="modal-content" method="post" action="<?php echo admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ); ?>" enctype="multipart/form-data" id="spiu-install-update-form">
						<div class="modal-header">
							<h5 class="modal-title"><?php esc_html_e( 'Install &amp; Update', 'simpods-mvc' ); ?> <span id="spiu-modal-product-title"></span></h5>
							<button type="button" class="close" data-dismiss="modal" aria-label="<?php esc_html_e( 'Close', 'simpods-mvc' ); ?>">
								<span aria-hidden="true">&times;</span>
							</button>
						</div>
						<div class="modal-body">
							<table class="w100 table-striped">
								<?php

								$for_licence	= 'spiu_licence_key';

								?>
								<tr>
									<td>

										<p class="mgt5"><label id="spiu-modal-info" for="<?php echo $for_licence; ?>"><?php esc_html_e( 'Don\'t have a licence key?', 'simpods-mvc' ); ?> <a href="https://simpodsmvc.me/my-account/simpods-licencing/"><?php esc_html_e( 'Get a free licence', 'simpods-mvc' ); ?></a></label></p>
									</td>
									<td>
										<h3 class="mgb5"><?php esc_html_e( 'Licence Key', 'simpods-mvc' ); ?></h3>
										<div id="spiu-licence-key-input-wrap">
											<input type="text" value="" name="<?php echo $for_licence; ?>" id="spiu-licence-key"  class="form-control" required />
											<span class="form-feedback d-none invalid-feedback" id="spiu_licence-error"><?php esc_html_e( 'Invalid licence key', 'simpods-mvc' ); ?></span>
										</div>
										<div id="spiu-licence-key-no-needed" class="d-none">
										<?php
										esc_html_e( 'Not required', 'simpods-mvc');
										?>
										</div>
									</td>
								</tr>
								<tr>
									<td colspan="2">
										<div id="spiu-modal-feedback"></div>
									</td>
								</tr>
								<?php
								//}
								?>

							</table>
						</div>
						<div class="modal-footer">
							<div class="">
								<div class=" ">
									<?php echo $nounc_field; ?>
									<p class="text-right w100">
										<?php esc_html_e( 'By clicking the "Install & Update" button below, you agree to submit your web address to Simpods MVC for licence validation.', 'simpods-mvc' ); ?>
										<br/>
										<?php esc_html_e( 'The plugin fold with the same component name will be deleted before installation. Please back up your site first.', 'simpods-mvc' ); ?>
										</p>
									<input type="hidden" id="spiu-product" name="spiu_product" value="" />
									<input type="hidden" id="spiu-fetch_file" name="spiu_fetch_file" value="" />
									<input type="hidden" id="spiu-center_nonce" name="spiu_center_nonce" value="" />
									<input type="hidden" id="spiu-action" name="spiu_action" value="spiu_install_update" />
									<input type="hidden" id="spiu-target_theme" name="spiu_target_theme" value="" />
									<input type="hidden" id="spiu-version" name="spiu_version" value="" />
									<input type="hidden" id="spiu-cat" name="spiu_cat" value="" />
									<input type="hidden" id="spiu-product_title" name="spiu_title" value="" />
									<input type="hidden" id="spiu-admin-page-slug" name="spiu_admin_page_slug" value="" />
									<input type="hidden" id="spiu-redirect_to_its_area" name="spiu_redirect_to_its_area" value="" />									
									<button type="submit" class="btn btn-simpods-blue alignright" id="spiu-submit" <?php echo $ctrl->disable; ?> ><?php esc_html_e( 'Install & Update', 'simpods-mvc' ); ?></button>

								</div>
							</div>

						</div>
						<span class="simpods-spinner d-none h-100" id="friend-spinner"></span>
					</form>

				</div>

			</div>
			<span class="simpods-spinner d-none h-100" id="spiu-check-update-spinner"></span>
		</div>
		<?php
		if( ( strtotime( $now ) - strtotime( $last_update ) ) > 604800 ){ // update when more than a week

			echo '
			<script type="text/javascript">
			jQuery( document ).ready( function( $ ) {
				$("#spiu-check-update-top-btn").click();
			})
			</script>
			';

		}
	}

	/**
	 * Checks new key validity and registers product
	 *
	 * @return object Registration object
	 * 	valid   bool   If key is valid
	 * 	status  string Status message
	 * 	expiry  string Expiry date
	 * 	url     string URL of ZIP to download
	 * 	version string version of CPD to install
	 * 	@author Dongjie Xu, Mark Wane
	 */
	public function register() {

		//$installing_bln = false;
		$return_data = (object)[
			'valid'   	=> false,
			'status'  	=> esc_html__( 'Failed to check key status. Please try again later.', 'simpods-mvc' ),
			'expiry'  	=> NULL,
			'url'     	=> NULL,
			'version' 	=> NULL,
			'local'		=> true,
			'eligible'	=> false,
			'get_key'	=> true,
		];

		// Check themes and plugins update are allowed. If not, do not process
		if( defined( 'DISALLOW_FILE_MODS' ) && DISALLOW_FILE_MODS == true ){	
			$return_data->status	=	esc_html__('Plugin and Theme Update and Installation has been disabled', 'simpods-mvc');
			return $return_data;
		}
		if ( isset( $_POST['action'] ) && $_POST['action'] == 'spiu_register_key' && isset( $_POST['product'] ) && trim( $_POST['product'] ) != '') {
			if ( isset( $_POST['security'] ) && wp_verify_nonce( $_POST['security'], 'simpods_install_update' ) ) {

				$this->key = trim( $_POST['key'] );

				$is_OK	= true;
				if( $_POST['get_key'] == 1 ){
					//check if the key is in the right format
					preg_match('/^[A-Za-z0-9]{4}-[A-Za-z0-9]{4}-[A-Za-z0-9]{4}-[A-Za-z0-9]{4}$/', $this->key, $match_arr );

					if ( 0 != count( $match_arr ) ) {
					} else {
						$is_OK	= false;
						$return_data->status = esc_html__( 'Invalid licence key', 'simpods-mvc' );
					}

				}

				if( $is_OK	== true ){

					$response_data = wp_safe_remote_post(
						$this->simpods_url,
						array(
							'body' => array(
								'action' 	=> 'simpods_key_verify',
								'key'    	=> $this->key,
								'url'    	=> site_url(),
								'product'	=> $_POST['product'],
								'security'	=> $_POST['security'],
							)
						)
					);
					
					// Failed to check status
					if ( is_wp_error( $response_data ) ) {
						return $return_data;
					}

					$return_data = json_decode( wp_remote_retrieve_body( $response_data ) );
					
					if ( wp_remote_retrieve_response_code( $response_data ) == 200 ) {
						if( !isset( $return_data->status ) || empty( $return_data->status ) ){
							$return_data->status = esc_html__( 'Verified.', 'simpods-mvc' );
						}
						$return_data->valid 	= true;
					}  else {
						if( !isset( $return_data->status ) || empty( $return_data->status ) ){
							$return_data->status = esc_html__( 'Failed to verify.', 'simpods-mvc' );
						}
						$return_data->valid 	= false;
						
					}

				}

				// check the required mini simpods version
				if( $return_data->status == 'Validated' ){
					
					if ( isset( $return_data->simpods_min_version_required ) && trim( $return_data->simpods_min_version_required ) != ''  ) {
						if( version_compare( SIMPODS_VERSION, $return_data->simpods_min_version_required ) < 0 ){
							$return_data->status = esc_html__( 'Simpods MVC minimum version required: ', 'simpods-mvc' ) . $return_data->simpods_min_version_required . esc_html__( '. Please update Simpods MVC first', 'simpods-mvc' );
							$return_data->valid 		= false;
							$return_data->eligible   = 0;
						}
						
					}
					
				}

			} else {

				$return_data->status = esc_html__( 'Sorry, your nonce did not verify.', 'simpods-mvc' );
			}
		}


		return $return_data;
	}

	/**
	 * fetch files and then install
	 * @author Dongjie Xu, Mark Wane
	 */
	public function install_update(){


		global $funs, $wp_filesystem;
		//$raw_dir 		= 	get_stylesheet_directory();

		$return_data		= array(
								'active'	=>  false,
								'success' 	=> 	false,
								'message'	=>	esc_html__('Missing parameters', 'simpods-mvc'),
								);

		// Check themes and plugins update are allowed. If not, do not process
		if( defined( 'DISALLOW_FILE_MODS' ) && DISALLOW_FILE_MODS == true ){	
			$return_data['message']	=	esc_html__('Plugin and Theme Update and Installation has been disabled', 'simpods-mvc');
			return $return_data;
		}			
		if ( isset( $_POST['spiu_action'] ) && $_POST['spiu_action'] == 'spiu_install_update' && isset( $_POST['spiu_product'] ) && trim( $_POST['spiu_product'] ) != '' && wp_http_validate_url( $_POST['spiu_fetch_file'] . '&nonce=' . $_POST['spiu_center_nonce'] ) ) {

			// if( $_POST['spiu_target_theme'] != 'main' && $_POST['spiu_target_theme'] != 'child-first' ) {
			// 	$return_data['active']	=	true;
			// 	$return_data['message']	=	esc_html__('Invalid theme path', 'simpods-mvc');
			// 	return $return_data;
			// }
			if ( isset( $_POST['spiu_install_update_nonce'] ) && wp_verify_nonce( $_POST['spiu_install_update_nonce'], 'simpods_install_update' ) ) {

				$return_data['active']	=	true;
				$current_url   			= admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' );
				$raw_dir 				= get_stylesheet_directory();
				//$upload_arr = wp_upload_dir();
				//$upload_dir = $upload_arr['basedir'];
				//$wp_content_dir 	= trailingslashit( $upload_arr['basedir'] );
				$wp_content_dir = trailingslashit( ABSPATH ) . 'wp-content/';
				$extra_fields	= array( 'spiu_licence_key', 'spiu_install_update_nonce', 'spiu_product', 'spiu_fetch_file', 'spiu_center_nonce', 'spiu_action', 'spiu-admin-page-slug', 'spiu-redirect_to_its_area' );	
				$is_credentialed 		= request_filesystem_credentials( $current_url, '', false, $wp_content_dir, $extra_fields );

				if ( false == $is_credentialed ) {
					// Cannot write directly, asking for FTP details

					$return_data['message']	=	esc_html__('Cannot write', 'simpods-mvc');
					return $return_data;
				}

				if ( !WP_Filesystem( $is_credentialed, $wp_content_dir ) ) {
					// Incorrect FTP details
					$is_credentialed = request_filesystem_credentials( $current_url, '', true, $wp_content_dir, $extra_fields );

					$return_data['message']	=	esc_html__('Incorrect FTP details', 'simpods-mvc');
					return $return_data;
				}


				global $wp_filesystem, $funs, $current_user;
				//SIMPODS_MVC_DIR
				//$simpods_location	= str_replace( '/mvc', '', rtrim( SIMPODS_MVC_DIR  , '/' ) );
				//$simpods_location	= str_replace( '\mvc', '', rtrim( SIMPODS_MVC_DIR  , '/' ) );//windows
				// $simpods_location	=	trailingslashit( SIMPODS_MVC_DIR );	
				// if( substr( $simpods_location, -5 ) == '/mvc/' || substr( $simpods_location, -5 ) == '\mvc/' ){
				// 	$simpods_location	= substr( $simpods_location, 0, strlen( $simpods_location ) - 5 );
				// } 				

				//$wp_content_dir 		= trailingslashit(  str_replace( ABSPATH, $wp_filesystem->abspath(), $raw_dir ) );

				$response_data 	= wp_remote_get(  $_POST['spiu_fetch_file'] . '&nonce=' . $_POST['spiu_center_nonce'] . '&site=' . urlencode( site_url() ) );
				$folder_name	= $_POST['spiu_product'] . '-temp'; // equivalent to the theme folder
				if( !$wp_filesystem->exists( $wp_content_dir . $folder_name ) ){
					wp_mkdir_p( $wp_content_dir . $folder_name );
				}

				$zip_file		= $folder_name . '/' . $_POST['spiu_product'] . '.zip';
				$response_code	= wp_remote_retrieve_response_code( $response_data ) ;

				if ( $response_code == 200 ) {

					$wp_filesystem->put_contents( $wp_content_dir . $zip_file, wp_remote_retrieve_body( $response_data ) );

					$unzipped_files = @unzip_file( trailingslashit( $wp_content_dir ) . $zip_file, $wp_content_dir . $folder_name );

					if ( is_wp_error( $unzipped_files ) ) {

						$return_data['message']	=	esc_html__('Errors in installation', 'simpods-mvc');
						return $return_data;
					} else {
						$is_deleted	=	$wp_filesystem->delete( $wp_content_dir . $zip_file );	// delete the zip file
						// where to install the files
						// $target_theme	=	$simpods_location; // default to where Simpods is
						// $is_copied	=	false;
						// if( $_POST['spiu_target_theme'] == 'child-first' ){ // targeting the child theme
						// 	if( is_child_theme() ){ // if a child theme is in use, use the child theme path
						// 		$target_theme	=	rtrim( get_stylesheet_directory() , '/' );
						// 	}
						// }
						
						// if( !is_dir( $target_theme . '/mvc/' ) ){
						// 	$return_data['message']	=	esc_html__('The directory for Simpods MVC is not found.', 'simpods-mvc');
						// 	return $return_data;
						// }
						$exist_products 	= $funs->simpods_select(
													array(
														'main_tb.`spiu_slug`' => $_POST['spiu_product']
													),
													array(
														'target_tb' => self::DB_TABLE,
														'limit' => 1
													)
												);
						// distribute files to their location
						// $tmpfolder_name	=	$wp_content_dir . trailingslashit( $folder_name );
						// $skipFile_str	=	$tmpfolder_name . $_POST['spiu_product'] . '-skip-overwrite.txt';
						// if( $wp_filesystem->exists( $skipFile_str ) ){
						// 	if( count( $exist_products ) && ! empty( $exist_products['spiu_installed_version'] ) ){ // if already installed, skip the files required
						// 		$delFiles_arr	= json_decode( $wp_filesystem->get_contents( $skipFile_str ) ) ;

						// 		// if updating simpods-mvc, avoid those files that can be customised.
						// 		if( is_array( $delFiles_arr )  ){
						// 			foreach( $delFiles_arr	as $toDel_str ){
						// 				$is_deleted	=	$wp_filesystem->delete( $tmpfolder_name . ltrim( $toDel_str, '/' ) );	// delete the files so they won't be copied
						// 			}
						// 		}
						// 	}
						// 	$is_deleted		=	$wp_filesystem->delete( $skipFile_str );	// delete the zip file
						// }
						// copy to the plugin dir
						//$upload_dir = 	wp_upload_dir(); 
						// remove existing file

						$is_deleted		=	$wp_filesystem->rmdir( WP_PLUGIN_DIR . '/' . $_POST['spiu_product'], true );
						$is_copied		=	copy_dir(
											    $wp_content_dir . $folder_name,
											    WP_PLUGIN_DIR
											);

						if( $is_copied ){// udpate completed, now update database

							$licence 				= '';
							
							if( isset( $_POST['spiu_licence_key'] ) ){
								$_POST['spiu_licence_key']	= trim( $_POST['spiu_licence_key'] );
								//check if the key is in the right format
								preg_match('/^[A-Za-z0-9]{4}-[A-Za-z0-9]{4}-[A-Za-z0-9]{4}-[A-Za-z0-9]{4}$/', $_POST['spiu_licence_key'] , $match_arr );

								if ( 0 != count( $match_arr ) ) {
									$licence 	= $_POST['spiu_licence_key'];
								}

							}
							
							$now 	= 	date( 'Y-m-d H:i:s' );
							$data	=	array(
												'spiu_title'				=>	$_POST['spiu_title'],
												'spiu_licence'				=>	$licence,
												'spiu_modified'				=>	$now,
												'spiu_modified_user'		=>	$current_user->ID,
												'spiu_installed_version'	=>	$_POST['spiu_version'],	
											);
							if( isset( $_POST['spiu_admin_page_slug'] ) && $_POST['spiu_admin_page_slug'] != ''  ){
								$data['spiu_admin_page_slug'] = $_POST['spiu_admin_page_slug'];
							}

							if( count( $exist_products ) ){
								if( $_POST['spiu_version'] > $exist_products[0]['spiu_latest_version'] ){
									$data['spiu_latest_version']	=	$_POST['spiu_version'];
								}
								$wheres	=	array(
													'id'	=>	$exist_products[0]['id'],
												);
								$funs->general_update( self::DB_TABLE, $data, $wheres );
							} else {
								$insert_data	= array(
								                	'spiu_slug'		=> 	$_POST['spiu_product'],
								                	'spiu_created'	=>	$now,
								                	'spiu_user'		=>	$current_user->ID,
								                	'spiu_status'	=>	'active',
								              );
								$data	= array_merge( $data, $insert_data );

								$id 	= $funs->general_insert( self::DB_TABLE, $data );

							}

						}

	
						$is_deleted		=	$wp_filesystem->rmdir( $wp_content_dir . $folder_name, true );
					//	$target			=	str_replace( ABSPATH, '', $target );
						$target 		= 	esc_html( $_POST['spiu_product'] );
						if( $is_copied	){
							$return_data['success'] 	= 	true;

							/* translators: %1$s: Component name, %2$s: Install location */
							$return_data['message']	= sprintf( esc_html__( '%1$s is successfully deployed to %2$s in the plugin folder.', 'simpods-mvc'), $_POST['spiu_title'], $target );

							if( isset( $_POST['spiu_admin_page_slug'] ) && $_POST['spiu_admin_page_slug'] != '' && isset( $_POST['spiu_redirect_to_its_area'] ) && $_POST['spiu_redirect_to_its_area'] == 1 ){
								$return_data['message'] .= ' ' . esc_html__( 'Please wait while we redirect you to its admin page.', 'simpods-mvc' );
								// $plugin_activated		 = activate_plugin( $_POST['spiu_product'] . '/' . $_POST['spiu_product'] . '.php' );
								// if ( is_wp_error( $plugin_activated ) ) {				
								// 	$return_data['message'] = esc_html__( 'Failed to active the plugin.', 'simpods-mvc' );
								// 	$return_data['success']	= false;									
								// }				
							}

						} else {
							/* translators: %s: Component name */
							$return_data['message']	= sprintf( esc_html__('Failed to deploy %s.', 'simpods-mvc'), $_POST['spiu_title'] );
						}

					}
				} else {
					/* translators: %s: Error code */
					$return_data['message']	= sprintf( esc_html__('Error: From Simpods - Error code %s', 'simpods-mvc'), $response_code  );
					return $return_data;
				}

			}

		}
		//echo $error_str;
		return $return_data;
	}
	/**
	 * Fetch simpods latest products information
	 * @author Dongjie Xu, Mark Wane
	 */
	public function check_updates(){

		global $funs;
		$return_data = (object)[
			'eligible' 	=> false,
			'status'  	=> esc_html__( 'Invalid parameters', 'simpods-mvc' ),
		];

		if ( isset( $_POST['action'] ) && $_POST['action'] == 'spiu_check_updates' ) {

			if ( isset( $_POST['security'] ) && wp_verify_nonce( $_POST['security'], 'simpods_install_update' ) ) {

				$response_data = wp_safe_remote_post(
					$this->simpods_url,
					array(
						'body' => array(
							'action' 	=> 'simpods_lastest_products',
							'security'	=> $_POST['security'],
						)
					)
				);

				// Failed to check status
				if ( is_wp_error( $response_data ) ) {
					$return_data->status = esc_html__( 'Failed to check Simpods products', 'simpods-mvc' );
					return $return_data;
				}

				$return_data = json_decode( wp_remote_retrieve_body( $response_data ) );

				if ( wp_remote_retrieve_response_code( $response_data ) == 200 ) {

					//$return_data->status = $return_data['remote_nonce'];
					//return $return_data->remote_nonce;
					if ( isset( $return_data->remote_nonce ) && wp_verify_nonce( $return_data->remote_nonce, 'simpods_install_update' ) && $return_data->eligible ) {
						$return_data->valid = true;

						// update database
						$is_updated	=	$this->insert_update( $return_data->items );

						if( ! $is_updated ){
							$return_data->status = esc_html__( 'Errors in updating database', 'simpods-mvc' );
						} else {
							$return_data->status = esc_html__( 'Products info updated, reloading...', 'simpods-mvc' );
						}
					} else {
						$return_data->status 	= esc_html__( 'Security error', 'simpods-mvc' );
						//remove items
						$return_data->items  	= array();
						$return_data->eligible  	= false;
					}
				}
			}

		}

		return $return_data;
	}

	/**
	 * Insert or update Simpods Products after fetching data from Simpods site
	 * @param  array $products Simpods products in categories
	 * @return boolean false if failed to update database
	 */
	public function insert_update( $products = array() ){

		global $wpdb, $current_user, $funs;

		$is_done	=	true;

		$now 	= 	date( 'Y-m-d H:i:s' );
		foreach( $products as $cat_str => $items ){
			foreach( $items as $item_obj ){

				$item_data 	= (array) $item_obj;
				$exist_products 	= $funs->simpods_select(
										array(
											'main_tb.`spiu_slug`' => $item_data['post_name'],
										),
										array(
											'target_tb' => self::DB_TABLE,
											'limit' => 1
										)
									);

				$data	=	array(
									'spiu_title'				=>	$item_data['post_title'],
									'spiu_category'				=>  $cat_str,
									'spiu_modified'				=>	$now,
									'spiu_modified_user'		=>	$current_user->ID,
									'spiu_licence_required' 	=>  $item_data['license_required'],
									'spiu_target_theme' 		=>  $item_data['target_theme'],
									'spiu_allow_installations' 	=>  $item_data['allow_installations'],
									'spiu_image' 				=>  $item_data['post_image'],
									'spiu_latest_version' 		=>  $item_data['latest_version'],
									'spiu_price' 				=>  $item_data['price'],
									'spiu_link' 				=>  $item_data['link'],
									'spiu_admin_page_slug' 		=>  $item_data['simpods_admin_page_slug'],
									'spiu_min_simpods_version'	=> 	$item_data['simpods_min_version_required'],
								);




				if( count( $exist_products ) ){

					$wheres		=	array(
										'id'	=>	$exist_products[0]['id'],
									);
					$success_bln	=	$funs->general_update( self::DB_TABLE, $data, $wheres );
					if( ! $is_done && ! $success_bln ){ // if already false, don't update
						$is_done	= false;
					}
				} else {
					$insert_data	= array(
					                	'spiu_slug'		=> 	$item_data['post_name'],
					                	'spiu_created'	=>	$now,
					                	'spiu_user'		=>	$current_user->ID,
					                	'spiu_status'	=>	'active',
						              );
					$data		= array_merge( $data, $insert_data );

					$id 		= $funs->general_insert( self::DB_TABLE, $data );
					if( ! $is_done && ! $id ){ // if already false, don't update
						$is_done	= false;
					}
				}
			}
		}
		update_option( 'spiu_last_update', $now, false );

		$simpods_data 	= $funs->simpods_select( array( 'main_tb.`spiu_slug`' => 'simpods-mvc' ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );
		if( count( $simpods_data ) ){

			$wheres		=	array(
								'id'	=>	$simpods_data[0]['id'],
							);
			$data		=	array(
									'spiu_installed_version' => SIMPODS_VERSION,
									);
			$success_bln	=	$funs->general_update( self::DB_TABLE, $data, $wheres );
			if( ! $is_done && ! $success_bln ){ // if already false, don't update
				$is_done	= false;
			}
		}

		return $is_done;
	}
}
