<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Linked Areas Class
 *
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 1.1.4 20/05/2015
 */
class MVC_Admin_Simpods_Linked_Areas extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-linked-areas';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_linked_areas';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_linked_areas';
	const PAGE_ICON = 'dashicons dashicons-admin-links';
	// The page's title to display in-page
	public static $page_title = 'Linked Areas';
	/* End Simpods MVC page variables */

	const FIELD_PREFIX    = 'spl_';
	const URL_VAR = 'simpods_linked_area';

	// Database table version
	private $database_version = 1.02;

	// Additional fields for pods table and functions
	public $custom_fields = array(
		'sp_linked_areas' => array( 'type' => 'text', 'settings' => '', 'default' => '' ),
	);

	private $capabilities = array(
								'simpods_assign_linked_area',
								'simpods_assign_private_linked_area',
								'simpods_assign_user_linked_area',
							);

	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Linked Areas', 'simpods-mvc' );

		// carry on if it is on the right section
		if( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_LINKED_AREAS' ) ) {
				define( 'SIMPODS_LINKED_AREAS', true );
			}

			$this->create_db();
			$this->actions();
			$this->filters();
			
		}

	}

	public function actions() {

		foreach ( MVC_Admin_Factory::$act_pod_tables as $table => $table_data ) {

			// after pod saved
			if ( $table_data['type'] == 'pod' ) {
				add_action( 'pods_api_post_save_pod_item_' . $table_data['pod'], array( $this, 'simpods_pods_post_save' ), 10, 3 );
			}

		}

		// check table fields when update pod editor
		add_action( 'save_post', array( $this, 'simpods_post_save' ), 10, 3 );

		add_action( 'pods_act_editor_before_sidebar', array( $this, 'linked_area_meta_box' ) );
		add_action( 'add_meta_boxes', array( $this, 'linked_area_meta_box' ) );
		// users
		add_action( 'user_new_form', array( $this, 'simpods_users_cats' ), 10, 1 );
		add_action( 'show_user_profile', array( $this, 'simpods_users_cats' ), 10, 1 );
		add_action( 'edit_user_profile', array( $this, 'simpods_users_cats' ), 10, 1 );
		add_action( 'user_register', array( $this, 'save_simpods_users_cats' ), 10, 1 );
		add_action( 'personal_options_update', array( $this, 'save_simpods_users_cats' ), 10, 1 );
		add_action( 'edit_user_profile_update', array( $this, 'save_simpods_users_cats' ), 10, 1 );

		add_action( 'restrict_manage_posts', array( $this, 'linked_area_list' ) );
	}

	public function filters() {
		add_filter( 'parse_query', array( $this, 'linked_area_filter' ), 0 );
		add_filter( 'admin_url', array( $this, 'url' ), 10, 2 );
	}

	public function enqueue() {}

	/**
	 * create the areas relationship database, important logic: each main can have more than one friend
	 */
	public function create_db() {

		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->database_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}
		$columns = array(
			'spl_friend' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spl_main' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spl_order' => array(
				'type'     => 'text',
				'settings' => '',
				'default'  => '',
			),
			'spl_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spl_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spl_modified_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spl_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spl_status' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "publish"',
			),
			'spl_input' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "checkbox"',
			),
			'spl_admin_only' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT ""',
			),			
		);
		$this->import_database( self::DB_TABLE, $columns );

		$this->add_capabilities( 'administrator', $this->capabilities );

		update_option( self::DB_TABLE . '_version', $this->database_version );
	}

	/**
	 * interface_fn: add an interface of this
	 */
	public function interface() {
		global $ctrl, $cross_admin, $funs;

		if( ! current_user_can( 'edit_posts' ) ){
			return false;
		}

		$settings_helper = new MVC_Admin_Settings_Helper();

		$tables = MVC_Admin_Factory::$act_pod_tables;

		$ctrl->notice .= $this->test_for_pods();
		$ctrl->forms();
		$ctrl->notice .= $this->simpods_admin_remove_item( self::DB_TABLE );

		$records 	 = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
		$count_data  = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE, 'count_only' => true ) );
		?>

		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
				<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1><?php echo self::$page_title; ?></h1>
					<?php echo $ctrl->notice; ?>
					<div id="simpods-la-add">
						<h2><?php esc_html_e( 'Create a Link', 'simpods-mvc') ?></h2>
						<?php $settings_helper->render_form( $ctrl->add_form_items, $ctrl->add_form_settings ); ?>
					</div>

					<?php if ( is_array( $records ) && !empty( $records ) ) { ?>
					<hr>
					<div id="simpods-la-table">
						<?php
						foreach ( $records as $index_int => &$record_data ) {
							$record_data = $funs->escape_attrs( $record_data );

							if ( !isset( $tables[ $record_data['spl_main'] ] ) ) {
								unset( $records[$index_int] );
								continue;
							}

							if ( $record_data['spl_input'] == 'radio' ) {
								$record_data['spl_input-formatted'] = esc_html__( 'One to many', 'simpods-mvc' );
							} else {
								$record_data['spl_input-formatted'] = esc_html__( 'Many to many', 'simpods-mvc' );
							}
							if ( $record_data['spl_admin_only'] == '0' ) {
								$record_data['spl_admin_only-formatted'] = esc_html__( 'No', 'simpods-mvc' );
							} else {
								$record_data['spl_admin_only-formatted'] = esc_html__( 'Yes', 'simpods-mvc' );
							}
							$main_tables 	= $tables[ $record_data['spl_main'] ];
							$friend_tables 	= $tables[ $record_data['spl_friend'] ];

							$main_tables['type'] 	= str_replace( '_', ' ', $main_tables['type'] );
							$friend_tables['type'] 	= str_replace( '_', ' ', $friend_tables['type'] );

							$record_data['spl_main-formatted']   = ucfirst( $main_tables['type'] ) . ': ' . $main_tables['title'];
							$record_data['spl_friend-formatted'] = ucfirst( $friend_tables['type'] ) . ': ' . $friend_tables['title'];
						}

						$headers = array(
							'spl_main-formatted'   		=> esc_html__( 'Main Table', 'simpods-mvc' ),
							'spl_friend-formatted' 		=> esc_html__( 'Friend Table', 'simpods-mvc' ),
							'spl_input-formatted'  		=> esc_html__( 'Relationship Type', 'simpods-mvc' ),
							'spl_admin_only-formatted'  => esc_html__( 'Admin Only', 'simpods-mvc' ),
						);
						$edit_fields = array(
							'spl_main',
							'spl_friend',
							'spl_input',
							'spl_admin_only',
							'id',
						);
						$edit_settings = array(
							'id-prefix' => 'simpods-la',
							'action'    => admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ),
							'title'     => esc_html__( 'Edit Linked Area', 'simpods-mvc' )
						);

						echo $this->simpods_table( $headers, $records, self::PAGE_SLUG, true, $edit_fields, $edit_settings );
						?>
					</div>
					<?php } ?>
					<script>var wp_errors = <?php echo is_wp_error( $ctrl->wp_errors ) ? wp_json_encode( $ctrl->wp_errors ) : "''"; ?>;</script>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Get all the ancestor tables for a specific table
	 *
	 * @param array $attributes The attributes of what to search for
	 * @return array The ancestor tables
	 */
	public function ancestors( $attributes ) {
		global $funs;
		$attributes =  wp_parse_args( $attributes, array(
			'table'     => '',
			'ancestors' => array(),
		) );

		$ancestors = $attributes['ancestors'];

		// find the friend
		$main_friends = $funs->simpods_select( array( 'spl_main' => array( $attributes['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		if ( $main_friends != NULL && count( $main_friends ) > 0 ) {
			array_push( $ancestors, $main_friends[0]['spl_friend'] );

			// loop the function itself to get all ancestors
			$ancestors = $this->ancestors( array( 'table' => $main_friends[0]['spl_friend'], 'ancestors' => $ancestors ) );
		}

		return $ancestors;
	}

	/**
	 * Gets items from a category
	 *
	 * @param string  $table        The category table to pull from
	 * @param unknown $category_table_id     The category table id. It could be pod id or posts, users
	 * @param integer $category_id     The category ID
	 * @param array   $attributes      Some attributes of what to pull
	 * @param boolean $can_echo_query Echo the query on screen
	 * @return array
	 */
	public function cat_items( $table, $category_table_id, $category_id, $attributes = array(), $can_echo_query = false ) {

		global $wpdb;

		$attributes =  wp_parse_args( $attributes, array(
			'order'    => 'ASC',
			'order_by' => '',
			'group_by' => '',
			'start'    => 0,
			'limit'    => 0,

		) );

		$params = array();

		$where = ' AND `sp_linked_areas` REGEXP "(:\"' . esc_sql( $category_table_id ) . '.' . intval( $category_id ) . '\";{1,})"';

		if ( count( $params ) > 0 ) {
			$query = $wpdb->prepare( 'SELECT
										 *
										 FROM `' . $wpdb->prefix . $table . '`

										 WHERE 1 = 1 ' . $where . ' ', $params );
		} else {
			$query = 'SELECT
										 *
										 FROM `' . $wpdb->prefix . $table . '`

										 WHERE 1 = 1 ' . $where . ' ';
		}

		if ( $can_echo_query ) {
			echo $query;
		}

		$items = $wpdb->get_results( $query, ARRAY_A );

		return $items;
	}

	/**
	 * Get all the decendant tables for a specific table
	 *
	 * @param array $attributes The attributes of what to search for
	 * @return array The decendant tables
	 */
	public function decendants( $attributes ) {

		global $funs;
		$attributes =  wp_parse_args( $attributes , array(
			'table'      => '',
			'decendants' => array(),
		) );

		$decendants = $attributes['decendants'];

		// find the main
		$main_friends = $funs->simpods_select( array( 'spl_friend' => array( $attributes['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		if ( $main_friends != NULL && count( $main_friends ) > 0 ) {
			array_push( $decendants, $main_friends[0]['spl_main'] );

			// loop the function itself to get all ancestors
			$decendants = $this->decendants( array( 'table' => $main_friends[0]['spl_main'], 'decendants' => $decendants ) );
		}

		return $decendants;
	}

	/**
	 * Filters the friend tables when a main table is selected
	 *
	 * @param array $attributes An array containing the main table name
	 * @return array
	 */
	public function filter_main_table( $attributes ) {
		global $funs;

		$filters =  wp_parse_args( $attributes, array(
			'table' => '',
		) );

		$friend_tables = MVC_Admin_Factory::$act_pod_tables;

		// remove existing friends from the list
		$items = $funs->simpods_select( array( 'spl_main' => array( $filters['table'] ) ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC' ) );

		if ( is_array( $items ) && count( $items ) > 0 && isset( $friend_tables[$items[0]['spl_friend']] ) ) {

			foreach ( $items as $k => $values ) {
				unset( $friend_tables[ $values['spl_friend'] ] );
			}

		}

		// unset the main table
		unset( $friend_tables[ $filters['table'] ] );

		$filters['table'] = esc_sql( $filters['table'] );

		return $friend_tables;
	}

	/**
	 * Filters search results
	 * @param  object $query_obj The query object that parsed the query.
	 * @return null
	 */
	public function linked_area_filter( $query_obj ) {
		global $funs;

		// find the pod table assets
		if ( isset( $_GET[self::URL_VAR] ) && is_numeric( $_GET[self::URL_VAR] ) ) {
			$pod_tables = $funs->find_a_pod_tb( 'pod', MVC_Admin_Factory::$post_type );

			if ( isset( $pod_tables['id'] ) ) {

				$linked_areas = $funs->simpods_select( array( 'spl_main' => 'pod_' . $pod_tables['id'] ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'spl_friend' ) );

				if ( isset( $linked_areas[0] ) && isset( MVC_Admin_Factory::$act_pod_tables[$linked_areas[0]['spl_friend']] ) ) {
					$pod_parent_tables = MVC_Admin_Factory::$act_pod_tables[$linked_areas[0]['spl_friend']];

					$assigned_data = $this->cat_items( $pod_tables['name'], $pod_parent_tables['id'], $_GET[self::URL_VAR] );

					if ( $assigned_data ) {
						$post_ids = Arrays::array_column( $assigned_data, 'id' );
					} else {
						$post_ids = array( -1 );
					}

					if ( count( $query_obj->query_vars['post__in'] ) > 0 ) {
						$post_ids = array_intersect( $query_obj->query_vars['post__in'], $post_ids );
					}

					if ( count( $post_ids ) == 0 ) {
						$post_ids = array( -1 );
					}

					$query_obj->query_vars['post__in'] = $post_ids;

					if ( isset( $_GET['orderby'] ) ) {
						$query_obj->set( 'orderby', $_GET['orderby'] );
					}

				}

			}

		}

	}

	/**
	 * Provide a combo box for filtering by module pages
	 *
	 * @return null
	 */
	public function linked_area_list() {

		global $funs;
		// find the pod table assets
		$pod_tables = $funs->find_a_pod_tb( 'pod', MVC_Admin_Factory::$post_type );

		if ( isset( $pod_tables['id'] ) ) {

			$linked_areas = $funs->simpods_select( array( 'spl_main' => 'pod_' . $pod_tables['id'] ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'spl_friend' ) );

			if ( isset( $linked_areas[0] ) && isset( MVC_Admin_Factory::$act_pod_tables[$linked_areas[0]['spl_friend']] ) ) {
				$pod_parent_table = MVC_Admin_Factory::$act_pod_tables[$linked_areas[0]['spl_friend']]['name'];

				$this->pages_list( esc_html__( 'Simpods Linked Areas', 'simpods-mvc' ), $pod_parent_table, self::URL_VAR );
			}

		}

	}

	/**
	* Add a Simpods Linked Areas meta box to relevant post types
	*
	* @return null
	*/
	public function linked_area_meta_box() {

		if( ! current_user_can( 'simpods_assign_linked_area' ) ){
			return false;
		}
		global $funs, $current_user;
		$is_allowed = true;

		if ( isset( $_GET['post'] ) ){
			$post_id = (int) $_GET['post'];
		} elseif ( isset( $_GET['id'] ) ){
			$post_id = (int) $_GET['id'];
		} else {
			$post_id = 0;
		}


		$tables = MVC_Admin_Factory::$act_pod_tables;

		if ( isset( $_GET['page'] ) && strpos( $_GET['page'], 'pods-manage-' ) === 0 ) {
			$table = substr( $_GET['page'], 12 );
		} else {
			$table = MVC_Admin_Factory::$post_type;
		}

		$idx = '';

		// find out the index in the table array
		foreach ( $tables as $key => $values ) {
			if ( $table == $values['pod'] ) {
				$idx = $key;
				break;
			}

		}

		// if no table found, return
		if ( $idx == '' ) {
			return;
		}

		$linked_areas = $funs->simpods_select( array( 'spl_main' => $idx ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'spl_friend' ) );

		// if friend table exist
		if ( is_array( $linked_areas ) && count( $linked_areas ) > 0 ) {
			foreach ( $linked_areas as $area_idx => $area_data ) {

				if( isset( $area_data['spl_admin_only'] ) && $area_data['spl_admin_only'] == 1 && ! in_array( 'administrator', $current_user->roles ) ){
					continue;
				}
				if( current_user_can( 'skip_simpods_linked_area_' . $tables[ $area_data['spl_main'] ]['name'] . '_' . $tables[ $area_data['spl_friend'] ]['name'] ) ) {
					continue;
				}			
				$area_data['main_tb_idx'] = $idx;
				if ( ( isset( $_GET['page'] ) && strpos( $_GET['page'], 'pods-manage-' ) === 0 ) ) {

					// build an array like the one for add_meta_box call back
					$args = array(
						'id'    => self::PAGE_SLUG . '-' . $area_data['spl_friend'],
						'title' => self::$page_title . ' (' . $tables[ $area_data['spl_friend'] ]['name'] . ')',
						'args'  => $area_data,
					);
					echo $this->simpods_box( $args );

				} else {

					// get the table info to check if it a pods table
					if ( $tables[ $idx ]['type'] != 'pod' ) {

						$screens = array( MVC_Admin_Factory::$post_type );

						add_meta_box(
							self::PAGE_SLUG . '-' . $area_data['spl_friend'],
							self::$page_title . ' (' . $tables[$area_data['spl_friend']]['title'] . ')',
							array( $this, 'simpods_post_type_cats' ),
							$screens,
							'side',
							'high',
							$area_data
						);

					}

				}

			}

		}

	}

	/**
	 * Save data when Linked Areas are used in the Users section
	 *
	 * @param integer $user_id User ID of the user
	 * @return null
	 */
	public function save_simpods_users_cats( $user_id ) {
		if ( !isset( $_POST['simpods_linked_area'] ) ) {
			return;
		}

		$index = '';
		$table = '';

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {

			if ( $values['pod'] == 'user' ) {
				$index = $key;
				break;
			}

		}

		if ( $index == '' ) {
			return;
		} else {
			$table = MVC_Admin_Factory::$act_pod_tables[ $index ]['name'];
		}

		$this->simpods_cats_save( $user_id, $table );

	}

	/**
	 * Prints out the metabox for Linked Areas
	 *
	 * @param array $args Data saved for the current item
	 * @return string The HTML for the box
	 */
	public function simpods_box( $args ) {
		global $current_user;

		if( isset( $args['args']['spl_admin_only'] ) && $args['args']['spl_admin_only'] == 1 && ! in_array( 'administrator', $current_user->roles ) ){
			return false;
		}
		if( ! current_user_can( 'simpods_assign_linked_area' ) ){
			return false;
		}		
		global $funs;

		$html   = '';
		$tables 	= MVC_Admin_Factory::$act_pod_tables;
		$area_data  = $args['args'];
		$table     	= $area_data['main_tb_idx'];

		$linked_areas = $funs->simpods_select( array( 'spl_friend' => $area_data['spl_friend'], 'spl_main' => $table ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );

		// if the All category not found, find other links, otherwise ignore then
		if ( !is_array( $linked_areas ) || count( $linked_areas ) == 0 ) {
			$linked_areas = $funs->simpods_select( array( 'spl_friend' => $area_data['spl_friend'], 'spl_main' => $table ), array( 'target_tb' => self::DB_TABLE ) );
		}

		if ( !is_array( $linked_areas ) || count( $linked_areas ) == 0 ) {
			return '';
		}

		$wheres = $this->get_the_ID( $tables[ $table ] );

		// get current item

		$this->update_columns( $tables[ $table ]['name'], $this->custom_fields );

		$items = $funs->simpods_select( $wheres, array( 'target_tb' => $tables[ $table ]['name'], 'limit' => 1 ) );


		$friend_title = $tables[ $linked_areas[0]['spl_friend'] ]['title'];
		$css_class    = $tables[ $table ]['pod'] == 'user' ? 'mgl10 pdb10' : '';

		if ( ( $tables[ $table ]['type'] == 'pod' || isset( $_GET['page'] ) ) || $tables[ $table ]['pod'] == 'user' ) {
			$html .= '
			<div class="postbox simpods_friend_area_cats" id="simpods_friend_area_cats_' . $tables[ $table ]['name'] . '">
				<button aria-expanded="true" class="handlediv button-link" type="button">
					<span class="screen-reader-text">' . esc_html__( 'Toggle panel', 'simpods-mvc' ) . ': ' . esc_html__( 'Categories', 'simpods-mvc' ) . '</span><span aria-hidden="true" class="toggle-indicator"></span>
				</button>
				<h2 class="hndle ui-sortable-handle ' . $css_class . '"><span>' . $friend_title . '</span></h2>
				<div class="inside">';
		}

		$html .= '
				<div class="categorydiv" id="simpods_links">

					<div class="tabs-panel" style="display: block;">

						<ul class="categorychecklist form-no-clear" data-wp-lists="list:category">
						';


		foreach ( $linked_areas as $k => $values ) {

			$heading 		= 'All';

			$friend_name 	= $tables[$linked_areas[0]['spl_friend']]['name'];

			$friend_items 	= $funs->simpods_select( array(), array( 'target_tb' => $friend_name ) );

			// make sure it is uploaded
			$html .= '<input type="hidden" name="simpods_linked_area[' . $tables[ $linked_areas[0]['spl_friend'] ]['id'] . '_unset]" value="0" autocomplete="off">';

			$i = 0;

			foreach ( $friend_items as $fk => $friend_item_data ) {

				$friend_item_data['id'] = $friend_item_data['id'];
				$pod_index = get_post_meta( $tables[ $linked_areas[0]['spl_friend'] ]['id'], 'pod_index', true );

				if ( !current_user_can( 'edit_post', $friend_item_data['id'] )  && !current_user_can( 'simpods_assign_private_linked_area' ) ) {
					continue;
				}

				if ( !empty( $pod_index ) ) {
					$friend_item_data['sp_title'] = $friend_item_data[$pod_index];
				}

				$parent_categories = array();

				if ( isset( $items[0] ) ) {
					$parent_categories = maybe_unserialize( $items[0]['sp_linked_areas'] );
				}

				if ( !is_array( $parent_categories ) ) {
					$parent_categories = array();
				}

				// check the boxes, with types
				$checked_style = in_array( $tables[$linked_areas[0]['spl_friend']]['id'] . '.' . $friend_item_data['id'], $parent_categories, true ) == true ? 'checked="checked"' : '';

				// add new will pass the filter - simpods_hierarchy
				if ( isset( $_GET[self::URL_VAR] ) && is_numeric( $_GET[self::URL_VAR] ) && $friend_item_data['id'] == $_GET[self::URL_VAR] ) {
					$checked_style = 'checked="checked"';
				}

				//input type
				$input_type = 'checkbox';
				$group     	= '';
				$input_name = 'simpods_linked_area[' . $tables[ $linked_areas[0]['spl_friend'] ]['id'] . '_' . $i . ']';

				if ( $linked_areas[0]['spl_input'] == 'radio' ) {
					$input_type = $linked_areas[0]['spl_input'];
					$group     	= 'group="simpods_category"';
					$input_name = 'simpods_linked_area[' . $tables[ $linked_areas[0]['spl_friend'] ]['id'] . '_0]';
				}

				$spl_title = '';

				if ( isset( $friend_item_data['sp_title'] ) ) {
					$spl_title = $friend_item_data['sp_title'];
				}

				$html .= '
											<li >
												<label class="selectit">
													<input type="' . $input_type . '" ' . $checked_style . ' ' . $group . ' name="' . $input_name . '" value="' . $friend_item_data['id'] . '" autocomplete="off">
													' . $spl_title . '
												</label>
											</li>
											';
				$i++;
			}

			if ( $i == 0 ) {
				$html .= '
											<li >
												' . esc_html__( 'No items', 'simpods-mvc' ) . '
											</li>
											';
			}

		}

		$html .= '
						</ul>
					</div>

				</div>';

		if ( ( $tables[ $table ]['type'] == 'pod' || isset( $_GET['page'] ) ) || $tables[ $table ]['pod'] == 'user' ) {
			$html .= '
				</div>
			</div>';
		}

		return $html;
	}

	/**
	 * Save or update the Linked Area data for normal post type
	 *
	 * @param integer $post_id Current post ID
	 * @param string   $table     Target table
	 * @return void
	 */
	public function simpods_cats_save( $post_id, $table ) {
		global $wpdb, $current_user, $funs;
					
		$wheres                      = array( 'id' => $post_id );
		$parent_categories           = array();
		$now                         = date( 'Y-m-d H:i:s' );
		$data                        = array();
		$data['spl_modified']        = $now;
		$data['spl_modified_author'] = $current_user->ID;

		$items = $funs->simpods_select( $wheres, array( 'target_tb' => $table, 'limit' => 1 ) );

		if ( isset( $items[0] ) ) {

			$parent_categories = maybe_unserialize( $items[0]['sp_linked_areas'] );

			if ( !is_array( $parent_categories ) ) {
				$parent_categories = array();
			}

			$removed = array();

			foreach ( $_POST['simpods_linked_area'] as $key => $category_id ) {

				$key_exploded = explode( '_', $key );

				// only remove the items from the current friend, but the others. In case some tables changed friends and want to change back, so the data are still there
				if ( !isset( $removed[ $key_exploded[0] ] ) ) {
					foreach ( $parent_categories as $parent_key => $parent_id ) {
						$parent_key_exploded = explode( '.', $parent_id );
						if ( $parent_key_exploded[0] == $key_exploded[0] ) {
							unset( $parent_categories[ $parent_key ] );
						}

					}

					$removed[ $key_exploded[0] ] = true;
				}

				if ( $category_id != 0 ) {
					array_push( $parent_categories, $key_exploded[0] . '.' . $category_id );
				}

			}

			$parent_categories = array_values( array_unique( $parent_categories ) );

			$data['sp_linked_areas'] = maybe_serialize( $parent_categories );

			$funs->general_update( $wpdb->prefix . $table, $data, $wheres );

		} else {

			foreach ( $_POST['simpods_linked_area'] as $key => $category_id ) {
				$key_exploded = explode( '_', $key );

				if ( $category_id != 0 ) {
					array_push( $parent_categories, $key_exploded[0] . '.' . $category_id );
				}

			}

			$data['sp_linked_areas'] = maybe_serialize( $parent_categories );
			$data['id']              = $post_id;
			$funs->general_insert( $wpdb->prefix . $table, $data );
		}

	}

	/**
	 * Save or update the Linked Area for advanced content type
	 *
	 * @param array   $pieces Current Pod details
	 * @param boolean $is_new  Is a new item or not
	 * @param integer $id     ID of item being updated
	 * @return void
	 */
	public function simpods_pods_post_save( $pieces, $is_new, $id ) {
		if ( isset( $_POST['_pods_pod'] ) && isset( $_POST['simpods_linked_area'] ) ) {

			$this->simpods_cats_save( $id, 'pods_' . $_POST['_pods_pod'] );

		}

	}

	/**
	 * Passes data to simpods_box_fn to create the Linked Area metabox for normal post type
	 *
	 * @param object $post     Current post
	 * @param array  $area_data Data saved for the current item
	 * @return null
	 */
	public function simpods_post_type_cats( $post, $area_data ) {
		echo $this->simpods_box( $area_data );
	}

	/**
	 * Updates a child post when a post is saved
	 *
	 * @param integer $post_id The post ID
	 * @param object  $post_obj   The post object
	 * @param boolean $update_bln Whether the post is being updated
	 * @return null
	 */
	public function simpods_post_save( $post_id, $post_obj, $update_bln ) {
		if ( !isset( $_POST['simpods_linked_area'] ) ) {
			return;
		}

		// if the post type is post or user, and s to the end to distinguish it from the pods post or user entension
		$post_type = MVC_Admin_Factory::$post_type;
		$table    = '';

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {

			if ( $values['pod'] == $post_type ) {
				$table = $values['name'];
				break;

			}

		}

		$this->simpods_cats_save( $post_id, $table );
	}

	/**
	 * Passes data to simpods_box_fn_fn to create the Linked Area metabox for the Users section
	 *
	 * @param object $args_obj The user object
	 * @return null
	 */
	public function simpods_users_cats( $args_obj ) {
		global $funs, $current_user;
		
		$tables = MVC_Admin_Factory::$act_pod_tables;
		$index  = '';

		foreach ( MVC_Admin_Factory::$act_pod_tables as $key => $values ) {

			if ( $values['pod'] == 'user' ) {
				$index = $key;
				break;
			}

		}

		$linked_areas = $funs->simpods_select( array( 'spl_main' => $index ), array( 'target_tb' => self::DB_TABLE, 'order' => 'DESC', 'group_by' => 'spl_friend' ) );

		if ( is_array( $linked_areas ) && count( $linked_areas ) > 0 ) {
			$user_id = isset( $args_obj->ID ) ? $args_obj->ID : -1;

			foreach ( $linked_areas as $area_idx => $area_data ) {
				
				if( current_user_can( 'skip_simpods_linked_area_' . $tables[ $area_data['spl_main'] ]['name'] . '_' . $tables[ $area_data['spl_friend'] ]['name'] ) ) {
					continue;
				}
				$area_data                = $area_data;
				$area_data['main_tb_idx'] = $key;
				$area_data['user_id']     = $user_id;
				$args                = array(
					'id'    => 'simpods_friend_area_cats_pod_' . $area_data['spl_friend'],
					'title' => $tables[ $area_data['spl_friend'] ]['name'],
					'args'  => $area_data,
				);

				echo $this->simpods_box( $args );
			}
		}

	}

	/**
	 * Append parameters to a provided URL. Limit to edit.php so it won't cause problems to the details
	 * @param  string $url The URL to append parameters to
	 * @return string The appended URL
	 */
	public function url( $url, $path ) {

		if ( isset( $_GET[self::URL_VAR] ) && is_numeric( $_GET[self::URL_VAR] ) && strpos( $url, 'edit.php' ) !== false ) {

			if ( strpos( $url, '?' ) !== false ) {
				$url .= '&';
			}

			$url .= self::URL_VAR . '=' . $_GET[self::URL_VAR];

		}

		return $url;
	}

}
