<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods MVC Settings Class
 *
 * @package Simpods MVC
 * @author Andrew Palfrey
 * @since 1.1.4 01/10/2017
 */
class MVC_Admin_Simpods_Settings extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-settings';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_settings';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_settings';
	const PAGE_ICON = 'dashicons dashicons-admin-settings';
	// The page's title to display in-page
	public static $page_title = 'Simpods Settings';
	/* End Simpods MVC page variables */
	// database table version
	private $db_version = 1.21;

	public function __construct( $init = true ) {
		self::$page_title = esc_html__( 'Simpods Settings', 'simpods-mvc' );

		// carry on if it is on the right section
		if ( $init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_SETTINGS' ) ) {
				define( 'SIMPODS_SETTINGS', true );
			}		
		}
	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}

	/**
	 * Create the database for this area
	 * @return null
	 */
	public function create_db() {
		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->db_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}

		global $funs, $wpdb, $simpods_setup;

		$db_columns = array(
			'sps_option' => array(
				'type' => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default' => '',
			),
			'sps_value' => array(
				'type' => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default' => '',
			),
			'sps_created' => array(
				'type' => 'datetime',
				'settings' => 'NULL',
				'default' => 'NULL',
			),
			'sps_modified' => array(
				'type' => 'datetime',
				'settings' => 'NULL',
				'default' => 'NULL',
			),
		);

		$this->import_database( self::DB_TABLE, $db_columns );

		$table = $wpdb->prefix . 'simpods_settings';
		$drop_columns = array(
			'sps_name',
			'sps_desc',
			'sps_type',
		);

		foreach ( $drop_columns as $column ) {
			$column_exists = $wpdb->get_results( $wpdb->prepare(
				"SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s AND COLUMN_NAME = %s ",
				DB_NAME, $table, $column
			) );

			if ( empty( $column_exists ) ) {
				continue;
			}
			
			$wpdb->query( "ALTER TABLE $table DROP COLUMN $column" );
		}
		
		foreach ( $simpods_setup->defaults['simpods_settings'] as $option => $val ) {
			$result = $funs->simpods_select( array( 'sps_option' => $option ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );
			
			if ( !$result ) {
				$db_data = array(
					'sps_option' => $option,
					'sps_value' => $val,
					'sps_created' => date( 'Y-m-d H:i:s' ),
					'sps_modified' => date( 'Y-m-d H:i:s' ),
				);
				$funs->general_insert( self::DB_TABLE, $db_data );
			}
		}

		update_option( self::DB_TABLE . '_version', $this->db_version );

		if ( !get_option( 'simpods_version' ) || get_option( 'simpods_version' ) !== SIMPODS_VERSION ) {
			update_option( 'simpods_version', SIMPODS_VERSION );
		}

	}

	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $ctrl, $cross_admin;

		$settings_helper = new MVC_Admin_Settings_Helper();

		$ctrl->forms();
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
				<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<div id="simpods-ss-settings">
						<h1><?php echo self::$page_title; ?></h1>
						<?php echo $ctrl->wp_notices; ?>
						<?php $settings_helper->render_form( $ctrl->form_items['main'], $ctrl->form_settings['main'] ); ?>
					</div>
				</div>
			</div>
		</div>
		<?php
	}

}
