<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Style & Script Admin class
 *
 * @package: Simpods MVC
 * @author: Andrew Palfrey
 * @since 1.1.4 18/09/2018
 */
class MVC_Admin_Simpods_Style_Script extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-style-script';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_style_script';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_style_script';
	const PAGE_ICON = 'dashicons dashicons-admin-appearance';
	// The page's title to display in-page
	public static $page_title = 'Style & Script';
	/* End Simpods MVC page variables */

	private $db_version = 1.0;

	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'Style & Script', 'simpods-mvc' );

		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_STYLE_SCRIPT' ) ) {
				define( 'SIMPODS_STYLE_SCRIPT', true );
			}

			$this->create_db();
		}

	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}

	/**
	 * Create the database for this area
	 * @return null
	 */
	public function create_db() {
		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->db_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}

		global $funs, $wpdb, $simpods_setup;

		$db_columns = array(
			'spss_scss_path' => array(
				'type'     => 'varchar(500)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spss_css_path' => array(
				'type'     => 'varchar(500)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spss_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spss_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spss_modified_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
		);

		$this->import_database( self::DB_TABLE, $db_columns );

		update_option( self::DB_TABLE . '_version', $this->db_version );
	}

	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $ctrl, $cross_admin, $funs;

		$settings_helper = new MVC_Admin_Settings_Helper();

		$ctrl->forms();
		$ctrl->wp_notices .= $this->simpods_admin_remove_item( self::DB_TABLE );

		// $records_arr = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
			<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1><?php echo self::$page_title; ?></h1>
					<?php echo $ctrl->wp_notices; ?>
					<div id="simpods-ss-settings">
						<!-- <h2><?php //esc_html_e( 'Style & Script Settings', 'simpods-mvc'); ?></h2> -->
						<?php $settings_helper->render_form( $ctrl->form_items['main'], $ctrl->form_settings['main'] ); ?>
					</div>
					<hr>
					<?php if ( is_array( $ctrl->records ) && !empty( $ctrl->records ) ) { ?>
					<hr>
					<div id="simpods-ss-table">
						<?php
						$headers = array(
							'spss_scss_path' => esc_html__( 'SCSS Folder Path', 'simpods-mvc' ),
							'spss_css_path'  => esc_html__( 'CSS Folder Path', 'simpods-mvc' ),
						);

						$edit = array(
							'spss_scss_path',
							'spss_css_path',
							'id',
						);

						$settings = array(
							'id-prefix' => 'simpods-ss',
							'action'    => admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ),
							'title'     => esc_html__( 'Edit Compile Location', 'simpods-mvc' )
						);

						echo $this->simpods_table( $headers, $ctrl->records, self::PAGE_SLUG, true, $edit, $settings );
						?>
					</div>
					<?php } ?>
					<script>var wp_errors = <?php echo is_wp_error( $ctrl->wp_errors ) ? wp_json_encode( $ctrl->wp_errors ) : "''"; ?>;</script>
				</div>
			</div>
		</div>
		<?php
	}

}