<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * User Code, generate codes to assign to users
 *
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 1.1.0 20/05/2015
 */
class MVC_Admin_Simpods_User_Code extends MVC_Admin_General {

	/* Start Simpods MVC page variables */
	// The page slug which will appear in the URL
	const PAGE_SLUG = 'simpods-user-code';
	// The table name to be created to hold the settings
	const DB_TABLE   = 'simpods_user_code';
	// The capabilities a user has to have to view/edit the settings
	const PAGE_CAP  = 'simpods_edit_user_code';
	const PAGE_ICON = 'dashicons dashicons-admin-network';
	// The page's title to display in-page
	public static $page_title = 'User Code';
	/* End Simpods MVC page variables */

	// database table version
	private $database_version = 1.0;
	const INSTALLED_SECTIONS 	= 'simpods_user_code_installed_sections';

	public  $update_vers 		= array(
										'update_user_pod'					=> 1.0004,
									);
	private $installed_vers 	= array();

	public function __construct( $can_init = true ) {
		self::$page_title = esc_html__( 'User Code', 'simpods-mvc' );

		// carry on if it is on the right section
		if ( $can_init ) {

			// for checking if this class is activated
			if ( !defined( 'SIMPODS_USER_CODE' ) ) {
				define( 'SIMPODS_USER_CODE', true );
			}

			$this->create_db();
			//$this->update_user_pod();
			$this->actions();
		}

	}

	/**
	 * WordPress actions to hook when the admin controller is loaded
	 * @return null
	 */
	public function actions() {
		//add_action( 'simpods-mvc-admin-user-code-update', array( $this, 'update_user_pod_table' ), 10, 3 );
	}

	/**
	 * WordPress filters to hook when the admin controller is loaded
	 * @return null
	 */
	public function filters() {}

	/**
	 * Scripts & styles to register & enqueue when the admin controller is loaded
	 * @return null
	 */
	public function enqueue() {}

	/**
	 * Create the database for this area
	 * @return null
	 */
	public function create_db() {

		if ( get_option( self::DB_TABLE . '_version', 0 ) >= $this->database_version && array_key_exists( self::DB_TABLE, unserialize( SIMPODS_ALL_TABLES )['act_tables'] ) ) {
			return;
		}
		$columns = array(
			'spuc_amount' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spuc_code' => array(
				'type'     => 'varchar(255)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spuc_user_ids' => array(
				'type'     => 'text',
				'settings' => '',
				'default'  => '',
			),
			'spuc_created' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spuc_modified' => array(
				'type'     => 'datetime',
				'settings' => 'NULL',
				'default'  => 'NULL',
			),
			'spuc_modified_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spuc_author' => array(
				'type'     => 'int(11)',
				'settings' => 'NOT NULL',
				'default'  => '',
			),
			'spuc_status' => array(
				'type'     => 'varchar(25)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT "publish"',
			),
			'spuc_note' => array(
				'type'     => 'text',
				'settings' => '',
				'default'  => '',
			),
			'spuc_expiry_date' => array(
				'type'     => 'date',
				'settings' => '',
				'default'  => 'DEFAULT "0000-00-00"',
			),
			'spuc_active' => array(
				'type'     => 'TINYINT(1)',
				'settings' => 'NOT NULL',
				'default'  => 'DEFAULT 1',
			),
		);

		$this->import_database( self::DB_TABLE, $columns );

		update_option( self::DB_TABLE . '_version', $this->database_version );
	}

	public function update_user_pod() {
		
		if ( ! function_exists( 'pods_api' ) ){
			return false;
		}
		$this->installed_vers = get_option( self::INSTALLED_SECTIONS, array() );
		if ( isset( $this->installed_vers['update_user_pod'] ) && $this->installed_vers['update_user_pod'] >= $this->update_vers['update_user_pod'] ) {
			return true;
		}			
		global $funs, $wpdb;
		$pods = array(
			'user'              => array(
				'create_extend'   => 'extend',
				'extend_pod_type' => 'user',
				'extend_storage'  => 'table',
			),
		);
		$fields = array(		
			'user'              => array(																						
			),					
		);
		$existing = false;
		if( isset( MVC_Admin_Parent_Factory::$act_tables['pods_user'] ) ){
			$existing = $funs->column_exist( 'pods_user', 'spuc_code' );
		}
		if( ! $existing ){
			$fields['user']['spuc_code'] = array(
											'name'    => 'spuc_code',
											'label'   => esc_html__( 'User Code', 'simpods-cpd' ),
											'type'    => 'simpodsareafield',
											'options' => array(
												'required'               		=> 0,
												'simpodsareafield_table_type'	=> 'other_tables',
												'simpodsareafield_other_table' 	=> 'simpods_user_code',
												'simpodsareafield_display_field'=> 'spuc_code',
												'simpodsareafield_input_type'	=> 'multi_select',
												'admin_only' => 1,	
											),
										);		
		}
		$success = true;
		//if ( ! pods_api()->pod_exists( array( 'name' => 'user' ) ) ){
		$fields_to_remove 	= array( 'author', 'permalink', 'created', 'modified' );
		$pod_ids			= $funs->create_pods( $pods, $fields_to_remove );
	
		
		if ( count( $pod_ids ) == 0 ){
			$success = false;
		}
		//}

		if ( ! $funs->create_fields( $fields, $pods, $pod_ids ) ){ echo 'ssss';
			$success = false;
		}
		if( $success ){
			$this->installed_vers['update_user_pod'] = $this->update_vers['update_user_pod'];
			update_option( self::INSTALLED_SECTIONS, $this->installed_vers );		
		}
		return $success;
	}
	/**
	 * Interface to display on the appropriate page
	 * @return null
	 */
	public function interface() {
		global $ctrl, $cross_admin, $funs;

		$settings_helper = new MVC_Admin_Settings_Helper();

		$ctrl->forms();
		$ctrl->notice .= $this->simpods_admin_remove_item( self::DB_TABLE );

		$records = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE ) );
		//$count_data   = $funs->simpods_select( array(), array( 'target_tb' => self::DB_TABLE, 'count_only' => true ) );
		?>
		<div class="wrap" id="simpods-wrap">
			<div class="row m-0">
				<?php echo $cross_admin->simpods_menu(); ?>
				<div id="simpods-component-panel" class="col-12 col-lg p-3">
					<h1><?php echo self::$page_title; ?></h1>
					<?php echo $ctrl->notice; ?>
					<?php
					if ( function_exists( 'pods_api' ) && ! isset( MVC_Admin_Parent_Factory::$act_tables['pods_user'] )  ){		
						if ( !isset( $_POST ) || !isset( $_POST['action'] ) || $_POST['action'] !== 'extend_user_table' || empty( $_POST['spuc_code'] )  ) {			
					?>
					<div id="simpods-uc-extend-user-table">
						<h2><?php esc_html_e( 'Database', 'simpods-mvc' ); ?></h2>
						<?php $settings_helper->render_form( $ctrl->settings_form_items, $ctrl->settings_form_settings ); ?>
					</div>					
					<?php
						}
					}
					?>
					<div id="simpods-uc-add">
						<h2><?php esc_html_e( 'Generate Code', 'simpods-mvc' ); ?></h2>
						<?php $settings_helper->render_form( $ctrl->add_form_items, $ctrl->add_form_settings ); ?>
					</div>

					<?php if ( is_array( $records ) && !empty( $records ) ) { ?>
					<hr>
					<div id="simpods-uc-table">
						<?php
						foreach ( $records as &$record_data ) {
							$record_data = $funs->escape_attrs( $record_data );

							$ids_imploded = '';
							$ids = array();

							if ( trim( $record_data['spuc_user_ids'] ) != '' ) {
								$ids = maybe_unserialize( $record_data['spuc_user_ids'] ); // old
								if ( is_array( $ids ) ) {
									$ids_imploded = implode( ',', $ids );
								} else {
									$ids_imploded = $record_data['spuc_user_ids']; // new one will be just 1,2,3
									$ids = explode( ',', $ids_imploded );
								}
							}

							$id_count = 0;

							if ( is_array( $ids ) ) {
								$id_count = count( $ids );
							}

							$date = 'N/A';
							$raw_date = '';

							$record_data['spuc_expiry_date-formatted'] = $record_data['spuc_expiry_date'];

							if ( $record_data['spuc_expiry_date'] !== '0000-00-00' ) {
								$raw_date = $record_data['spuc_expiry_date'];
								$date = date( get_option( 'date_format' ), strtotime( $record_data['spuc_expiry_date'] ) );
								$record_data['spuc_expiry_date'] = date( get_option( 'date_format' ), strtotime( $record_data['spuc_expiry_date'] ) );
							} else {
								$record_data['spuc_expiry_date'] = '';
							}

							if ( !is_numeric( $record_data['spuc_active'] ) ) {
								$record_data['spuc_active'] = 0;
							} else {
								$record_data['spuc_active'] = intval( $record_data['spuc_active'] );
							}

							$record_data['spuc_user_ids']              = $ids_imploded;
							$record_data['spuc_expiry_date-formatted'] = $date;
							$record_data['spuc_usage']                 = $id_count . ' / ' . absint( $record_data['spuc_amount'] );
							$record_data['spuc_active-formatted']      = $record_data['spuc_active'] == 1 ? esc_html__( 'Yes', 'simpods-mvc' ) : esc_html__( 'No', 'simpods-mvc' );
						}

						$headers = array(
							'id'                  		 => esc_html__( 'ID', 'simpods-mvc' ),
							'spuc_code'                  => esc_html__( 'Code', 'simpods-mvc' ),
							'spuc_user_ids'              => esc_html__( 'Users', 'simpods-mvc' ),
							'spuc_usage'                 => esc_html__( 'Usage', 'simpods-mvc' ),
							'spuc_note'                  => esc_html__( 'Notes', 'simpods-mvc' ),
							'spuc_expiry_date-formatted' => esc_html__( 'Expiry Date', 'simpods-mvc' ),
							'spuc_active-formatted'      => esc_html__( 'Active', 'simpods-mvc' ),
						);

						$edit_values = array(
							'spuc_code',
							'spuc_amount',
							'spuc_active',
							'spuc_user_ids',
							'spuc_expiry_date',
							'spuc_note',							
							'id',
						);

						$edit_settings = array(
							'id-prefix' => 'simpods-uc',
							'action'    => admin_url( 'admin.php?page=' . self::PAGE_SLUG . '' ),
							'title'     => esc_html__( 'Update User Code', 'simpods-mvc' )
						);

						echo $this->simpods_table( $headers, $records, self::PAGE_SLUG, true, $edit_values, $edit_settings );
						?>
					</div>
					<?php } ?>
					<script>var wp_errors = <?php echo is_wp_error( $ctrl->wp_errors ) ? wp_json_encode( $ctrl->wp_errors ) : "''"; ?>;</script>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Update `spuc_code` in user pods table if it exists
	 * Used with simpods-mvc-admin-user-code-update action
	 *
	 * @param boolean $is_done    If the update was successful
	 * @param integer $id      The ID of the user code row
	 * @param array   $user_ids Array of IDs for users assigned to the code
	 * @deprecated
	 * @return null
	 */
	public function update_user_pod_table( $is_done, $id, $user_ids ){
		if ( ! $is_done || ! function_exists( 'pods_api' ) || ! pods_api()->pod_exists( array( 'name' => 'user' ) ) ){
			// Update failed or user pods does not exist
			return;
		}

		global $funs;

		$pod_table = 'pods_user';

		// Find code for ID
		$codes = $funs->simpods_select(
			array(
				'id' => $id
			),
			array(
				'target_tb' => self::DB_TABLE
			)
		);
		if ( ! is_array( $codes ) || ! array_key_exists( 0, $codes ) || ! array_key_exists( 'spuc_code', $codes[0] ) ){
			return;
		}
		$code = $codes[0]['spuc_code'];

		// Find existing users with code
		$existing_users = $funs->simpods_select(
			array(),
			array(
				'target_tb' => $pod_table,
				'search'    => 'AND FIND_IN_SET( "' . esc_sql( $code ) . '", `spuc_code` )'
			)
		);
		// Check existing users with the code
		foreach ( $existing_users as $users ){
			if ( ! in_array( $users['id'], $user_ids ) ){
				// User is no longer assigned to this code, remove it
				$codes = explode( ',', $users['spuc_code'] );
				$idx = array_search( $code, $codes );
				unset( $codes[ $idx ] );
				$codes_imploded = implode( ',', $codes );

				$funs->general_update(
					$pod_table,
					array(
						'spuc_code' => $codes_imploded
					),
					array(
						'id' => $users['id']
					)
				);
			} else {
				// User is already assinged to the code, skip over them
				$idx = array_search( $users['id'], $user_ids );
				unset( $user_ids[ $idx ] );
			}
		}

		// Add the code to newly assigned users
		foreach ( $user_ids as $user_id ){
			$users = $funs->simpods_select(
				array(
					'id' => $user_id
				),
				array(
					'target_tb' => $pod_table
				)
			);

			if ( ! is_array( $users ) || ! array_key_exists( 0, $users ) ){
				continue;
			}
			$users = $users[0];

			$codes   = explode( ',', $users['spuc_code'] );
			$codes[] = $code;
			$codes_imploded   = implode( ',', $codes );

			$funs->general_update(
				$pod_table,
				array(
					'spuc_code' => $codes_imploded
				),
				array(
					'id' => $users['id']
				)
			);
		}
	}

}
