<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * User Settings Helper
 *
 * For managing Simpods Settings tables on a per user basis
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Mark Wane
 * @since: 18/01/2018
 */
class MVC_Admin_User_Settings_Helper extends MVC_Admin_Settings_Helper {

	/**
	 * Get rows from a Simpods MVC user settings table by prefix
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_settings_by_prefix_fn-user/
	 * @param  string 	$table 		Name of table (without table_prefix)
	 * @param  string 	$prefix 	Prefix of setting name
	 * @param  int 		$user_id 		ID of user for setting (default null)
	 * @return array 					Associate array of row objects keyed by name
	 */
	public function get_settings_by_prefix( $table, $prefix, $user_id = null ) {

		global $funs;
		$results = $funs->simpods_select(
			array(
				'name' => $prefix . '%',
				'user' => $user_id,
			),
			array( 'target_tb' => $table )
		);

		$return_data = array();

		foreach ( $results as $row ) {
			$row        = (object) $row;
			$row->value = maybe_unserialize( $row->value );

			$return_data[$row->name] = $row;

			wp_cache_set( $row->name, $row->value, $table . $user_id );
		}

		return $return_data;
	}

	/**
	 * Get all rows for a user from a simpods user settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_settings_for_form_fn/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  int 		$user_id 	ID of user for setting (default null)
	 * @return array 				Associate array of row objects keyed by name
	 */
	public function get_settings( $table, $user_id = null ) {

		global $funs;

		$results = $funs->simpods_select(
			array( 'user' => $user_id ),
			array( 'target_tb' => $table )
		);

		$return_data = array();

		foreach ( $results as $row ) {
			$row        = (object) $row;
			$row->value = maybe_unserialize( $row->value );

			$return_data[$row->name] = $row;

			wp_cache_set( $row->name, $row->value, $table . $user_id );
		}

		return $return_data;
	}

	/**
	 * Insert a hidden setting row in a Simpods MVC user settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/insert_fn-user/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  string 	$name 	Name of option to insert
	 * @param  mixed 	$value 		Value of option
	 * @param  int 		$user_id 	ID of user for setting
	 * @return null
	 */
	public function insert( $table, $name, $value, $user_id = null ) {
		global $funs;

		$funs->general_insert(
			$table,
			array(
				'name'  => $name,
				'user'  => $user_id,
				'type'  => 'hidden',
				'value' => maybe_serialize( $value ),
			)
		);

		wp_cache_set( $name, $value, $table . $user_id );
	}

	/**
	 * Update a row in a Simpods MVC user settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/update_fn-user/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  string 	$name 	Name of option to insert
	 * @param  mixed 	$value 		Value of option
	 * @param  int 		$user_id 	ID of user for setting
	 * @return null
	 */
	public function update( $table, $name, $value, $user_id = null ) {
		global $wpdb;

		$wpdb->update(
			$wpdb->prefix . $table,
			array( 'value' => maybe_serialize( $value ) ),
			array(
				'name' => $name,
				'user' => $user_id,
			),
			array( '%s' ),
			array( '%s', '%d' )
		);

		wp_cache_set( $name, $value, $table . $user_id );
	}

	/**
	 * Remove a setting row from a Simpods MVC user settings table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/delete_fn-user/
	 * @param  string 	$table 	Name of table (without table_prefix)
	 * @param  string 	$name 	Name of option to insert
	 * @param  int 		$user_id 	ID of user for setting
	 * @return null
	 */
	public function delete( $table, $name, $user_id = null ) {
		global $funs;

		$funs->simpods_delete(
			$table,
			array(
				'name' => $name,
				'user' => $user_id,
			)
		);

		wp_cache_delete( $name, $table . $user_id );
	}

}
