<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Class to create a new Pods field type: PodsField_Simpodsareafield
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 09/02/2016
 */
class PodsField_Simpodsareafield extends PodsField {
	/**
	 * Whether this field is running under 1.x deprecated forms
	 *
	 * @var bool
	 * @since 1.0
	 */
	public static $deprecated = false;
	/**
	 * Field Type Group
	 *
	 * @var string
	 * @since 2.0
	 */
	public static $group = 'Simpods';
	/**
	 * Field Type Identifier, has to be one word
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $type = 'simpodsareafield';
	/**
	 * Option Name to save to postmeta table
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $type_table = 'simpodsareafield_table';
	/**
	 * input name
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $input = 'pods-simpods-area-field';
	/**
	 * Field Type Label
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $label = 'Area Field';
	/**
	 * Field Type Preparation
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $prepare = '%s';
	/**
	 * Pod Types supported on (true for all, false for none, or give array of specific types supported)
	 *
	 * @var array|bool
	 * @since 1.0.0
	 */
	public static $pod_types = true;
	/**
	 * API caching for fields that need it during validate/save
	 *
	 * @var \PodsAPI
	 * @since 1.0.0
	 */
	private static $api = false;
	/**
	 * Tables caching for Advanced Custom Type tables
	 *
	 * @var array
	 * @since 1.0.0
	 */
	public static $act_tables = array();
	/**
	 * Do things like register/enqueue scripts and stylesheets
	 *
	 * @return \PodsField
	 * @since 1.0
	 */
	public function __construct() {
		global $funs;
		self::$group = esc_html__( 'Simpods MVC', 'simpods-mvc' );
		self::$label = esc_html__( 'Area Field', 'simpods-mvc' );
		//self::$act_tables = MVC_Admin_Factory::$act_tables;
		if ( is_admin() && ! $funs->is_frontend_ajax() ) {
			self::$act_tables = MVC_Admin_Factory::$act_tables;
		} else {
			self::$act_tables = MVC_Front_Factory::$act_tables;
		}
	}
	/**
	 * Add options and set defaults for field type, shows in admin area
	 *
	 * @return array $options
	 * @since 1.0
	 * @see PodsField::ui_options
	 * @use pods_tables() has to call the function rather than using the static one. The static one doesn't include all tables after saving
	 */
	public function options() {

		$tables = MVC_Admin_Factory::$act_tables;
		if ( isset( $_GET['id'] ) && is_numeric( $_GET['id'] ) && isset( $tables['pod_' . $_GET['id']] ) ) {
			unset( $tables['pod_' . $_GET['id']] );
		}
		$lists = array();
		if ( count( $tables ) == 0 ) {
			$lists[0] = esc_html__( 'No Pods Table Available', 'simpods-mvc' );
		} else {
			foreach ( $tables as $table_data ) {
				$lists[ $table_data['pod'] ] = $table_data['title'];
			}
		}
		$options = array(
			self::$type . '_table_type' => array(
				'label'      => esc_html__( 'Table Type', 'simpods-mvc' ),
				'description' 	=> esc_html__( 'Pods Tables are created by Pods Framework. If choose Other Tables, you can enter any table name.', 'simpods-cpd' ),
				'default'    => 'pods_table',
				'type'       => 'pick',
				'data' => array(					
					'pods_tables' => esc_html__( 'Pods Tables', 'simpods-mvc' ),
					'other_tables' 	=> esc_html__( 'Other Tables', 'simpods-mvc' ),
				),								
				'dependency' => true,
			),	
			self::$type . '_table' => array(
				'label'      => esc_html__( 'Pods Table', 'simpods-mvc' ),
				'default'    => 0,
				'type'       => 'pick',
				'data'       => $lists,
				'depends-on' => array( self::$type . '_table_type' => 'pods_tables' ),
			),
			self::$type . '_other_table' => array(
				'label' => esc_html__( 'Table Name', 'simpods-mvc' ),
				'description' 	=> esc_html__( 'Table name without the table prefix. Please make sure the "id" field exists in the table.', 'simpods-cpd' ),
				'depends-on' => array( self::$type . '_table_type' => 'other_tables' ),
				'default' => '',
				'type' => 'text'
			),				
			self::$type . '_display_field' => array(
				'label' => esc_html__( 'Display Field', 'simpods-mvc' ),
				'description' 	=> esc_html__( 'Which field to fetch the data to display. If empty for Pods Table, the first admin column will be used.', 'simpods-cpd' ),				
				'default' => '',
				'type' => 'text'
			),			
			self::$type . '_input_type' => array(
				'label'      => esc_html__( 'Input Type', 'simpods-mvc' ),
				'default'    => 'single_select',
				'type'       => 'pick',
				'data' => array(					
					'single_select' => esc_html__( 'single select', 'simpods-mvc' ),
					'multi_select' 	=> esc_html__( 'multi-select', 'simpods-mvc' ),
					'text' 			=> esc_html__( 'text', 'simpods-mvc' ),
					'textarea' 		=> esc_html__( 'textarea', 'simpods-mvc' ),
				),								
				//'dependency' => true,
			),	
			/*self::$type . '_select2' => array(
				'label'      	=> esc_html__( 'Enable Search And Pick (Select2)', 'simpods-mvc' ),
				'default' 		=> 0,
				'type' 			=> 'boolean',				
				'depends-on' 	=> array( self::$type . '_input_type' => array( 'single_select', 'multi_select' ) ),
			),*/						
		);
		return $options;
	}
	/**
	 * Options for the Admin area, defaults to $this->options()
	 *
	 * @return array $options
	 * @since 1.0
	 * @see PodsField::options
	 */
	public function ui_options() {
		return $this->options();
	}
	/**
	 * Define the current field's schema for DB table storage
	 *
	 * @param array $options
	 * @return string
	 * @since 1.0
	 */
	public function schema( $options = null ) {
		$schema = 'longtext';
		return $schema;
	}
	/**
	 * Define the current field's preparation for sprintf
	 *
	 * @param array $options
	 * @return array
	 * @since 1.0
	 */
	public function prepare( $options = null ) {
		$format = self::$prepare;
		return $format;
	}
	/**
	 * Change the value of the field
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0.0
	 */
	public function value( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Change the way the value of the field is displayed with Pods::get
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0
	 */
	public function display( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Customize output of the form field
	 *
	 * @param string $name
	 * @param mixed $value
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return void
	 * @since 1.0
	 */
	public function input( $name, $value = null, $options = null, $pod = null, $id = null ) {
		global $funs;
		$is_allowed = true;
		if ( !defined( 'SIMPODS_AREA_FIELD' ) || !is_user_logged_in() || !current_user_can( 'edit_posts' ) ) {
			// action before the iframe
			$is_allowed = false;
		}
		$is_allowed = apply_filters( 'simpods_load_area_field_allow', $is_allowed, $_GET );
		if ( !$is_allowed ) {
			echo apply_filters( 'simpods_load_area_field_allow_msg', esc_html__( 'You do not have permission to edit this item.', 'simpods-mvc') );
		} else {
			$options = (array) $options;
			$form_field_type = PodsForm::$field_type;
			if ( is_array( $value ) ) {
				$value =  implode( '', $value );
			}
			$value = trim( $value );
			$pod_label  = '';
			//$saved_table_id = intval( $options[self::$type_table] );
			if( $options['simpodsareafield_table_type'] == 'other_tables' ){
				$table 	= esc_sql( $options['simpodsareafield_other_table'] );
				
			} else {
				$table 	= $options[ self::$type_table ];
				
				// Convert from old ID format
				if ( is_numeric( $table ) ) {
					$saved_table_id = $table;
					$post_data = get_post( $saved_table_id, ARRAY_A );
					if ( is_array( $post_data ) && $post_data['post_type'] == '_pods_pod' ) {
						$table 	= $post_data['post_name'];
						$pod_label	= $post_data['post_title'];
					}
				} else {
					$target_pod  = pods_api()->load_pod( $table );
					if( $target_pod ){
						$saved_table_id 	= $target_pod['id'];
						$pod_label		= $target_pod['label'];
					}
				}
			}
			if ( $table != '' ) {
				$ids 			= Arrays::string_to_array( $value );
				$html 			= '';		
				$display_field 	= $options['simpodsareafield_display_field'];
				$other_tables 	= array();
				if( isset( $options['simpodsareafield_input_type'] ) && ( $options['simpodsareafield_input_type'] == 'single_select' || $options['simpodsareafield_input_type'] == 'multi_select' ) ){
					
					if( $options['simpodsareafield_table_type'] == 'other_tables' ){
						$fields = array(
									);
						$attrs  = array(
										'target_tb' => $options['simpodsareafield_other_table'],
									);
						$other_tables = $funs->simpods_select( $fields, $attrs  );
						
					} else {
						$params = array(
							 'limit'   => -1 // Return all rows 
						);
						// Run the find						
						$items 			= pods( $table, $params );	
						
						if( $items ){ 
							$other_tables = $items->data();
							if( empty( $display_field ) ){
								$display_field  = MVC_Admin_Factory::$act_tables['pod_' . $saved_table_id]['name_field']; //$items->api->pod_data['options']['ui_fields_manage'][0];
							} 
						}
					}
					
					$multi_select2	= '';
					if( ! empty( $other_tables ) ){  
						if( $options['simpodsareafield_input_type'] == 'single_select' ){
							$html 	= '<select name="' . $name . '" class="simpods-area-field-' . esc_attr( $name ) . '-select2" >';//multiple="multiple"[]
						} else {
							$html 	= '<select name="' . $name . '[]" class="simpods-area-field-' . esc_attr( $name ) . '-select2" multiple="multiple" >';
							$multi_select2	= 'multiple:true';
						}
						
						foreach( $other_tables as $item ){
							$item 	 = (array) $item; 
							$item_id = '';
							//$title   = '';
							$title 	 = $item[ $display_field ];
							if( !isset( $item[ 'ID' ] ) && isset( $item[ 'id' ] ) && is_numeric( $item[ 'id' ] ) ){ // pod or custom table
								$item_id 	= $item[ 'id' ];
								//$item_arr 	= (array) $item;
								//$title 		= $item[ $display_field ];
								//$title 		= $item->$items->api->pod_data['options']['pod_index'];
							} elseif ( isset( $item[ 'ID' ] ) && is_numeric( $item[ 'ID' ] ) ) {
								$item_id 	= $item[ 'ID' ];
								//$title 		= $item[ 'post_title' ];
							} else {
								continue;
							}
							$selected = '';
							if( in_array( $item_id, $ids ) ){
								$selected = 'selected="selected"';
							}
							$html .= '<option value="' . intval( $item_id ) . '" ' . $selected . '>' . esc_attr( $title  ) . '</option>';
						}	 
						$html 	.= '</select>';					
						echo '<span class="">' . $html . '</span>';
						//if( isset( $options['simpodsareafield_select2'] ) && $options['simpodsareafield_select2'] == 1 ){
						echo '<script>
								jQuery(document).ready(function( $ ) {
								    $(".simpods-area-field-' . esc_attr( $name ) . '-select2").select2({
									    tags: true,
									    tokenSeparators: [","],
									    placeholder: "' . esc_html__( 'Select - ', 'simpods-mvc' ) . esc_attr( $pod_label ) . '",
										allowClear: true,
										' . $multi_select2 . '
									});
								});
							  </script>';
						//}
					} else {
						esc_html__( 'No items available', 'simpods-mvc' );
					}
				} else {												
					$target_table = 'pods_' . $table;
					if( $options['simpodsareafield_table_type'] == 'other_tables' ){
						$target_table = $options['simpodsareafield_other_table'];
					} 	
					if( ! empty( $ids ) ){
						$fields = array(
										'id'	=> $ids,
									);
						$attrs  = array(
										'target_tb' => $target_table,
									);
						$other_tables = $funs->simpods_select( $fields, $attrs  );
					}	
					if( empty( $display_field ) ){
						$display_field  = MVC_Admin_Factory::$act_tables['pod_' . $saved_table_id]['name_field']; //$items->api->pod_data['options']['ui_fields_manage'][0];
					} 
					$k       = 0;
					foreach ( $other_tables as $item ) {
						
						if ( $k == 1 ) {
							$html .= '<div style="display:none;" id="simpods-area-field-' . esc_attr( $name ) . '-saved-items">';
						}

						$display = $item[ $display_field ];
						$html .= '
								<div class="simpods-info">
									<strong>' . $item[ 'id' ] . '</strong>: ' . $display . '
									<div class="simpods-info-del alignright"  data-target="' . esc_attr( $name ) . '" data-id="' . $item[ 'id' ] . '">
										<span class="dashicons dashicons-trash"></span>
									</div>
								</div>';
						$k ++;
					
					}
					if ( $k > 1 ) {
						$html .= '</div><div class="aligncenter" id="' . $name . '-more-btn" ><div class="simpods-pods-btn" data-target="simpods-area-field-' . esc_attr( $name ) . '-saved-items"><span class="dashicons dashicons-arrow-down-alt2"></span></div></div>
							<script>
							jQuery(document).ready(function( $ ) {
								$(".simpods-pods-btn").click( function(){
									if( $( "#" + $( this ).data("target") ).css("display") == "none" ){
										$( "#" + $( this ).data("target") ).show("fast");
										$( this ).find(".dashicons").removeClass("dashicons-arrow-down-alt2");
										$( this ).find(".dashicons").addClass("dashicons-arrow-up-alt2");
									} else {
										$( "#" + $( this ).data("target") ).hide("fast");
										$( this ).find(".dashicons").removeClass("dashicons-arrow-up-alt2");
										$( this ).find(".dashicons").addClass("dashicons-arrow-down-alt2");
									}
								});
							});
						  	</script>
						';
					}
					$html .= '
						<script>
						jQuery(document).ready(function( $ ) {
							$(".simpods-info-del").click( function(){
								
								var ids_arr = $( "#" + $( this ).data("target") ).val().replace(/\s/g, "").split(",");
								for( var i = 0; i < ids_arr.length; i ++ ){
									if( parseInt( ids_arr[ i ] ) == $( this ).data("id") ){
										ids_arr.splice(i,1);
										$( this ).parent(".simpods-info").remove();
										break;
									}																	
								}
								$( "#" + $( this ).data("target") ).val( ids_arr.join(",") ) ;
								
								if( ids_arr.length == 0 ){
									console.log( "#' . esc_attr( $name ) . '-more-btn" );
									$( "#' . esc_attr( $name ) . '-more-btn" ).remove();
								}
							});
						});
					  	</script>
					';	
					if( $options['simpodsareafield_input_type'] == 'textarea' ){				
						echo '<span class=""><textarea id="' . $name . '" name="' . $name . '" placeholder="' . esc_html__( 'IDs: Separated by comma', 'simpods-mvc' ) . '" autocomplete="off"  />' . esc_attr( $value ) . '</textarea></span>';
					} else {			
						echo '<span class=""><input type="text" id="' . $name . '" name="' . $name . '" value="' . esc_attr( $value ) . '" placeholder="' . esc_html__( 'IDs: Separated by comma', 'simpods-mvc' ) . '" autocomplete="off"  /></span>';
					}
					echo $html;
				}
			} else {
				esc_html_e( 'No Advanced Content Type Table Selected', 'simpods-mvc' );
			}
		}
	}
	/**
	 * Get the data from the field, run when loading an area
	 *
	 * @param string $name The name of the field
	 * @param string|array $value The value of the field
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @param boolean $in_form
	 * @return array Array of possible field data
	 * @since 1.0
	 */
	public function data( $name, $value = null, $options = null, $pod = null, $id = null, $in_form = true ) {
		return (array) $value;
	}
	/**
	 * Build regex necessary for JS validation
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param string $pod
	 * @param int $id
	 * @return bool
	 * @since 1.0
	 */
	public function regex( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return false;
	}
	/**
	 * Validate a value before it's saved
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param int $id
	 * @param array $params
	 * @return bool
	 * @since 1.0
	 */
	public function validate( $value, $name = null, $options = null, $fields = null, $pod = null, $id = null, $params = null ) {
		// if( 'simpodsareafield' == $options['type'] && 1 == $options['unique'] ){
		// 	global $wpdb;
		// 	$query = $wpdb->prepare( 'SELECT * FROM `' . $wpdb->prefix . 'pods_' . $options['pod'] . '` WHERE `id` != %d AND FIND_IN_SET( %d, `' . esc_sql( $name ) . '` ) LIMIT 0,1', array( $id, $value  ) );
		// 	$exist = $wpdb->get_results( $query, ARRAY_A );			
		// 	if( ! empty( $exist ) ){
		// 		//echo esc_html__('The chosen value has been assigned to the item with ID ', 'simpods-mvc' ) . $exist[0]['id'] . '. ';	
		// 		return false;
		// 	} 		
		// }
			
		return true;
	}
	/**
	 * Change the value or perform actions after validation but before saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return mixed
	 * @since 1.0
	 */
	public function pre_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		if( is_array( $value ) ){
			$value = implode( ',' , $value );
		}
		return $value;
	}
	/**
	 * Save the value to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return bool|void Whether the value was saved
	 * @since 1.0.0
	 */
	public function save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		return null;
	}
	/**
	 * Perform actions after saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @since 1.0
	 */
	public function post_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {}
	/**
	 * Perform actions before deleting from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param null $options
	 * @param string $pod
	 * @since 1.0
	 */
	public function pre_delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Delete the value from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @since 1.0.0
	 */
	public function delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Perform actions after deleting from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @since 1.0
	 */
	public function post_delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Customize the Pods UI manage table column output
	 *
	 * @param int $id
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @since string Value to be shown in the UI
	 * @since 1.0
	 */
	public function ui( $id, $value, $name = null, $options = null, $fields = null, $pod = null ) {
		return $value;
	}

}
