<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Class to create a new Pods field type: PodsField_Simpodspodsbuttonfield
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/07/2016
 */
class PodsField_Simpodsbuttonfield extends PodsField {
	/**
	 * Whether this field is running under 1.x deprecated forms
	 *
	 * @var bool
	 * @since 1.0
	 */
	public static $deprecated = false;
	/**
	 * Field Type Group
	 *
	 * @var string
	 * @since 2.0
	 */
	public static $group = 'Simpods';
	/**
	 * Field Type Identifier, has to be one word
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $type = 'simpodsbuttonfield';
	/**
	 * input name
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $input = 'pods-simpods-button-field';
	/**
	 * Field Type Label
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $label = 'Button Field';
	/**
	 * Field Type Preparation
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $prepare = '%s';
	/**
	 * Pod Types supported on (true for all, false for none, or give array of specific types supported)
	 *
	 * @var array|bool
	 * @since 1.0.0
	 */
	public static $pod_types = true;
	/**
	 * API caching for fields that need it during validate/save
	 *
	 * @var \PodsAPI
	 * @since 1.0.0
	 */
	private static $api = false;
	/**
	 * Tables caching for Advanced Custom Type tables
	 *
	 * @var array
	 * @since 1.0.0
	 */
	public static $act_tables = array();
	/**
	 * Do things like register/enqueue scripts and stylesheets
	 *
	 * @return \PodsField
	 * @since 1.0
	 */
	public function __construct() {
		global $funs;
		self::$group = esc_html__( 'Simpods MVC', 'simpods-mvc' );
		self::$label = esc_html__( 'Button Field', 'simpods-mvc' );
		//self::$act_tables = MVC_Admin_Factory::$act_tables;
		if ( is_admin() && ! $funs->is_frontend_ajax() ) {
			self::$act_tables = MVC_Admin_Factory::$act_tables;
		} else {
			self::$act_tables = MVC_Front_Factory::$act_tables;
		}
	}
	/**
	 * Add options and set defaults for field type, shows in admin area
	 *
	 * @return array $options
	 * @since 1.0
	 * @see PodsField::ui_options
	 * @use pods_tables() has to call the function rather than using the static one. The static one doesn't include all tables after saving
	 */
	public function options() {
		$tag_list = array( 'div' => 'div', 'span' => 'span', 'button' => 'button' );
		$options = array(
			self::$type . '_wrap' => array(
				'label'      => esc_html__( 'Wrap it by', 'simpods-mvc' ),
				'default'    => 0,
				'type'       => 'pick',
				'data'       => $tag_list,
				'dependency' => false,
			),
            self::$type . '_link' => array(
                'label' 	=> esc_html__( 'Link', 'simpods-mvc' ),
                'default' 	=> '',
                'type' 		=> 'text',
            ),
            self::$type . '_link_target' => array(
                'label' 		=> esc_html__( 'Open Link In', 'simpods-mvc' ),
                'default' 		=> '',
                'data'       	=> array(
                					'' 			=> esc_html__( 'N/A', 'simpods-mvc' ),
                					'_self' 	=> esc_html__( 'Same frame', 'simpods-mvc' ),
                					'_blank' 	=> esc_html__( 'New window or tab', 'simpods-mvc' ),
                					'_parent' 	=> esc_html__( 'Parent frame', 'simpods-mvc' ),
                					'_top' 		=> esc_html__( 'The full body of the window', 'simpods-mvc' ),
                				),
                'type' 			=> 'pick',
            ),
		);
		return $options;
	}
	/**
	 * Options for the Admin area, defaults to $this->options()
	 *
	 * @return array $options
	 * @since 1.0
	 * @see PodsField::options
	 */
	public function ui_options() {
		return $this->options();
	}
	/**
	 * Define the current field's schema for DB table storage
	 *
	 * @param array $options
	 * @return string
	 * @since 1.0
	 */
	public function schema( $options = null ) {
		$schema = 'VARCHAR(1000)';
		return $schema;
	}
	/**
	 * Define the current field's preparation for sprintf
	 *
	 * @param array $options
	 * @return array
	 * @since 1.0
	 */
	public function prepare( $options = null ) {
		$format = self::$prepare;
		return $format;
	}
	/**
	 * Change the value of the field
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0.0
	 */
	public function value( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Change the way the value of the field is displayed with Pods::get
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0
	 */
	public function display( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Customize output of the form field
	 *
	 * @param string $name
	 * @param mixed $value
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return void
	 * @since 1.0
	 */
	public function input( $name, $value = null, $options = null, $pod = null, $id = null ) {
		$is_allowed = true;
		if ( !defined( 'SIMPODS_BUTTON_FIELD' ) || !is_user_logged_in() || !current_user_can( 'edit_posts' ) ) {
			// action before the iframe
			$is_allowed = false;
		}
		$is_allowed = apply_filters( 'simpods_load_button_field_allow', $is_allowed, $_GET );
		if ( !$is_allowed ) {
			echo apply_filters( 'simpods_load_button_field_allow_msg', esc_html__( 'You do not have permission to edit this item.', 'simpods-mvc') );
		} else {
			$options 	= 	(array) $options;
			$loader_html	= '<div class="mg5 alignleft hidden simpods-button-ajax-loader ' . esc_attr( $this->replace_schars( $options['label'] ) ) . '"><img src="' . admin_url( 'images/wpspin_light.gif' ) . '"/> </div>';
			if( isset( $options[ self::$type . '_wrap'] ) && empty( $options[ self::$type . '_wrap'] ) != '' ){
				$wrap_tag = esc_html( $options[ self::$type . '_wrap'] ) ;
			} else {
				$wrap_tag = 'div';
			}
			$btn_html	=	'<' . $wrap_tag . ' class="alignleft ' . self::$input . ' ' . esc_attr( $this->replace_schars( $options['label'] ) ) . ' button">' . esc_attr( $options['label'] ) . '</' . $wrap_tag . '>';
			if( isset( $options[ self::$type . '_link_target'] ) && !empty( $options[ self::$type . '_link_target'] ) ){
				if( isset( $options[ self::$type . '_link'] ) && !empty( $options[ self::$type . '_link'] ) ){
					$btn_html	=	'<a href="' . esc_url( $options[ self::$type . '_link'] ) . '" target="' . $options[ self::$type . '_link_target'] . '" class="' . self::$type . '_link">' . $btn_html . '</a>';
				}
			}
			echo $btn_html . $loader_html . '<input type="hidden" name="' . $name . '" value="1">';
		}
	}
	/**
	 * Get the data from the field, run when loading an area
	 *
	 * @param string $name The name of the field
	 * @param string|array $value The value of the field
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @param boolean $in_form
	 * @return array Array of possible field data
	 * @since 1.0
	 */
	public function data( $name, $value = null, $options = null, $pod = null, $id = null, $in_form = true ) {
		return (array) $value;
	}
	/**
	 * Build regex necessary for JS validation
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param string $pod
	 * @param int $id
	 * @return bool
	 * @since 1.0
	 */
	public function regex( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return false;
	}
	/**
	 * Validate a value before it's saved
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param int $id
	 * @param array $params
	 * @return bool
	 * @since 1.0
	 */
	public function validate( $value, $name = null, $options = null, $fields = null, $pod = null, $id = null, $params = null ) {
		return true;
	}
	/**
	 * Change the value or perform actions after validation but before saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return mixed
	 * @since 1.0
	 */
	public function pre_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		return $value;
	}
	/**
	 * Save the value to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return bool|void Whether the value was saved
	 * @since 1.0.0
	 */
	public function save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		return null;
	}
	/**
	 * Perform actions after saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @since 1.0
	 */
	public function post_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {}
	/**
	 * Perform actions before deleting from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param null $options
	 * @param string $pod
	 * @since 1.0
	 */
	public function pre_delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Delete the value from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @since 1.0.0
	 */
	public function delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Perform actions after deleting from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @since 1.0
	 */
	public function post_delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Customize the Pods UI manage table column output
	 *
	 * @param int $id
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @since string Value to be shown in the UI
	 * @since 1.0
	 */
	public function ui( $id, $value, $name = null, $options = null, $fields = null, $pod = null ) {
		return $value;
	}

	/**
	 * remove special characters
	 * @param  string $a_string a string
	 * @param  string $replace a string to replace the special characters
	 * @return string special characters in the string replaced by the $replace
	 */
	public function replace_schars( $a_string, $replace = '-' ) {
	   $string = str_replace(' ', $replace, $a_string ); // Replaces all spaces with hyphens.
	   return preg_replace('/[^A-Za-z0-9\-]/', $replace, $a_string ); // Removes special chars.
	}
}
