<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Class to create a new Pods field type: PodsField_Simpodsheaderfield
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/07/2016
 */
class PodsField_Simpodsheaderfield extends PodsField {
	/**
	 * Whether this field is running under 1.x deprecated forms
	 *
	 * @var bool
	 * @since 1.0
	 */
	public static $deprecated = false;
	/**
	 * Field Type Group
	 *
	 * @var string
	 * @since 2.0
	 */
	public static $group = 'Simpods';
	/**
	 * Field Type Identifier, has to be one word
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $type = 'simpodsheaderfield';
	/**
	 * input name
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $input = 'pods-simpods-header-field';
	/**
	 * Field Type Label
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $label = 'Header Field';
	/**
	 * Field Type Preparation
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $prepare = '%s';
	/**
	 * Pod Types supported on (true for all, false for none, or give array of specific types supported)
	 *
	 * @var array|bool
	 * @since 1.0.0
	 */
	public static $pod_types = true;
	/**
	 * API caching for fields that need it during validate/save
	 *
	 * @var \PodsAPI
	 * @since 1.0.0
	 */
	private static $api = false;
	/**
	 * Tables caching for Advanced Custom Type tables
	 *
	 * @var array
	 * @since 1.0.0
	 */
	public static $act_tables = array();
	/**
	 * Do things like register/enqueue scripts and stylesheets
	 *
	 * @return \PodsField
	 * @since 1.0
	 */
	public function __construct() {
		global $funs;
		self::$group = esc_html__( 'Simpods MVC', 'simpods-mvc' );
		self::$label = esc_html__( 'Header Field', 'simpods-mvc' );
		if ( is_admin() && ! $funs->is_frontend_ajax() ) {
			self::$act_tables = MVC_Admin_Factory::$act_tables;
		} else {
			self::$act_tables = MVC_Front_Factory::$act_tables;
		}
	}
	/**
	 * Add options and set defaults for field type, shows in admin area
	 *
	 * @return array $options
	 * @since 1.0
	 * @see PodsField::ui_options
	 * @use pods_tables() has to call the function rather than using the static one. The static one doesn't include all tables after saving
	 */
	public function options() {
		$list_tags = array( 'h1' => esc_html__( 'Header 1', 'simpods-mvc' ), 'h2' => esc_html__( 'Header 2', 'simpods-mvc' ), 'h3' => esc_html__( 'Header 3', 'simpods-mvc' ), 'h4' => esc_html__( 'Header 4', 'simpods-mvc' ) );
		$options = array(
			self::$type . '' => array(
				'label'      => esc_html__( 'Header Type', 'simpods-mvc' ),
				'default'    => 0,
				'type'       => 'pick',
				'data'       => $list_tags,
				'dependency' => true,
			),
		);
		return $options;
	}
	/**
	 * Options for the Admin area, defaults to $this->options()
	 *
	 * @return array $options
	 * @since 1.0
	 * @see PodsField::options
	 */
	public function ui_options() {
		return $this->options();
	}
	/**
	 * Define the current field's schema for DB table storage
	 *
	 * @param array $options
	 * @return string
	 * @since 1.0
	 */
	public function schema( $options = null ) {
		$schema = 'VARCHAR(1000)';
		return $schema;
	}
	/**
	 * Define the current field's preparation for sprintf
	 *
	 * @param array $options
	 * @return array
	 * @since 1.0
	 */
	public function prepare( $options = null ) {
		$format = self::$prepare;
		return $format;
	}
	/**
	 * Change the value of the field
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0.0
	 */
	public function value( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Change the way the value of the field is displayed with Pods::get
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0
	 */
	public function display( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Customize output of the form field
	 *
	 * @param string $name
	 * @param mixed $value
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return void
	 * @since 1.0
	 */
	public function input( $name, $value = null, $options = null, $pod = null, $id = null ) {
		$is_allowed = true;
		if ( !defined( 'SIMPODS_HEADER_FIELD' ) ) {
			// action before the iframe
			$is_allowed = false;
		}
		$is_allowed = apply_filters( 'simpods_load_header_field_allow', $is_allowed, $_GET );
		if ( !$is_allowed ) {
			echo apply_filters( 'simpods_load_header_field_allow_msg', esc_html__( 'You do not have permission to edit this item.', 'simpods-mvc') );
		} else {
			$options = (array) $options;
			echo '<style>
					tr.pods-form-ui-row-type-simpodsheaderfield{
						position: relative;
						height: 90px;
						border: none;
					}
					.pods-form-ui-row-type-simpodsheaderfield th,
					.pods-form-ui-row-type-simpodsheaderfield td .pods-form-ui-label{
						display:none;
					}
					.pods-form-ui-row-type-simpodsheaderfield td{
						width:100%;
						position:absolute;
					}
				 </style>';
			isset( $options['simpodsheaderfield'] ) && $options['simpodsheaderfield'] != '' ? $heading_tag = $options['simpodsheaderfield'] : 'h1';
			echo '<' . $heading_tag . ' class="' . self::$input . '">' . esc_attr( $options['label'] ) . '</' . $heading_tag . '><input type="hidden" name="' . $name . '" value="1">';
		}
	}
	/**
	 * Get the data from the field, run when loading an area
	 *
	 * @param string $name The name of the field
	 * @param string|array $value The value of the field
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @param boolean $in_form
	 * @return array Array of possible field data
	 * @since 1.0
	 */
	public function data( $name, $value = null, $options = null, $pod = null, $id = null, $in_form = true ) {
		return (array) $value;
	}
	/**
	 * Build regex necessary for JS validation
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param string $pod
	 * @param int $id
	 * @return bool
	 * @since 1.0
	 */
	public function regex( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return false;
	}
	/**
	 * Validate a value before it's saved
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param int $id
	 * @param array $params
	 * @return bool
	 * @since 1.0
	 */
	public function validate( $value, $name = null, $options = null, $fields = null, $pod = null, $id = null, $params = null ) {
		return true;
	}
	/**
	 * Change the value or perform actions after validation but before saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return mixed
	 * @since 1.0
	 */
	public function pre_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		return $value;
	}
	/**
	 * Save the value to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return bool|void Whether the value was saved
	 * @since 1.0.0
	 */
	public function save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		return null;
	}
	/**
	 * Perform actions after saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @since 1.0
	 */
	public function post_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {}
	/**
	 * Perform actions before deleting from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param null $options
	 * @param string $pod
	 * @since 1.0
	 */
	public function pre_delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Delete the value from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @since 1.0.0
	 */
	public function delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Perform actions after deleting from the DB
	 *
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @since 1.0
	 */
	public function post_delete( $id = null, $name = null, $options = null, $pod = null ) {}
	/**
	 * Customize the Pods UI manage table column output
	 *
	 * @param int $id
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @since string Value to be shown in the UI
	 * @since 1.0
	 */
	public function ui( $id, $value, $name = null, $options = null, $fields = null, $pod = null ) {
		return $value;
	}

}
