<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
* Class to create a new Pods field type: PodsField_Simpodspodsfile, adapted from pods file field
*
* @package Simpods MVC
* @author Dongjie Xu
* @since 1.0.0 19/04/2016
*/
class PodsField_Simpodspodsfile extends PodsField {
	/**
	 * Whether this field is running under 1.x deprecated forms
	 *
	 * @var bool
	 * @since 1.0
	 */
	public static $deprecated = false;
	/**
	 * Field Type Group
	 *
	 * @var string
	 * @since 2.0
	 */
	public static $group = 'Simpods';
	/**
	 * Field Type Identifier
	 *
	 * @var string
	 * @since 2.0
	 */
	public static $type = 'simpodspodsfile';
	/**
	 * input name
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $input = 'pods-simpods-file-field';
	/**
	 * Field Type Label
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $label = 'File';
	/**
	 * Field Type Preparation
	 *
	 * @var string
	 * @since 1.0
	 */
	public static $prepare = '%s';
	/**
	 * Pod Types supported on (true for all, false for none, or give array of specific types supported)
	 *
	 * @var array|bool
	 * @since 1.0.0
	 */
	public static $pod_types = true;
	/**
	 * API caching for fields that need it during validate/save
	 *
	 * @var \PodsAPI
	 * @since 2.3
	 */
	protected static $api = false;
	/**
	 * Tables caching for Advanced Custom Type tables
	 *
	 * @var array
	 * @since 1.0.0
	 */
	public static $act_tables = array();
	/**
	 * Do things like register/enqueue scripts and stylesheets
	 *
	 * @since 2.0
	 */
	public $simpods_files     		= array();
	public $simpods_files_options 	= array();
	public function __construct() {
		self::$group = esc_html__( 'Simpods MVC', 'simpods-mvc' );
		self::$label = esc_html__( 'Files Field', 'simpods-mvc' );
		$this->actions();
		$this->filters();
	}
	public function actions() {
		global $funs;
		if ( is_admin() && ! $funs->is_frontend_ajax() ) {
			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ), 11 );
			add_action( 'admin_enqueue_scripts', array( $this, 'localize' ), 12 );
		} else {
			add_action( 'wp_enqueue_scripts', array( $this, 'enqueue' ), 11 );
			add_action( 'wp_enqueue_scripts', array( $this, 'localize' ), 12 );
		}
		add_action( 'pods_api_post_save_pod_item', array( $this, 'check_unzip' ), 10, 3 );
	}
	public function filters() {}
	public function enqueue() {
		if ( isset( $_GET ) &&  isset( $_GET['page'] ) && $_GET['page'] == 'pods' ) {
			return;
		}
		global $funs;
		wp_enqueue_media();
		$depths	= array( 'js' => array(), 'css' =>  array(), 'php' =>  array() );
		//$depths['js'] = MVC_Admin_Parent_Factory::$scripts;
		$files  = array( self::$type => SIMPODS_MVC_URL . '/admin/js/pods_fields/podsfield-simpodspodsfile.js' );
		$depths = $funs->include( $files );
		//MVC_Admin_Parent_Factory::$scripts = $depths['js'];
		wp_localize_script(
			self::$type . '',
			self::$type . '_set',
			array(
				'deselect'  => esc_html__( 'Deselect', 'simpods-mvc' ),
				'site_url'  => site_url(),
				'ajaxurl'   => admin_url( 'admin-ajax.php' ),
				'nonce'     => wp_create_nonce( 'simpods-file-nonce' ),
			)
		);
	}
	/**
	 * Add admin_init actions
	 *
	 * @since 2.3
	 */
	public function admin_init() {}
	public function update_label() {
		return '';
	}
	/**
	 * Add options and set defaults to
	 *
	 * @param array $options
	 * @since 2.0
	 */
	public function options() {
		$sizes = get_intermediate_image_sizes();
		$image_sizes = array();
		foreach ( $sizes as $size ) {
			$image_sizes[ $size ] = ucwords( str_replace( '-', ' ', $size ) );
		}
		$options = array(
			self::$type . '_format_type' => array(
				'label' => esc_html__( 'Upload Limit', 'simpods-mvc' ),
				'default' => 'single',
				'type' => 'pick',
				'data' => array(
					'single' => esc_html__( 'Single File', 'simpods-mvc' ),
					'multi'  => esc_html__( 'Multiple Files', 'simpods-mvc' )
				),
				'dependency' => true
			),
			self::$type . '_uploader' => array(
				'label' => esc_html__( 'File Uploader', 'simpods-mvc' ),
				'default' => 'attachment',
				'type' => 'pick',
				'data' => apply_filters(
					'pods_form_ui_field_file_uploader_options',
					array(
						'attachment' => esc_html__( 'Attachments (WP Media Library)', 'simpods-mvc' ),
					)
				),
				'dependency' => true
			),
			self::$type . '_attachment_tab' => array(
				'label' => esc_html__( 'Attachments Default Tab', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_uploader' => 'attachment' ),
				'default' => 'upload',
				'type' => 'pick',
				'data' => array(
					// keys MUST match WP's router names
					// 'upload' => esc_html__( 'Upload File', 'simpods-mvc' ),
					'browse' => esc_html__( 'Media Library', 'simpods-mvc' )
				)
			),
			self::$type . '_edit_title' => array(
				'label' => esc_html__( 'Editable Title', 'simpods-mvc' ),
				'default' => 1,
				'type' => 'boolean'
			),
			self::$type . '_extract_zip' => array(
				'label' => esc_html__( 'Unzip the Zipped File to A asset_{attachment_id} Folder', 'simpods-mvc' ),
				'default' => 0,
				'type' => 'boolean'
			),
			/* Don't delete, may work on them later
			self::$type . '_linked' => array(
				'label' => esc_html__( 'Link to File in editor', 'simpods-mvc' ),
				'default' => 0,
				'type' => 'boolean'
			),
			self::$type . '_limit' => array(
				'label' => esc_html__( 'Max Number of Files', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_format_type' => 'multi' ),
				'default' => 0,
				'type' => 'number'
			),
			self::$type . '_restrict_filesize' => array(
				'label' => esc_html__( 'Restrict File Size', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_uploader' => 'plupload' ),
				'default' => '10MB',
				'type' => 'text'
			),*/
			/*self::$type . '_type' => array(
				'label' => esc_html__( 'Restrict File Types', 'simpods-mvc' ),
				'default' => apply_filters( 'pods_form_ui_field_file_type_default', 'images' ),
				'type' => 'pick',
				'data' => apply_filters(
					'pods_form_ui_field_file_type_options',
					array(
						'images' => esc_html__( 'Images (jpg, jpeg, png, gif)', 'simpods-mvc' ),
						'video' => esc_html__( 'Video (mpg, mov, flv, mp4, etc..)', 'simpods-mvc' ),
						'audio' => esc_html__( 'Audio (mp3, m4a, wav, wma, etc..)', 'simpods-mvc' ),
						'text' => esc_html__( 'Text (txt, csv, tsv, rtx, etc..)', 'simpods-mvc' ),
						'any' => esc_html__( 'Any Type (no restriction)', 'simpods-mvc' ),
						'other' => esc_html__( 'Other (customize allowed extensions)', 'simpods-mvc' )
					)
				),
				'dependency' => true
			),
			self::$type . '_allowed_extensions' => array(
				'label' => esc_html__( 'Allowed File Extensions', 'simpods-mvc' ),
				'description' => esc_html__( 'Separate file extensions with a comma (ex. jpg,png,mp4,mov)', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_type' => 'other' ),
				'default' => apply_filters( 'pods_form_ui_field_file_extensions_default', '' ),
				'type' => 'text'
			),*/
			/*self::$type . '_image_size' => array(
				'label' => esc_html__( 'Excluded Image Sizes', 'simpods-mvc' ),
				'description' => esc_html__( 'Image sizes not to generate when processing the image', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_type' => 'images' ),
				'default' => 'images',
				'type' => 'pick',
				'pick_format_type' => 'multi',
				'pick_format_multi' => 'checkbox',
				'data' => apply_filters(
					'pods_form_ui_field_file_image_size_options',
					$image_sizes
				)
			),*/
			self::$type . '_add_button' => array(
				'label' => esc_html__( 'Add Button Text', 'simpods-mvc' ),
				'default' => esc_html__( 'Add File', 'simpods-mvc' ),
				'type' => 'text'
			),
			self::$type . '_modal_title' => array(
				'label' => esc_html__( 'Modal Title', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_uploader' => 'attachment' ),
				'default' => esc_html__( 'Attach a file', 'simpods-mvc' ),
				'type' => 'text'
			),
			self::$type . '_modal_add_button' => array(
				'label' => esc_html__( 'Modal Add Button Text', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_uploader' => 'attachment' ),
				'default' => esc_html__( 'Add File', 'simpods-mvc' ),
				'type' => 'text'
			),
			self::$type . '_new_folder' => array(
				'label' => esc_html__( 'Save the files into new folders', 'simpods-mvc' ),
				'default' => 0,
				'type' => 'boolean',
				'dependency' => true
			),
			self::$type . '_folder' => array(
				'label' => esc_html__( 'Folder', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_new_folder' => 1 ),
				'description' => esc_html__( 'Create a new folder to save files e.g. new_folder/new_sub_folder. It does not apply to month- and year-based folders', 'simpods-mvc'),
				'default' => '',
				'type' => 'text'
			),
			self::$type . '_sub_folder' => array(
				'label' => esc_html__( 'Sub-folders', 'simpods-mvc' ),
				'depends-on' => array( self::$type . '_new_folder' => 1 ),
				'description' => esc_html__( 'Create sub-folders with parent IDs. If adding a new post, simpods-file-sub-folder-0 will be used instead. When updating a post, it will be saved to its own folder name with this pattern: type-id.', 'simpods-mvc'),
				'default' => 0,
				'type' => 'boolean'
			),
			self::$type . '_protect_folder' => array(
				'label' => esc_html__( 'Protect the folder', 'simpods-mvc'),
				'default' => 0,
				'depends-on' => array( self::$type . '_new_folder' => 1 ),
				'description' => esc_html__( 'No external access: people cannot access the files outside your site. Paterned name: The file name will be renamed with a pattern regularly. If you change your options, you have to upload a file to update the settings', 'simpods-mvc'),
				'default' => 'no',
				'type' => 'pick',
				'data' => array(
					'no'              		=> esc_html__( 'No', 'simpods-mvc' ),
					'no_external_access' 	=> esc_html__( 'No external access', 'simpods-mvc' ),
					'patterned_code_in_url'	=> esc_html__( 'Patterned Code in URL', 'simpods-mvc' ),
					//'logged_in_users' => esc_html__( 'Logged in users', 'simpods-mvc' ),
					//'administrators'  => esc_html__( 'Administrators', 'simpods-mvc' ),
					//'assigned_user'   => esc_html__( 'Assigned user. Files uploaded on the user profile page only. Not for nested repeater fields', 'simpods-mvc' ),
				),
				'dependency' => true
			),
			// self::$type . '_allow_local' => array(
			// 	'label' => esc_html__( 'Allow local access', 'simpods-mvc'),
			// 	'default' => 0,
			// 	'depends-on' => array( self::$type . '_protect_folder' => array( 'logged_in_users' )	), //, 'administrators', 'assigned_user'
			// 	'description' => esc_html__( 'Allow checks to be bypassed when file is accessed by something else on the server', 'simpods-mvc'),
			// 	'default' => 0,
			// 	'type' => 'boolean'
			// )
		);
		if ( !pods_version_check( 'wp', '3.5' ) ) {
			unset( $options[ self::$type . '_linked' ] );
			unset( $options[ self::$type . '_modal_title' ] );
			unset( $options[ self::$type . '_modal_add_button' ] );
			$options[ self::$type . '_attachment_tab' ][ 'default' ] = 'type';
			$options[ self::$type . '_attachment_tab' ][ 'data' ] = array(
				'type' => esc_html__( 'Upload File', 'simpods-mvc' ),
				'library' => esc_html__( 'Media Library', 'simpods-mvc' )
			);
		}
		return $options;
	}
	/**
	 * Define the current field's schema for DB table storage
	 *
	 * @param array $options
	 * @return array
	 * @since 2.0
	 */
	public function schema( $options = null ) {
		$schema = 'TEXT';
		return $schema;
	}
	/**
	 * Define the current field's preparation for sprintf
	 *
	 * @param array $options
	 * @return array
	 * @since 1.0
	 */
	public function prepare( $options = null ) {
		$format = self::$prepare;
		return $format;
	}
	/**
	 * Change the value of the field
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null|string
	 * @since 1.0.0
	 */
	public function value( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return $value;
	}
	/**
	 * Change the way the value of the field is displayed with Pods::get
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @return mixed|null
	 * @since 2.0
	 */
	public function display( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		if ( is_array( $value ) && !empty( $value ) ) {
			if ( isset( $value[ 'ID' ] ) )
				$value = wp_get_attachment_url( $value[ 'ID' ] );
			else {
				$attachments = $value;
				$value = array();
				foreach ( $attachments as $v ) {
					if ( !is_array( $v ) ) {
						$value[] = $v;
					} else if ( isset( $v[ 'ID' ] ) ) {
						$value[] = wp_get_attachment_url( $v[ 'ID' ] );
					}
				}
				$value = implode( ' ', $value );
			}
		}
		return $value ;
	}
	/**
	 * Customize output of the form field
	 *
	 * @param string $name
	 * @param mixed $value
	 * @param array $options
	 * @param array $pod
	 * @param int $id
	 * @since 2.0
	 */
	public function input( $name, $value = null, $options = null, $pod = null, $id = null ) {
		$parents  = $this->parent_id();
		// convert the $value to the array required by pods
		$value      = maybe_unserialize( $value );
		$values = array();
		if ( is_array( $value ) ) {
			foreach( $value as $key => $value_details ){
				$values[ $key ] = $key;
			}
		}
		$value   =  $values;
		$options = (array) $options;
		$form_field_type = PodsForm::$field_type;
		if ( !is_admin() ) {
			include_once ABSPATH . '/wp-admin/includes/template.php';
			if ( is_multisite() )
				include_once ABSPATH . '/wp-admin/includes/ms.php';
		}
		if ( ( ( defined( 'PODS_DISABLE_FILE_UPLOAD' ) && true === PODS_DISABLE_FILE_UPLOAD )
			   || ( defined( 'PODS_UPLOAD_REQUIRE_LOGIN' ) && is_bool( PODS_UPLOAD_REQUIRE_LOGIN ) && true === PODS_UPLOAD_REQUIRE_LOGIN && !is_user_logged_in() )
			   || ( defined( 'PODS_UPLOAD_REQUIRE_LOGIN' ) && !is_bool( PODS_UPLOAD_REQUIRE_LOGIN ) && ( !is_user_logged_in() || !current_user_can( PODS_UPLOAD_REQUIRE_LOGIN ) ) ) )
			 && ( ( defined( 'PODS_DISABLE_FILE_BROWSER' ) && true === PODS_DISABLE_FILE_BROWSER )
				  || ( defined( 'PODS_FILES_REQUIRE_LOGIN' ) && is_bool( PODS_FILES_REQUIRE_LOGIN ) && true === PODS_FILES_REQUIRE_LOGIN && !is_user_logged_in() )
				  || ( defined( 'PODS_FILES_REQUIRE_LOGIN' ) && !is_bool( PODS_FILES_REQUIRE_LOGIN ) && ( !is_user_logged_in() || !current_user_can( PODS_FILES_REQUIRE_LOGIN ) ) ) )
		) {
			?>
		<p>You do not have access to upload / browse files. Contact your website admin to resolve.</p>
		<?php
			return;
		}
		// Use plupload if attachment isn't available
		if ( 'attachment' == pods_var( self::$type . '_uploader', $options ) && ( !is_user_logged_in() || ( !current_user_can( 'upload_files' ) && !current_user_can( 'edit_files' ) ) ) ){
			$field_type = 'plupload';
		} else if ( 'plupload' == pods_var( self::$type . '_uploader', $options ) ){
			$field_type = 'plupload';
		} else if ( 'attachment' == pods_var( self::$type . '_uploader', $options ) ) {
			if ( !pods_version_check( 'wp', '3.5' ) || !is_admin() ) {
				// @todo test frontend media modal
				$field_type = 'attachment';
			} else {
				$field_type = 'media';
			}
		}
		else {
			// Support custom File Uploader integration
			do_action( 'pods_form_ui_field_file_uploader_' . pods_var( self::$type . '_uploader', $options ), $name, $value, $options, $pod, $id );
			do_action( 'pods_form_ui_field_file_uploader', pods_var( self::$type . '_uploader', $options ), $name, $value, $options, $pod, $id );
			return;
		}
		$field_options = array();
		foreach ( $options as $key => $option ) {
			if ( strpos( $key, 'simpodspodsfile_' ) === 0 ) {
				$field_options[ $key ] = $option;
			}
		}
		$this->simpods_files_options[ $name ] = $field_options;
		$inline_css = '';
		if ( !is_array( $value ) || count( $value ) == 0 ) {
			$inline_css = 'display:none;';
		}
		?>
		<div class="pods-form-ui-field pods-dfv-field">
			<div class="pods-ui-file-list pods-field-template-rows">
				<ul class="pods-dfv-list <?php echo $name; ?>-list" style="<?php echo $inline_css; ?>">
					<?php
					//$upload_arr = wp_upload_dir();
					if ( is_array( $value ) && count( $value ) > 0 ) {
						foreach ( $value as $key => $file_id ) {
						$file_html  = '';
						$title   	= get_the_title( $file_id );
						$file_types = wp_check_filetype( get_attached_file( $file_id ) );
						$file_url  	= wp_get_attachment_url( $file_id );
						$file_link 	= $file_url;
						$icon_url  	= '';
						$icon_data  = wp_get_attachment_image_src( $file_id, 'thumbnail' );
						if( is_array( $icon_data ) && count( $icon_data ) > 0 ){
							$icon_url  = $icon_data[0];
						}
						
						if ( $file_types ) {
							if ( stripos( $file_types['type'], 'image' ) === false ) {								
								if ( stripos( $file_types['type'], 'audio' ) !== false ) {									
									$icon_url = rtrim( site_url(), '/' ) . '/wp-includes/images/media/audio.png';
								} elseif ( stripos( $file_types['type'], 'video' ) !== false ) {
									
									$icon_url = rtrim( site_url(), '/' ) . '/wp-includes/images/media/video.png';
								} else {								
									$icon_url = rtrim( site_url(), '/' ) . '/wp-includes/images/media/document.png';
								}
							}
														
						}
						$file_html = '<img class="pinkynail" src="' . $icon_url . '" alt="Icon">';
						$this->simpods_files[] = array(
							'target'   		=> $name,
							'file_id'   	=> $file_id,
							'file_html'     => $file_html,
							'file_url'    	=> $file_link,
							'deselect' 	=> esc_html__('Deselect', 'simpods-mvc'),
						);
						}
					}
					echo '<script type="text/javascript">';
					echo 'var simpods_files_parents = ' . json_encode( $parents ) . ';';
					echo 'var simpods_files = ' . json_encode( $this->simpods_files ) . ';';
					echo 'var simpods_files_options = ' . json_encode( $this->simpods_files_options ) . ';';
					echo '</script>';
					?>
				</ul>
			</div>
			<div class="pods-ui-region"></div>
			<div class="pods-ui-form">
				<div>
					<a class="button simpodspodsfile-btn" href="#" data-target="<?php echo $name; ?>" ><?php esc_html_e('Add File', 'simpods-mvc'); ?></a>
				</div>
			</div>
		</div>
		<?php
	}
	/**
	 * return parent id
	 */
	public function parent_id() {
		$data 		= array('type' => '', 'id' => 0 );
		$file_name 	= basename( $_SERVER['PHP_SELF'] );
		if ( defined( 'IS_PROFILE_PAGE' ) && IS_PROFILE_PAGE ) {// If is current user's profile (profile.php)
			$data['id']   = get_current_user_id();
			$data['type'] = 'user';
		} else if ( $file_name == 'user-edit.php' && isset( $_GET['user_id'] ) && is_numeric($_GET['user_id']) ) {// If is another user's profile page
			$data['id']   = ( int )$_GET['user_id'];
			$data['type'] = 'user';
		} else if ( $file_name == 'post.php' && isset( $_GET['post'] ) && is_numeric($_GET['post']) ){ //wordpress post id
			$data['id']   = ( int )$_GET['post'];
			$data['type'] = 'post';
		} else if ( $file_name == 'admin.php' && isset( $_GET['page'] ) && strpos( $_GET['page'], 'pods-' ) === 0 && isset( $_GET['id'] ) && is_numeric($_GET['id']) ){
		//pods post id
			$data['id']   = ( int )$_GET['id'];
			$data['type'] = 'pods';
		} else if ( isset( $_GET['page'] ) && $_GET['page'] == 'panda-pods-repeater-field' && isset( $_GET['postid'] ) && is_numeric($_GET['postid']) ){
		//Panda pods repeater field post id
			$data['id']   = ( int )$_GET['postid'];
			$data['type'] = 'panda';
		} else {
			$data['id']   = 0;
			$data['type'] = 'simpods-file-sub-folder';
		}
		return $data;
	}
	/**
	 * Build regex necessary for JS validation
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param string $pod
	 * @param int $id
	 * @return bool
	 * @since 2.0
	 */
	public function regex( $value = null, $name = null, $options = null, $pod = null, $id = null ) {
		return false;
	}
	/**
	 * Validate a value before it's saved
	 *
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param int $id
	 * @param null $params
	 * @return bool
	 * @since 2.0
	 */
	public function validate( $value, $name = null, $options = null, $fields = null, $pod = null, $id = null, $params = null ) {
		// check file size
		// check file extensions
		return true;
	}
	/**
	 * Change the value or perform actions after validation but before saving to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @return mixed
	 * @since 2.0
	 */
	public function pre_save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		// when change from pods file / image to simpods pods file, the db field is not created. Check and create here
		global $cross_admin;
		foreach( $fields as $key => $value_details ){
			if( $value_details['type'] == 'simpodspodsfile' ){
				// extract the attachment id from the array
				$custom_fields = array(
					$value_details['name'] => array( 'type' => 'TEXT', 'settings' => 'NOT NULL', 'default' => '' ),
				);
				$cross_admin->update_columns( 'pods_' . $pod['name'],  $custom_fields, false );
			}
		}
		// Array ( [883] => Array ( [id] => 883 [title] => Ascend Logo - Light ) )
		$value = maybe_serialize( $value );
		return $value;
	}
	/**
	 * Check if field is Simpods File and if Extract ZIP is checked
	 *
	 * @param  mixed[] $pieces  the pod data
	 * @param  boolean $is_new_item is it a new item
	 * @param  int $id  id of the item if not new
	 * @return null
	 */
	public function check_unzip( $pieces, $is_new_item, $id ) {
		foreach ( $pieces as $key => $value_details ) {
			if ( $key == 'fields' ) {
				foreach ( $value_details as $sub_key => $sub_values ) {
					if ( $sub_values['type'] == 'simpodspodsfile' && $sub_values['options']['simpodspodsfile_extract_zip'] ) {
						if ( isset( $sub_values['value'] ) && is_array( $sub_values['value'] ) ) {
							foreach ( $sub_values['value'] as $sub_sub_key => $sub_sub_values ) {
								$this->unzip( intval( $sub_sub_key ), $sub_values['options'] );
							}
						}
					}
				}
			}
		}
	}
	/**
	 * Extract uploaded ZIP file
	 *
	 * This function is to add custom orderby to query_post for most viewed
	 *
	 * @param int $attID_int ID of file to extract
	 * @param array $options Settings from the Pods field to protect the file. Leave it to later.
	 */
	public function unzip( $attID_int, $options = array() ) {
		global $wp_filesystem;
		$defaults = array(
			'simpodspodsfile_protect_folder' 	=> '',
			'simpodspodsfile_allow_local' 		=> 0,
		);
		$options    = wp_parse_args( $options, $defaults );
		if ( is_numeric( $attID_int ) && $attID_int > 0 ) {
			$file_url = wp_get_attachment_url( $attID_int );
			if ( $file_url != '' ) {
				$upload_dir_data 	= wp_upload_dir();
				$file_name      	= str_replace( $upload_dir_data['url'] . '/', '', $file_url );
				$file_name_parts    = explode( '.', $file_name );
				if ( $file_name_parts[1] == 'zip' ) {
					$path_and_file = str_replace( $upload_dir_data['url'], $upload_dir_data['path'], $file_url );
					// delete the directory for update
					//if ( $upload_dir_data['basedir'] . '/asset_' . $attID_int ) {
					$this->delete_files( $upload_dir_data['basedir'] . '/asset_' . $attID_int );
					//}
					//echo $path_and_file;
					if ( $wp_filesystem->exists( $path_and_file ) ) {
						$unzip_to_dir = $upload_dir_data['basedir'] . '/asset_' . $attID_int;
						// create object
						$zip_archive = new ZipArchive();
						// open archive
						if ( $zip_archive->open( $path_and_file ) !== TRUE ) {
							trigger_error( esc_html__( 'Could not open archive ', 'simpods-mvc' ) . $path_and_file );
						}
						// extract contents to destination directory
						$zip_archive->extractTo( $unzip_to_dir );
						// close archive
						$zip_archive->close();
					}
				} else {
					// delete the directory if no file saved
					if ( $upload_dir_data['basedir'] . '/asset_' . $attID_int ) {
						$this->delete_files( $upload_dir_data['basedir'] . '/asset_' . $attID_int );
					}
				}
			} else {
				// delete the directory if no file saved
				if ( $upload_dir_data['basedir'] . '/asset_' . $attID_int ) {
					$this->delete_files( $upload_dir_data['basedir'] . '/asset_' . $attID_int );
				}
				// http://stackoverflow.com/questions/3349753/delete-directory-with-files-in-it
			}
		}
	}
	/*
	 * php delete function that deals with directories recursively
	 *
	 * @param string $target File path to delete
	 */
	private function delete_files( $target ) {
		//global $funs;	
		//$ready = $funs->activate_wp_filesystem( $target );
				
		$ready = simpods_wp_filesystem( $target );
		if( $ready ){
			global $wp_filesystem;
			//$wp_filesystem_base = WP_Filesystem_Base(false);		
			if( $wp_filesystem->is_dir( $target ) ){
				$wp_filesystem->delete( $target , true, 'd');
			} else {
				$wp_filesystem->delete( $target , false, 'f');
			}			
		}				
	}
	/**
	 * Save the value to the DB
	 *
	 * @param mixed $value
	 * @param int $id
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @param object $params
	 * @since 2.3
	 */
	public function save( $value, $id = null, $name = null, $options = null, $fields = null, $pod = null, $params = null ) {
		return null;
		if ( empty( self::$api ) ) {
			self::$api = pods_api();
		}
		// File title / field handling
		foreach ( $value as $id ) {
			$title = false;
			if ( is_array( $id ) ) {
				if ( isset( $id['title'] ) && 0 < strlen( trim( $id['title'] ) ) ) {
					$title = trim( $id['title'] );
				}
				if ( isset( $id['id'] ) ) {
					$id = (int) $id['id'];
				} else {
					$id = 0;
				}
			}
			if ( empty( $id ) ) {
				continue;
			}
			// Update the title if set
			if ( false !== $title && 1 == pods_var( self::$type . '_edit_title', $options, 0 ) ) {
				$attachment_data = array(
					'ID'         => $id,
					'post_title' => $title,
				);
				self::$api->save_wp_object( 'media', $attachment_data );
			}
		}
	}
	/**
	 * Customize the Pods UI manage table column output
	 *
	 * @param int $id
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $fields
	 * @param array $pod
	 * @return mixed|void
	 * @since 2.0
	 */
	public function ui( $id, $value, $name = null, $options = null, $fields = null, $pod = null ) {
		if ( empty( $value ) ) {
			return;
		}
		if ( !empty( $value ) && isset( $value[ 'ID' ] ) ) {
			$value = array( $value );
		}
		$image_size = apply_filters( 'pods_form_ui_field_file_ui_image_size', 'thumbnail', $id, $value, $name, $options, $pod );
		return $this->images( $id, $value, $name, $options, $pod, $image_size );
	}
	/**
	 * Return image(s) markup
	 *
	 * @param int $id
	 * @param mixed $value
	 * @param string $name
	 * @param array $options
	 * @param array $pod
	 * @param string $image_size
	 * @return string
	 * @since 2.3
	 */
	public function images( $id, $value, $name = null, $options = null, $pod = null, $image_size = null ) {
		$images = '';
		if ( empty( $value ) || !is_array( $value ) ) {
			return $images;
		}
		foreach ( $value as $v ) {
			$images .= pods_image( $v, $image_size );
		}
		return $images;
	}
	/**
	 * Handle file row output for uploaders
	 *
	 * @param array $attributes
	 * @param int $limit
	 * @param bool $editable
	 * @param int $id
	 * @param string $icon
	 * @param string $name
	 * @return string
	 * @since 2.0
	 */
	public function markup( $attributes, $limit = 1, $editable = true, $id = null, $icon = null, $name = null, $linked = false, $link = null ) {
		// Preserve current file type
		$field_type = PodsForm::$field_type;
		ob_start();
		if ( empty( $id ) ) {
			$id = '{{id}}';
		}
		if ( empty( $icon ) ) {
			$icon = '{{icon}}';
		} else{
			$icon = esc_url( $icon );
		}
		if ( empty( $name ) ) {
			$name = '{{name}}';
		}
		if ( empty( $link ) ) {
			$link = '{{link}}';
		}
		$editable = (boolean) $editable;
		$linked = (boolean) $linked;
		?>
	<li class="pods-file hidden" id="pods-file-<?php echo esc_attr( $id ); ?>">
		<?php echo PodsForm::field( $attributes[ 'name' ] . '[' . $id . '][id]', $id, 'hidden' ); ?>
		<ul class="pods-file-meta media-item">
			<?php if ( 1 != $limit ) { ?>
				<li class="pods-file-col pods-file-handle"><?php esc_html_e( 'Handle', 'simpods-mvc' ); ?></li>
			<?php } ?>
			<li class="pods-file-col pods-file-icon">
				<img class="pinkynail" src="<?php echo $icon; ?>" alt="Icon" />
			</li>
			<li class="pods-file-col pods-file-name">
				<?php
				if ( $editable ) {
					echo PodsForm::field( $attributes[ 'name' ] . '[' . $id . '][title]', $name, 'text' );
				} else {
					echo ( empty( $name ) ? '{{name}}' : $name );
				}
				?>
			</li>
			<li class="pods-file-col pods-file-delete"><a href="#delete"><?php esc_html_e( 'Delete', 'simpods-mvc' ); ?></a></li>
			<?php
				if ( $linked ) {
			?>
				<li class="pods-file-col pods-file-download"><a href="<?php echo esc_url( $link ); ?>" target="_blank"><?php esc_html_e( 'Download', 'simpods-mvc' ); ?></a></li>
			<?php
				}
			?>
		</ul>
	</li>
	<?php
		PodsForm::$field_type = $field_type;
		return ob_get_clean();
	}
	/**
	 * Handle plupload AJAX
	 *
	 * @since 2.3
	 */
	public function admin_ajax_upload() {
		if ( !isset( $_POST['action'] ) || $_POST['action'] != 'admin_ajax_upload' ) {
			die();
		}
		pods_session_start();
		// Sanitize input
		$params = pods_unslash( (array) $_POST );
		foreach ( $params as $key => $value ) {
			if ( 'action' == $key )
				continue;
			unset( $params[ $key ] );
			$params[ str_replace( '_podsfix_', '', $key ) ] = $value;
		}
		$params = (object) $params;
		$methods = array(
			'upload',
		);
		if ( !isset( $params->method ) || !in_array( $params->method, $methods ) || !isset( $params->pod ) || !isset( $params->field ) || !isset( $params->uri ) || empty( $params->uri ) ) {
			pods_error( esc_html__( 'Invalid AJAX request', 'simpods-mvc' ), PodsInit::$admin );
		} else if ( !empty( $params->pod ) && empty( $params->field ) ) {
			pods_error( esc_html__( 'Invalid AJAX request', 'simpods-mvc' ), PodsInit::$admin );
		} else if ( empty( $params->pod ) && !current_user_can( 'upload_files' ) ) {
			pods_error( esc_html__( 'Invalid AJAX request', 'simpods-mvc' ), PodsInit::$admin );
		}
		// Flash often fails to send cookies with the POST or upload, so we need to pass it in GET or POST instead
		if ( is_ssl() && empty( $_COOKIE[ SECURE_AUTH_COOKIE ] ) && !empty( $_REQUEST[ 'auth_cookie' ] ) ) {
			$_COOKIE[ SECURE_AUTH_COOKIE ] = $_REQUEST[ 'auth_cookie' ];
		} else if ( empty( $_COOKIE[ AUTH_COOKIE ] ) && !empty( $_REQUEST[ 'auth_cookie' ] ) ) {
			$_COOKIE[ AUTH_COOKIE ] = $_REQUEST[ 'auth_cookie' ];
		}
		if ( empty( $_COOKIE[ LOGGED_IN_COOKIE ] ) && !empty( $_REQUEST[ 'logged_in_cookie' ] ) ) {
			$_COOKIE[ LOGGED_IN_COOKIE ] = $_REQUEST[ 'logged_in_cookie' ];
		}
		global $current_user;
		unset( $current_user );
		/**
		 * Access Checking
		 */
		$upload_disabled = false;
		if ( defined( 'PODS_DISABLE_FILE_UPLOAD' ) && true === PODS_DISABLE_FILE_UPLOAD ) {
			$upload_disabled = true;
		} else if ( defined( 'PODS_UPLOAD_REQUIRE_LOGIN' ) && is_bool( PODS_UPLOAD_REQUIRE_LOGIN ) && true === PODS_UPLOAD_REQUIRE_LOGIN && !is_user_logged_in() ) {
			$upload_disabled = true;
		} else if ( defined( 'PODS_UPLOAD_REQUIRE_LOGIN' ) && !is_bool( PODS_UPLOAD_REQUIRE_LOGIN ) && ( !is_user_logged_in() || !current_user_can( PODS_UPLOAD_REQUIRE_LOGIN ) ) ) {
			$upload_disabled = true;
		}
		$uid = session_id();
		if ( is_user_logged_in() ) {
			$uid = 'user_' . get_current_user_id();
		}
		$nonce_check = 'pods_upload_' . (int) $params->pod . '_' . $uid . '_' . $params->uri . '_' . (int) $params->field;
		if ( true === $upload_disabled || !isset( $params->_wpnonce ) || false === wp_verify_nonce( $params->_wpnonce, $nonce_check ) ) {
			pods_error( esc_html__( 'Unauthorized request', 'simpods-mvc' ), PodsInit::$admin );
		}
		$pod = array();
		$field = array(
			'type' => 'file',
			'options' => array()
		);
		$api = pods_api();
		$api->display_errors = false;
		if ( !empty( $params->pod ) ) {
			$pod = $api->load_pod( array( 'id' => (int) $params->pod ) );
			$field = $api->load_field( array( 'id' => (int) $params->field ) );
			if ( empty( $pod ) || empty( $field ) || $pod[ 'id' ] != $field[ 'pod_id' ] || !isset( $pod[ 'fields' ][ $field[ 'name' ] ] ) ) {
				pods_error( esc_html__( 'Invalid field request', 'simpods-mvc' ), PodsInit::$admin );
			}
			if ( !in_array( $field[ 'type' ], PodsForm::file_field_types() ) ) {
				pods_error( esc_html__( 'Invalid field', 'simpods-mvc' ), PodsInit::$admin );
			}
		}
		$method = $params->method;
		// Cleaning up $params
		unset( $params->action );
		unset( $params->method );
		unset( $params->_wpnonce );
		$params->post_id = pods_var( 'post_id', $params, 0, null, true );
		/**
		 * Upload a new file (advanced - returns URL and ID)
		 */
		if ( 'upload' == $method ) {
			$file = $_FILES[ 'Filedata' ];
			$limit_size = pods_var( $field[ 'type' ] . '_restrict_filesize', $field[ 'options' ] );
			if ( !empty( $limit_size ) ) {
				if ( false !== stripos( $limit_size, 'MB' ) ) {
					$limit_size = (float) trim( str_ireplace( 'MB', '', $limit_size ) );
					$limit_size = $limit_size * 1025 * 1025; // convert to KB to B
				}
				else if ( false !== stripos( $limit_size, 'KB' ) ) {
					$limit_size = (float) trim( str_ireplace( 'KB', '', $limit_size ) );
					$limit_size = $limit_size * 1025 * 1025; // convert to B
				}
				else if ( false !== stripos( $limit_size, 'GB' ) ) {
					$limit_size = (float) trim( str_ireplace( 'GB', '', $limit_size ) );
					$limit_size = $limit_size * 1025 * 1025 * 1025; // convert to MB to KB to B
				}
				else if ( false !== stripos( $limit_size, 'B' ) )
					$limit_size = (float) trim( str_ireplace( 'B', '', $limit_size ) );
				else
					$limit_size = wp_max_upload_size();
				if ( 0 < $limit_size && $limit_size < $file[ 'size' ] ) {
					/* translators: %s: File size */
					$error = esc_html__( 'File size too large, max size is %s', 'simpods-mvc' );
					$error = sprintf( $error, pods_var( $field[ 'type' ] . '_restrict_filesize', $field[ 'options' ] ) );
					pods_error( '<div style="color:#FF0000">' . esc_html__( 'Error', 'simpods-mvc' ) . ': ' . $error . '</div>' );
				}
			}
			$limit_file_type = pods_var( $field[ 'type' ] . '_type', $field[ 'options' ], 'images' );
			if ( 'images' == $limit_file_type ) {
				$limit_types = 'jpg,jpeg,png,gif';
			} else if ( 'video' == $limit_file_type ) {
				$limit_types = 'mpg,mov,flv,mp4';
			} else if ( 'audio' == $limit_file_type ) {
				$limit_types = 'mp3,m4a,wav,wma';
			} else if ( 'text' == $limit_file_type ) {
				$limit_types = 'txt,rtx,csv,tsv';
			} else if ( 'any' == $limit_file_type ) {
				$limit_types = '';
			} else {
				$limit_types = pods_var( $field[ 'type' ] . '_allowed_extensions', $field[ 'options' ], '', null, true );
			}
			$limit_types = trim( str_replace( array( ' ', '.', "\n", "\t", ';' ), array( '', ',', ',', ',' ), $limit_types ), ',' );
			if ( pods_version_check( 'wp', '3.5' ) ) {
				$mime_types = wp_get_mime_types();
				if ( in_array( $limit_file_type, array( 'images', 'audio', 'video' ) ) ) {
					$new_limit_types = array();
					foreach ( $mime_types as $type => $mime ) {
						if ( 0 === strpos( $mime, $limit_file_type ) ) {
							$type = explode( '|', $type );
							$new_limit_types = array_merge( $new_limit_types, $type );
						}
					}
					if ( !empty( $new_limit_types ) )
						$limit_types = implode( ',', $new_limit_types );
				}
				else if ( 'any' != $limit_file_type ) {
					$new_limit_types = array();
					$limit_types = explode( ',', $limit_types );
					foreach ( $limit_types as $k => $limit_type ) {
						$found = false;
						foreach ( $mime_types as $type => $mime ) {
							if ( 0 === strpos( $mime, $limit_type ) ) {
								$type = explode( '|', $type );
								foreach ( $type as $t ) {
									if ( !in_array( $t, $new_limit_types ) )
										$new_limit_types[] = $t;
								}
								$found = true;
							}
						}
						if ( !$found )
							$new_limit_types[] = $limit_type;
					}
					if ( !empty( $new_limit_types ) )
						$limit_types = implode( ',', $new_limit_types );
				}
			}
			$limit_types = explode( ',', $limit_types );
			$limit_types = array_filter( array_unique( $limit_types ) );
			if ( !empty( $limit_types ) ) {
				$ok = false;
				foreach ( $limit_types as $limit_type ) {
					$limit_type = '.' . trim( $limit_type, ' .' );
					$pos = ( strlen( $file[ 'name' ] ) - strlen( $limit_type ) );
					if ( $pos === stripos( $file[ 'name' ], $limit_type ) ) {
						$ok = true;
						break;
					}
				}
				if ( false === $ok ) {
					/* translators: %s: List of file types */
					$error = esc_html__( 'File type not allowed, please use one of the following: %s', 'simpods-mvc' );
					$error = sprintf( $error, '.' . implode( ', .', $limit_types ) );
					pods_error( '<div style="color:#FF0000">' . esc_html__( 'Error', 'simpods-mvc' ) . ': ' . $error . '</div>' );
				}
			}
			$custom_handler = apply_filters( 'pods_upload_handle', null, 'Filedata', $params->post_id, $params, $field );
			if ( null === $custom_handler ) {
				$linked = pods_var( $field[ 'type' ] . '_linked', $field[ 'options' ], 0 );
				$attachment_id = media_handle_upload( 'Filedata', $params->post_id );
				if ( is_object( $attachment_id ) ) {
					$errors = array();
					foreach ( $attachment_id->errors[ 'upload_error' ] as $error_code => $error_message ) {
						$errors[] = '[' . $error_code . '] ' . $error_message;
					}
					pods_error( '<div style="color:#FF0000">' . esc_html__( 'Error', 'simpods-mvc' ) . ': ' . implode( '</div><div>', $errors ) . '</div>' );
				}
				else {
					$attachment = get_post( $attachment_id, ARRAY_A );
					$attachment[ 'filename' ] = basename( $attachment[ 'guid' ] );
					$thumb = wp_get_attachment_image_src( $attachment[ 'ID' ], 'thumbnail', true );
					$attachment[ 'thumbnail' ] = $thumb[ 0 ];
					$attachment[ 'link' ] = '';
					if ( $linked ) {
						$attachment[ 'link' ] = wp_get_attachment_url( $attachment[ 'ID' ] );
					}
					$attachment = apply_filters( 'pods_upload_attachment', $attachment, $params->post_id );
					wp_send_json( $attachment );
				}
			}
		}
		session_write_close();
		die(); // KBAI!
	}
	/**
	 * localize javascript variables
	 * @return null
	 */
	public function localize(){
		if ( isset( $_GET ) &&  isset( $_GET['page'] ) && $_GET['page'] == 'pods' ) {
			return;
		}
		wp_localize_script(
				self::$type . '',
				'simpods_files',
				$this->simpods_files
		);
	}
}
