<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * collection of functions for generic purposes for each section
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 05/08/2014
 */
abstract class MVC_Front_General {

	abstract public function actions();
	abstract public function filters();
	//abstract public function enqueue();

	/**
	 * [pagenate_fn description]
	 * @param  [type] $url      [description]
	 * @param  [type] $url_query      [description]
	 * @param  [type] $page_idx     [description]
	 * @param  [type] $pages_count    [description]
	 * @param  string $page_name [description]
	 * @return [type]
	 */
	public function pagenate( $url, $url_query, $page_idx, $pages_count, $page_name = 'paging' ) {
		$url = esc_attr( $url );
		$url_query = esc_attr( $url_query );

		$html = '
			<div class="pagenate">';

		if ( $page_idx > 1 ) {

			$html .= '<div class="pull-left pagenating-link"><a href="' . $url . '?' . $url_query . '' . esc_attr( $page_name ) . '=1" title="' . esc_html__( 'Go to the first page', 'simpods-mvc' ) . '" class="first-page btn btn-default">&#60;&#60;</a>
			<a href="' . $url . '?' . $url_query . '' . esc_attr( $page_name ) . '=' . ( $page_idx - 1 ) . '" title="' . esc_html__( 'Go to the previous page', 'simpods-mvc' ) . '" class="prev-page btn btn-default"> &#60;</a></div>';

		}

		$inputs 		= parse_str( $url_query, $fields );
		$inputs_html 	= '';

		if ( count( $inputs ) > 0 ) {

			foreach ( $inputs as $key => $value ) {

				if ( is_array( $value ) ) {

					foreach ( $value as $sub_value ) {
						$inputs_html .= '<input name="' . esc_attr( $key ) . '[]" value="' . esc_attr( $sub_value ) . '" />';
					}

				} else {
					$inputs_html .= '<input name="' . esc_attr( $key ) . '" value="' . esc_attr( $value ) . '" />';
				}

			}

		}

		$html .= '<div class="pull-left">
						<div class="pagenate-form">
							<form action="' . $url . '?' . $url_query . '" method="get" >
								<div class="pull-left">
									<input class="paging_input mgl5 mgr5 nav-input" type = "text" value="' . $page_idx . '" name="' . esc_attr( $page_name ) . '" title="' . esc_html__( 'Current page', 'simpods-mvc' ) . '" class="current_page" />
								</div>
								<div class="pull-left">
									<div class="total_pages mgl5 mgr10 mgt5"> / ' . $pages_count . '</div>
								</div>
								<div class="pull-left">
									<input type="submit" value="Go" class="paging_go mgr5 btn btn-primary "/>
								</div>
								' . $inputs_html . '
							</form>
						</div>
					</div>';

		if (  ( $page_idx ) < $pages_count ) {
			$html .= '<div class="pull-left pagenating-link"> <a href="' . $url . '?' . $url_query . '' . esc_attr( $page_name ) . '=' . ( $page_idx + 1 ) . '" title="' . esc_html__( 'Go to the next page', 'simpods-mvc' ) . '" class="next-page btn btn-default"> &#62; </a>
			<a href="' . $url . '?' . $url_query . '' . esc_attr( $page_name ) . '=' . ( $pages_count ) . '" title="' . esc_html__( 'Go to the last page', 'simpods-mvc' ) . '" class="last-page btn btn-default">&#62;&#62;</a></div>';
		}

		$html .= '
			</div>';

		return $html;
	}

	/**
	 * Return a Pod settings with files links if there is a field for files. It has been used by components.
	 * @param  string $pod_slug a Pod slug
	 * @return array
	 */
	public function pods_settings( $pod_slug = '' ) {

		if ( function_exists( 'pods_query' ) ) {
			$settings = array();

			if ( $pod_slug == '' ) {
				return $settings;
			}

			$a_pod = pods( $pod_slug );

			if ( $a_pod ) {
				$fields = $a_pod->fields();

				foreach ( $fields as $key => $field_data ) {

					$settings[ $key ] = get_option( $pod_slug . '_' . $key, '' );

					if ( $field_data['type'] == 'file' ) {

						if ( is_array( $settings[ $key ] ) && count( $settings[ $key ] ) > 0 ) {
							$files = array();

							if ( $field_data['options']['file_type'] == 'images' ) {

								foreach ( $settings[ $key ] as $i => $id ) {
									$files[ $i ]['large']  = pods_image_url( $id, 'large' );
									$files[ $i ]['medium'] = pods_image_url( $id, 'medium' );
									$files[ $i ]['thumb']  = pods_image_url( $id, 'thumbnail' );
								}

							} else {

								foreach ( $settings[ $key ] as $i => $id ) {
									$files[ $i ] = wp_get_attachment_url( $id );
								}

							}

							$settings[ $key . '_files' ] = $files;
						}

					}

				}

			}

			return $settings;
		}

	}

}