<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods Pods Files Front class
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 03/05/2018
 */
class MVC_Front_Pods_Files extends MVC_Front_General {

	public $custom_dir = '';
	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor,
	 *  			  $can_init set to true will run automatically
	 */
	public function __construct( $can_init = true ) {

		if ( $can_init ) {

			//$this->actions();
			$this->filters();

		}

	}

	/**
	 * actions_fn: WP actions to hook when an instance of the class is created
	 */
	public function actions() {
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue' ) );
	}

	/**
	 * filters_fn: WP filters to hook when an instance of the class is created
	 */
	public function filters() {}

	/**
	 * enqueue_fn: Scripts and styles to enqueue when an instance of the class is created
	 */
	public function enqueue() {}

	public function set_custom_dir( $dir ) {
		$this->custom_dir = $dir;
	}

	/**
	* Creates files/directories and protects them
	*
	* @param string  $folder     The folder to change place items
	* @param string  $protected_target  How to protect the items
	* @return null
	*/
	public function create_files( $folder, $protected_target = '' ) {
		global $pagenow, $funs;
		// Install files and folders for uploading files and prevent hotlinking
		$upload_dir = wp_upload_dir();

		$files  = array(

			array(
				'base'    => $upload_dir['basedir'] . '/' . $folder,
				'file'    => '.htaccess',
				'content' => '',
			)		
		);

		$folders = explode( '/', $folder );
		// may not be array any more? Check.

		if ( 'no_external_access' == $protection ) { 
			$doamin 	= $funs->get_domain();
			$ip_server 	= $_SERVER['SERVER_ADDR']; 
			$files[0]['content'] = '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://www.' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://www.' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://www.' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://www.' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteRule ^ /index.php [NC,L]' . PHP_EOL . '</IfModule>';
		}
		if( 'patterned_code_in_url' == $protection ){
			$pattern_params = $simpods_global->get_simpods_patterned_code_for_url();
			if( ! empty( $pattern_params['pattern_data'] ) ){
				$pattern_option = $pattern_params['pattern_data'];
				if( ! empty( $pattern_option['pattern'] ) ){
					$files[0]['content'] = '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL . 'RewriteCond %{QUERY_STRING} !^code=' . $pattern_option['pattern'] . '$' .PHP_EOL . 'RewriteRule ^ /index.php [NC,L]' . PHP_EOL . '</IfModule>';
				}
			}
		}

		global $wp_filesystem;

		if ( ! $wp_filesystem->is_dir( $upload_dir['basedir'] . '/' . $folder ) ){
			$wp_filesystem->mkdir( $upload_dir['basedir'] . '/' . $folder );
		}

		foreach ( $files as $file ) {
			$base 		= str_replace( ABSPATH, $wp_filesystem->abspath(), $file['base'] );
			$file_path 	= trailingslashit( $base ) . $file['file'];

			if ( $wp_filesystem->exists( $base ) ){
				if ( $wp_filesystem->exists( $file_path ) ){
					$wp_filesystem->delete( $file_path );
				}
			} else {
				$wp_filesystem->mkdir( $base );
			}

			$wp_filesystem->put_contents( $file_path, $file['content'], FS_CHMOD_FILE );
		}
	}

	/**
	 * Uploads a file
	 *
	 * @param string $holder  The file input name
	 * @param integer $post_id Post ID
	 * @param string $can_create_thumbnail   Whether to create a thumbnail
	 * @param array $post_data Data for the attachment
	 * @param array $overrides    Overrides for wp_handle_upload
	 * @return integer The attachment ID
	 */
	public function insert_attachment( $holder, $post_id, $can_create_thumbnail = 'false', $post_data = array(), $overrides = array( 'test_form' => false ) ) {
		// check to make sure its a successful upload

		if ( $_FILES[$holder]['error'] !== UPLOAD_ERR_OK ) {
			__return_false();
		}

		require_once ABSPATH . "wp-admin" . '/includes/image.php';
		require_once ABSPATH . "wp-admin" . '/includes/file.php';
		require_once ABSPATH . "wp-admin" . '/includes/media.php';

		$attach_id = media_handle_upload( $holder, $post_id, $post_data, $overrides );

		if ( $can_create_thumbnail ) {
			update_post_meta( $post_id, '_thumbnail_id', $attach_id );
		}

		return $attach_id;
	}

	/**
	 * Change upload directory
	 *
	 * @param array $dirs Upload folder data array
	 * @return array New upload folder data array
	 */
	public function new_upload_dir( $dirs ) {
		// $dirs already you might want to use
		return array(
			'path'    => $dirs['basedir'] . '/' . $this->custom_dir,
			'url'     => $dirs['url'] . '/' . $this->custom_dir,
			'subdir'  => '/' . $this->custom_dir,
			'basedir' => $dirs['error'],
			'error'   => $dirs['error'],
		);
	}

	/**
	 * Used for media_handle_upload to overwrite files with the same name
	 *
	 * @param string $dirs  Directory array
	 * @param string $name File name
	 * @param string $ext_str  File extension
	 * @return string The new name
	 */
	// public function unique_name( $dirs, $name, $file_extension ) {
	// 	return $name;
	// }

}
