<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Pods class
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 26/03/2019
 */
class MVC_Front_Pods extends MVC_Front_General {

	const PAGE_SLUG  = '';
	const DB_TABLE    = '';	
	public $field_types = array('avatar','boolean','code','color', 'currency','date','datetime','email','file','html','number','paragraph','password','phone','pick','slug','taxonomy','text','time','website','wysiwyg','simpodsareafield', 'simpodsheaderfield', 'simpodspodsfile', 'simpodsworkflow');

	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor
	 */
	public function __construct( $can_init = true ) {
		// carry on if it is on the right section
		if ( $can_init ) {

			$this->actions();
			$this->filters();

		}

	}

	public function actions() {}

	public function filters() {
		add_filter( 'pods_admin_setup_edit_field_options', array( $this, 'extra_options' ), 10, 2 );
		add_filter( 'pods_meta_post_fields', array( $this, 'filter_fields' ), 10, 5 );
		add_filter( 'pods_meta_meta_post_fields', array( $this, 'filter_fields' ), 10, 2 );
		add_filter( 'pods_form_ui_field_pandarepeaterfield', array( $this, 'toggle_fields' ), 10, 6 );
		add_filter( 'pods_form_ui_field_simpodspodsfile', array( $this, 'toggle_fields' ), 10, 6 );

		$count 	=  count( $this->field_types );
		for ( $i = 0; $i < $count; $i++ ) {
			add_filter( 'pods_form_ui_field_' . $this->field_types[$i], array( $this, 'toggle_fields' ), 10, 6 );
		}

	}

	public function enqueue() {}


	/**
	 * Toggle the field based on the Simpods Conditions set for the field
	 *
	 * @param string  $output  The HTML output
	 * @param string  $name    The name of the pods field
	 * @param string  $value   The value of the pods field
	 * @param array   $options Options for the pods field
	 * @param object  $pod     The pods object
	 * @param integer $id      The post ID
	 * @return void
	 */
	public function toggle_fields( $output, $name, $value, $options, $pod, $id ) {

		$field_name 	= substr( $name, strlen( 'pods_field_' ) );

		$options = array();
		if ( isset( $pod->fields ) && is_array( $pod->fields ) ) {
			foreach ( $pod->fields as $key => $values ) {
				if ( $key == $field_name ) {
					// get the conditions
					$options = $values['options'] ;
					break;
				}
			}
		} else {
			return $output;
		}

		foreach ( $options as $key => $option_data ) {

			if ( 'simpods_condition' == $key ) {

				$output .= '<script>jQuery(document).ready( function($) { ';

				if ( trim( $option_data ) ) {
					$name = str_replace( '_', '-', str_replace( 'pods_field_', 'pods-form-ui-row-name-', $name ) );
					$values    = explode( PHP_EOL, $option_data );

					foreach ( $values as $value ) {
						$targets 	= explode( '|', $value );
						$count 		= count( $targets );
						if ( $count >= 2 ) {

							$picked_field = pods_field( $pod->pod, $id, $targets[0] );
							if( '' == $picked_field && isset( $pod->fields['test_user_only']['options']['default_value'] ) ){ // use default value if no values saved
								$picked_field	=	$pod->fields['test_user_only']['options']['default_value'];
							}
							$target_id 	= '#pods-form-ui-pods-field-' . str_replace( '_', '-', esc_attr( $targets[0] ) );
							$conditions = '';
							$match_js  	= '';
							for ( $i = 1; $i < $count; $i++ ) {
								$conditions .= '$("' . $target_id . '").val() == "' . $targets[$i] . '"';
								$match_js .= '"' . $picked_field . '" == "' . $targets[$i] . '"';
								if ( $i < $count - 1 ) {
									$conditions .= ' || ';
									$match_js .= ' || ';
								}
							}
							$toggle_js	=	'if( ' . $conditions . ' ){
												$( ".' . $name . '" ).removeClass("hidden");
											} else {
												$( ".' . $name . '" ).addClass("hidden");
											};';

							$output .= 'if( ' . $match_js . ' ){							
											$( ".' . $name . '" ).removeClass("hidden");
										} else {											
											$( ".' . $name . '" ).addClass("hidden");
										};
										var toggled_bln = false;
										$( "' . $target_id . '" ).on("change",function(){		// for other versions 									
											' . $toggle_js . '
											toggled_bln = true;
										});	
										if( toggled_bln == false ){ // if on doesn\'t work, try live

											//$( "' . $target_id . '" ).live("change",function(){	// for jQuery 1.3	
											$(document.body).on("change",  "' . $target_id . '", function( ){												
												' . $toggle_js . '
											});							
										}						
										';
						}

					}

				}

				$output .= '});</script>';

			}

		}

		return $output;
	}

}
