<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods Global Shortcodes class
 *
 * @version: 1.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 02/04/2019
 */
class MVC_Front_Simpods_Global_Shortcodes extends MVC_Front_General {

	/**
	 * Constructor
	 * @param Boolean $can_init toggle to run the functions in the constructor,
	 *  			  $can_init set to true will run automatically
	 */
	public function __construct( $can_init = true ) {

		if ( $can_init ) {

			$this->actions();
			$this->filters();
			
		}
		$this->add_shortcodes();
	}

	/**
	 * actions_fn: WP actions to hook when an instance of the class is created
	 */
	public function actions() {
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue' ) );
	}

	/**
	 * filters_fn: WP filters to hook when an instance of the class is created
	 */
	public function filters() {}

	/**
	 * enqueue_fn: Scripts and styles to enqueue when an instance of the class is created
	 * Simpods will auto include js and css files with the same template name
	 */
	public function enqueue() {}


	/**
	 * add_shortcodes_fn: add shortcodse 
	 */
	public function add_shortcodes(){
					
		add_shortcode( 'simpods-blog-fields', array( $this, 'simpods_blog_fields' ) );
		add_shortcode( 'simpods-page-link', array( $this, 'simpods_page_link' ) );
		add_shortcode( 'simpods-year', array( $this, 'simpods_year' ) );
	}

	 /**
	  * get a field value from a table
	  * @param array $attributes = array(
										'autoprefix' => 1, // auto adding table prefix to the table, so this function can be used to any table, 1 for base_prefix, 2 for current blog
										'table' 	 => '', a table in the database
										'field' 	 => '', a field in the table
										'where'		 => '', where to get replace current_user | current_page with ID
										'default' 	 => '', default value for the field
										'output'  	 => 'value', value|checked|selected|disabled|date	 
										'sql'		 => 0, 	show sql				
	  								  )
	  */
	 public function simpods_blog_fields( $attributes ){
		 
		global $wpdb, $current_user;
		$return = '';
		$attributes   = shortcode_atts( array(
			'autoprefix' => 1,
			'table' 	 => '',
			'field' 	 => '',
			'where'		 => 0,
			'order'		 => '',
			'default' 	 => '',
			'output'  	 => 'value',
			'sql'		 => 0,
		), $attributes );
		if( $attributes['autoprefix'] == 1 ){
			$attributes['table'] = $wpdb->base_prefix . $attributes['table']; 	
		}
		if( $attributes['autoprefix'] == 2 ){
			$attributes['table'] = $wpdb->prefix . $attributes['table']; 	
		}		
		$params 	 = array();
		$is_for_role = false;
		if( trim( $attributes['where'] ) != '' ){			
			$user_id = is_user_logged_in()? $current_user->ID : 0;
			$attributes['where']  = str_replace( array( 'current_user', 'current_page', '&#038;', '&amp;' ), array( $user_id, MVC_Front_Factory::$current_page_id, '&', '&' ), $attributes['where'] );
			parse_str( $attributes['where'], $wheres  );
			
			$where = '';
			if( is_array( $wheres ) ) {
				foreach( $wheres as $key => $value ){
					// exclude
					$not = strpos( $key, '_exc_' ) === 0? 'NOT ' : '';
					if( $not != '' ){			
						$key = substr( $key, 6 );
					}
						
					$prefix = strpos( $key, '_find_' ) === 0? '%%' : '';	
					if( $prefix != '' ){			
						$key  = substr( $key, 6 );			
					}					
					// if it is looking up the role
					
					if( substr( $attributes['table'], -8 ) == 'usermeta' && $key == 'meta_key' && $value == 'role' ){
						$value 		= $wpdb->prefix . 'capabilities'; 	
						$is_for_role	= true;					
					}
					
					if( is_numeric( $value ) ){
						
						$where .= ' AND ' . $key . ' ' . $not . 'LIKE "' . $prefix . '%d' . $prefix . '"';
						array_push( $params, $value );					
						
					} else if( is_string( $value ) ){
						
						$where .= ' AND ' . $key . ' ' . $not . 'LIKE "' . $prefix . '%s' . $prefix . '"';
						array_push( $params, $value );					
					}				
					
					if( is_array( $value ) && count( $value ) > 0 ){
						//$not 	= $not == '!'? 'NOT ' : '';
						$values 	= $this->escape( $value );				
						$values_imploded     = '("' . implode( '","', $values ) . '")';
						$where .= ' AND ' . $key . ' ' . $not . 'IN '. $values_imploded . '';										
					}							
				}	
			}			
			$attributes['where'] = ' WHERE 1= 1 ' . $where; 	
			
		}	
		$order	=	'';
		if( trim( $attributes['order'] ) != '' ){			
			
			$attributes['order'] 	= esc_sql( $attributes['order']  );
			$orders 		= explode(',', $attributes['order'] );
			if( is_array( $orders ) ) {
				foreach( $orders as $order_query ){
					if( trim( $order_query ) != '' ){
						$order_query_exploded = explode( ' ', $order_query );

						$order	.=	'`' . trim( $order_query_exploded[0] ) . '` ';
						
						if( isset( $order_query_exploded[1] ) && in_array( strtolower( trim( $order_query_exploded[1] ) ), array('asc', 'desc') ) ){
							$order .= $order_query_exploded[1] . ', '; 
						} else{
							$order .= 'ASC, '; 
						}
						
					}
				}
			}
			$order	= 'ORDER BY ' . rtrim( $order, ', ' );
		}

		if( count( $params ) > 0 ){
			$query  = $wpdb->prepare( 'SELECT `' . esc_sql( $attributes['field'] ) . '` FROM `' . $attributes['table'] . '` ' . $attributes['where'] . ' ' . $order . ' LIMIT 0, 1', $params );
		} else {
			$query = 'SELECT `' . esc_sql( $attributes['field'] ) . '` FROM `' . $attributes['table'] . '` ' . $attributes['where'] . ' ' . $order . ' LIMIT 0, 1' ;
		}		
		if( $attributes['sql'] == 1 ){
			echo $query;
		}
		//return $query;
		$items = $wpdb->get_results( $query , ARRAY_A );
		
		if( count( $items ) > 0 ){
				
			//print_r( $items );	
			if( $is_for_role ){
				$roles = maybe_unserialize(  $items[0][ $attributes['field'] ] );	
				foreach( $roles as $key => $role ){
					$return = $key;
					break;	
				}
			}		
			if( trim( $items[0][ $attributes['field'] ] ) == '' ){
				$return = esc_attr( $attributes['default'] );	
			} else {
				$return = esc_attr( $items[0][ $attributes['field'] ] );					
			}
			// for checkbox and radio
			if( $attributes['output'] == 'checked' ){
				if( $attributes['default'] == $items[0][ $attributes['field'] ] ){
					$return = 'checked="checked"';
				}
			}
			if( $attributes['output'] == 'selected' ){
				if( $attributes['default'] == $items[0][ $attributes['field'] ] ){
					$return = 'selected="selected"';
				}
			}
			// if a value saved, send back disabled
			if( $attributes['output'] == 'disabled' ){
				if( count( $items ) > 0 && $items[0][ $attributes['field'] ] != '' ){
					$return = 'disabled="disabled"';
				}
			}	
			if( $attributes['output'] == 'date' ){
				if( count( $items ) > 0 && $items[0][ $attributes['field'] ] != '' ){
					if( $items[0][ $attributes['field'] ] != '0000-00-00' ){
						$return = date( 'd-m-Y', strtotime( $items[0][ $attributes['field'] ] ) );
					} else {
						$return = '';	
					}
				}
			}	
		} else {
			$return = esc_attr( $attributes['default'] );	
		}
		
		return $return ;			 
	 }
 	 /**
	  * get page by page id, wp post types only
	  * @param array $attributes = array(
										'id' 		 => 1, // page id  	
										'html' 		 => 0 // naked link not in html				
										'title'		 => ''
	  								  )
	  */
	 public function simpods_page_link( $attributes ){
		$return = '';
		$attributes   = shortcode_atts( array(
			'id' 		 => 1, // page id  	
			'html' 		 => 0, // naked link not in html				
			'title'		 => 'link',
		), $attributes );		 
		if( !is_numeric( $attributes['id'] ) ){
			return '';	
		}
		$link_html = get_permalink( $attributes['id'] );
		if( $attributes['html'] ){
			$link_html = '<a href="' . esc_url( $link_html ) . '">' . esc_html( $attributes['title'] ) . '</a>';				
		}
		return $link_html;
		
	 }	

	 /**
      * Return the current year
      *
      */
	 public function simpods_year(){
	 	return '<span class="simpods-year">' . date('Y') . '</span>';
	 }
}
