<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods Hierarchy Front class
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 14/03/2016
 */
class MVC_Front_Simpods_Hierarchy extends MVC_Front_General {

	public function __construct( $can_init = true ) {

		// carry on if it is on the right section

		if ( $can_init ) {
			$this->actions();
			$this->filters();
		}

	}

	public function actions() {}

	public function filters() {}

	public function enqueue() {}

	/**
	 * Save or update the Hierarchy data for normal post type
	 *
	 * @param array   $attributes      Attributes for what to update
	 * @param boolean $can_echo_query Echo the query
	 * @return void
	 */
	public function simpods_cats_save( $attributes, $can_echo_query = false ) {
		global $wpdb, $current_user, $funs;

		$defaults = array(
			'simpods_category' => array(), //array( parent table id => parent table item id)
			'post_id'          => 0,
			'target_tb'        => '',
			'author_id'        => $current_user->ID,
		);

		// Parse incoming $args into an array and merge it with $defaults
		$attributes = wp_parse_args( $attributes, $defaults );

		if ( !is_numeric( $attributes['post_id'] ) || !is_numeric( $attributes['author_id'] ) ) {
			return '';
		}

		$attributes['target_tb']    = esc_sql( $attributes['target_tb'] );
		$wheres                     = array( 'id' => $attributes['post_id'] );
		$parent_categories          = array();
		$now                        = date( 'Y-m-d h:i:s' );
		$data                       = array();
		$data['sp_modified']        = $now;
		$data['sp_modified_author'] = $attributes['author_id'];

		$items = $funs->simpods_select( $wheres, array( 'target_tb' => $attributes['target_tb'], 'limit' => 1 ) );

		if ( isset( $items[0] ) ) {

			$parent_categories = maybe_unserialize( $items[0]['sp_categories'] );

			if ( !is_array( $parent_categories ) ) {
				$parent_categories = array();
			}

			$removed = false;

			foreach ( $attributes['simpods_category'] as $key => $id ) {

				$key_exploded = explode( '_', $key );

				// only remove the items from the current parent, but the others. In case some tables changed parents and want to change back, so the data are still there

				if ( !$removed ) {

					foreach ( $parent_categories as $parent_key => $parent_id ) {
						$parent_key_exploded = explode( '.', $parent_id );

						if ( $parent_key_exploded[0] == $key_exploded[0] ) {
							unset( $parent_categories[$parent_key] );
						}

					}

					$removed = true;
				}

				if ( $id != 0 ) {
					array_push( $parent_categories, intval( $key_exploded[0] ) . '.' . intval( $id ) );
				}

			}

			$parent_categories = array_values( array_unique( $parent_categories ) );

			$data['sp_categories'] = maybe_serialize( $parent_categories );

			if ( empty( $items[0]['sp_author'] ) ) {
				$data['sp_author'] = $current_user->ID;
			}

			if ( $items[0]['sp_created'] == '0000-00-00 00:00:00' ) {
				$data['sp_created'] = $now;
			}

			return $funs->general_update( $wpdb->prefix . $attributes['target_tb'], $data, $wheres );

		} else {
			$data['sp_author']  = $attributes['author_id'];
			$data['sp_created'] = $now;

			foreach ( $attributes['simpods_category'] as $key => $id ) {
				$key_exploded = explode( '_', $key );

				if ( $id != 0 ) {
					array_push( $parent_categories, intval( $key_exploded[0] ) . '.' . intval( $id ) );
				}

			}

			$data['sp_categories'] = maybe_serialize( $parent_categories );
			$data['id']            = $attributes['post_id'];
			return $funs->general_insert( $wpdb->prefix . $attributes['target_tb'], $data );
		}

	}

}
