<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Simpods User Code Class
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 11/04/2016
 */
class MVC_Front_Simpods_User_Code extends MVC_Front_General {

	const DB_TABLE   = 'simpods_user_code';	

	public function __construct( $can_init = true ) {

		// carry on if it is on the right section

		if ( $can_init ) {
			$this->actions();
			$this->filters();
		}

	}

	/**
	 * actions_fn wp actions to hook when an instance of the class is created
	 */
	public function actions() {
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue' ) );
	}

	/**
	 * filters_fn wp filters to hook when an instance of the class is created
	 */
	public function filters() {}

	/**
	 * enqueue_fn scripts and styles to enqueue when an instance of the class is created
	 */
	public function enqueue() {}

	/**
	 * Compatability shim to convert parameters to new standard
	 *
	 * @param integer $user_id The user ID
	 * @param string  $code   The user code
	 * @return null
	 */
	public function assign_sp_code( $user_id, $code ){
		$this->assign_user_to_code( array(
			'user_id' => $user_id,
			'code'    => $code
		) );
	}

	/**
	 * Assign the simpods user code to each user in the pods user table
	 * @internal
	 * @todo Merge with assign_user_to_code_fn
	 *
	 * @param integer $user_id The user ID
	 * @param string  $code   The user code
	 * @return null
	 */
	public function assign_code_to_user( $user_id, $code ) {
		global $wpdb, $funs;
		$table = $wpdb->prefix . 'pods_user';

		// check if table exists
		$is_table = $wpdb->query( 'SHOW TABLES LIKE "' . $table . '"' );

		if ( $is_table ) {

			$fields  	= $funs->get_fields( $table );
			$successful = false;

			if ( is_array( $fields ) && count( $fields ) > 0 ) {

				foreach ( $fields as $field_data ) {

					if ( $field_data['Field'] == 'spuc_code' ) {
						$successful = true;
					}

				}

				// if the field spuc_code doesn't exist

				if ( !$successful ) {
					$inserted = $wpdb->query( 'ALTER TABLE `' . $table . '` ADD `spuc_code` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL AFTER `' . $fields[count( $fields ) - 1]['Field'] . '`' );

					if ( $inserted ) {
						$successful = true;
					}

				}
				// when the field spuc_code exists
				if ( $successful ) {
					$code_id  	= 0;
					$codes 	= $this->get_code_details( $code );
					if( ! empty( $codes ) ){
						$code_id 	= $codes[0]['id']; 
						$code_ids   = array();
						$codes_imploded 	= '';
						// Find existing codes
						$existing_codes = $wpdb->get_col( $wpdb->prepare( 'SELECT `spuc_code` FROM `' . $wpdb->prefix . 'pods_user' . '` WHERE `id` = %d ', array( $user_id ) ), 0 );
						if ( ! empty( $existing_codes ) && ! empty( $existing_codes[0] ) ){
							$code_ids  = Arrays::string_to_array( $existing_codes[0] ); // explode( ',', $existing_codes[0] );
							if ( $code_id && in_array( $code_id, $code_ids ) ){
								// User already has this code, no change required
								return true;
							} 
						} 
						$code_ids[] = $code_id;
						$codes_imploded 	= implode( ',', $code_ids );

						if( ! empty( $codes_imploded ) ){
							$wpdb->query( $wpdb->prepare( 'INSERT IGNORE INTO `' . $wpdb->prefix . 'pods_user' . '` SET `id` = %d;', array( $user_id ) ) );
							$wpdb->query( $wpdb->prepare( 'UPDATE `' . $wpdb->prefix . 'pods_user' . '` SET `spuc_code` = "%s" WHERE `id` = %d;', array( $codes_imploded, $user_id ) ) );
							return true;
						}
					}
				}

			}

		}
		return false;

	}

	/**
	 * Assgin a user to a code
	 *
	 * @param  array $arrs_arr Code and user id
	 * @return boolean
	 */
	public function assign_user_to_code( $attributes = array() ) {
		global $wpdb, $current_user, $funs;

		$defaults = array(
			'code'    => '',
			'user_id' => 0,
		);
		$attributes         = wp_parse_args( $attributes, $defaults );
		$successful      = 0;
		$attributes['code'] = trim( $attributes['code'] );

		if ( $attributes['code'] !== '' && is_int( $attributes['user_id'] ) && $attributes['user_id'] != 0 ) {
			//$codes = $funs->simpods_select( array( 'spuc_code' => esc_sql( $attributes['code'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );
			$codes 	= $this->get_code_details( $attributes['code'] );
			if ( count( $codes ) > 0 ) {
				$codes[0]['spuc_user_ids'] = trim( trim( $codes[0]['spuc_user_ids'] ), ',' );
				$ids                       = array();

				if ( $codes[0]['spuc_user_ids'] != '' ) {
					$ids = maybe_unserialize( $codes[0]['spuc_user_ids'] ); // old

					if ( !is_array( $ids ) ) {
						$ids = explode( ',', $codes[0]['spuc_user_ids'] ); // new
					}

				}

				array_push( $ids, $attributes['user_id'] );

				$ids     = array_map( 'trim', $ids );
				$ids     = array_unique( $ids );
				$successful = $wpdb->query( $wpdb->prepare( 'UPDATE `' . $wpdb->prefix . self::DB_TABLE . '` SET `spuc_user_ids` = "%s" WHERE `id` ="%d";', array( implode( ',', $ids ), $codes[0]['id'] ) ) );

				// Also save code to the user
				$this->assign_code_to_user( $attributes['user_id'], $attributes['code'] );
			}

		}

		return $successful;
	}

	/**
	 * Get the assigned code for a user
	 *
	 * @param array $attributes The User ID
	 * @return array User codes
	 */
	public function assigned_code( $attributes = array() ) {
		global $wpdb, $current_user, $funs;

		$defaults = array(
			'user_id' => 0,
		);
		$attributes = wp_parse_args( $attributes, $defaults );

		if ( $attributes['user_id'] == 0 && is_user_logged_in() ) {
			$attributes['user_id'] = $current_user->ID;
		} else {
			return false;
		}

		if ( !is_numeric( $attributes['user_id'] ) ) {
			return false;
		}

		$fields = array(
			//'_find_spuc_user_ids'	=> 'i:' . $attributes['user_id'] . ';',
		);

		$attrs = array(
			'target_tb' => self::DB_TABLE,
			'search'    => ' AND FIND_IN_SET("' . intval( $attributes['user_id'] ) . '" , `spuc_user_ids` ) ',
		);

		$codes = $funs->simpods_select( $fields, $attrs );

		return $codes;
	}

	/**
	 * Check if a code is valid
	 *
	 * @param array $attributes Array to local the code
	 * @return array Status of the code
	 */
	public function check_code( $attributes = array() ) {
		global $wpdb, $current_user, $funs;

		$defaults = array(
			'code' => '',
		);
		$attributes = wp_parse_args( $attributes, $defaults );

		$returns = array(
			'exist'    => false,
			'full'     => false,
			'inactive' => false,
			'expired'  => false,
			'can_use'  => false,
			'details'  => array(),
		);
		$attributes['code'] = trim( $attributes['code'] );

		if ( $attributes['code'] !== '' ) {
			//$codes = $funs->simpods_select( array( 'spuc_code' => esc_sql( $attributes['code'] ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );
			$codes 	= $this->get_code_details( $attributes['code'] );
			if ( count( $codes ) > 0 ) {
				$ids                      = array();
				$returns['exist']          = true;
				$codes[0]['spuc_user_ids'] = trim( trim( $codes[0]['spuc_user_ids'] ), ',' );

				if ( $codes[0]['spuc_user_ids'] != '' ) {
					$ids = maybe_unserialize( $codes[0]['spuc_user_ids'] ); // old

					if ( !is_array( $ids ) ) {
						$ids = explode( ',', $codes[0]['spuc_user_ids'] ); // new
					}

				}

				if ( count( $ids ) >= $codes[0]['spuc_amount'] ) {
					$returns['full'] = true;
				}

				if ( $codes[0]['spuc_active'] !== 1 ) {
					$returns['inactive'] = true;
				}

				if ( isset( $codes[0]['spuc_expiry_date'] ) && $codes[0]['spuc_expiry_date'] != '0000-00-00' ) {

					if ( strtotime( $codes[0]['spuc_expiry_date'] ) < time() ) {
						$returns['expired'] = true;
					}

				}

				if ( !$returns['full'] && $returns['inactive'] && !$returns['expired'] ) {
					$returns['can_use'] = true;
				}

				$returns['details'] = $codes[0];
			}

		}

		return $returns;

	}
	/**
	 * Get the details of a code
	 *
	 * @param string $code
	 * @return array $code_details 
	 */
	public function get_code_details( $code ){
		global $funs;
		$code_details = $funs->simpods_select( array( 'spuc_code' => esc_sql( $code ) ), array( 'target_tb' => self::DB_TABLE, 'limit' => 1 ) );
		return $code_details;
	}
}
