<?php
/** 
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 20/05/2015
 */
/**
 * Include all files in a directory and subdirectories
 * @param  string $parent_dir  The directory in the parent theme
 * @param  string $child_dir   The directory in the child theme
 * @return null
 */
function simpods_include_directory_files( $parent_dir, $child_dir ) {
	global $wp_filesystem;
	$parent_dir = trailingslashit( $parent_dir );
	if ( isset( $_SERVER['HTTP_REFERER'] ) ) {
		// load cross site ajax
		$page_ids 	= simpods_referer_page_id();
		$ajax_path 	= '';
		if ( isset( $_POST['ajax_path'] ) ) {
			$ajax_path  = $_POST['ajax_path'];
		}
		if ( isset( $_GET['ajax_path'] ) ){
			$ajax_path  = $_GET['ajax_path'];
		}
		if( ! empty( $ajax_path ) ){
			$ajax_path  = $_POST['ajax_path'];
			$parent_url = $parent_dir . $ajax_path;
			$child_url  = $child_dir . $ajax_path;
			if ( $wp_filesystem->exists( $child_url ) && !$wp_filesystem->is_dir( $child_url ) ) {
				include_once $child_url;
			} elseif ( $wp_filesystem->exists( $parent_url ) && !$wp_filesystem->is_dir( $parent_url ) ) {
				include_once $parent_url;
			}
		} else {
			simpods_load_template_ajax( $parent_dir, $child_dir, $page_ids );
		}
	}
}
/**
 * Assign current page id to $current_page_id
 * @return array
 */
function simpods_referer_page_id() {
	$return_ids = array(
		'page_id' => 0,
	);
	$referer_url = wp_get_referer();
	$post_id     = url_to_postid( $referer_url );
	if ( $post_id ) {
		$return_ids['page_id'] = $post_id;
		return $return_ids;
	}
	// when it is custom post type and others
	$return_ids = array(
		'page_id'   => 0,
		'author_id' => 0,
		'tax_id'    => 0,
		'wp'        => false,
	);
	$current_page_id   = 0;
	$current_author_id = 0;
	$current_tag_id    = 0;
	$site_url          = str_replace( array( 'http://' . $_SERVER['SERVER_NAME'], 'https://' . $_SERVER['SERVER_NAME'] ), '', site_url() );
	$page_slug = str_replace( $site_url, '', $_SERVER['HTTP_REFERER'] );
	$page_slug = str_replace( array( 'http://' . $_SERVER['SERVER_NAME'], 'https://' . $_SERVER['SERVER_NAME'] ), '', $page_slug );
	// remove from #comment-page-
	if ( strpos( $page_slug, '/page/' ) !== false ) {
		$page_slug = substr( $page_slug, 0, strpos( $page_slug, '/page/' ) ) . '/';
	}
	// remove from #comment-page-
	if ( strpos( $page_slug, '/comment-page-' ) !== false ) {
		$page_slug = substr( $page_slug, 0, strpos( $page_slug, 'comment-page-' ) ) . '/';
	}
	$url_parts = explode( '?', $page_slug );
	// remove first slash
	if ( substr( $url_parts[0], 0, 1 ) == '/' ) {
		$url_parts[0] = substr( $url_parts[0], 1, -1 );
	}
	// remove last slash
	if ( substr( $url_parts[0], -1, 1 ) == '/' ) {
		$url_parts[0] = substr( $url_parts[0], 0, -1 );
	}
	$slug_parts = explode( '/', $url_parts[0] );
	// if not the home page
	$page_info = false;
	if ( $slug_parts[count( $slug_parts ) - 1] != '' ) {
		// construct page path
		$page_path = implode( '/', $slug_parts );
		$page_info = get_page_by_path( $page_path );
		if ( $page_info ) {
			$current_page_id = $page_info->ID;
		} else {
			$post_types = get_post_types();
			$type_found = false;
			foreach ( $post_types as $post_type ) {
				$page_info = get_page_by_path( $slug_parts[count( $slug_parts ) - 1], OBJECT, $post_type );
				if ( $page_info ) {
					//if the wordpress link is the same as the url, it is the right one
					if ( isset( $_SERVER['REQUEST_SCHEME'] ) && get_permalink( $page_info->ID ) == $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"] ) {
						$current_page_id = $page_info->ID;
						$type_found     = true;
						break;
					}
				}
			}
			//author
			if ( count( $slug_parts ) == 2 && $slug_parts[0] == 'author' ) {
				$user_info = get_user_by( 'slug', $slug_parts[1] );
				if ( $user_info ) {
					$current_author_id = $user_info->ID;
				}
			}
			// if not found, try to use the last second one as tag
			if ( !$type_found && count( $slug_parts ) == 2 ) {
				$term = '';
				if ( $slug_parts[count( $slug_parts ) - 2] == 'tag' ) {
					$term = 'post_tag';
				} else if ( $slug_parts[count( $slug_parts ) - 2] == 'category' ) {
					$term = 'category';
				} else if ( $slug_parts[count( $slug_parts ) - 2] == 'link_category' ) {
					$term = 'link_category';
				} else {
					$term = 'nav_menu';
				}
				$page_info = get_term_by( 'slug', $slug_parts[count( $slug_parts ) - 1], $term );
				if ( $page_info ) {
					$type_found     = true;
					$current_tag_id = $page_info->term_id;
				}
			}
			// if not found, try to use the last second one as the post type
			if ( !$type_found && count( $slug_parts ) >= 2 ) {
				$page_info = get_page_by_path( $slug_parts[count( $slug_parts ) - 1], OBJECT, $slug_parts[count( $slug_parts ) - 2] );
				if ( $page_info ) {
					$type_found     = true;
					$current_page_id = $page_info->ID;
				}
			}
		}
	}
	if ( $slug_parts[0] == '' && !is_404() ) {
		$current_page_id = get_option( 'page_on_front' );
	}
	$return_ids = array(
		'page_id'   => $current_page_id,
		'author_id' => $current_author_id,
		'tag_id'    => $current_tag_id,
	);
	return $return_ids;
}
/**
 * load Ajax file based on which template calls
 * @param  string $parent_dir      parent theme path
 * @param  string $child_dir child theme path
 * @param  array  $ids      ids
 * @return null
 */
function simpods_load_template_ajax( $parent_dir, $child_dir, $ids ) {
    global $wpdb, $funs, $current_template, $wp_filesystem;
    $current_blog_id = get_current_blog_id();
    $current_page_id = $ids['page_id'];
    $template_slug   = '';
    if ( !headers_sent() ) {
        if ( session_status() == PHP_SESSION_NONE ) {
            @session_start();
        }
    } 
    if ( $current_page_id ) {
        $template_slug = get_page_template_slug( $current_page_id );
        if ( $template_slug == '' ) { // when it is failed to work, for example custom post types
            $meta_items = $wpdb->get_results( $wpdb->prepare( 'SELECT `meta_value` FROM `' . $wpdb->prefix . 'postmeta' . '` WHERE `post_id` = "%d" AND `meta_key` = "_wp_page_template" LIMIT 0, 1', array( $current_page_id ) ), ARRAY_A );
            if ( $meta_items ) {
                if ( isset( $meta_items[0]['meta_value'] ) ) {
                    $template_slug = $meta_items[0]['meta_value'];
                    if ( $meta_items[0]['meta_value'] == 'default' ) {
                        $template_slug = 'page.php';
                    }
                }
            }
            if ( $template_slug == '' ) {
                $post_type = get_post_type( $current_page_id );
                if ( $post_type ) {
                    $template_slug = 'single-' . $post_type . '.php';
                }
            }
        }
    } else if ( isset( $_SESSION['current_template'] ) && !empty( $_SESSION['current_template'] ) ) {
        // if another page open, this may not work
        $template_slug = $_SESSION['current_template'];
    } else {
        // ajax will come to this as well
        $taxonomy = simpods_get_taxonomy();
        if ( $taxonomy ) {
            $template_slug = $taxonomy;
            if ( $template_slug == 'taxonomy-category.php' ) {
                $template_slug = 'category.php';
            }
        } else {
            // if not post type or page. load the a page with the same php name
            preg_match( '/([^\/]+)\.php/i', $_SERVER['PHP_SELF'], $url_match );
            if ( isset( $url_match[1] ) && trim( $url_match[1] ) != '' ) {
                $template_slug = esc_attr( $url_match[1] ) . '.php';
            } else {
                $template_slug = 'archive.php';
            }
        }
        if ( is_multisite() ) {
            $template_slug = '' . get_current_blog_id() . '/' . $template_slug;
        }
    }
    session_write_close();
    $template_slug 		= apply_filters( 'simpods_load_template_ajax', $template_slug );

    $file          = '';
    // themes first
    $parent_url    = trailingslashit( $parent_dir ) . $template_slug;
    $child_url     = trailingslashit( $child_dir ) . $template_slug;
    if ( $wp_filesystem->exists( $child_url ) ) {
        $file = $child_url;
    } elseif ( $wp_filesystem->exists( $parent_url ) ) {
        $file = $parent_url;
    } else {
        $plugin = get_option( 'simpods_' . $template_slug, '' ); // Backward compatibility
        if ( $plugin != '' && $wp_filesystem->exists( $plugin ) ) {
            $file = $plugin;
        }
    }       

    if ( $wp_filesystem->exists( $file ) ) {
        include_once $file;
        simpods_initialise_ajax_class( $template_slug );
    }
	simpods_load_plugin_ajaxes( $template_slug );
    // template ajax from simpods plugins
 //    $component_plugins 	= SIMPODS_COMPONENT_PLUGINS;
	// if ( is_array( $component_plugins ) ) {
	// 	require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
	// 	foreach ( $component_plugins as $plugin_name => $v_arr ) {
	// 		if( is_plugin_active( $plugin_name . '/' . $plugin_name . '.php' ) ){ 
	// 			// Load the same page controllers from plugins, namespace required
	// 			//$full_file = ABSPATH . 'wp-content/plugins/' . $plugin_name . '/mvc/ajax/ctrl_mvc_front_' . rtrim( rtrim( $template_slug, '.class.php' ), '.php' ) . '.class.php';
	// 			$file      = ABSPATH . 'wp-content/plugins/' . $plugin_name . '/mvc/ajax/' . $template_slug ;
	// 			// if ( ( $wp_filesystem->exists( $full_file ) && !$wp_filesystem->is_dir( $full_file ) ) && ( !$wp_filesystem->exists( $file ) || $wp_filesystem->is_dir( $file ) ) ) { // if the file with the short name not there but the full name one exists, use the full name one			
	// 			// 	$file 		= $full_file;
	// 			// }	

	// 			if ( $wp_filesystem->exists( $file ) && $template_slug != '' && !$wp_filesystem->is_dir( $file ) ) {					
	// 				include_once $file;	
	// 				simpods_initialise_ajax_class( $template_slug, $plugin_name );
	// 			}	
				
	// 		}	
	// 	}	
	// }

}

/**
 * Return taxonomy template file names
 * @param  array  $args [description]
 * @return array
 */
function simpods_get_taxonomy( $args = array() ) {
	global $wpdb;
	$args =  wp_parse_args( $args, array(
		'template' => true,
	) );
	$page_url = $_SERVER['REQUEST_URI'];
	if ( strpos( $_SERVER['REQUEST_URI'], '?' ) !== false ) {
		$page_url = substr( $page_url, 0, strpos( $_SERVER['REQUEST_URI'], '?' ) );
	}
	if ( strpos( $page_url, '#' ) !== false ) {
		$page_url = substr( $page_url, 0, strpos( $page_url, '#' ) );
	}
	$page_url_parts = explode( '/', $page_url );
	if ( $page_url_parts[count( $page_url_parts ) - 1] == '' ) {
		$page_slug = $page_url_parts[count( $page_url_parts ) - 2];
		$tax_index = 3;
	} else {
		$page_slug = $page_url_parts[count( $page_url_parts ) - 1];
		$tax_index = 2;
	}
	$taxonomy = $page_url_parts[count( $page_url_parts ) - $tax_index];
	// pods use underscore by default instead of dash
	$pods_taxonomy = str_replace( '-', '_', $page_url_parts[count( $page_url_parts ) - $tax_index] );
	$db_query = $wpdb->prepare( 'SELECT tt_tb.`taxonomy` AS `taxonomy`, t_tb.`term_id` AS `term_id` FROM `' . $wpdb->prefix . 'terms` AS t_tb INNER JOIN `' . $wpdb->prefix . 'term_taxonomy` AS tt_tb ON tt_tb.`term_id` = t_tb.`term_id` WHERE t_tb.`slug` = "%s" AND tt_tb.`taxonomy` in ( "%s", "%s" ) LIMIT 0, 1;', array( $page_slug, $pods_taxonomy, $taxonomy ) );
	$db_results = $wpdb->get_results( $db_query, ARRAY_A );
	if ( $args['template'] ) {
		return $db_results ? 'taxonomy-' . $db_results[0]['taxonomy'] . '.php' : false;
	} else {
		return $db_results ? $db_results[0] : false;
	}
}
function inc_php_in_dirs( $parent_dir, $child_dir ) {	
	simpods_include_directory_files( $parent_dir, $child_dir );
}
function referer_page_id() {	
	return simpods_referer_page_id();
}
function load_template_ajax( $parent_dir, $child_dir, $ids ) {	
	simpods_load_template_ajax( $parent_dir, $child_dir, $ids );
}
// ajax action/filter chain order: theme > ( child theme overwrite parent theme ) > plugin > simpods mvc	
if ( isset( $_REQUEST ) && count( $_REQUEST ) > 0 && wp_doing_ajax() ) {
	if ( isset( $_SERVER['HTTP_REFERER'] ) ) {
		if ( $funs->is_frontend_ajax() ) {
			global $cross_site;
			// $lib_files = array(
			// 	'mvc_library_arrays' => SIMPODS_MVC_URL . '/library/php/arrays.class.php',
			// );
			// $funs->include( $lib_files, true, array() );
			$cross_site = MVC_Front_Factory::build( 'MVC_Front_Cross_Site' );
			$cross_site->for_ajax();		

			// Load template ajax by themes and plugins
			$parent_dir = get_template_directory() . '/mvc/ajax/';
			$child_dir  = get_stylesheet_directory() . '/mvc/ajax/';
			simpods_include_directory_files( $parent_dir, $child_dir );

			// Load component ajax
			// $component_load = SIMPODS_COMPONENTS; //Depreciated,
			// if ( !empty( $component_load ) && !empty( $component_load['front'] ) && !empty( $component_load['front']['ajax'] ) ) {
			// 	$funs->include( $component_load['front']['ajax'], false, array() );
			// }
			// Load registered ajax by plugins						
			$plugin_load = array( 'where' => 'front', 'type' => 'ajax' );
			$funs->load_simpods_plugin_files( $plugin_load );			


			// Load cross site ajax here so it can access global $cross_site; Rewritable.
			$ajax_files = array(
				'cross-site-ajax' => '/mvc/ajax/ajax-cross-site.php'
			);
			$funs->include( $ajax_files, true, array() );
			if( class_exists ( 'Ajax_Cross_Site' ) ){					
				new Ajax_Cross_Site();
			}  			

		} else {
			global $cross_admin;
			// $lib_files = array(
			// 	'mvc_library_arrays' => SIMPODS_MVC_URL . '/library/php/arrays.class.php',
			// );
			// $funs->include( $lib_files, true, array() );
			$cross_admin = MVC_Admin_Factory::build( 'MVC_Admin_Cross_Admin' );
 
			//$cross_admin = new MVC_Admin_Cross_Admin( false );
			$cross_admin->for_ajax();
			// Load cross site ajax here so it can access global $cross_admin
	
			// Load component ajax
			// $component_load = SIMPODS_COMPONENTS; //get_option( 'simpods_components_load' );
			// if ( !empty( $component_load ) && !empty( $component_load['admin'] ) && !empty( $component_load['admin']['ajax'] ) ) {
			// 	$funs->include( $component_load['admin']['ajax'], true, array(), true );
			// }
			// Load registered ajax by plugins			
			$plugin_load = array(
				'where' => 'admin',
				'type' => 'ajax'
			);		
			// registered ones only, plugin ajaxes are auto-initialised
			$funs->load_simpods_plugin_files( $plugin_load );

			if ( !headers_sent() ) {
				if (session_status() == PHP_SESSION_NONE) {
				    @session_start();
				}
			}
			if ( isset( $_SESSION['cur_admin_page'] ) && !empty( $_SESSION['cur_admin_page'] ) ) {
				$file = $_SESSION['cur_admin_page'] . '.php';
			} else {
				$referer_url = wp_get_referer();
				parse_str( substr( $referer_url, strpos( $referer_url, '?' ) + 1 ), $page_query );
				if ( isset( $page_query['page'] ) ) {
					$file = esc_attr( $page_query['page'] ) . '.php';
				} else {
					$file = strtok( basename( $_SERVER['HTTP_REFERER'] ), '?' );
				}
			}

			// load cross simpods jax
			if( strpos( $file, 'simpods-') === 0 ){
				$cross_simpods_ajax_files = array(
					'simpods-cross-simpods-ajax' => SIMPODS_MVC_URL . '/admin/ajax/simpods-cross-simpods.php',
				);
				$funs->include( $cross_simpods_ajax_files, false );	
				if( class_exists ( 'Ajax_Simpods_Cross_Simpods' ) ){					
					new Ajax_Simpods_Cross_Simpods();
				}  							
			}
			// load current page ajax 
			$admin_ajax_files = array(
				'current-admin-page-ajax' => '/mvc/admin/ajax/' . $file
			);
			$funs->include( $admin_ajax_files, true, array() );
			simpods_initialise_ajax_class( $file );
			$is_admin = true;
			simpods_load_plugin_ajaxes( $file, $is_admin );			

			// Load admin ajax. Rewritable. 
			$ajax_files = array(
				'cross-admin-ajax' => '/mvc/admin/ajax/ajax-cross-admin.php'
			);
			$funs->include( $ajax_files, true, array() );
			if( class_exists ( 'Ajax_Cross_Admin' ) ){					
				New Ajax_Cross_Admin();
			}  				
			// Load admin ajax. Edited by the Simpods team only
			$ajax_files = array(
				'simpods-cross-admin-ajax' => SIMPODS_MVC_URL . '/admin/ajax/simpods-cross-admin.php'
			);
			$funs->include( $ajax_files, false, array() );	
			if( class_exists ( 'Ajax_Simpods_Cross_Admin' ) ){					
				new Ajax_Simpods_Cross_Admin();
			}  
			
		}
	}
}

/**
 * Initialise a template ajax class
 * 
 * @param string $template_slug like template-name.php
 */ 
function simpods_initialise_ajax_class( $template_slug, $plugin_folder = '' ){ 
	$template_name = substr( $template_slug , 0, -4 );        	
	$ajax_class_name	= str_replace( '-', '_', $template_name	);
	$ajax_class_name	= ucwords( $ajax_class_name, '_' ); // e.g. Ajax_Simpods_Booking 
	if( '' !== $plugin_folder ){
		$namespace 	= str_replace( '-', '_', $plugin_folder ); // use plugin_name as namespace. Namespace doesn't take dash
		$ajax_class_name 		= $namespace . '\\' . $ajax_class_name;	
	}
	if( class_exists ( $ajax_class_name ) ){					
		$reflection  = new ReflectionClass( $ajax_class_name );
		$class_valid = !$reflection->isAbstract() && !$reflection->isInterface() && $reflection->isInstantiable();
		if ( $class_valid ) {
			New $ajax_class_name();		
		}
	}  	
}

/**
 * Load the ajax file of the current template
 * 
 */
function simpods_load_plugin_ajaxes( $template_slug, $is_admin = false ){
	global $wp_filesystem;
    // template ajax from simpods plugins
    $component_plugins 	= SIMPODS_COMPONENT_PLUGINS; 
	if ( is_array( $component_plugins ) ) {
		$admin_folder = '';
		if( $is_admin ){
			$admin_folder = '/admin';
		}
		require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		foreach ( $component_plugins as $plugin_name => $plugin_details ) {
			if( is_plugin_active( $plugin_name . '/' . $plugin_name . '.php' ) ){ 
				$file      = ABSPATH . 'wp-content/plugins/' . $plugin_name . '/mvc'. $admin_folder . '/ajax/' . $template_slug ;

				if ( $wp_filesystem->exists( $file ) && $template_slug != '' && !$wp_filesystem->is_dir( $file ) ) {					
					include_once $file;	
					simpods_initialise_ajax_class( $template_slug, $plugin_name );
				}	
				
			}	
		}	
	}
}	