<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Arrays
 *
 * @version 1.0.0
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 16/02/2016
 */
class Arrays {
	/**
	 * function array_column_fn: works like the array_column in PHP 5.5
	 *
	 * @para array  $a_arr two dimensions array
	 * @para string $column column name in the second dimension
	 * @return a new single dimension array with values from the targeted column in the second dimension
	 */
	public static function array_column( $a_arr, $column ) {
		$ret_arr = array();
		foreach ( $a_arr as $row ) {
			is_array( $row ) ? $ret_arr[] = $row[$column] : $ret_arr[] = $row->$column;
		}
		return $ret_arr;
	}
	/**
	 * sort2D_fn: sort 2 dimensions array by column name. e.g. array( 0 => array( 'featured' => 1 ), 1=> array('featured' => 2)
	 * 
	 * @link https://www.php.net/manual/zh/function.array-multisort.php
	 **/
	public static function sort2D( $a_arr, $column_name ) {
		$args = func_get_args();
		$data = array_shift( $args );
		foreach ( $args as $n => $field ) {
			if ( is_string( $field ) ) {
				$tmp = array();
				foreach ( $data as $key => $row ) {
					$tmp[ $key ] = $row[ $field ];
				}
				$args[$n] = $tmp;
			}
		}
		$args[] = &$data;
		call_user_func_array( 'array_multisort', $args );
		return array_pop( $args );
	}
	/**
	 * remove_reindex_fn: remove an element and return a reindexed array. e.g. array( 2,3,5) remove 3 , return array( 2,5)
	 **/
	public static function remove_reindex( $a_arr, $value ) {
		$b_arr   = array();
		$idx = array_search( $value, $a_arr );
		if ( $idx !== false ) {
			array_splice( $a_arr, $idx, 1 );
		}
		foreach ( $a_arr as $k => $array_value ) {
			$b_arr[] = '' . $array_value . '';
		}
		return $b_arr;
	}
	/**
	 * array_flips_fn: flip multi-associated array
	 */
	public static function array_flips( $a_arr ) {
		$b_arr = array();
		foreach ( $a_arr as $key => $v_arr ) {
			for ( $i = 0; $i < count( $v_arr ); $i++ ) {
				$b_arr[$i][$key] = $v_arr[$i];
			}
		}
		return $b_arr;
	}
	/**
	 * col_key_col_val_fn: use one column as key one column as value from an associated array
	 */
	public static function col_key_col_val( $a_arr, $key_column, $value_column, $options = array( 'tolower' => false ) ) {
		$b_arr = array();
		foreach ( $a_arr as $key => $v_arr ) {
			if ( is_array( $v_arr ) && isset( $v_arr[$key_column] ) && isset( $v_arr[$value_column] ) ) {
				if ( $options['tolower'] ) {
					$v_arr[$key_column] = strtolower( $v_arr[$key_column] );
				}
				$b_arr[$v_arr[$key_column]] = $v_arr[$value_column];
			}
		}
		return $b_arr;
	}
	/**
	 * col_key_fn: use one column as key instead of the origianl ones
	 */
	public static function col_key( $a_arr, $key_column ) {
		$b_arr = array();
		foreach ( $a_arr as $key => $v_arr ) {
			
			if ( is_array( $v_arr ) && isset( $v_arr[ $key_column ] ) ) {
				$b_arr[ $v_arr[ $key_column ] ] = $v_arr;
			}
			
			if ( is_object( $v_arr ) && isset( $v_arr->$key_column ) ) {
				$b_arr[ $v_arr->$key_column ] = $v_arr;
			}
		}
		return $b_arr;
	}
	/**
	 * Convert a string to an array. This function applys trim to its values 
	 * so you don't need to worry about the problems caused by space values, 
	 * also eliminates the problem that explode an empty string will return an one-element array.
	 * 
	 * @param string $a_string
	 * @param string $separator a string to separate $a_string
	 * 
	 * @return array $return 
	 */ 
	public static function string_to_array( $a_string = '', $separator = ',' ){
		$return	= array();
		if( is_string( $a_string ) || is_numeric( $a_string ) ){
			$a_string = trim( $a_string ) ;
			if( $a_string != '' ){
				$return = array_map( 'trim', explode( $separator, $a_string ) );										
			} else {
				$return = array();
			}					
			$count = count( $return );
			if( $count > 0 && $return[ $count - 1 ] === '' ){
				array_pop( $return ); // remove the last empty item with empty value;
			}			
		}
		return $return;
	}
	/**
	 * To check if Array B contains Array A
	 * 
	 * @param array $a_array Array A
	 * @param array $b_array Array B
	 * 
	 * @return true|array If Array B contains Array A, return true, else, return the missing ones
	 * 
	 */ 
	public static function a_in_b( $a_array = array(), $b_array = array() ){
		$difference = array();
		if( is_array( $a_array ) && is_array( $b_array ) ){
			$intersect 	= array_intersect( $a_array, $b_array );
							
			$difference	= array_diff( $a_array, $intersect  );
		}
		if( empty( $difference ) ){ // a is in b
			return true;
		}
		return $difference;
	}
	/**
	 * Get the percentage of similarity of two arrays
	 * 
	 * @param array $a Array A
	 * @param array $b Array B	 
	 * @return number|false  the percentage of similarity of two arrays or false if either $a or $b is not an array
	 */ 
	public static function similarity( $a = array() , $b = array(), $key_to_key = true ){
		if( is_array( $a ) && is_array( $b ) ){
			$a_length = count( $a );
			$found_count = 0;
			if( $key_to_key ){
				foreach ( $a as $k => $v ) {
				    if ( isset( $b[ $k ] ) && $v == $b[ $k ] ){
				    	$found_count ++;	
				    } 
				}
			} else {
				// foreach ( $a as $k ) {
				//     if ( in_array( $k, $b ) ){
				//     	$found_count ++;	
				//     } 
				// }	
				
				for( $i = $a_length - 1; $i >= 0; $i -- ){
					$b_length = count( $b );
				    for( $j = 0; $j < $b_length; $j ++ ){
				    	if( $a[ $i ] == $b[ $j ] ){
				    		unset( $a[ $i ] ); // found, usset it so it won't participate in the next loop
				    		unset( $b[ $j ] ); 
				    		$found_count ++;	
				    		break;
				    	}			    
				    } 
				    $b = array_values( $b ); // reindex
				}							
			}
			
			return ( $found_count / $a_length ) *100;		
		}
		return false;
	}
	/**
	 * Recursive array_map for multidimensional arrays
	 *
	 * @param string $callback The function to apply to the array
	 * @param array  $array    The array to be processed
	 * @param author Andrew Palfrey
	 * @return array           The final array after processing
	 */
	public static function array_map_r( $callback, $array ) {
		$result = array();
		foreach ( $array as $key => $val ) {
			$result[$key] = ( is_array( $val ) ? $this->array_map_r( $callback, $val ) : $callback( $val ) );
		}
		return $result;
	}
	/**
	 * Checks if the given key or index exists multidimensionally in an array
	 * 
	 * @link https://www.sitepoint.com/community/t/recursive-isset-or-array-key-exists/4496/6
	 * @param mixed $needle   The key to search for
	 * @param array $haystack The array to search
	 * @param author Andrew Palfrey
	 * @return boolean        True if exists, false if not
	 */
	public static function array_key_exists_r( $needle, $haystack ) {
		$result = array_key_exists( $needle, $haystack );
		if ( $result ) {
			return $result;
		}
		foreach ( $haystack as $v ) {
			if ( is_array( $v ) ) {
				$result = $this->array_key_exists_r( $needle, $v );
			}
			if ( $result ) {
				return $result;
			}
		}
		return $result;
	}
	/**
	 * Recursively get keys from a multidimensional array
	 *
	 * @param array $array The array to get keys from
	 * @param author Andrew Palfrey
	 * @return void
	 */
	public static function array_keys_r( $array ) {
		$keys = array();
		foreach ( $array as $key => $value ) {
			$return[] = $key;
			if ( is_array( $value ) ) {
				$return = array_merge( $return, $this->array_keys_r( $value ) );
			}
		}
		return $return;
	}
	/**
	 * Check if needles are in array, changes between all items and any items with the all parameter
	 *
	 * @param array   $needles  Array of needles
	 * @param array   $haystack Array to search
	 * @param boolean $all      Whether to search all or not (true = all, false = any)
	 * @param author Andrew Palfrey
	 * @return void
	 */
	private static function multi_in_array( $needles, $haystack, $all = true ) {
		if ( $all ) {
			return !array_diff( $needles, $haystack );
		} else {
			return !!array_intersect( $needles, $haystack );
		}
	}
	/**
	 * Recursively checks if a value exists in a multisimensional array
	 *
	 * @param mixed   $needle   The value to search for
	 * @param array   $haystack The array to search
	 * @param boolean $strict   Checks the types of the needle in the haystack
	 * @param author Andrew Palfrey
	 * @return void
	 */
	public static function in_array_r( $needle, $haystack, $strict = false ) {
		foreach ( $haystack as $item ) {
			if ( ( $strict ? $item === $needle : $item == $needle ) || ( is_array( $item ) && $this->in_array_r( $needle, $item, $strict ) ) ) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Flattens a multi-dimentional array and returns it.
	 * 
	 * @link https://gist.github.com/SeanCannon/6585889
	 * @param  array $array The multi-dimentional array you would like to flatten
	 * @return array        The flattened array
	 */
	public static function array_flatten( $array = null ) {
		$result = array();
		if ( !is_array( $array ) ) {
			$array = func_get_args();
		}
		foreach ( $array as $key => $value ) {
			if ( is_array( $value ) ) {
				$result = array_merge( $result, $this->array_flatten( $value ) );
			} else {
				$result = array_merge( $result, array( $key => $value ) );
			}
		}
		return $result;
	}
	/**
	 * Checks an array for multiple keys
	 * 
	 * @link https://stackoverflow.com/a/13169599
	 * @param array $keys The keys to search for
	 * @param array $arr  The array to search
	 * @return boolean    Whether all keys exist or not
	 */
	public static function array_keys_exists( $keys, $arr ) {
		return !array_diff_key( array_flip( $keys ), $arr );
	}			
	/**
	 * Sort a 2 dimentional associated array
	 * 	 	 
	 * @param array $arr  The array to sort e.g array( 'key1' => array( 'sort_me' => 1 ),  'key2' => array( 'sort_me' => 2 ), )
	 * @param string $key, the key to sort
	 * @param number $default_priority the default priority if not set
	 * @return array the sorted array
	 */
	public static function sort_2d_associated( $arr, $key, $default_priority = 1 ){
		foreach ( $arr as $k => $sub_arr ) {
			if( !isset( $sub_arr[ $key ] ) ){
				$arr[ $k ][ $key ] = $default_priority;
			}
		}
		uasort( $arr, function( $a, $b ) use ( $key ) {					
		    return strcmp( $a[ $key ], $b[ $key ] );
		});
		return $arr;
	}

}