<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods Autoload
 *
 * @version 1.0.0
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 10/01/2019
 */
class Simpods_Autoload {
    private static $called = false;
    private static $models      = array();
    public static $included = array();
    public static $classes  = array();
    // private $parentDir_str = '';
    // private $childDir_str  = '';
    // Core model classes should not be overwritten
    private static $front_core_models = array(
        'mvc-front-general.class.php'                   => 1,
        'mvc-front-parent-cross-site.class.php'         => 1,
        'mvc-front-parent-factory.class.php'            => 1,
        'mvc-front-pods.class.php'                      => 1,
        'mvc-front-pods-files.class.php'                => 1,        
        'mvc-front-pods-header-field.class.php'         => 1,
        'mvc-front-simpods-global-shortcodes.class.php' => 1,
        'mvc-front-simpods-guest-accounts.class.php'    => 1,
        'mvc-front-simpods-hierarchy.class.php'         => 1,
        'mvc-front-simpods-linked-areas.class.php'      => 1,
        'mvc-front-simpods-user-code.class.php'         => 1,
        'mvc-front-simpods-login-attempts.class.php'    => 1,
    );
    private static $admin_core_models = array(
        'mvc-admin-general.class.php'                    => 1,
        'mvc-admin-parent-cross-admin.class.php'         => 1,
        'mvc-admin-parent-factory.class.php'             => 1,
        'mvc-admin-pods.class.php'                       => 1,
        'mvc-admin-pods-area-field.class.php'            => 1,
        'mvc-admin-pods-button-field.class.php'          => 1,
        'mvc-admin-pods-files.class.php'                 => 1,
        'mvc-admin-pods-header-field.class.php'          => 1,
        'mvc-admin-settings-helper.class.php'            => 1,
        'mvc-admin-simpods-custom-controllers.class.php' => 1,
        'mvc-admin-simpods-email.class.php'              => 1,
        'mvc-admin-simpods-guest-accounts.class.php'     => 1,
        'mvc-admin-simpods-hierarchy.class.php'          => 1,
        'mvc-admin-simpods-home.class.php'               => 1,
       // 'mvc-admin-simpods-install-update.class.php'     => 1,
        'mvc-admin-simpods-linked-areas.class.php'       => 1,
        'mvc-admin-simpods-login-attempts.class.php'     => 1,   
        'mvc-admin-simpods-settings.class.php'           => 1,
        'mvc-admin-simpods-style-script.class.php'       => 1,
        'mvc-admin-simpods-user-code.class.php'          => 1,
        'mvc-admin-user-settings-helper.class.php'       => 1,
        'podsfield_simpodsareafield.php'                 => 1,
        'podsfield_simpodsbuttonfield.php'               => 1,
        'podsfield_simpodsheaderfield.php'               => 1,
        'podsfield_simpodspodsfile.php'                  => 1,
    );
    public function __construct() {
        if ( !self::$called ) {
            self::autoload();
            self::$called = true;
        }
    }
    final private function __clone() {}
    /**
     * Autoload model folders, admin and frontend
     *
     * @return null
     */
    final private static function autoload() {
        global $funs;
        $admin_folder = '';
        if ( is_admin() ) {
            if ( !$funs->is_frontend_ajax() ) {
                $admin_folder = 'admin/';
            }
        }
        $template_dir   = get_template_directory() . '/mvc/' . $admin_folder . 'models';
        $stylesheet_dir = get_stylesheet_directory() . '/mvc/' . $admin_folder . 'models';
        $plugin_dir     = SIMPODS_MVC_DIR .'/' . $admin_folder . 'models';
        self::$models = array(
            'plugin' => array(),
            'parent' => array(),
            'child'  => array(),
        );
        self::get_model_info( $plugin_dir, 'plugin' );
        // add all files to the self::$models array. the directory may not exist if we embed simpods mvc in t
        //if ( is_dir( $template_dir ) ) {
        self::get_model_info( $template_dir, 'parent' );
        require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        // load component plugins
        $component_plugins = SIMPODS_COMPONENT_PLUGINS; //get_option( 'simpods_plugins', array() );

        if ( is_array( $component_plugins ) ) {
           
                foreach ( $component_plugins as $plugin_slug => $plugin_info ) {

                    // if plugin not active, don't include . All Simpods plugin main file should be named by its plugin folder name.
                    if ( is_plugin_active( $plugin_slug . '/' . $plugin_slug . '.php' ) ) {   
                        $plugin_dir = ABSPATH . 'wp-content/plugins/' . $plugin_slug . '/mvc/global/models';
                        // always treat component plugins as parent ones so they are always loaded
                        self::get_model_info( $plugin_dir, 'plugin' );
                    }
                }
                foreach ( $component_plugins as $plugin_slug => $plugin_info ) {
                    // if plugin not active, don't include . All Simpods plugin main file should be named by its plugin folder name.
                    if ( is_plugin_active( $plugin_slug . '/' . $plugin_slug . '.php' ) ) {                          
                        $plugin_dir = ABSPATH . 'wp-content/plugins/' . $plugin_slug . '/mvc/' . $admin_folder . 'models';
                        // always treat component plugins as parent ones so they are always loaded
                        self::get_model_info( $plugin_dir, 'plugin' );
                    }
                }

        }
        //}
       
        if (  CHILD_THEME_ACTIVE ) { 
            // only do it if a child theme is active
            self::get_model_info( $stylesheet_dir, 'child' );
            spl_autoload_register(
                function ( $class ) {            
                    self::load_class( self::$models['plugin'], $class, 'plugin' );
                    self::load_class( self::$models['parent'], $class, 'parent' );
                    self::load_class( self::$models['child'], $class, 'child' );
                }
            );
        } else {
            spl_autoload_register(
                function ( $class ) {
                    self::load_class( self::$models['plugin'], $class, 'plugin' );
                    self::load_class( self::$models['parent'], $class, 'parent' );
                }
            );
        }
    }
    /**
     * Register classes in model folders, admin and frontend
     *
     * @param string $dir   directory
     * @param string $location parent|child
     * @return null
     */
    final private static function get_model_info( $dir, $location ) {
        global $wp_filesystem;
        if ( !$wp_filesystem->exists( $dir ) ) {
            return;
        }
        $files = scandir( $dir );
        // reverse array so folders go to the bottom
        $files = array_reverse( $files );
        unset( $files[array_search( '.', $files, true )] );
        unset( $files[array_search( '..', $files, true )] );
        // prevent empty ordered elements
        if ( count( $files ) < 1 ) {
            return;
        }
        foreach ( $files as $file ) {
            if ( is_dir( $dir . '/' . $file ) ) {
                //if multisite, only load files from the current blog, use blog id to find out
                $load_file = true;
                if ( is_multisite() ) {
                    if ( is_numeric( $file ) && get_current_blog_id() != $file ) {
                        //load file and build MVC_Front_Cross_Blog
                        $load_file = false;
                    }
                }
                if ( $load_file ) {
                    self::get_model_info( $dir . '/' . $file, $location );
                }
            } else {
                // don't need to use multidimentional array as all file name must be unqiue
                if ( substr( $file, -10 ) != '.class.php' ) {
                    continue;
                }
                self::$models[ $location ][ $file ] = $dir . '/' . $file;
            }
            $class = substr( $file, 0, -10 );
            if ( !in_array( $class, self::$classes ) ) {
                array_push( self::$classes, str_replace( '-', '_', $class ) ); // class name doesn't accept -
            }
        }
    }
    /**
     * Only load the class required. 
     * 
     * @param array $files files, directories in a directory
     * @param string $class the class name to create an instance. e.g. MVC_Front_A_Class 
     * @param string $location parent|child theme
     */
    final private static function load_class( $files, $class = '', $location = 'parent' ) {
        $class = strtolower( $class );
        $class_file = $class . '.class.php';
        $available  = false;    
        if ( isset( $files[ $class_file ] ) && !is_array( $files[ $class_file ] ) ) {
            $available = true;    
          
        } else { 
            // file named with dash instead of underscore
            $class_file = str_replace( '_', '-', $class_file );
            if ( isset( $files[ $class_file ] ) && !is_array( $files[ $class_file ] ) ) {  
                $available  = true;        
            }
        }
        if ( true === $available ) { // if can be found and not an array
            // Intend to load all classes
            $include_file = true;
            // avoid overwritting core classes
            $in_core      = false;
            // check if someone is trying to overwrite the core classes in the child
            if ( is_admin() ) { 
                if ( isset( self::$admin_core_models[ $class_file ] ) ) { // in core, don't overwrite. If trying to overwrite, a Fatal error will appear                    
                    $in_core = false;
                }
            } else {
                if ( isset( self::$front_core_models[ $class_file ] ) ) { // in core, don't overwrite. If trying to, a Fatal error will appear
                    $in_core = false;
                }
            }
            
            // If loading a plugin file, the file is in parent, child and not in core, don't load it       
            if ( $location == 'plugin' ) {              
                if( ! $in_core && ( isset( self::$models['parent'][ $class_file ] ) || isset( self::$models['child'][ $class_file ] ) ) ){ // not in core but in parent and child, don't load
                    $include_file = false;                   
                }

            }
            // If loading a parent file, the file is in child and in core, don't load it       
            if ( $location == 'parent' ) {
                if( $in_core || isset( self::$models['child'][ $class_file ] ) ){ // not in core but in parent and child, don't load
                    $include_file = false;                   
                }
            }      
            // If loading a parent file, the file is in child and in core, don't load it       
            if ( $location == 'child' ) {
                if( $in_core ){ // not in core but in parent and child, don't load
                    $include_file = false;                   
                }
            }                  
            //     
             // child theme activated, load all classes the child theme one

            // if ( CHILD_THEME_ACTIVE ) {               
            //     if ( $location == 'parent' && isset( self::$models['child'][ $class_file ] ) ) {
            //         // check if someone is trying to overwrite the core classes in the child
            //         if ( is_admin() ) {
            //             if ( !isset( self::$admin_core_models[ $class_file ] ) ) { // in core, don't overwrite. If trying to overwrite, a Fatal error will appear
            //                 $include_file = false;
            //             }
            //         } else {
            //             if ( !isset( self::$front_core_models[ $class_file ] ) ) { // in core, don't overwrite. If trying to, a Fatal error will appear
            //                 $include_file = false;
            //             }
            //         }
            //     }
            // } 
            // always load core model classes from parent
            if ( $include_file ) {
                include_once $files[ $class_file ];
                array_push( self::$included, $class_file );
            }
        } else {
            // try sub-directories in the directory
            foreach ( $files as $file ) {
                if ( is_array( $file ) ) {
                    // load files for subfolders
                    self::load_class( $file, $class, $location );
                }
            }
        }
    }
}
new Simpods_Autoload();
