<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods Controllers Class
 *
 * @version 2.0.0
 * @package Simpods MVC
 * @author Andrew Palfrey
 * @since 28/09/2018
 */
class Simpods_Controllers {

	const DB_TABLE = 'simpods_custom_controllers';

	/**
	 * Look up the simpods custom controllers table to see if it is logged as a custom one
	 *
	 * @param  string $path The part from URL after the domain. e.g [domain]/events/list/, enter "events/list"
	 * @return string|boolean the custom controllers path or false
	 */
	final public function look_up( $path = '' ) {
		global $funs;

		if ( !get_option( self::DB_TABLE . '_table_created' ) ) {
			return false;
		}

		if ( $path == '' ) {

			$http_host = '';
			if( isset( $_SERVER['HTTP_HOST'] ) ){ // sometimes it is not available and produce an error
				$http_host = $_SERVER['HTTP_HOST'];
			}
			$url = ( isset( $_SERVER['HTTPS'] ) ? 'https' : 'http' ) . '://' . $http_host . $_SERVER['REQUEST_URI'];
			$url_parts = parse_url( $url );

			if ( isset( $url_parts['query'] ) ) {
				$url = str_replace( '?' . $url_parts['query'], '', $url );
			}

			$path = trim( str_replace( site_url(), '', $url ), '/' );

			$fields = array(
				'spcc_where'   => ( is_admin() ) ? 'admin' : 'frontend',
				'spcc_look_up' => $path,
			);
			$args   = array(
				'target_tb' => self::DB_TABLE,
				'limit'     => 1,
			);

			$controller_item = $funs->simpods_select( $fields, $args );

			if ( $controller_item ) {
				return $controller_item[0]['spcc_controller_path'];
			} else {
				$path_parts = explode( '/', $path );
				$path_count  = count( $path_parts );

				//search wild card
				while ( count( $path_parts ) > 1 ) {
					// pop out the last and change it to wild card
					array_pop( $path_parts );
					$path = implode( '/', $path_parts );

					$limit = $path_count - count( $path_parts );
					for ( $i = 0; $i < $limit; $i++ ) {
						$path .= '/*';
					}

					$fields['spcc_look_up'] = $path;

					$controller_item = $funs->simpods_select( $fields, $args );

					if ( $controller_item ) {
						return $controller_item[0]['spcc_controller_path'];
					}

				}

			}

		}

		return false;
	}

}
