<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Handling emails
 *
 * @version: 2.0.0
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 06/02/2018
 */
class Simpods_Email {

	public $from_site = true;
	const EMAIL_VERSION = '2.0.0';
	public function __construct( $init = true ) {

	}

	/**
	 * Send an outlook appointment
	 *
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/outlook_appointment_fn/
	 * @link   https://www.exchangecore.com/blog/sending-outlookemail-calendar-events-php/
	 * @param  string $domain        Domain where the outlook exchange is hosted
	 * @param  string $from_name     Name of the sender
	 * @param  string $from_address  Email address of the sender
	 * @param  string $to_name       Name of the recipient
	 * @param  string $to_address    Email address of the recipient
	 * @param  string $start_time    Appointment start time (American or ISO8601 format)
	 * @param  string $end_time      Appointment end time (American or ISO8601 format)
	 * @param  string $subject       Appointment subject
	 * @param  string $email_content Email content
	 * @param  string $location      Appointment Location
	 * @param  string $timezone      Timezone
	 * @return boolean
	 */
	final public function outlook_appointment( $domain, $from_name, $from_address, $to_name, $to_address, $start_time, $end_time, $subject, $email_content, $location, $timezone = 'Etc/UTC' ) {

	   // $domain = 'exchangecore.com';
		$timezone_definition = $this->get_timezone_definition( $timezone );
	    //Create Email Headers
	    $mime_boundary = "----Simpods MVC----" . MD5( TIME() );

	    $headers = "From: " . $from_name . " <" . $from_address . ">\n";
	    $headers .= "Reply-To: " . $from_name . " <" . $from_address . ">\n";
	    $headers .= "MIME-Version: 1.0\n";
	    $headers .= "Content-Type: multipart/alternative; boundary=\"$mime_boundary\"\n";
	    $headers .= "Content-class: urn:content-classes:calendarmessage\n";

	    //Create Email Body (HTML)
	    $message = "--$mime_boundary\r\n";
	    $message .= "Content-Type: text/html; charset=UTF-8\n";
	    $message .= "Content-Transfer-Encoding: 8bit\n\n";
	    $message .= "<html>\n";
	    $message .= "<body>\n";
	    $message .= '<p>' . $email_content . '</p>';
	    $message .= "</body>\n";
	    $message .= "</html>\n";
	    $message .= "--$mime_boundary\r\n";

	    $ical = 'BEGIN:VCALENDAR' . "\r\n" .
	    'PRODID:-//University of Central Lancashire//Simpods MVC ' . self::EMAIL_VERSION . '//EN' . "\r\n" .
	    'VERSION:2.0' . "\r\n" .
	    'METHOD:REQUEST' . "\r\n" .
	    $timezone_definition . "\r\n" .
	    'BEGIN:VEVENT' . "\r\n" .
	    'ORGANIZER;CN="' . $from_name . '":MAILTO:' . $from_address . "\r\n" .
	    'ATTENDEE;CN="' . $to_name . '";ROLE=REQ-PARTICIPANT;RSVP=TRUE:MAILTO:' . $to_address . "\r\n" .
	    'LAST-MODIFIED:' . date("Ymd\TGis") . "\r\n" .
	    'UID:'.date("Ymd\TGis", strtotime( $start_time )).rand()."@" . $domain ."\r\n" .
	    'DTSTAMP:'.date("Ymd\TGis"). "\r\n" .
	    'DTSTART;TZID="' . $timezone . '":'.date("Ymd\THis", strtotime( $start_time )). "\r\n" .
	    'DTEND;TZID="' . $timezone . '":'.date("Ymd\THis", strtotime( $end_time )). "\r\n" .
	    'TRANSP:OPAQUE'. "\r\n" .
	    'SEQUENCE:1'. "\r\n" .
	    'SUMMARY:' . $subject . "\r\n" .
	    'LOCATION:' . $location . "\r\n" .
	    'CLASS:PUBLIC'. "\r\n" .
	    'PRIORITY:5'. "\r\n" .
	    'BEGIN:VALARM' . "\r\n" .
	    'TRIGGER:-PT15M' . "\r\n" .
	    'ACTION:DISPLAY' . "\r\n" .
	    'DESCRIPTION:Reminder' . "\r\n" .
	    'END:VALARM' . "\r\n" .
	    'END:VEVENT'. "\r\n" .
	    'END:VCALENDAR'. "\r\n";
	    $message .= 'Content-Type: text/calendar;name="meeting.ics";method=REQUEST'."\n";
	    $message .= "Content-Transfer-Encoding: 8bit\n\n";
	    $message .= $ical;

	    // Warning: DO NOT USE wp_mail. It won't work.
	    $mailsent = mail( $to_address, $subject, $message, $headers );

		return ( $mailsent ) ? ( true ) : ( false );
	}

	/**
	 * Send normal HTML email
	 *
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/email_fn/
	 * @param  string $from_name     Name of the sender
	 * @param  string $from_address  Email address of the sender
	 * @param  string $to_name       Name of the recipient
	 * @param  string $to_address    Email address of the recipient
	 * @param  string $subject       Subject of the email
	 * @param  string $email_content The email content
	 * @param  string $cc_address    CC email addresses
	 * @param  string $bcc_address   BCC email addresses
	 * @return boolean
	 */
	final public function send_email( $from_name, $from_address, $to_name, $to_address, $subject, $email_content, $cc_address = '', $bcc_address = '' ) {
		//Create Email Headers
		global $current_user, $funs;

		$headers     = '';
		$headers_set = false;

		//if user has logged in

		if ( is_user_logged_in() ) {
			$find = array(
				'{current_user_login}',
				'{current_user_firstname}',
				'{current_user_lastname}',
				'{current_user_display_name}',
			);
			$replace = array(
				$current_user->user_login,
				$current_user->user_firstname,
				$current_user->user_lastname,
				$current_user->display_name,
			);
			$from_name = str_replace( $find, $replace, $from_name );
			$to_name   = str_replace( $find, $replace, $to_name );

			$find = array(
				'{current_user_email}',
			);
			$replace = array(
				$current_user->user_email,
			);
			$from_address = str_replace( $find, $replace, $from_address );
			$to_address   = str_replace( $find, $replace, $to_address );
			$cc_address   = str_replace( $find, $replace, $cc_address );
			$bcc_address  = str_replace( $find, $replace, $bcc_address );

			$find = array(
				'{current_user_login}',
				'{current_user_firstname}',
				'{current_user_lastname}',
				'{current_user_display_name}',
				'{current_user_level}',
				'{current_user_id}',
				'{current_user_email}',
			);
			$replace = array(
				$current_user->user_login,
				$current_user->user_firstname,
				$current_user->user_lastname,
				$current_user->display_name,
				$current_user->user_level,
				$current_user->ID,
				$current_user->user_email,
			);
			$subject     = str_replace( $find, $replace, $subject );
			$email_content = str_replace( $find, $replace, $email_content );
		}

		$find = array(
			'{current_date_dmy}',
			'{current_date_mdy}',
			'{current_date_ymd}',
		);
		$replace = array(
			date('d/m/Y'),
			date('m/d/Y'),
			date('Y/m/d'),
		);

		$subject       = str_replace( $find, $replace, $subject );
		$email_content = str_replace( $find, $replace, $email_content );

		if ( !empty( $from_name ) && is_email( $from_address ) ) {
			$headers .= 'From: ' . $from_name . ' <' . $from_address . '>' . "\r\n";
			$headers_set = true;
		}

		if ( empty( $from_name ) && is_email( $from_address ) ) {
			$from_name = get_bloginfo( 'name' );
			$headers .= 'From: ' . $from_name . ' <' . $from_address . '>' . "\r\n";
			$headers_set = true;
		}

		// check if it is in this format name <email>;
		preg_match( '/(?:"?([^"]*)"?\s)?(?:<?(.+@[^>]+)>?)/', $from_address, $from_matches );

		if ( empty( $from_name ) && !is_email( $from_address ) && count( $from_matches ) > 0 ) {
			$headers .= 'From: ' . $from_address . '' . "\r\n";
			$headers_set = true;
		}

		if ( !$headers_set && $this->from_site ) {
			$headers .= $this->from_admin();
		}

		if ( !empty( $to_address ) ) {
			$to_addresses = explode( ',', $to_address );
			$to_addresses = array_map( 'trim', $to_addresses );

			foreach ( $to_addresses as $to ) {

				if ( $this->is_email( $to ) ) {
					$to_address = implode( ',', $to_addresses );
				}

			}

		}

		if ( !empty( $cc_address ) ) {

			$to_addresses = explode( ',', $cc_address );
			$to_addresses = array_map( 'trim', $to_addresses );
			$cc_header    = 'Cc: ';

			foreach ( $to_addresses as $to ) {

				if ( $this->is_email( $to ) ) {
					$headers .= $cc_header . $to . '' . "\r\n";
				}

			}

		}

		if ( !empty( $bcc_address ) ) {

			$to_addresses = explode( ',', $bcc_address );
			$to_addresses = array_map( 'trim', $to_addresses );
			$bcc_header   = 'Bcc: ';

			foreach ( $to_addresses as $to ) {
				
				if ( $this->is_email( $to ) ) { 
					$headers .= $bcc_header . $to . '' . "\r\n";
				}

			}

		}

		$email_content = trim( $email_content );

		if ( trim( $to_name ) != '' ) {

			$email_content = str_replace( '{name}', $to_name, $email_content ); 
			//$toName_str	=	$this->magic_fields['{name}']; 
		} 
		// if ( trim( $to_name ) != '' && $prepend_name ) {
		// 	$email_content = trim( $to_name ) . "<br/><br/>" . $email_content;
		// }
		// To fix the problem that some servers don't send Bcc emails if To is empty
		if( empty( $to_address ) ){
			$to_address = 'noreply@' . $funs->get_domain() ; 
		}

		$headers .= "Content-type: text/html;";

		$email_sent = wp_mail( $to_address, $subject, $email_content, $headers );
		
		// if failed try plain text
		if ( !$email_sent ) {
			$email_sent = wp_mail( $to_address, $subject, wp_strip_all_tags( $email_content ) );
		}

		return $email_sent ? true : false;
	}

	final public function email( $from_name, $from_address, $to_name, $to_address, $subject, $email_content, $cc_address = '', $bcc_address = '' ) {
		
		return $this->send_email( $from_name, $from_address, $to_name, $to_address, $subject, $email_content, $cc_address, $bcc_address );
	}

	/**
	 * Send an email using a template
	 *
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/template_email/
	 * @param  string $template_id         Slug of the template
	 * @param  array  $replace    Associative array of template tags and content to insert
	 * @param  string $to_address    Email of the recipient
	 * @param  string $to_name     Name of the recipient
	 * @param  string $from_address  Email address of the sender
	 * @param  string $from_name   Name of the sender
	 * @param  string $cc_address  CC email addresses
	 * @param  string $bcc_address BCC email addresses
	 * @param  string $email_pod        Name of the pods email table
	 * @return [type]
	 */
	final public function send_template_email( $template_id, $replace = array(), $to_address = '', $to_name = '', $from_address = '', $from_name = '', $cc_address = '', $bcc_address = '', $email_pod = 'simpods_email' ) {

		if ( !function_exists( 'pods' ) ) {
			return false;
		}

		$id_field    = is_int( $template_id ) ? 'id' : 'permalink';
		$template_id = esc_sql( $template_id );

		$pod = pods( $email_pod, array( 'where' => 't.' . $id_field . ' = "' . $template_id . '"' ), true );

		if ( $pod === false || !$pod->fetch() ) {

			if ( $email_pod === 'simpods_email' ) { // if failed try email pods, as some old projects uing it.
				$email_pod = 'email';
				$pod       = pods( $email_pod, array( 'where' => 't.' . $id_field . ' = "' . $template_id . '"' ), true );

				if ( $pod === false || !$pod->fetch() ) {
					return false;
				}

			} else {
				return false;
			}

		}

		$email_template = $pod->display( 'content' );
		$subject        = $pod->display( 'subject' );

		$email_content = str_replace( array_keys( $replace ), $replace, $email_template );

		$email_content = do_shortcode( $email_content );

		if ( !$this->is_email( $to_address ) ) {                           // if not email, use those from the template
			$to_address = wp_strip_all_tags( $pod->display( 'to_email' ), true ); // pods won't allow name <email@domain.com>, so it is ok to remove all tags

			if ( $to_address == '' ) { // email_fn will check if it is a valid email
				$to_address = get_option( 'admin_email' );
			}

		}

		if ( empty( $to_address ) ) {
			$to_address = get_bloginfo( 'name' );
		}

		if ( !$this->is_email( $from_address ) ) {
			$from_address = $pod->display( 'from_email' );

			if ( !$this->is_email( $from_address ) ) {
				$from_address = get_option( 'admin_email' );
			}

		}

		if ( empty( $from_name ) ) {
			$from_name = get_bloginfo( 'name' );
		}

		return $this->email( $from_name, $from_address, $to_name, $to_address, $subject, $email_content, $cc_address, $bcc_address );
	}

	final public function template_email( $template_id, $replace = array(), $to_address = '', $to_name = '', $from_address = '', $from_name = '', $cc_address = '', $bcc_address = '', $email_pod = 'simpods_email' ) {
		// _simpods_deprecated_message();
		return $this->send_template_email( $template_id, $replace, $to_address, $to_name, $from_address, $from_name, $cc_address, $bcc_address, $email_pod );
	}

	/**
	 * Check whether an email address is valid or not
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/is_email_fn/
	 * @param  string  $email Email address to test
	 * @return boolean
	 */
	final public function is_email( $email ) {

		if ( is_email( $email ) ) {
			return true;
		} else {
			preg_match( '/(?:"?([^"]*)"?\s)?(?:<?(.+@[^>]+)>?)/', $email, $email_matches );

			if ( isset( $email_matches[2] ) && is_email( $email_matches[2] ) ) {
				return true;
			}

		}

		return false;
	}

	/**
	 * Get an iCal timezone definition
	 * @param  string $timezone Name of the timezone in the IANA time zone database (e.g. Europe/London)
	 * @return string
	 */
	private function get_timezone_definition( $timezone ) {
		$timezone_definition = get_transient( 'timezone_def_' . $timezone );

		if ( $timezone_definition === false ) {
			$timezone_url        = 'http://tzurl.org/zoneinfo-outlook/' . $timezone;
			$timezone_definition = wp_remote_retrieve_body( wp_remote_get( $timezone_url ) );

			$timezone_start = strpos( $timezone_definition, 'BEGIN:VTIMEZONE' );
			$timezone_end   = strpos( $timezone_definition, 'END:VTIMEZONE' ) + strlen( 'END:VTIMEZONE' );

			$timezone_definition = substr( $timezone_definition, $timezone_start, $timezone_end - $timezone_start );

			set_transient( 'timezone_def_' . $timezone, $timezone_definition, 3 * MONTH_IN_SECONDS );
		}

		return $timezone_definition;
	}

	/**
	 * construct the email header for From
	 * @return string
	 */
	private function from_admin() {
		$headers = '';

		$site_name   = wp_strip_all_tags( get_bloginfo( 'name' ) );
		$admin_email = get_bloginfo( 'admin_email' );

		if ( !empty( $site_name ) && is_email( $admin_email ) ) {
			$headers = 'From: ' . $site_name . ' <' . $admin_email . '>' . "\r\n";
		}

		if ( empty( $site_name ) && is_email( $admin_email ) ) {
			$headers = 'From: ' . $admin_email . "\r\n";
		}

		return $headers;
	}

}

$GLOBALS['simpods_email'] = new Simpods_Email();

/** Usage example **/
/*
$domain 		= 'destination.com'; // Domain where the outlook exchange is hosted
$from_name 		= 'someone';
$from_address 	= 'from@domain.com';  // has to be an email from the same domain for noral html email.
$to_name 		= 'somebody';
$to_address 	= 'to@domain.com';
$start_time 	= '02/11/2018 18:00:00';   // American format
$end_time 		= '02/11/2018 19:00:00';   // American format
$subject 		= 'Simpods booking appointment';
$location 		= '';
$description 	= apply_filters( 'the_content', 'HI' );

// outlook_appointment
$simpods_email->outlook_appointment( $domain, $from_name, $from_address, $to_name, $to_address, $start_time, $end_time, $subject, $description, $location);

// normal html email
$simpods_email->email( $from_name, $from_address, $to_name, $to_address, $subject, $description);
*/
