<?php
/**
 * File access control script
 *
 * Restricts access to media files to authorised users.
 * Must be placed in a sub-directory of uploads along with the accompaning .htaccess and restriction definition (simpods_file.json) files.
 *
 * @version: 2.0.1
 * @package: Simpods MVC
 * @author: Mark Wane, Dongjie Xu
 * @since: 16/01/2018
 * @deprecated
 */

$script_path = explode( '/', __DIR__ );

while ( 'wp-content' != array_pop( $script_path ) ) {} // NOOP - just need to reach the wp installation root

require_once implode( '/', $script_path ) . '/wp-load.php';

$home_link = ' <a href="' . home_url() . '">' . esc_html__( 'Return to home', 'simpods-mvc' ) . '</a>';

// Forbid direct access (prevent the script downloading itself)
if ( $_SERVER['SCRIPT_NAME'] == $_SERVER['REQUEST_URI'] ) {
	wp_die( esc_html__( 'File cannot be accessed.', 'simpods-mvc' ) . $home_link, 403 );
}

// Covert the URL into a filepath
$script_name = explode( '/', $_SERVER['SCRIPT_NAME'] );

array_pop( $script_name );

$file_name = str_replace( implode( '/', $script_name ), '',  $_SERVER['REQUEST_URI'] );
$file_path = __DIR__ . $file_name;
$ext_regex = '/(mp4\?|m4a\?|mp3\?|ogg\?|wav\?)/i';

// Remove anyting after and including ? if it is a mp4, so it can be loaded in the Media library
if ( preg_match( $ext_regex, $file_path ) > 0 ) {
	$file_path = substr( $file_path, 0, strpos( $file_path, '?' ) );
}

// Forbid access to the restrictions file
if ( $file_name == '/simpods_file.json' ) {
	wp_die( esc_html__( 'File cannot be accessed.', 'simpods-mvc' ) . $home_link, 403 );
}

$restrictions = json_decode( file_get_contents ( 'simpods_file.json' ), true );

if ( !is_array( $restrictions ) || !array_key_exists( 'type', $restrictions ) ) {
	wp_die( esc_html__( 'Restricton not defined. erg', 'simpods-mvc' ) . $home_link, 500 );
}

$access_allowed = false;

// Allow files to be accessed by local scripts
if ( array_key_exists( 'local', $restrictions ) && $restrictions['local'] && $_SERVER['SERVER_ADDR'] == $_SERVER['REMOTE_ADDR'] ) {
	$access_allowed = true;
} else {
	switch ( $restrictions['type'] ) {
		case 'login':
			$access_allowed = is_user_logged_in();
			break;
		case 'cap':
			$access_allowed = current_user_can( $restrictions['cap'] );
			break;
		case 'user':
			$access_allowed = in_array( get_current_user_id(), $restrictions['users'] );
			break;
		default:
			wp_die( esc_html__( 'Restricton not defined.', 'simpods-mvc' ) . $home_link, 500 );
	}
}

if ( !$access_allowed ) {
	wp_die( esc_html__( 'You do not have permission to access this file.', 'simpods-mvc' ) . $home_link, 403 );
}

$content_type =  mime_content_type( $file_path );

if ( $content_type == 'application/octet-stream' ) {
	if ( strpos( $file_path, '.mp3' ) !== false ) {
		$content_type = 'audio/mpeg';
	}
	
}

//https://github.com/benjipott/video.js-chromecast/issues/45
if( 'video/mp4' == $content_type ){
	// $file=$file_path;
	// $fp = @fopen($file, 'rb');
	// $size = @filesize($file); // File size
	// $length = $size;           // Content length
	// $start = 0;               // Start byte
	// $end = $size - 1;       // End byte
	// //ob_get_clean();

	// header('Content-type: video/mp4');
	// header("Cache-Control: max-age=2592000, public");
	// header("Expires: " . gmdate('D, d M Y H:i:s', time() + 2592000) . ' GMT');
	// header("Last-Modified: " . gmdate('D, d M Y H:i:s', @filemtime($file)) . ' GMT');
	// header("Accept-Ranges: bytes 0-$end");
	// // header("Accept-Ranges: bytes");
	// if (isset($_SERVER['HTTP_RANGE'])) {
	// 	$c_start = $start;
	// 	$c_end = $end;
	// 	list(, $range) = explode('=', $_SERVER['HTTP_RANGE'], 2);
	// 	if (strpos($range, ',') !== false) {
	// 		header('HTTP/1.1 416 Requested Range Not Satisfiable');
	// 		header("Content-Range: bytes $start-$end/$size");
	// 		exit;
	// 	}
	// 	if ($range == '-') {
	// 		$c_start = $size - substr($range, 1);
	// 	} else {
	// 		$range = explode('-', $range);
	// 		$c_start = $range[0];
	// 		$c_end = (isset($range[1]) && is_numeric($range[1])) ? $range[1] : $size;
	// 	}
	// 	$c_end = ($c_end > $end) ? $end : $c_end;
	// 	if ($c_start > $c_end || $c_start > $size - 1 || $c_end >= $size) {
	// 		header('HTTP/1.1 416 Requested Range Not Satisfiable');
	// 		header("Content-Range: bytes $start-$end/$size");
	// 		exit;
	// 	}
	// 	$start = $c_start;
	// 	$end = $c_end;
	// 	$length = $end - $start + 1;
	// 	fseek($fp, $start);
	// 	header('HTTP/1.1 206 Partial Content');
	// 	header("Content-Range: bytes $start-$end/$size");
	// 	header("Content-Length: " . $length);
	// } else {
	// 	header("Content-Length: " . $length);
	// 	header("Accept-Ranges: bytes");
	// }
	// readfile( $file_path );

	// ob_start();
	// $buffer = 1024 * 8;
	// while (!feof($fp) && ($p = ftell($fp)) <= $end) {
	// 	if ($p + $buffer > $end) {
	// 		$buffer = $end - $p + 1;
	// 	}
	// 	set_time_limit(0);
	// 	$data = @stream_get_contents($fp, $buffer);

	// 	echo $data;
	// 	//echo fread($fp, $buffer);
	//    // ob_flush();
	// 	//flush();
	// }
	// fclose($fp);
	//exit();
}					
	

// Set type and size header
header( 'Content-Type: ' . $content_type );
header( 'Content-Length: ' . filesize( $file_path ) );

// Generate caching params and set headers
$last_modified = gmdate( 'D, d M Y H:i:s', filemtime( $file_path ) );
$etag          = '"' . sha1_file( $file_path ) . '"';

header( 'Last-Modified: ' . $last_modified . ' GMT' );
header( 'ETag: ' . $etag );
header( 'Cache-Control: private, must-revalidate' );

// Check caching headers send by client
if ( isset( $_SERVER['HTTP_IF_NONE_MATCH'] ) ) {
	// Check ETag
	$client_etag = stripslashes( $_SERVER['HTTP_IF_NONE_MATCH'] );
	$client_etag = substr( $client_etag, 0, strpos( $client_etag, '-' ) );

	if ( trim( $client_etag, '"' ) == trim( $etag, '"' ) ) {
		status_header( 304 );
		exit;
	}

} else if ( isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) ) {
	// Compare modified dates if no ETag
	$client_modified = strtotime( trim( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) );
	$last_modified   = strtotime( $last_modified );

	if ( $client_modified >= $last_modified ) {
		status_header( 304 );
		exit;
	}

}

// Access allowed and no cache match
readfile( $file_path );
