<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Form Creator
 * TODO: Re-develop in 3.0.0
 *
 * @version: 1.1.4
 * @package: Simpods MVC
 * @author: Andrew Palfrey & Luke Nelson
 * @since: 29/04/2018
 */
class Simpods_Forms {

	public $bootstrap_version   = 'bs4';
	private $defaults = array(
		'label'       => false,
		'type'        => 'text',
		'input-group' => false,
		'card'        => false,
		'id'          => false,
		'class'       => false,
		'placeholder' => false,
		'required'    => false,
		'readonly'    => false,
		'disabled'    => false,
		'minlength'   => false,
		'maxlength'   => false,
		'length'      => false,
		'min'         => false,
		'max'         => false,
		'date-time'   => false,
		'data-value'  => false,
		'pattern'     => false,
		'accept'      => false,
		'custom'      => false,
		'options'     => array(),
	);
	private $errors_defaults = array(
		'errors' => array(),
	);

	public function __construct( $can_init = true ) {
		global $funs;
		$settings = $funs->settings;

		/**
		 * Stores the default error messages in the variable.
		 * Could not be placed in when the variable is declared since
		 * errors are thrown when putting a function in a class' variable.
		 */
		$this->errors_defaults['errors'] = array(
			'error'          => esc_html__( 'This field is required.', 'simpods-mvc' ),
			'invalid'        => esc_html__( 'This field is not valid.', 'simpods-mvc' ),
			'min-max'        => esc_html__( 'This field length has to be between x and y.', 'simpods-mvc' ),
			'min-max-length' => esc_html__( 'This field length has to be between x and y characters long.', 'simpods-mvc' ),
			'length'         => esc_html__( 'This field length has to be x characters long.', 'simpods-mvc' ),
		);

		if ( !empty( $settings ) && isset( $settings['style_include_bootstrap'] ) && is_string( $settings['style_include_bootstrap'] ) ) {

			if ( $settings['style_include_bootstrap'] !== 'no' && $settings['style_include_bootstrap'] !== '' ) {
				$this->bootstrap_version = $settings['style_include_bootstrap'];
			}

		}

	}

	/**
	 * Generates simple fields; Text, Number, Email, Password, URL, File
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function basic_field( $name = '', $value = '', $attributes = array() ) {
		$css_class  = 'form-control';
		$html_attrs   = '';
		$html = '';

		if ( $attributes['class'] ) {
			$css_class .= ' ' . $attributes['class'];
		}

		if ( $value !== '' ) {
			$value = 'value="' . $value . '" ';
		}

		if ( $attributes['placeholder'] ) {
			$html_attrs .= 'placeholder="' . $attributes['placeholder'] . '" ';
		}

		if ( $attributes['minlength'] ) {
			$html_attrs .= 'minlength="' . $attributes['minlength'] . '" ';
		}

		if ( $attributes['maxlength'] ) {
			$html_attrs .= 'maxlength="' . $attributes['maxlength'] . '" ';
		}

		if ( $attributes['length'] ) {
			$html_attrs .= 'data-length="' . $attributes['length'] . '" ';
		}

		if ( $attributes['min'] ) {
			$html_attrs .= 'min="' . $attributes['min'] . '" ';
		}

		if ( $attributes['max'] ) {
			$html_attrs .= 'max="' . $attributes['max'] . '" ';
		}

		if ( $attributes['date-time'] ) {
			$html_attrs .= 'data-type="' . $attributes['date-time'] . '" ';
		}

		if ( $attributes['data-value'] ) {
			$html_attrs .= 'data-value="' . $attributes['data-value'] . '" ';
		}

		if ( $attributes['pattern'] ) {
			$html_attrs .= 'pattern="' . $attributes['pattern'] . '" ';
		}

		if ( $attributes['accept'] ) {
			$html_attrs .= 'accept="' . $attributes['accept'] . '" ';
		}

		if ( $attributes['readonly'] ) {
			$html_attrs .= 'readonly ';
		}

		if ( $attributes['required'] ) {
			$html_attrs .= 'required ';
		}

		if ( $attributes['disabled'] ) {
			$html_attrs .= 'disabled ';
		}

		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$base_css_class = 'card';

			if ( $this->bootstrap_version == 'bs3' ) {
				$base_css_class = 'panel';
			}

			$card_css_class = $base_css_class;

			if ( isset( $attributes['card']['theme'] ) && $attributes['card']['theme'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['theme'];
			} else {

				if ( $this->bootstrap_version == 'bs3' ) {
					$card_css_class .= ' panel-default';
				}

			}

			if ( isset( $attributes['card']['class'] ) && $attributes['card']['class'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['class'];
			}

			$html .= '<div class="' . $card_css_class . '">';

			if ( isset( $attributes['card']['header'] ) ) {
				$html .= $this->card( 'header', $attributes );
			}

			$html .= '<div class="' . $base_css_class . '-body">';

			if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
				$html .= '<div class="input-group';

				if ( isset( $attributes['input-group']['class'] ) ) {
					$html .= ' ' . $attributes['input-group']['class'];
				}

				$html .= '">';

				if ( isset( $attributes['input-group']['before'] ) ) {
					$html .= $this->input_group( 'before', $attributes );
				}

			}
		} else {
			if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
				$html .= '<div class="input-group">';

				if ( isset( $attributes['input-group']['before'] ) ) {
					$html .= $this->input_group( 'before', $attributes );
				}

			} else {
				if ( $attributes['label'] ) {
					$html .= $this->label( $attributes );
				}
			}

		}

		if ( $attributes['custom'] && $attributes['type'] == 'file' ) {
			$html .= '<div class="custom-file">';
			$css_class .= ' custom-file-input';
		}

		$html .= '<input type="' . $attributes['type'] . '" name="' . $name . '" id="' . $attributes['id'] . '" class="' . $css_class . '" ' . $value . $html_attrs . '/>';

		if ( $attributes['custom'] && $attributes['type'] == 'file' ) {
			$html .= '<label for="' . $attributes['id'] . '" class="custom-file-label">' . esc_html__( 'Choose File...', 'simpods-mvc' ) . '</label>';
			$html .= '</div>';
		}

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {

			if ( isset( $attributes['input-group']['after'] ) ) {
				$html .= $this->input_group( 'after', $attributes );
			}

			$html .= '</div>';
		}

		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$html .= '</div>';

			if ( isset( $attributes['card']['footer'] ) ) {
				$html .= $this->card( 'footer', $attributes );
			}

			$html .= '</div>';
		}

		$html .= $this->write_errors( $name, $attributes );

		return $html;
	}

	/**
	 * Generates checkbox & radio fields (including Bootstrap 4 custom versions).
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function check_radio_field( $name = '', $value = '', $attributes = array() ) {
		$css_class      = '';
		$html_attrs     = '';
		$min_max      	= '';
		$html      		= '';
		$min_max_added 	= false;

		if ( $attributes['custom'] ) {
			$css_class = 'custom-control-input';
		}

		if ( $attributes['class'] ) {
			$css_class .= ' ' . $attributes['class'];
		}

		if ( $attributes['min'] ) {
			$min_max .= 'min="' . $attributes['min'] . '" ';
		}

		if ( $attributes['max'] ) {
			$min_max .= 'max="' . $attributes['max'] . '" ';
		}

		if ( $attributes['required'] ) {
			$html_attrs .= 'required ';
		}

		if ( $attributes['disabled'] ) {
			$html_attrs .= 'disabled ';
		}

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
			$html .= '<div class="input-group">';

			if ( isset( $attributes['input-group']['before'] ) ) {
				$html .= $this->input_group( 'before', $attributes );
			}

		} else
		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$base_css_class = 'card';

			if ( $this->bootstrap_version == 'bs3' ) {
				$base_css_class = 'panel';
			}

			$card_css_class = $base_css_class;

			if ( isset( $attributes['card']['theme'] ) && $attributes['card']['theme'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['theme'];
			} else {

				if ( $this->bootstrap_version == 'bs3' ) {
					$card_css_class .= ' panel-default';
				}

			}

			if ( isset( $attributes['card']['class'] ) && $attributes['card']['class'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['class'];
			}

			$html .= '<div class="' . $card_css_class . '">';

			if ( isset( $attributes['card']['header'] ) ) {
				$html .= $this->card( 'header', $attributes );
			}

			$html .= '<div class="' . $base_css_class . '-body">';
		} else {

			if ( $attributes['label'] ) {
				$html .= $this->label( $attributes );
			}

		}

		foreach ( $attributes['options'] as $option_key => $option_value ) {
			$checked_html 	= '';
			$id      		= '';
			$label   		= '';
			$option_css_class 	= $css_class;

			if ( is_array( $value ) ) {

				if ( in_array( $option_key, $value ) ) {
					$checked_html = ' checked';
				}

			} else {

				if ( $option_key == $value ) {
					$checked_html = ' checked';
				}

			}

			if ( is_array( $option_value ) ) {

				if ( isset( $option_value['id'] ) ) {
					$id = $option_value['id'];
				}

				if ( isset( $option_value['label'] ) ) {
					$label = $option_value['label'];
				}

				if ( isset( $option_value['class'] ) ) {
					$option_css_class .= ' ' . $option_value['class'];
				}

			} else {
				$id    = sanitize_title( $option_key );
				$label = $option_value;
			}

			if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
				$form_control_css_class = 'form-control';

				if ( $attributes['disabled'] ) {
					$form_control_css_class .= ' disabled';
				}

				$html .= '<div class="' . $form_control_css_class . '">';
			}

			if ( $attributes['custom'] ) {
				$html .= '<div class="custom-control custom-' . $attributes['type'] . '">';
			} else {
				$html .= '<div class="' . $attributes['type'] . '">';
			}

			$html .= '<input type="' . $attributes['type'] . '" name="' . $name . '" id="' . $id . '" class="' . $option_css_class . '" value="' . $option_key . '" ' . $html_attrs;

			if ( $attributes['type'] == 'checkbox' && !$min_max_added ) {
				$html .= $min_max;
				$min_max_added = true;
			}

			$html .= $checked_html . '/>';

			if ( $attributes['custom'] ) {
				$html .= '<label for="' . $id . '" class="custom-control-label">' . $label . '</label>';
			} else {
				$html .= '<label for="' . $id . '">' . $label . '</label>';
			}

			$html .= '</div>';

			if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
				$html .= '</div>';
			}

		}

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {

			if ( isset( $attributes['input-group']['after'] ) ) {
				$html .= $this->input_group( 'after', $attributes );
			}

			$html .= '</div>';
		} else
		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$html .= '</div>';

			if ( isset( $attributes['card']['footer'] ) ) {
				$html .= $this->card( 'footer', $attributes );
			}

			$html .= '</div>';
		}

		$html .= $this->write_errors( $name, $attributes );

		return $html;
	}

	/**
	 * Generates select fields (including Bootstrap 4 custom versions).
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function select_field( $name = '', $value = '', $attributes = array() ) {
		$css_class  = 'form-control';
		$html_attrs   = '';
		$html = '';

		if ( $attributes['custom'] ) {
			$css_class .= ' custom-select';
		}

		if ( $attributes['class'] ) {
			$css_class .= ' ' . $attributes['class'];
		}

		if ( $attributes['minlength'] && $attributes['multiple'] ) {
			$html_attrs .= 'minlength="' . $attributes['minlength'] . '" ';
		}

		if ( $attributes['maxlength'] && $attributes['multiple'] ) {
			$html_attrs .= 'maxlength="' . $attributes['maxlength'] . '" ';
		}

		if ( $attributes['length'] && $attributes['multiple'] ) {
			$html_attrs .= 'data-length="' . $attributes['length'] . '" ';
		}

		if ( $attributes['multiple'] ) {
			$html_attrs .= 'multiple ';
		}

		if ( $attributes['readonly'] ) {
			$html_attrs .= 'readonly ';
		}

		if ( $attributes['required'] ) {
			$html_attrs .= 'required ';
		}

		if ( $attributes['disabled'] ) {
			$html_attrs .= 'disabled ';
		}

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
			$html .= '<div class="input-group">';

			if ( isset( $attributes['input-group']['before'] ) ) {
				$html .= $this->input_group( 'before', $attributes );
			}

		} else
		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$base_css_class = 'card';

			if ( $this->bootstrap_version == 'bs3' ) {
				$base_css_class = 'panel';
			}

			$card_css_class = $base_css_class;

			if ( isset( $attributes['card']['theme'] ) && $attributes['card']['theme'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['theme'];
			} else {

				if ( $this->bootstrap_version == 'bs3' ) {
					$card_css_class .= ' panel-default';
				}

			}

			if ( isset( $attributes['card']['class'] ) && $attributes['card']['class'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['class'];
			}

			$html .= '<div class="' . $card_css_class . '">';

			if ( isset( $attributes['card']['header'] ) ) {
				$html .= $this->card( 'header', $attributes );
			}

			$html .= '<div class="' . $base_css_class . '-body">';
		} else {

			if ( $attributes['label'] ) {
				$html .= $this->label( $attributes );
			}

		}

		$html .= '<select name="' . $name . '" id="' . $attributes['id'] . '" class="' . $css_class . '" ' . $html_attrs . '>';

		if (  (  ( is_string( $value ) && $value == '' ) || ( is_array( $value ) && empty( $value ) ) ) && !$attributes['multiple'] ) {
			$html .= '<option value="" selected>' . esc_html__( 'Please select one...', 'simpods-mvc' ) . '</option>';
		} else {

			if ( !$attributes['multiple'] ) {
				$html .= '<option value="">' . esc_html__( 'Please select one...', 'simpods-mvc' ) . '</option>';
			}

		}

		foreach ( $attributes['options'] as $option_key => $label ) {
			$selected_html = '';

			if ( is_array( $value ) ) {

				if ( in_array( $option_key, $value ) ) {
					$selected_html = ' selected';
				}

			} else {

				if ( $option_key == $value ) {
					$selected_html = ' selected';
				}

			}

			$html .= '<option value="' . $option_key . '"' . $selected_html . '>' . $label . '</option>';
		}

		$html .= '</select>';

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {

			if ( isset( $attributes['input-group']['after'] ) ) {
				$html .= $this->input_group( 'after', $attributes );
			}

			$html .= '</div>';
		} else
		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$html .= '</div>';

			if ( isset( $attributes['card']['footer'] ) ) {
				$html .= $this->card( 'footer', $attributes );
			}

			$html .= '</div>';
		}

		$html .= $this->write_errors( $name, $attributes );

		return $html;
	}

	/**
	 * Generates textarea fields.
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function textarea_field( $name = '', $value = '', $attributes = array() ) {
		$css_class  = 'form-control';
		$html_attrs   = '';
		$html = '';

		if ( $attributes['class'] ) {
			$css_class .= ' ' . $attributes['class'];
		}

		if ( $attributes['rows'] ) {
			$html_attrs .= 'rows="' . $attributes['rows'] . '" ';
		}

		if ( $attributes['placeholder'] ) {
			$html_attrs .= 'placeholder="' . $attributes['placeholder'] . '" ';
		}

		if ( $attributes['minlength'] ) {
			$html_attrs .= 'minlength="' . $attributes['minlength'] . '" ';
		}

		if ( $attributes['maxlength'] ) {
			$html_attrs .= 'maxlength="' . $attributes['maxlength'] . '" ';
		}

		if ( $attributes['length'] ) {
			$html_attrs .= 'data-length="' . $attributes['length'] . '" ';
		}

		if ( $attributes['readonly'] ) {
			$html_attrs .= 'readonly ';
		}

		if ( $attributes['required'] ) {
			$html_attrs .= 'required ';
		}

		if ( $attributes['disabled'] ) {
			$html_attrs .= 'disabled ';
		}

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {
			$html .= '<div class="input-group">';

			if ( isset( $attributes['input-group']['before'] ) ) {
				$html .= $this->input_group( 'before', $attributes );
			}

		} else
		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$base_css_class = 'card';

			if ( $this->bootstrap_version == 'bs3' ) {
				$base_css_class = 'panel';
			}

			$card_css_class = $base_css_class;

			if ( isset( $attributes['card']['theme'] ) && $attributes['card']['theme'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['theme'];
			} else {

				if ( $this->bootstrap_version == 'bs3' ) {
					$card_css_class .= ' panel-default';
				}

			}

			if ( isset( $attributes['card']['class'] ) && $attributes['card']['class'] !== '' ) {
				$card_css_class .= ' ' . $attributes['card']['class'];
			}

			$html .= '<div class="' . $card_css_class . '">';

			if ( isset( $attributes['card']['header'] ) ) {
				$html .= $this->card( 'header', $attributes );
			}

			$html .= '<div class="' . $base_css_class . '-body">';
		} else {

			if ( $attributes['label'] ) {
				$html .= $this->label( $attributes );
			}

		}

		$html .= '<textarea name="' . $name . '" id="' . $attributes['id'] . '" class="' . $css_class . '" ' . $html_attrs . '>' . $value . '</textarea>';

		if ( $attributes['input-group'] && is_array( $attributes['input-group'] ) ) {

			if ( isset( $attributes['input-group']['after'] ) ) {
				$html .= $this->input_group( 'after', $attributes );
			}

			$html .= '</div>';
		} else
		if ( $attributes['card'] && is_array( $attributes['card'] ) ) {
			$html .= '</div>';

			if ( isset( $attributes['card']['footer'] ) ) {
				$html .= $this->card( 'footer', $attributes );
			}

			$html .= '</div>';
		}

		$html .= $this->write_errors( $name, $attributes );

		return $html;
	}

	/**
	 * Generates labels for the fields (both basic and input groups).
	 * @param  array   $attributes       The options to apply to the field
	 * @param  boolean $is_input_group Whether the label is in an input group or not
	 * @return string
	 */
	private function label( $attributes = array(), $is_input_group = false ) {
		$html   = '';
		$label    = '';
		$id       = '';
		$css_class    = '';
		$required_html = '';
		$defaults  = array(
			'id'    => false,
			'text'  => false,
			'class' => false,
		);

		if ( $attributes['required'] ) {
			$required_html = ' <span class="required">*</span>';
		}

		if ( $is_input_group ) {

			if ( $this->bootstrap_version == 'bs4' ) {
				$html .= '<span class="input-group-text">';
			}

		}

		if ( is_array( $attributes['label'] ) ) {
			$label_arr = wp_parse_args( $attributes['label'], $defaults );

			if ( $label_arr['id'] ) {
				$id = ' id="' . $label_arr['id'] . '"';
			}

			if ( $label_arr['class'] && !$is_input_group ) {
				$css_class .= ' class="col-form-label ' . $label_arr['class'] . '"';
			} else {

				if ( $is_input_group ) {

					if ( $label_arr['class'] ) {
						$css_class .= ' class="mgb0 ' . $label_arr['class'] . '"';
					} else {
						$css_class .= ' class="mgb0"';
					}

				}

			}

			$html .= '<label for="' . $attributes['id'] . '"' . $id . $css_class . '>' . $label_arr['text'] . $required_html . '</label>';
		} else {
			$html .= '<label for="' . $attributes['id'] . '" class="mgb0">' . $attributes['label'] . $required_html . '</label>';
		}

		if ( $is_input_group ) {

			if ( $this->bootstrap_version == 'bs4' ) {
				$html .= '</span>';
			}

		}

		return $html;
	}

	/**
	 * Generates input group addons (both before and after the field).
	 * @param  string $type Whether the addon is before or after the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function input_group( $type = 'before', $attributes = array() ) {
		$html 	= '';
		$items  = array();

		if ( $type == 'before' ) {

			if ( $this->bootstrap_version == 'bs3' ) {
				$html .= '<div class="input-group-addon">';
			} else
			if ( $this->bootstrap_version == 'bs4' ) {
				$html .= '<div class="input-group-prepend">';
			}

			$items = $attributes['input-group']['before'];
		} else
		if ( $type == 'after' ) {

			if ( $this->bootstrap_version == 'bs3' ) {
				$html .= '<div class="input-group-addon">';
			} else
			if ( $this->bootstrap_version == 'bs4' ) {
				$html .= '<div class="input-group-append">';
			}

			$items = $attributes['input-group']['after'];
		}

		if ( is_array( $items ) ) {

			foreach ( $items as $item ) {

				if ( $item == 'label' ) {
					$html .= $this->label( $attributes, true );
				} else {
					$html .= $item;
				}

			}

		} else {

			if ( $items == 'label' ) {
				$html .= $this->label( $attributes, true );
			} else {
				$html .= $items;
			}

		}

		$html .= '</div>';

		return $html;
	}

	/**
	 * Generates card/panel headers and footers.
	 * @param  string $type Whether the item is the header or the footer
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function card( $type = 'header', $attributes = array() ) {
		$html    = '';
		$base_css_class = 'card';
		$item      = '';

		if ( $this->bootstrap_version == 'bs3' ) {
			$base_css_class = 'panel';
		}

		if ( $type == 'header' ) {
			$item = $attributes['card']['header'];

			if ( $this->bootstrap_version == 'bs3' ) {
				$type = 'heading';
			}

		} else
		if ( $type == 'footer' ) {
			$item = $attributes['card']['footer'];
		}

		$html .= '<div class="' . $base_css_class . '-' . $type . '">';

		if ( $item == 'label' ) {
			$html .= $this->label( $attributes );
		} else {
			$html .= $item;
		}

		$html .= '</div>';

		return $html;
	}

	/**
	 * Generates the error message blocks for each field
	 * @param  string $name The name of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	private function write_errors( $name = '', $attributes = array() ) {
		$html = '';

		foreach ( $attributes['errors'] as $error => $message ) {
			$id    = $name . '-' . $error;
			$css_class = '';

			if ( $this->bootstrap_version == 'bs3' ) {
				$css_class .= 'help-block hidden';
			} else
			if ( $this->bootstrap_version == 'bs4' ) {
				$css_class .= 'form-feedback d-none';
			}

			$html .= '<span id="' . $id . '" class="' . $css_class . '">' . $message . '</span>';
		}

		return $html;
	}

	/**
	 * Called by the template, prepares the generation of a 'text' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function text( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'text',
			'id'   => $name,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if (  ( $attributes['minlength'] || $attributes['maxlength'] ) && !isset( $attributes['errors']['min-max-length'] ) ) {
			//$error = esc_html__( 'This field length has to be ', 'simpods-mvc' );

			if ( $attributes['minlength'] && $attributes['maxlength'] ) {
				/* translators: %1$d: minimum number, %2$d: maximum number */
				$error = sprintf( esc_html__( 'This field length has to be between %1$d and %2$d characters long.', 'simpods-mvc' ), $attributes['minlength'], $attributes['maxlength'] );
				//$error .= esc_html__( 'between ', 'simpods-mvc' ) . $attributes['minlength'] . esc_html__( ' and ', 'simpods-mvc' ) . $attributes['maxlength'];
			} else
			if ( $attributes['minlength'] && !$attributes['maxlength'] ) {
				$error = esc_html( sprintf(
					/* translators: %d minimum number */
					_n(
						'This field length has to be at least %d character long.',
						'This field length has to be at least %d characters long.',
						$attributes['minlength'],
						'simpods-mvc'
					),
					$attributes['minlength']
				) );
				//$error .= esc_html__( 'at least ', 'simpods-mvc' ) . $attributes['minlength'];
			} else
			if ( !$attributes['minlength'] && $attributes['maxlength'] ) {
				$error = esc_html( sprintf(
					/* translators: %d maximum number */
					_n(
						'This field length has to be at most %d character long.',
						'This field length has to be at most %d characters long.',
						$attributes['minlength'],
						'simpods-mvc'
					),
					$attributes['minlength']
				) );
				//$error .= esc_html__( 'at most ', 'simpods-mvc' ) . $attributes['maxlength'];
			}

			//$error .= esc_html__( ' characters long.', 'simpods-mvc' );

			$attributes['errors']['min-max-length'] = $error;
		}

		if ( $attributes['length'] && !isset( $attributes['errors']['length'] ) ) {
			$attributes['errors']['length'] = esc_html( sprintf(
				/* translators: %d required number */
				_n(
					'This field length has to be %d character long.',
					'This field length has to be %d characters long.',
					$attributes['length'],
					'simpods-mvc'
				),
				$attributes['length']
			) );
		}

		if (  ( $attributes['date-time'] || $attributes['pattern'] ) && !isset( $attributes['errors']['invalid'] ) ) {

			if ( $attributes['date-time'] ) {
				/* translators: %s field format */
				$attributes['errors']['invalid'] = sprintf( esc_html__( 'This field is not a valid %s.', 'simpods-mvc' ), str_replace( '-', '/', $attributes['date-time'] ) );
			} else
			if ( $attributes['pattern'] ) {
				/* translators: %s field format pattern */
				$attributes['errors']['invalid'] = sprintf( esc_html__( 'This field is not valid based on the "%s" pattern', 'simpods-mvc' ), '<code>' . str_replace( '-', '/', $attributes['pattern'] ) . '</code>' );
				/* $attributes['errors']['invalid'] = esc_html__( 'This field is not valid based on the "', 'simpods-mvc' );
				$attributes['errors']['invalid'] .= '<code>' . str_replace( '-', '/', $attributes['pattern'] ) . '</code>';
				$attributes['errors']['invalid'] .= esc_html__( '"pattern', 'simpods-mvc' ); */
			}

		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->basic_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a 'number' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function number( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'number',
			'id'   => $name,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if ( !isset( $attributes['errors']['invalid'] ) ) {
			$attributes['errors']['invalid'] = esc_html__( 'This is not a valid number.', 'simpods-mvc' );
		}

		if (  ( $attributes['min'] || $attributes['max'] ) && !isset( $attributes['errors']['min-max'] ) ) {
			//$error = esc_html__( 'This field has to be ', 'simpods-mvc' );

			if ( $attributes['min'] && $attributes['max'] ) {
				/* translators: %1$d: minimum number, %2$d: maximum number */
				$error = sprintf( esc_html__( 'This field has to be between %1$d and %2$d.', 'simpods-mvc' ), $attributes['min'], $attributes['max'] );
				//$error .= esc_html__( 'between ', 'simpods-mvc' ) . $attributes['min'] . esc_html__( ' and ', 'simpods-mvc' ) . $attributes['max'] . '.';
			} else
			if ( $attributes['min'] && !$attributes['max'] ) {
				/* translators: %d: minimum number */
				$error = sprintf( esc_html__( 'This field has to be greater than or equal to %d.', 'simpods-mvc' ), $attributes['min'] );
				//$error .= esc_html__( 'more than or equal to ', 'simpods-mvc' ) . $attributes['min'] . '.';
			} else
			if ( !$attributes['min'] && $attributes['max'] ) {
				/* translators: %d: maximum number */
				$error = sprintf( esc_html__( 'This field has to be less than or equal to %d.', 'simpods-mvc' ), $attributes['max'] );
				//$error .= esc_html__( 'less than or equal to ', 'simpods-mvc' ) . $attributes['max'] . '.';
			}

			$attributes['errors']['min-max'] = $error;
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->basic_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of an 'email' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function email( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'email',
			'id'   => $name,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if ( !isset( $attributes['errors']['invalid'] ) ) {
			$attributes['errors']['invalid'] = esc_html__( 'This is not a valid email address.', 'simpods-mvc' );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->basic_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a 'password' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function password( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'password',
			'id'   => $name,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if ( !isset( $attributes['errors']['invalid'] ) ) {
			$attributes['errors']['invalid'] = esc_html__( 'This is not a valid password.', 'simpods-mvc' );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->basic_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a 'url' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function url( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'url',
			'id'   => $name,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if ( !isset( $attributes['errors']['invalid'] ) ) {
			$attributes['errors']['invalid'] = esc_html__( 'This is not a valid URL.', 'simpods-mvc' );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->basic_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a 'file' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function file( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'file',
			'id'   => $name,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if ( !isset( $attributes['errors']['invalid'] ) ) {
			$attributes['errors']['invalid'] = esc_html__( 'This is not a valid file type.', 'simpods-mvc' );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->basic_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a set of 'checkbox' fields
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function checkbox( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'checkbox',
			'id'   => str_replace( '[', '', str_replace( ']', '', $name ) ),
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if (  ( $attributes['min'] || $attributes['max'] ) && !isset( $attributes['errors']['invalid'] ) ) {
			//$error = esc_html__( 'You have to select ', 'simpods-mvc' );

			if ( $attributes['min'] && $attributes['max'] ) {
				/* translators: %1$d: minimum number, %2$d: maximum number */
				$error = sprintf( esc_html__( 'You have to select between %1$d and %2$d options.', 'simpods-mvc' ), $attributes['min'], $attributes['max'] );
				//$error .= esc_html__( ' ', 'simpods-mvc' ) . $attributes['min'] . esc_html__( ' and ', 'simpods-mvc' ) . $attributes['max'];
			} else
			if ( $attributes['min'] && !$attributes['max'] ) {
				$error = esc_html( sprintf(
					/* translators: %d minimum number */
					_n(
						'You have to select at least %d option.',
						'You have to select at least %d options.',
						$attributes['min'],
						'simpods-mvc'
					),
					$attributes['min']
				) );
				//$error .= esc_html__( 'at least ', 'simpods-mvc' ) . $attributes['min'];
			} else
			if ( !$attributes['min'] && $attributes['max'] ) {
				$error = esc_html( sprintf(
					/* translators: %d maximum number */
					_n(
						'You have to select at most %d option.',
						'You have to select at most %d options.',
						$attributes['max'],
						'simpods-mvc'
					),
					$attributes['max']
				) );
				//$error .= esc_html__( 'at most ', 'simpods-mvc' ) . $attributes['max'];
			}

			//$error = esc_html__( ' options.', 'simpods-mvc' );

			$attributes['errors']['invalid'] = $error;
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->check_radio_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a set of 'radio' fields
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function radio( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'type' => 'radio',
			'id'   => str_replace( '[', '', str_replace( ']', '', $name ) ),
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( !isset( $attributes['errors']['invalid'] ) ) {
			$attributes['errors']['invalid'] = esc_html__( 'Please select an option.', 'simpods-mvc' );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->check_radio_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a 'select' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function select( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'id'       => $name,
			'multiple' => false,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if (  ( $attributes['minlength'] || $attributes['maxlength'] ) && !isset( $attributes['errors']['min-max-length'] ) ) {
			//$error = esc_html__( 'You have to select ', 'simpods-mvc' );

			if ( $attributes['minlength'] && $attributes['maxlength'] ) {
				/* translators: %1$d: minimum number, %2$d: maximum number */
				$error = sprintf( esc_html__( 'You have to select between %1$d and %2$d options.', 'simpods-mvc' ), $attributes['minlength'], $attributes['maxlength'] );
				//$error .= esc_html__( 'between ', 'simpods-mvc' ) . $attributes['minlength'] . esc_html__( ' and ', 'simpods-mvc' ) . $attributes['maxlength'];
			} else
			if ( $attributes['minlength'] && !$attributes['maxlength'] ) {
				/* translators: %d minimum number */
				$error = esc_html( sprintf(
					_n(
						'You have to select at least %d option.',
						'You have to select at least %d options.',
						$attributes['minlength'],
						'simpods-mvc'
					),
					$attributes['minlength']
				) );
				//$error .= esc_html__( 'at least ', 'simpods-mvc' ) . $attributes['minlength'];
			} else
			if ( !$attributes['minlength'] && $attributes['maxlength'] ) {
				/* translators: %d maximum number */
				$error = esc_html( sprintf(
					_n(
						'You have to select at most %d option.',
						'You have to select at most %d options.',
						$attributes['maxlength'],
						'simpods-mvc'
					),
					$attributes['maxlength']
				) );
				//$error .= esc_html__( 'at most ', 'simpods-mvc' ) . $attributes['maxlength'];
			}

			//$error .= esc_html__( ' options.', 'simpods-mvc' );

			$attributes['errors']['min-max-length'] = $error;
		}

		if ( $attributes['length'] && !isset( $attributes['errors']['length'] ) ) {
			/* translators: %d required number */
			$attributes['errors']['length'] = esc_html( sprintf(
				_n(
					'You have to select $d option.',
					'You have to select $d options.',
					$attributes['length'],
					'simpods-mvc'
				),
				$attributes['length']
			) );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->select_field( $name, $value, $attributes );
	}

	/**
	 * Called by the template, prepares the generation of a 'textarea' field
	 * @param  string $name The name of the field
	 * @param  string $value  The value of the field
	 * @param  array  $attributes The options to apply to the field
	 * @return string
	 */
	public function textarea( $name = '', $value = '', $attributes = array() ) {
		$defaults = array(
			'id'   => $name,
			'rows' => 3,
		);

		$attributes = wp_parse_args( $attributes, $defaults );
		$attributes = wp_parse_args( $attributes, $this->defaults );

		if ( !isset( $attributes['errors'] ) ) {
			$attributes['errors'] = array();
		}

		if ( $attributes['required'] && !isset( $attributes['errors']['error'] ) ) {
			$attributes['errors']['error'] = $this->errors_defaults['errors']['error'];
		}

		if (  ( $attributes['minlength'] || $attributes['maxlength'] ) && !isset( $attributes['errors']['min-max-length'] ) ) {
			//$error = esc_html__( 'This field length has to be ', 'simpods-mvc' );

			if ( $attributes['minlength'] && $attributes['maxlength'] ) {
				/* translators: %1$d: minimum number, %2$d: maximum number */
				$error = sprintf( esc_html__( 'This field length has to be between %1$d and %2$d characters long.', 'simpods-mvc' ), $attributes['minlength'], $attributes['maxlength'] );
				//$error .= esc_html__( 'between ', 'simpods-mvc' ) . $attributes['minlength'] . esc_html__( ' and ', 'simpods-mvc' ) . $attributes['maxlength'];
			} else
			if ( $attributes['minlength'] && !$attributes['maxlength'] ) {
				/* translators: %d minimum number */
				$error = esc_html( sprintf(
					_n(
						'This field length has to be at least %d character long.',
						'This field length has to be at least %d characters long.',
						$attributes['minlength'],
						'simpods-mvc'
					),
					$attributes['minlength']
				) );
				//$error .= esc_html__( 'at least ', 'simpods-mvc' ) . $attributes['minlength'];
			} else
			if ( !$attributes['minlength'] && $attributes['maxlength'] ) {
				/* translators: %d maximum number */
				$error = esc_html( sprintf(
					_n(
						'This field length has to be at most %d character long.',
						'This field length has to be at most %d characters long.',
						$attributes['minlength'],
						'simpods-mvc'
					),
					$attributes['minlength']
				) );
				//$error .= esc_html__( 'at most ', 'simpods-mvc' ) . $attributes['maxlength'];
			}

			//$error .= esc_html__( ' characters long.', 'simpods-mvc' );

			$attributes['errors']['min-max-length'] = $error;
		}

		if ( $attributes['length'] && !isset( $attributes['errors']['length'] ) ) {
			/* translators: %d required number */
			$attributes['errors']['length'] = esc_html( sprintf(
				_n(
					'This field length has to be $d character long.',
					'This field length has to be $d characters long.',
					$attributes['length'],
					'simpods-mvc'
				),
				$attributes['length']
			) );
			//$attributes['errors']['length'] = esc_html__( 'This field length has to be ', 'simpods-mvc' ) . $attributes['length'] . esc_html__( ' characters long.', 'simpods-mvc' );
		}

		$attributes = wp_parse_args( $attributes, $this->errors_defaults );

		return $this->textarea_field( $name, $value, $attributes );
	}

}

$GLOBALS['simpods_forms'] = new Simpods_Forms();

class_alias( 'Simpods_Forms', 'MVC_Front_Forms' );
