<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Simpods functions, a collection of functions that are needed before the MVC
 *
 * Global instance avalible in global $funs
 * 
 * @package Simpods MVC
 * @author Dongjie Xu
 * @since 26/09/2017
 */
class Simpods_Functions {
	public $settings = array();
	/**
	 * include or enqueue files.
	 * @param  array $files    files array array( 'name' => 'mvc/hi.php')
	 * @param  boolean $is_child_first if true, and if the child file exist, include the child one instead of the parent one
	 * @param  array $depths depth array to enqueue the files
	 * @param  boolean $can_show_error echo errors
	 * @return array $depths the populated depth array
	 */
	public function __construct() {
		$this->settings = $this->get_options();
		if( !defined( 'SIMPODS_ALL_TABLES' ) ){
			$tables = $this->get_tables();
			define( 'SIMPODS_ALL_TABLES', serialize( $tables ) );
		}
	}
	/**
	 * Retrieves contents of the main Simpods MVC settings table
	 *
	 * @return mixed[] associative array of settings
	 */
	private function get_options() {
		if ( !get_option( 'simpods_settings_table_created' ) ) {
			return array();
		}
		global $wpdb;
		$query      = 'SELECT * FROM `' . $wpdb->prefix . 'simpods_settings` ';
		$db_options = $wpdb->get_results( $query, ARRAY_A );
		$options    = array();
		foreach ( $db_options as $option ) {
			$key           = $option['sps_option'];
			$val           = $option['sps_value'];
			$options[ $key ] = $val;
		}
		return $options;
	}
	/**
	 * Loads additional files
	 * JavaScript and CSS files are enqueued, PHP files are included
	 *
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/include_fn/
	 * @param  array   $files      Associative array of files: handle => filepath
	 * @param  boolean $is_child_first If files can be used from the child theme (default true)
	 * @param  array   $depths      Arrays of CSS, JS & PHP dependency handles (default empty)
	 * @param  boolean $can_show_error    If an error should be shown for non-existent files (default false)
	 * @param  array   $attrs      Array of attributes, see online documentation (default empty)
	 * @return array   $depths	   Arrays of CSS, JS & PHP dependency handles
	 */
	public function include( $files, $is_child_first = true, $depths = array(), $can_show_error = false, $attrs = array() ) {
		global $wp_filesystem;
		$depths_copied = $depths;
		$defaults = array(
			'js'  => array(),
			'css' => array(),
			'php' => array(),
		);
		if ( is_admin() ) {
			if ( class_exists( 'MVC_Admin_Factory' ) ) {
				$defaults = array(
					'js'  => MVC_Admin_Parent_Factory::$scripts,
					'css' => MVC_Admin_Parent_Factory::$styles,
					'php' => MVC_Admin_Factory::$php_files,
				);
			}
		} else {
			if ( class_exists( 'MVC_Front_Factory' ) ) {
				$defaults = array(
					'js'  => MVC_Front_Factory::$scripts,
					'css' => MVC_Front_Factory::$styles,
					'php' => MVC_Front_Factory::$php_files,
				);
			}
		} 
		/**
		 * Parse incoming $args into an array and merge it with $defaults
		 */
		$depths = wp_parse_args( $depths, $defaults );
		foreach ( $files as $key => $file ) {
			if ( !is_string( $file ) ) {
				continue;
			}
			if( strpos( $file, WP_CONTENT_URL ) !== 0 ){ //WP_SITEURL has the priority over WP_CONTENT_URL. When they are not the same, files are not loaded.
				$file = '/' . ltrim( $file, '/' ); 
			}
			$key     = str_replace( '/', '-', $key );
			$file_url     = '';
			$file_location = '';
			if ( !empty( $this->settings ) && isset( $this->settings['style_load_min'] ) && $this->settings['style_load_min'] == '1' ) {
				$locations = $this->fetch_path_and_url( $file, false, $is_child_first );
			} else {
				
				$locations = $this->fetch_path_and_url( $file, true, $is_child_first );
			}

			$file_url      = $locations['file_url'];
			$file_location = $locations['file_path'];
			
			if ( $file_url != '' ) {
				if ( substr( $file_url, -3 ) === '.js' ) {
					// if ( substr( $key, -3 ) == '-js' ) { // avoid -js-js
					// 	//$key .= '-js';
					// 	$key = substr( $key, 0, -3 );
					// }
					
					if ( !in_array( $key, $depths['js'] ) ) {
						$version      	= time() ; // No cache by default
						$version 		= apply_filters( 'simpods_include_js_version', $version );						
						$is_in_footer 	= false;
						if ( isset( $attrs[ $key ] ) ) {
							if ( isset( $attrs[ $key ]['version'] ) && $attrs[ $key ]['version'] !== false ) {
								$version = $attrs[ $key ]['version'];
							}
							if ( isset( $attrs[ $key ]['in_footer'] ) && $attrs[ $key ]['in_footer'] ) {
								$is_in_footer = true;
							}
						}
						wp_register_script( $key, $file_url, $depths['js'], $version, $is_in_footer );
						if( ! isset( $attrs[ $key ] ) || empty( $attrs[ $key ]['register_only'] ) ){	// If register only, don't enqueue. Useful for shortcodes: you can enqueue its scripts only when a shortcode is present 											
							wp_enqueue_script( $key );		
							
						}  
						array_push( $depths['js'], $key );
					}
				}
				if ( substr( $file_url, -4 ) === '.css' ) {
					/*if ( substr( $key, -4 ) !== '-css' ) {
						$key .= '-css';
					}*/
					// if ( substr( $key, -4 ) == '-css' ) { // avoid -css-css
					// 	$key = substr( $key, 0, -4 );
					// }
					if ( !in_array( $key, $depths['css'] ) ) {
						$version    = time() ; // No cache by default
						$version 	= apply_filters( 'simpods_include_css_version', $version );
						$media 		= false;
						if ( isset( $attrs[ $key ] ) ) {
							if ( isset( $attrs[ $key ]['version'] ) && $attrs[ $key ]['version'] !== false ) {
								$version = $attrs[ $key ]['version'];
							}
							if ( isset( $attrs[ $key ]['media'] ) && $attrs[ $key ]['media'] ) {
								$media = true;
							}
						}
						wp_register_style( $key, $file_url, $depths['css'], $version, $media );
						if( ! isset( $attrs[ $key ] ) || empty( $attrs[ $key ]['register_only'] ) ){	// If register only, don't enqueue. Useful for shortcodes: you can enqueue its styles only when a shortcode is present						
							wp_enqueue_style( $key );
							
						} 
						array_push( $depths['css'], $key );
					}
				}

				if ( strpos( $file_location, '.php' ) ) {					
					if( $wp_filesystem->exists( $file_location ) ){
						include_once $file_location;
						array_push( $depths['php'], $file_location );
					}
				}
			} else {
				if ( $can_show_error ) {
					echo esc_attr( $file ) . ' ' . esc_html__( 'not found', 'simpods-mvc' );
				}
			}
		}
		if ( is_admin() ) {
			if ( class_exists( 'MVC_Admin_Factory' ) ) {
				MVC_Admin_Parent_Factory::$scripts  = $depths['js']; 
				MVC_Admin_Parent_Factory::$styles   = $depths['css'];
				MVC_Admin_Factory::$php_files = $depths['php'];
			}
		} else {
			if ( class_exists( 'MVC_Front_Factory' ) ) {
				MVC_Front_Factory::$scripts  = $depths['js'];
				MVC_Front_Factory::$styles   = $depths['css'];
				MVC_Front_Factory::$php_files = $depths['php'];
			}
		}

		return $depths;
	}
	/**
	 * return files locations
	 * @param  string  $file    file name, e.g. a.php, a.js, a.css
	 * @param  boolean $can_skip_minified skip checking minified js and css or not
	 * @param  boolean $is_child_first If true, it means the file can be overwritten based on the this order child > parent > simpods plugin
	 * @return array   return file path and url
	 */
	public function fetch_path_and_url( $file, $can_skip_minified = false, $is_child_first = true ){
		global $wp_filesystem; 
		$returns  = array(
			'file_url'     => '',
			'file_path' => '',
		);
		$file_url     = '';
		$file_path = '';
		$try_min  = false;
		$original_file = $file;
		$priority_url  = '';
		$secondary_url = '';
		$third_url = '';
		$priority_path  = '';
		$secondary_path = '';
		$third_path 		= '';
		if( strpos( $file, WP_CONTENT_URL ) !== 0 ){
			$file = '/' . ltrim( $file, '/' );
		} 
		//minified versions take higher priority
		if( ! $can_skip_minified ){
			if( substr( $file, -3 ) === '.js' && substr( $file, -7 ) !== '.min.js' ){
				$file   = substr( $file, 0, -3 ) . '.min.js';
				$try_min	= true;
			}
			if( substr( $file, -4 ) === '.css' && substr( $file, -8 ) !== '.min.css' ){
				$file   = substr( $file, 0, -4 ) . '.min.css';
				$try_min	= true;
			}
		} 
		if( stripos( $file, WP_PLUGIN_URL ) !== false || stripos( $file, WP_PLUGIN_DIR ) !== false ){ // this is a plugin file
			
			// remove WP_PLUGIN_DIR and WP_PLUGIN_URL
			$plugin_url			= WP_PLUGIN_URL . '/';
			$file_short_path 	= str_replace( $plugin_url, '', $file ); 
			$plugin_uri			= WP_PLUGIN_DIR . '/';
			$file_short_path 	= str_replace( $plugin_uri, '', $file_short_path ); 

			$priority_url  		= ltrim( WP_PLUGIN_URL . '/' . $file_short_path, '/');		
			
			$priority_path  	= WP_PLUGIN_DIR . '/' . $file_short_path;		
			$priority_path  	= str_replace( '//', '/', $priority_path );						
			// $file_brs 	= explode( '/', $file_short_path );
			// $file		= $plugin_url . implode( '/', $file_brs ); 		
			
			//$priority_url  	= ltrim( $file, '/');
			//$priority_path  = str_replace( WP_CONTENT_URL, WP_CONTENT_DIR, $file );
			//$priority_path  = str_replace( '//', '/', $priority_path );
		
		} else {
			if ( $is_child_first ) {
				$priority_url  	= get_stylesheet_directory_uri() . $file;
				$priority_path  = get_stylesheet_directory() . $file;
				$secondary_url 	= get_template_directory_uri() . $file;
				$secondary_path = get_template_directory() . $file;
				$third_url 		= SIMPODS_URL . $file;
				$third_path 	= SIMPODS_DIR . $file;
			} else {
				$priority_url  	= SIMPODS_URL . $file;
				$priority_path  = SIMPODS_DIR . $file;
				$secondary_url 	= get_template_directory_uri() . $file;
				$secondary_path = get_template_directory() . $file;
				$third_url 		= get_stylesheet_directory_uri() . $file;
				$third_path 	= get_stylesheet_directory() . $file;		
			}
		}

		if ( $wp_filesystem->exists( $priority_path ) ) { 
			$file_url  = $priority_url;
			$file_path 	= $priority_path;
		} else if ( $wp_filesystem->exists( $secondary_path ) ) { 
			$file_url  = $secondary_url;
			$file_path 	= $secondary_path;			
		} else { 
	 		if ( $wp_filesystem->exists( $third_path ) ) { 
				$file_url  = $third_url;
				$file_path 	= $third_path;		
			}
		}

		$returns['file_url']     = $file_url;
		$returns['file_path'] = $file_path;
		
		// if the minified version not found, try the original files
		if ( $try_min == true && $returns['file_url'] == '' ) {
			$returns	= $this->fetch_path_and_url( $original_file, true, $is_child_first );
		}
		return $returns;
	}
	/**
	 * Determine if an Ajax request is coming from the front-end or back-end
	 *
	 * @link https://simpodsmvc.me/docs/function-reference/functions/is_frontend_ajax_fn/
	 * @link https://snippets.khromov.se/determine-if-wordpress-ajax-request-is-a-backend-of-frontend-request/
	 * @author Stanislav Khromov
	 * @return boolean True if front-end, False if back-end
	 */
	public function is_frontend_ajax() {
		$script_filename = isset( $_SERVER['SCRIPT_FILENAME'] ) ? $_SERVER['SCRIPT_FILENAME'] : '';
		// Try to figure out if frontend AJAX request... If we are DOING_AJAX; let's look closer
		if (  ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
			// From wp-includes/functions.php, wp_get_referer() function.
			// Required to fix: https://core.trac.wordpress.org/ticket/25294
			$ref = '';
			if ( !empty( $_REQUEST['_wp_http_referer'] ) ) {
				$ref = wp_unslash( $_REQUEST['_wp_http_referer'] );
			} elseif ( !empty( $_SERVER['HTTP_REFERER'] ) ) {
				$ref = wp_unslash( $_SERVER['HTTP_REFERER'] );
			}
			$base_name = basename( $script_filename );
			// If referer does not contain admin URL and we are using the admin-ajax.php endpoint, this is likely a frontend AJAX request
			if ( ( ( strpos( $ref, admin_url() ) === false ) && ( $base_name === 'admin-ajax.php' ) || $base_name == 'async-upload.php' ) ) {
				return true;
			}
		}
		// If no checks triggered, we end up here - not an AJAX request.
		return false;
	}
	/**
	 * Return the file path of a Controller
	 * This function takes Multisites and Child Themes into account
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_final_path_fn/
	 * @param  string  $folder 	  Folder if exists
	 * @param  string  $file   	  File path
	 * @param  string  $is_url    	  If true, return the URL instead of the file path (default false)
	 * @param  boolean $is_relative  Whether to include directory or not (default false)
	 * @param  boolean $is_multisites_ignored  Whether to ignore Multisite (default false)
	 * @param  boolean $to_get_details   Whether to return the file path details or just the path (default false)
	 * @return mixed             	  If $to_get_details is false, returns a string of the final file URI. Otherwse, returns an array of details
	 */
	public function get_final_path( $folder = '', $file = '', $is_url = false, $is_relative = false, $is_multisites_ignored = false, $to_get_details = false ) {
		global $wp_filesystem;
		$original_file = $file;
		if ( is_multisite() && ! $is_multisites_ignored ) {
			$file = '' . get_current_blog_id() . '/' . ltrim( $file, '/' );
		}
		$return_data = array(
			'found' => false,
			'uri'   => '',
			'url'   => '',
			'path'  => '',
			'where' => '',
		); 
	
		if( stripos( $folder, WP_PLUGIN_DIR ) !== false || stripos( $file, WP_PLUGIN_DIR ) !== false  ){ // specified to be in a plugin folder or a plugin file, just return it.
			if( ! empty( $folder ) ){
				$final_url  = trailingslashit( $folder ) . $file; 
			} else {
				$final_url  = $file; 
			}
			//echo $final_url .'<br/>';
			if( $wp_filesystem->exists( $final_url ) ){
				
				$return_data['found'] 	= true;
				$return_data['where'] 	= 'plugin';
				$return_data['path']  	= $final_url;				
				$return_data['uri'] 	= $final_url;
				$return_data['url'] 	= trailingslashit( str_replace( WP_CONTENT_DIR, WP_CONTENT_URL,  $folder ) ) . trim( $file, '/' );	
				
			}
			if ( !$to_get_details ) {
				return $final_url;
			} else {
				return $return_data;
			}				
		}

		$template_dir = '';
		$style_dir = '';
		$simpods_dir 	=	'';
		if ( !$is_relative ) {
			$template_dir = rtrim( get_template_directory(), '/' );
			$style_dir = rtrim( get_stylesheet_directory(), '/' );
			$template_uri = rtrim( get_template_directory_uri(), '/' );
			$style_uri = rtrim( get_stylesheet_directory_uri(), '/' );
			$simpods_dir = SIMPODS_DIR;
		}
		$file  = trim( $file, '/' );
		$folder= trim( $folder, '/' );
		$final_path = trailingslashit( $template_dir . '/' . $folder ) . $file;
		$child_path = trailingslashit( $style_dir . '/' . $folder ) . $file;
		$where 		= '';
		$does_final_path_exist = $wp_filesystem->exists( $final_path );
		$does_child_path_exist = $wp_filesystem->exists( $child_path );

		if ( is_child_theme() !== false && $does_child_path_exist && !$wp_filesystem->is_dir( $child_path ) ) {
			$return_data['found'] 	= true;
			$final_path           	= $child_path;
			$where 					= 'child';
		} else
		if ( $does_final_path_exist && !$wp_filesystem->is_dir( $final_path ) ) {
			$return_data['found'] 	= true;
			$where 					= 'parent';

		} else {

			// if both not exist,  try the simpods mvc plugin
			$final_url  = trailingslashit( $simpods_dir . '/' . $folder ) . $file;
			$final_path 	= $final_url;
			//echo $final_url .'<br/>';
			if( $wp_filesystem->exists( $final_url ) ){	
				
				$where 		= 'plugin-simpods';
				$return_data['found'] = true;
			}
		}
		// if not exists, and it is a multisite, try without blog id
		if ( $return_data['found'] == false && is_multisite() ) {
			$final_path = trailingslashit( $template_dir . '/' . $folder ) . trim( $original_file, '/' );
			$child_path = trailingslashit( $style_dir . '/' . $folder ) . trim( $original_file, '/' );
			$child_bln = true;
			$does_final_path_exist = $wp_filesystem->exists( $final_path );
			$does_child_path_exist = $wp_filesystem->exists( $child_path );
			if ( is_child_theme() !== false && $does_child_path_exist && !$wp_filesystem->is_dir( $child_path ) ) {
				$return_data['found'] 	= true;
				$final_path           	= $child_path;
				$where 					= 'child';
			} else
			if ( $does_final_path_exist && !$wp_filesystem->is_dir( $final_path ) ) {
				$return_data['found'] 	= true;
				$where 					= 'parent';
			} else {

				// if both not exist,  try the simpods mvc plugin
				$final_url  = trailingslashit( $simpods_dir . '/' . $folder ) . $file;
				$final_path  = $final_url;
				//echo $final_url .'<br/>';
				if( $wp_filesystem->exists( $final_url ) ){							
					$where 		= 'plugin';
					$return_data['found'] = true;
				} 
			}
		}

		if ( !$is_relative ) {
			if ( $is_url ) {
				if ( $where == 'child' ) {
					$final_path = $style_uri . '/' . trim( $folder, '/' ) . '/' . trim( $file, '/' );
				}
				if ( $where == 'parent' ) {
					$final_path = $template_uri . '/' . trim( $folder, '/' ) . '/' . trim( $file, '/' );
				}
				if ( $where == 'plugin' ) {
					$final_path = $simpods_dir . '/' . trim( $folder, '/' ) . '/' . trim( $file, '/' );
				}				
			}
		} else {
			$final_path = '/' . trim( $folder, '/' ) . '/' . trim( $file, '/' );
		}
		$final_path = apply_filters( 'simpods_get_final_path', $final_path );
		if ( 'child' == $where ) {
			$return_data['uri'] = rtrim( get_stylesheet_directory(), '/' ) . '/' . $folder . '/' . $file;
			$return_data['url'] = rtrim( get_stylesheet_directory_uri(), '/' ) . '/' . $folder . '/' . $file;
		} 
		if ( 'parent' == $where ) {
			$return_data['uri'] = rtrim( get_template_directory(), '/' ) . '/' . $folder . '/' . $file;
			$return_data['url'] = rtrim( get_template_directory_uri(), '/' ) . '/' . $folder . '/' . $file;
		}
		if ( 'plugin-simpods' == $where ) {

			$return_data['uri'] = SIMPODS_DIR . '/' . $folder . '/' . $file;
			$return_data['url'] = SIMPODS_URL . '/' . $folder . '/' . $file;

		}
		$return_data['where'] = $where;
		$return_data['path']  = $final_path;

		if ( !$to_get_details ) {
			return $final_path;
		} else {
			return $return_data;
		}
	}
	/**
	 * Escape data
	 * The function will recurse over arrays
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/escape_fn/
	 * @param  mixed $data Data to escape
	 * @return mixed 		   Escaped data
	 */
	public function escape( $data ) {
		if ( is_array( $data ) ) {
			foreach ( $data as $key => $value ) {
				$key = esc_sql( $key );
				if ( is_array( $value ) ) {
					$value = $this->escape( $value );
				} else {
					// esc_sql a boolean will return error
					if ( is_string( $value ) ) {
						$value = esc_sql( $value );
					}
				}
				$data[ $key ] = $value;
			}
		} else {
			if ( is_string( $data ) ) {
				$data = esc_sql( $value );
			}
		}
		return $data;
	}
	/**
	 * Escape attributes
	 * The function will recurse over arrays
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/escape_attrs_fn/
	 * @param  mixed $data Data to escape
	 * @return mixed 		   Escaped data
	 */
	public function escape_attrs( $data ) {
		if ( is_array( $data ) ) {
			foreach ( $data as $key => $value ) {
				$key = esc_attr( $key );
				if ( is_array( $value ) ) {
					$value = $this->escape_attrs( $value );
				} else {
					// esc_sql a boolean will return error
					if ( is_string( $value ) ) {
						$value = esc_attr( $value );
					}
				}
				$data[ $key ] = $value;
			}
		} else {
			if ( is_string( $data ) ) {
				$data = esc_attr( $value );
			}
		}
		return $data;
	}
	/**
	 * Get tables from the database
	 * By default this function only returns pods tables
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_tables_fn/
	 * @param  boolean $can_return_all 	If true, return all tables (default false)
	 * @return array 				  	Array of tables
	 */
	public function get_tables( $can_return_all = false ) {
		if ( $can_return_all ) {
			$cache_name = 'all';
		} else {
			$cache_name = 'pods';
		}
		// $items = wp_cache_get( $cache_name, 'simpodsTables' ); // doesn't seem to work
		// if ( false !== $items ){
		// 	return $items;
		// }
		global $wpdb;
		$act_tables    = array();
		$act_pod_tables = array();
		$tables    = get_option( 'simpods_all_tables', array() );
		$run 		   = false;
		if( ( isset( $_GET ) && isset( $_GET['page'] ) && ( ( $_GET['page'] == 'pods' && isset( $_GET['do'] ) && $_GET['do'] == 'save' ) || strpos( $_GET['page'], 'simpods-' ) === 0 ) ) ){
			$run = true;
			
		}
		// only do this on the pods page or simpods page
		if( $tables === false || $run === true ){
			$sql       = 'SHOW TABLES LIKE "%"';
			$tables    = $wpdb->get_results( $sql );
			update_option( 'simpods_all_tables', $tables ); // Panda Pods Repeater Field will check and use this first, don't change it.
		} 
		$act_tables 	= get_option( 'simpods_act_tables', array() );
		$act_pod_tables 	= get_option( 'simpods_act_pod_tables', array() );
		$pods_table_ids = array();
		if( $act_tables === false || $act_pod_tables === false || $run === true ){			
			foreach ( $tables as $idx_int => $table_obj ) {
				foreach ( $table_obj as $table_name ) {
					if ( 0 === strpos( $table_name, $wpdb->prefix ) ){
						$table = substr_replace( $table_name, '', 0, strlen( $wpdb->prefix ) );
					} else {
						$table = $table_name;
					}
					// return all tables
					if ( $can_return_all ) {
						array_push( $act_tables, $table );
					} else {
						// only return wp_posts, wp_users and pods tables
						$name_field = 'sp_title';
						// only return pods tables
						$table_info    = $this->get_pods_tb_info( $table );
						$name_field = get_post_meta( $table_info['id'], 'pod_index', true );
						if( ! empty( $table_info['id'] ) ){
							array_push( $pods_table_ids, $table_info['id'] );
						}
						$name_label = '';
						if ( $name_field == '' ) {
							if ( $table_info['type'] == 'post_type' ) {
								$name_field = 'post_title';
							} else if ( $table_info['type'] == 'user' ) {
								$name_field = 'display_name';								
							} else {
								$name_field = 'sp_title';
							}
						} else {
							$query 		= $wpdb->prepare(
															'SELECT ps_tb.post_title
															 FROM `' . $wpdb->posts . '` AS ps_tb																		  
															 WHERE ps_tb.`post_name` = %s AND ps_tb.`post_parent` = %d AND ps_tb.`post_type` = "_pods_field" LIMIT 0, 1', 
															 array( 
															 	$name_field, 
															 	$table_info['id'] 
															 ) 
															);
							$items 		= $wpdb->get_results( $query , ARRAY_A ); 	
							if( $items && count( $items ) > 0 ){
								$name_label	=	 $items[0]['post_title'];
							}							
						}							
						$title	=	'';
						if( isset( $table_info['title'] ) ){
							$title	=	$table_info['title'];
						}
						$act_tables[ $table ] = array(
							'id'         => $table_info['id'],
							'name'       => $table,
							'title'      => $title,
							'pod'        => $table_info['name'],
							'type'       => $table_info['type'],
							'name_field' => $name_field,
							'name_label' => $name_label,
						);								
						if ( strpos( $table_name, $wpdb->prefix . 'pods_' ) === 0 ) {
							$with_ids = array(
								'id'         => $table_info['id'],
								'name'       => $table,
								'title'      => $title,
								'pod'        => $table_info['name'],
								'type'       => $table_info['type'],
								'name_field' => $name_field,
								'name_label' => $name_label,
							);
							$act_pod_tables[ 'pod_' . $table_info['id'] ] 	= $with_ids;
							$act_tables[ 'pod_' . $table_info['id'] ] 		= $with_ids;
						}
					}
				}
			}
			//remove those deleted tables
			foreach( $act_tables as $k => $item ){
				if( preg_match( '/^(pod_){1}\d*$/s',  $k, $match) ){
					$id = substr( $k, 4 );
					if( ! in_array( $id, $pods_table_ids ) ){
						unset( $act_tables[ $k ] );
					}
				}
			}
			foreach( $act_pod_tables as $k => $item ){
				if( preg_match( '/^(pod_){1}\d*$/s',  $k, $match) ){
					$id = substr( $k, 4 );
					if( ! in_array( $id, $pods_table_ids ) ){
						unset( $act_pod_tables[ $k ] );
					}
				}
			}			
			// only do this on the pods page or simpods page
			
			update_option( 'simpods_act_tables', $act_tables );
			update_option( 'simpods_act_pod_tables', $act_pod_tables );
		} 
		if ( $can_return_all ) {
			$return_data = $act_tables;
		} else {
			$return_data = array(
				'act_tables' 		=> $act_tables,
				'act_pod_tables' 	=> $act_pod_tables,
			);
		}		
		//wp_cache_add( $cache_name, $return_data, 'simpodsTables', MINUTE_IN_SECONDS );
		return $return_data;
	}
	/**
	 * Get entries from a table, taking Simpods Hierachy, Simpods Linked Areas, Pods into account.
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/simpods_select_fn/
	 * @param  array   $fields    Fields to search
	 * @param  array   $attributes      SQL attributes array (see online documentation for details)
	 * @param  boolean $can_echo_query If true the query will be echoed out (default false)
	 * @param  boolean $can_prefix  If true, prefixes table names with the WordPress table prefix (default true)
	 * @return array   				  Array of found rows. If there are none, the array will be empty.
	 */
	final public function simpods_select( $fields = array(), $attributes = array(), $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		$defaults = array(
			'target_tb'    => '',         // The target database table
			'limit'        => 0,          // Max number of results to return
			'order'        => 'ASC',      // Search order. _prev_ and _next_ only work with ASC
			'order_by'     => '',         // Field to order the results by. Can be string or an array: array( array( field_name, direction, escape field name or not ), ... ), e.g. array( array( 'main_tb.`name`', 'ASC', false ), array( 'pos_tb.`post_title`', 'DESC', false ) )
			'group_by'     => '',         // Group your search results
			'having'	   => '',		  // Restrict the results returned by the GROUP BY	
			'start'        => 0,          // Which row to start the search
			'cat_areas'    => array(),    // For Simpods Hierarchy: array( array( 'join' => AND|OR, 'cattb' => number, 'catid' => number ) )
			'inc_cats'     => false,      // If true, all Simpods Hierarchy categories will be unserialized and included
			'linked_areas' => array(),    // For Simpods Linked Area: array( array( 'join' => AND|OR, 'cattb' => number, 'catid' => number ) )
			'inc_areas'    => false,      // If true, all Simpods Linked Areas will be unserialized and included
			'scheduled'    => false,      // If scheduled, outdated posts won't be included. Only work if the Simpods Schedule Component is available.
			'post_status'  => array(),    // Post status, e.g. 'publish', 'pending' etc.
			'count_only'   => false,      // If true, the search will return the number of how many rows found
			'term_id'      => 0,          // The term id, like the post category id or custom taxonomy id.
			'search'       => '',         // Extra bits that are not included in the function, expect escaped. e.g. AND main_tb.field = "something" AND post_tb.post_name LIKE "%%something%%"
			'join'         => '',         // Extra join queries, expect escaped
			'fields'       => '',         // Extra fields to return, usually needed when you use 'search' and 'join' above string or array e.g.
			'fields_only'  => false,	  // If true, only the fields in the fields option will be returned.
			'results_type' => ARRAY_A,    // ARRAY_A | ARRAY_N | OBJECT | OBJECT_K
			'return_query' => false,	  // Return query for debugging	
		);
		// Parse incoming $args into an array and merge it with $defaults
		$attributes  = wp_parse_args( $attributes, $defaults );
		$table = $attributes['target_tb'];
		if ( $attributes['target_tb'] == '' ) {
			return;
		}
		$no_prex_target_tb 	   = $attributes['target_tb'];
		if( stripos( $attributes['target_tb'], $wpdb->prefix ) === 0 ){
			$no_prex_target_tb = substr( $no_prex_target_tb, strlen( $wpdb->prefix ) );
		}
		$attributes['target_tb'] = esc_sql( stripslashes( $attributes['target_tb'] ) );
		if ( $can_prefix && stripos( $attributes['target_tb'], $wpdb->prefix ) !== 0 ) {
			$attributes['target_tb'] = $wpdb->prefix . $attributes['target_tb'];
		}
		if ( is_admin() && ! $this->is_frontend_ajax() ) {
			$tables = MVC_Admin_Factory::$act_tables;
		} else {
			$tables = MVC_Front_Factory::$act_tables;
		}		
		$param  = array();
		$where = '';
		if ( is_array( $fields ) ) {
			foreach ( $fields as $key => $value ) {
				// Exclude
				$not = strpos( $key, '_exc_' ) === 0 ? 'NOT ' : '';
				if ( $not != '' ) {
					$key = substr( $key, 5 );
				}
				$to_find = strpos( $key, '_find_' ) === 0 ? '%%' : '';
				if ( $to_find != '' ) {
					$key = substr( $key, 6 );
				}
				// Prev and next only work with ASC
				$to_find_prev = strpos( $key, '_prev_' ) === 0 ? true : false;
				if ( $to_find_prev ) {
					$key = substr( $key, 6 );
				}
				$to_find_next = strpos( $key, '_next_' ) === 0 ? true : false;
				if ( $to_find_next ) {
					$key = substr( $key, 6 );
				}
				$search_table = 'main_tb';
				// if not specified to which table (no dot), assign it to main_tb
				if ( strpos( $key, '.' ) === false && is_string( $key )  ) {
					$key = 'main_tb.' . $key;
				} else {
					$search_table = substr( $key, 0, strpos( $key, '.' ) );
				}
				$key = esc_sql( $key );
				// if looking for previous or next row
				if ( $to_find_prev || $to_find_next ) {
					if ( $to_find_next ) {
						if ( is_numeric( $value ) ) {
							// if putting a dot at the end of an number, like 24., strpos will return false so it is treated as an integer
							$int_or_float = strpos( $value, '.' ) !== false ? '%f' : '%d';
							$where .= ' AND ' . $key . ' = ( SELECT min(' . $key . ') from ' . $attributes['target_tb'] . ' WHERE ' . $key . ' > ' . $int_or_float . ' )';
							array_push( $param, $value );
						} else if ( is_string( $value ) ) {
							$where .= ' AND ' . $key . ' = ( SELECT min(' . $key . ') from ' . $attributes['target_tb'] . ' WHERE ' . $key . ' > "%s" )';
							array_push( $param, $value );
						}
					} else {
						if ( is_numeric( $value ) ) {
							$int_or_float = strpos( $value, '.' ) !== false ? '%f' : '%d';
							$where .= ' AND ' . $key . ' = ( SELECT max(' . $key . ') from ' . $attributes['target_tb'] . ' WHERE ' . $key . ' < ' . $int_or_float . ' )';
							array_push( $param, $value );
						} else if ( is_string( $value ) ) {
							$where .= ' AND ' . $key . ' = ( SELECT max(' . $key . ') from ' . $attributes['target_tb'] . ' WHERE ' . $key . ' < "%s" )';
							array_push( $param, $value );
						}
					}
				} else {
					if ( is_numeric( $value ) ) {
						$int_or_float = strpos( $value, '.' ) !== false ? '%f' : '%d';
						$be_like = $to_find == '' && $not == '' ? '=' : 'LIKE';
						$where .= ' AND ' . $key . ' ' . $not . ' ' . $be_like . ' "' . $to_find . '' . $int_or_float . '' . $to_find . '"';
						array_push( $param, $value );
					} else if ( is_string( $value ) ) {
						$where .= ' AND ' . $key . ' ' . $not . ' LIKE "' . $to_find . '%s' . $to_find . '"';
						array_push( $param, $value );
					} else if ( is_array( $value ) && count( $value ) > 0 ) {
						$value = $this->escape( $value );
						$in_sql  = '("' . implode( '","', $value ) . '")';
						$where .= ' AND ' . $key . ' ' . $not . ' IN ' . $in_sql . '';
					}
				}
			}
		}
		$limit_sql = '';
		if ( $attributes['limit'] != '' ) {
			$limit_sql = 'LIMIT ' . esc_sql( $attributes['start'] ) . ', ' . esc_sql( $attributes['limit'] ) . '';
		}
		$order_sql = '';
		if ( $attributes['order_by'] != '' ) {
			if ( is_array( $attributes['order_by'] ) ) {
				foreach ( $attributes['order_by'] as $orders ) {
					if ( isset( $orders[0] ) ) {
						if ( $order_sql != '' ) {
							$order_sql .= ', ';
						}
						if ( isset( $orders[2] ) && $orders[2] == false ) {
							// skip escape
							$order_sql .= ' ' . $orders[0] . ' ';
						} else {
							$order_sql .= ' ' . esc_sql( $orders[0] ) . ' ';
						}
						if ( isset( $orders[1] ) ) {
							if ( in_array( strtoupper( $orders[1] ) , array('ASC', 'DESC') ) ) {
								$order_sql .= ' ' . $orders[1] . ' ';
							} else if ( $orders[1] == false ) {
								$order_sql .= '';
							} else {
								$order_sql .= ' ASC ';
							}
						} else {
							$order_sql .= ' ASC ';
						}
					}
				}
				$order_sql = 'ORDER BY ' . $order_sql;
			} else {
				if ( $attributes['order_by'] == 'random' ) {
					$order_sql = 'ORDER BY RAND()';
				} else {
					if ( $attributes['order'] == false ) {
						$attributes['order']	=	'';
					} else {
						if ( $attributes['order'] != 'ASC' ) {
							$attributes['order'] = 'DESC';
						}
					}
					$order_sql = 'ORDER BY ' . esc_sql( $attributes['order_by'] ) . ' ' . $attributes['order'] . '';
				}
			}
		}
		$extra_sql  = '';
		$join_sql   = '';
		$table_type = '';
		// get the table info to check if it a pods table
		if ( function_exists( 'pods_query' ) ) {
			$table_info = $this->get_pods_tb_info( $attributes['target_tb'] );
			if ( is_array( $table_info ) ) {
				$table_type = $table_info['type'];
				// Custom Post Type
				if ( $table_type == 'post_type' ) {
					$join_sql .= 'INNER JOIN ' . $wpdb->prefix . 'posts AS post_tb ON post_tb.`ID` =  main_tb.`id`';
					$extra_sql .= ', post_tb.* , post_tb.`post_title` AS sp_title ';
					if ( $order_sql == '' ) {
						//$order_sql = 'ORDER BY post_tb.`menu_order` ' . $attributes['order'] . '';
					} else {
						if ( $attributes['order_by'] != 'random' ) {
							//$order_sql .= ', post_tb.`menu_order` ' . $attributes['order'];
						}
					}
				}
				if ( $table_type == 'user' ) {
					$join_sql .= 'INNER JOIN ' . $wpdb->base_prefix . 'users AS user_tb ON user_tb.`ID` =  main_tb.`id`';
					$extra_sql .= ', user_tb.`display_name` AS display_name , user_tb.`display_name` AS sp_title ';
				}
				if ( $table_type == 'pod' ) {
					$table_data = $this->find_a_pod_tb( 'name', $attributes['target_tb'] );
					if ( isset( $table_data['name_field'] ) && $table_data['name_field'] != '' ) {
						$extra_sql .= ', main_tb.`' . $table_data['name_field'] . '` AS sp_title ';
					}
				}
			}
		}
		if ( $attributes['term_id'] != 0 ) {
			if( is_numeric( $attributes['term_id'] ) ){ 
				$join_sql .= ' INNER JOIN `' . $wpdb->prefix . 'term_relationships` AS tr_tb ON tr_tb.object_id = main_tb.`id`
								INNER JOIN `' . $wpdb->prefix . 'term_taxonomy` AS tt_tb ON tt_tb.`term_taxonomy_id` = tr_tb.`term_taxonomy_id`
								INNER JOIN `' . $wpdb->prefix . 'terms` AS t_tb ON t_tb.`term_id` = tt_tb.term_id';
				$where .= ' AND t_tb.`term_id` = ' . $attributes['term_id'] . ' ';
			}
			if( is_array( $attributes['term_id'] ) && ! empty( $attributes['term_id'] ) ){
				
				$multi_term	= '';
				$multi_join = '';
				foreach( $attributes['term_id'] as $i => $term_id ){
					if( ! is_numeric( $term_id ) ){
						continue;
					}
					if( $i > 0 ){
						$multi_term .= ' OR ';
					}
					$multi_join .= ' INNER JOIN `' . $wpdb->prefix . 'term_relationships` AS tr_tb_' . $i . ' ON tr_tb_' . $i . '.object_id = main_tb.`id`
									INNER JOIN `' . $wpdb->prefix . 'term_taxonomy` AS tt_tb_' . $i . ' ON tt_tb_' . $i . '.`term_taxonomy_id` = tr_tb_' . $i . '.`term_taxonomy_id`
									INNER JOIN `' . $wpdb->prefix . 'terms` AS t_tb_' . $i . ' ON t_tb_' . $i . '.`term_id` = tt_tb_' . $i . '.term_id';
					$multi_term .= ' t_tb_' . $i . '.`term_id` = ' . $term_id . ' ';					
				}
				// if there are some numbers in the array, update the join and where query
				if( $multi_term != '' ){
					$join_sql 		.= $multi_join;
					$where 		.= ' AND (' . $multi_term . ') ';
					if( '' == trim( $attributes['group_by'] ) ){
						$attributes['group_by'] = ' main_tb.`id` ';
					} else {
						$attributes['group_by'] .= ', main_tb.`id` ';
					}
				}
			}
		}
		$groupby_sql = '';
		if ( $attributes['group_by'] != '' ) {
			$groupby_sql = 'GROUP BY( ' . esc_sql( $attributes['group_by'] ) . ' )';
			if( $attributes[ 'having' ] != '' ){
				$groupby_sql .= ' HAVING ' . esc_sql( $attributes[ 'having' ] ) . ' ';
			}
		}
		$join_sql      .= ' ' . $attributes['join'] . ' ';
		$fields_only_sql	=	'';
		if ( !empty( $attributes['fields'] ) ) {
			if ( is_array( $attributes['fields'] ) ) {
				foreach ( $attributes['fields'] as $exField_str ) {
					$fields_only_sql .= $exField_str  . ',';
				}
				$fields_only_sql	= rtrim( $fields_only_sql, ',' ) ;
			} else {
				$fields_only_sql = $attributes['fields'] ;
			}
		}
		if( !$attributes['fields_only'] && $fields_only_sql != '' ){
			$extra_sql  = rtrim( trim( $extra_sql ), ',' ) ;
			$extra_sql .= ',' . ltrim( trim( $fields_only_sql ), ',' ) . ' ';
		}
		// hierachy
		if ( is_array( $attributes['cat_areas'] ) ) {
			$link_areas_sql = '';
			$i            = 0;
			foreach ( $attributes['cat_areas'] as $linked_area_sqls ) {
				// if first one, no join
				if ( $i == 0 ) {
					$linked_area_sqls['join'] = '';
				}
				if ( !isset( $linked_area_sqls['tb_alias'] ) || trim( $linked_area_sqls['tb_alias'] ) == '' ) {
					$table_alias = 'main_tb';
				} else {
					$table_alias = esc_sql( $linked_area_sqls['tb_alias'] );
				}
				if ( isset( $linked_area_sqls['join'] ) && in_array( strtoupper( $linked_area_sqls['join'] ), array( 'AND', 'OR', '' ) ) ) {
					if ( isset( $linked_area_sqls['cattb'] ) && is_numeric( $linked_area_sqls['cattb'] ) && isset( $linked_area_sqls['catid'] ) && is_numeric( $linked_area_sqls['catid'] ) ) {
						$link_areas_sql .= $linked_area_sqls['join'] . ' ' . $table_alias . '.`sp_categories` REGEXP "(:\"' . intval( $linked_area_sqls['cattb'] ) . '.' . intval( $linked_area_sqls['catid'] ) . '\";{1,})" AND sp_tb.sp_parent = "pod_' . intval( $linked_area_sqls['cattb'] ) . '" AND sp_tb.sp_child = "pod_' . $tables[ $no_prex_target_tb ]['id'] . '" ';
					}
										
				}
				$i++;
			}
			if ( $link_areas_sql != '' ) {
				$join_sql  .= ' INNER JOIN ' . $wpdb->prefix . 'simpods_area_categories' . ' AS sp_tb ';
				$where .= ' AND ( ' . $link_areas_sql . '  ) ';
			}
		}
		// linked areas
		if ( is_array( $attributes['linked_areas'] ) ) {
			$link_areas_sql = '';
			$i             = 0;
			foreach ( $attributes['linked_areas'] as $linked_area_sqls ) {
				// if first one, no join
				if ( $i == 0 ) {
					$linked_area_sqls['join'] = '';
				}
				if ( !isset( $linked_area_sqls['tb_alias'] ) || trim( $linked_area_sqls['tb_alias'] ) == '' ) {
					$table_alias = 'main_tb';
				} else {
					$table_alias = esc_sql( $linked_area_sqls['tb_alias'] );
				}
				if ( isset( $linked_area_sqls['join'] ) && in_array( strtoupper( $linked_area_sqls['join'] ), array( 'AND', 'OR', '' ) ) ) {
					if ( isset( $linked_area_sqls['cattb'] ) && is_numeric( $linked_area_sqls['cattb'] ) && isset( $linked_area_sqls['catid'] ) && is_numeric( $linked_area_sqls['catid'] ) ) {
						$link_areas_sql .= $linked_area_sqls['join'] . ' ' . $table_alias . '.`sp_linked_areas` REGEXP "(:\"' . intval( $linked_area_sqls['cattb'] ) . '.' . intval( $linked_area_sqls['catid'] ) . '\";{1,})" AND spl_tb.spl_main = "pod_' . $tables[ $no_prex_target_tb ]['id'] . '" AND spl_tb.spl_friend = "pod_' . intval( $linked_area_sqls['cattb'] ) . '" ';
					}
				}
				$i++;
			}
			if ( $link_areas_sql != '' ) {
				$join_sql  .= ' INNER JOIN ' . $wpdb->prefix . 'simpods_linked_areas' . ' AS spl_tb ';
				$where .= ' AND ( ' . $link_areas_sql . ' ) ';
			}
		}
		
		if ( $attributes['scheduled'] == true ) {
			$where .= ' AND ( main_tb.`sp_end` LIKE "0000-00-00 00:00:00" || ( main_tb.`sp_end` NOT LIKE "0000-00-00 00:00:00" AND main_tb.`sp_end` >= CURDATE() ) ) ';
		}
		if ( $attributes['count_only'] === false ) {
			if ( $attributes['fields_only'] ) {
				$fields_sql = $fields_only_sql;
			} else {
				// $fields_only_sql has been merged into $extra_sql
				$fields_sql = 'main_tb.* ' . $extra_sql;
			}
			if ( function_exists( 'pods_query' ) ) {
				if ( substr( $table, 0, 5 ) == 'pods_') {
					$child_pod = pods( substr( $table, 5 ) );
					if ( is_object( $child_pod ) ) {
						$i = 0;
						foreach ( $child_pod->fields as $key => $field_data ) {
							if ( is_array( $field_data ) ) {
								$relate_picks	 = array( 'user', 'post_type', 'pod', 'media' );
								if ( ( isset( $field_data['type'] ) && $field_data['type'] == 'file' ) || ( isset( $field_data['type'] ) && $field_data['type'] == 'pick' && in_array( $field_data['pick_object'], $relate_picks ) ) ) {
									$fields_sql .= ',(
													SELECT GROUP_CONCAT( psl' . $i .  '_tb.related_item_id )
													FROM `' . $wpdb->prefix . 'podsrel` AS psl' . $i .  '_tb
													WHERE psl' . $i .  '_tb.pod_id = "' . $child_pod->pod_id . '"
													AND psl' . $i .  '_tb.field_id = "' . $field_data['id'] . '"
													AND psl' . $i .  '_tb.item_id = main_tb.id
													GROUP BY psl' . $i .  '_tb.item_id
													) AS ' . $key;
									$i++;
								}
							}
						}
					}
				}
			}
		} else {
			$fields_sql = ' COUNT( main_tb.`id` ) AS "count"';
		}
		$where .= ' ' . $attributes['search'] . ' ';
		if ( count( $param ) > 0 ) {
			$query = $wpdb->prepare( 'SELECT ' . $fields_sql . '
										 FROM `' . $attributes['target_tb'] . '` AS main_tb
										 ' . $join_sql . '
										 WHERE 1=1 ' . $where . ' ' . $groupby_sql . ' ' . $order_sql . ' ' . $limit_sql . ';', $param );
		} else {
			$query = 'SELECT ' . $fields_sql . '
										 FROM `' . $attributes['target_tb'] . '` AS main_tb
										 ' . $join_sql . '
										 WHERE 1=1 ' . $where . ' ' . $groupby_sql . ' ' . $order_sql . ' ' . $limit_sql . ';';
		}
		if ( $can_echo_query ) {
			echo $query;
		}
		if ( $attributes['return_query'] ) {
			return $query;
		}
		$items = $wpdb->get_results( $query, $attributes['results_type'] );
		// if convert sp area categories required
		if ( $attributes['inc_cats'] == true ) {
			$category_idx = 0;
			for ( $i = 0; $i < count( $items ); $i++ ) {
				if ( isset( $items[$i]['sp_categories'] ) ) {
					$items[$i]['sp_cats'] = $this->convert_sp_items( $items[$i]['sp_categories'], array( 'inc_cats' => true ) );
				}
			}
		}
		// if convert sp area categories required
		if ( $attributes['inc_areas'] == true ) {
			$category_idx = 0;
			for ( $i = 0; $i < count( $items ); $i++ ) {
				if ( isset( $items[$i]['sp_linked_areas'] ) ) {
					$items[$i]['sp_areas'] = $this->convert_sp_items( $items[$i]['sp_linked_areas'], array( 'inc_areas' => true ) );
				}
			}
		}
		return $items;
	}
	/**
	 * Insert new data into a database
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/general_insert_fn/
	 * @param  string  $table    	Target table (without table_prefix)
	 * @param  array   $data     	Data to save to database
	 * @param  boolean $can_echo_query   If true the query will be echoed out (default false)
	 * @param  boolean $can_prefix 	If true, prefixes table names with the WordPress table prefix (default true)
	 * @return mixed 					Returns inserted ID, if failed, false will be returned.
	 */
	final public function general_insert( $table, $data, $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		$table = esc_sql( stripslashes( $table ) );
		if ( $can_prefix && stripos( $table, $wpdb->prefix ) !== 0 ) {
			$table = $wpdb->prefix . $table;
		}
		// bind the fields
		$bond_data = $this->bind( $table, $data, false );
		// check if exists
		// fields
		$fields_sql = '';
		$fields 	= array();
		// data type defined by wordpress
		$value_types 	= array();
		$val_types_sql 	= '';
		// data
		$values = array();
		$item_processed	= false;
		if ( isset( $bond_data['bond'] ) ) {
			$count 	=	count( $bond_data['bond'] );	
			if ( $count > 0 ) {
				for ( $i = 0; $i < $count; $i++ ) {
					array_push( $fields, '`' . $bond_data['bond'][$i][0] . '`' );
					if ( is_numeric( $bond_data['bond'][ $i ][1] ) && strpos( $bond_data['bond'][ $i ][1] , '0' ) !== 0 ) {
						$value_type = strpos( $bond_data['bond'][ $i ][1], '.' ) !== false ? '%f' : '%d';
						array_push( $value_types, $value_type );
					} else {
						array_push( $value_types, '%s' );
					}
					array_push( $values, $bond_data['bond'][$i][1] );
				}
				$fields_sql   = implode( ',', $fields );
				$val_types_sql = implode( ',', $value_types );
				if ( count( $values ) > 0 ) {
					$query = $wpdb->prepare( 'INSERT INTO `' . $table . '` ( ' . $fields_sql . ' ) VALUES ( ' . $val_types_sql . ' );', $values );
					$item_processed = $wpdb->query( $query );
				}
			}
			$count 	=	count( $bond_data['bond'] );	
			if ( $count == 0 ) {
				$query = esc_html__( 'Simpods - general_insert_fn - no fields matched', 'simpods-mvc' );
			}
		} else {
			$query = esc_html__( 'Simpods - general_insert_fn - table not found or no fields matched', 'simpods-mvc' );
		}
		if ( $can_echo_query ) {
			echo $query;
		}
		if ( $item_processed ) {
			return $wpdb->insert_id;
		}
		return false;
	}
	/**
	 * Update a database record
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/general_update_fn/
	 * @param  string  	$table    	Target table (without table_prefix)
	 * @param  array  	$data      	Data to save to database
	 * @param  array  	$wheres     	Locate the entries to update
	 * @param  boolean 	$can_echo_query  If true the query will be echoed out (default false)
	 * @param  boolean 	$can_prefix 	If true, prefixes table names with the WordPress table prefix (default true)
	 * @return boolean 					Returns a boolean of whether the update was successful or not
	 */
	final public function general_update( $table, $data, $wheres, $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		$table = esc_sql( stripslashes( $table ) );
		if ( $can_prefix && stripos( $table, $wpdb->prefix ) !== 0 ) {
			$table = $wpdb->prefix . $table;
		}
		// bind the fields
		$bond_data  = $this->bind( $table, $data, false );
		$bond_wheres = $this->bind( $table, $wheres, false );
		// data
		$values = array();
		// string for update sql query
		$update_sql = '';
		$updates 	= array();
		$count 		= count( $bond_data['bond'] );
		for ( $i = 0; $i < $count; $i++ ) {
			// data type defined by wordpress
			if ( is_numeric( $bond_data['bond'][ $i ][1] ) && strpos( $bond_data['bond'][ $i ][1] , '0' ) !== 0 ) {
				$value_type = strpos( $bond_data['bond'][$i][1], '.' ) !== false ? '%f' : '%d';
			} else {
				$value_type = '"%s"';
			}
			array_push( $updates, '`' . $bond_data['bond'][$i][0] . '` = ' . $value_type );
			array_push( $values, $bond_data['bond'][$i][1] );
		}
		$update_sql = implode( ',', $updates );
		$search_sql = '';
		$searches 	= array();
		$count 		= count( $bond_wheres['bond'] );
		for ( $i = 0; $i < $count; $i++ ) {
			// data type defined by wordpress
			if ( is_numeric( $bond_wheres['bond'][ $i ][1] ) && strpos( $bond_wheres['bond'][ $i ][1] , '0' ) !== 0 ) {
				is_int( $bond_wheres['bond'][$i][1] ) ? $value_type = '%d' : $value_type = '%f';
			} else {
				$value_type = '"%s"';
			}
			array_push( $searches, '`' . $bond_wheres['bond'][$i][0] . '` = ' . $value_type );
			array_push( $values, $bond_wheres['bond'][$i][1] );
		}
		$search_sql = implode( ' AND ', $searches );
		if ( count( $values ) > 0 ) {
			$query = $wpdb->prepare( 'UPDATE `' . $table . '` SET ' . $update_sql . ' WHERE ' . $search_sql . ';', $values );
		} else {
			$query = 'UPDATE `' . $table . '` SET ' . $update_sql . ' WHERE ' . $search_sql . ';';
		}
		if ( $can_echo_query ) {
			echo $query;
		}
		$item_processed = $wpdb->query( $query );
		return $item_processed;
	}
	/**
	 * Delete database records
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/general_delete_fn/
	 * @param  string  $table       Target table (without table_prefix)
	 * @param  string  $field      	Field to search (default 'ID')
	 * @param  integer $search_what      Search term, number for ID, sting for slug etc
	 * @param  boolean $can_echo_query  	If true the query will be echoed out (default false)
	 * @param  boolean $can_prefix 	If true, prefixes table names with the WordPress table prefix (default true)
	 * @return boolean 					Whether the delete was succesful or not
	 */
	final public function general_delete( $table, $field = 'ID', $search_what = 0, $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		if ( is_user_logged_in() ) {
			if ( $can_prefix && stripos( $table, $wpdb->prefix ) !== 0 ) {
				$table = $wpdb->prefix . $table;
			}
			if ( is_numeric( $search_what ) ) {
				// if putting a dot at the end of an number, like 24., strpos will return false so it is treated as an integer
				$int_or_float  = strpos( $search_what, '.' ) !== false ? '%f' : '%d';
				$query = $wpdb->prepare( 'DELETE FROM `' . $table . '` WHERE `' . esc_sql( $field ) . '` = ' . $int_or_float . ' ;' , array( $search_what ) );
			} else if ( is_string( $search_what ) ) {
				$query = $wpdb->prepare( 'DELETE FROM `' . $table . '` WHERE `' . esc_sql( $field ) . '` = %s ;' , array( $search_what ) );
			}
			$deleted = $wpdb->query( $query );
			if ( $can_echo_query ) {
				echo $query ;
			}
			return $deleted;
		}
	}
	/**
	 * Delete database records. If a pods table is linked, delete associated data as well,
	 * $pod->delete() is implemented. If it is normal post, wp_delete_post() is implemented.
	 * If it is a custom table, it will bind the fields and then delete.
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/simpods_delete_fn/
	 * @param  string 	$table 		Target table (without table_prefix)
	 * @param  array 	$wheres     	Locate the entries to delete
	 * @param  boolean 	$can_echo_query 	If true the query will be echoed out (default false)
	 * @param  boolean 	$can_prefix 	If true, prefixes table names with the WordPress table prefix (default true)
	 * @return boolean 			 		Whether the records have been deleted or not
	 */
	final public function simpods_delete( $table, $wheres, $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		$table = esc_sql( stripslashes( $table ) );
		if ( $can_prefix && stripos( $table, $wpdb->prefix ) !== 0 ) {
			$table 	= $wpdb->prefix . $table;
		}
		$deleted   = false;
		$query = esc_html__( 'Item not found.', 'simpods-mvc' );
		// find it first
		$items = $this->simpods_select( $wheres, array( 'target_tb' => $table, 'limit' => 1 ) );
		if ( count( $items ) > 0 ) {
			// if deleting from the posts table
			if ( $table == $wpdb->prefix . 'posts' ) {
				if ( function_exists( 'pods_query' ) ) {
					//if pods is avalilable && it is deleting from the posts table, use pods delete function so data in the connected pods table will be deleted too
					$pod     = pods( $items[0]['post_type'], $items[0]['ID'] );
					$deleted = $pod->delete();
					if ( $deleted ) {
						$query = esc_html__('Deleted by Pods', 'simpods-mvc');
					} else {
						$query = esc_html__('Failed to delete by Pods', 'simpods-mvc');
					}
				} else {
					// use wordpress delete function so meta data can be deleted. Bypass trash and force deletion
					$deleted = wp_delete_post( $items[0]['ID'], 1 );
					if ( $deleted ) {
						$query = esc_html__('Deleted by wp_delete_post()', 'simpods-mvc');
					} else {
						$query = esc_html__('Failed to delete by wp_delete_post()', 'simpods-mvc');
					}
				}
			} else {
				// if it is a pods advanced table,
				if ( strpos( $table, $wpdb->prefix . 'pods_' ) === 0 && isset( $items[0]['id'] ) ) {
					$pod     = pods( substr( $table, strlen( $wpdb->prefix . 'pods_' ) ), $items[0]['id'] );
					$deleted = $pod->delete();
					if ( $deleted ) {
						$query = esc_html__('Deleted by Pods - advanced content type', 'simpods-mvc');
					} else {
						$query = esc_html__('Failed to delete by Pods - advanced content type', 'simpods-mvc');
					}
				} else {
					// custom delete
					// bind the fields as ID may not in the where_arr
					$bond_data = $this->bind( $table, $wheres, false );
					$where  = '1 = 1';
					$values = array();
					for ( $i = 0; $i < count( $bond_data['bond'] ); $i++ ) {
						if ( is_numeric( $bond_data['bond'][ $i ][1] ) && strpos( $bond_data['bond'][ $i ][1] , '0' ) !== 0 ) {
							$value_type = strpos( $bond_data['bond'][ $i ][1], '.' ) !== false ? '%f' : '%d';
						} else {
							$value_type = '%s';
						}
						array_push( $values, $bond_data['bond'][ $i ][1] );
						$where .= ' AND `' . $bond_data['bond'][ $i ][0] . '` = ' . $value_type;
					}
					$query = $wpdb->prepare( 'DELETE FROM `' . $table . '` WHERE ' . $where, $values );
					$deleted   = $wpdb->query( $query );
					if ( $deleted ) {
						$query .= ' - ' . esc_html__( 'successful', 'simpods-mvc' );
					} else {
						$query .= ' - ' . esc_html__( 'failed', 'simpods-mvc' );
					}
				}
			}
		}
		if ( $can_echo_query ) {
			echo $query;
		}
		return $deleted;
	}
	/**
	 * Bind posted data keys with table field names
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/bind_fn/
	 * @param  string  $table   	Target table (without table_prefix)
	 * @param  array   $data    	Data to save, in $_POST format
	 * @param  boolean $to_lower 	If true, change keys to lowercase (default true)
	 * @param  boolean $can_echo_query   If true the query will be echoed out (default false)
	 * @return array|false Returns false if failed, array is succeeds
	 */
	public function bind( $table, $data, $to_lower = true, $can_echo_query = false ) {
		//$type_str N = normal array; I= array for inserting data function; U=array for updating data function
		$bind_fields  		= array();
		$bind_fields_alt 	= array();
		$binds_data = false;
		$fields = $this->get_fields( $table, $can_echo_query );
		if ( $can_echo_query ) {
			print( $fields );
		}
		if ( $fields ) {
			for ( $i = 0; $i < count( $fields ); $i++ ) {
				foreach ( $data as $key => $value ) {
					if ( $to_lower ) {
						$key = strtolower( $key );
					}
					if ( $key == $fields[$i]['Field'] ) {
						$bind_fields[ $key ] = $value;
						array_push( $bind_fields_alt, array( $key, $value ) );
					}
				}
			}
		}
		if ( count( $bind_fields ) == 0 ) {
		} else {
			$binds_data = array( 'normal' => $bind_fields, 'bond' => $bind_fields_alt );
		}
		return $binds_data;
	}
	/**
	 * Check if a table column exists
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/column_exist_fn/
	 * @param  string  $table     	Table name
	 * @param  string  $column_str    	Column name
	 * @param  boolean $can_echo_query 	If true the query will be echoed out (default false)
	 * @param  boolean $can_prefix  	If true, prefixes table names with the WordPress table prefix (default true)
	 * @return boolean            		True if column exists
	 */
	public function column_exist( $table, $column_str, $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		if ( $can_prefix ) {
			$table = $wpdb->prefix . esc_sql( $table );
		} else {
			$table = esc_sql( $table );
		}
		$query = 'SHOW COLUMNS FROM `' . $table . '` LIKE "' . esc_sql( $column_str ) . '"';
		if ( $can_echo_query ) {
			echo $query;
		}
		$result_bln = $wpdb->query( $query );
		return boolval( $result_bln );
	}
	/**
	 * Get field names of a table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_fields_fn/
	 * @param  string  $table    	Target table (without table_prefix)
	 * @param  boolean $can_echo_query   If true the query will be echoed out (default false)
	 * @param  boolean $can_prefix 	If true, prefixes table names with the WordPress table prefix (default true)
	 * @return array 					Return the field names
	 */
	public function get_fields( $table, $can_echo_query = false, $can_prefix = true ) {
		global $wpdb;
		$table = esc_sql( stripslashes( $table ) );
		if ( $can_prefix && stripos( $table, $wpdb->prefix ) !== 0 ) {
			$table = $wpdb->prefix . $table;
		}
		$query = 'SHOW FIELDS FROM `' . $table . '`';
		if ( $can_echo_query ) {
			echo $query;
		}
		$items = wp_cache_get( $table, 'getFields' );
		if ( false === $items ) {
			$items = $wpdb->get_results( $query, ARRAY_A );
			wp_cache_add( $table, $items, 'getFields', MINUTE_IN_SECONDS );
		}
		return $items;
	}
	/**
	 * Get pods table info
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/get_pods_tb_info_fn/ ‎
	 * @param  string  $table   	Target table (without table_prefix)
	 * @param  boolean $can_echo_query 	If true the query will be echoed out (default false)
	 * @return mixed 					Returns false if failed, array is succeeds
	 */
	private function get_pods_tb_info( $table, $can_echo_query = false ) {
		global $wpdb;
		// if prefix found, removed it as it is added below
		if ( strpos( $table, $wpdb->prefix ) === 0 ) {
			$table = substr( $table, strlen( $wpdb->prefix ) );
		}
		$table_info = array( 'id' => 0, 'name' => '', 'type' => '' );
		if ( strpos( $table, 'pods_' ) === 0 ) {
			$table = substr( $table, 5 );
		
			//$table_info = wp_cache_get( $table, 'pods_tb_info' );
			//if ( $table_info === false ) {
				$query = $wpdb->prepare( 'SELECT ps_tb.*, pm_tb.`meta_value` AS type
												 FROM `' . $wpdb->prefix . 'posts` AS ps_tb
												 LEFT JOIN `' . $wpdb->prefix . 'postmeta` AS pm_tb ON ps_tb.`ID` = pm_tb.`post_id` AND pm_tb.`meta_key` = "type"
												 WHERE ps_tb.`post_name` = "%s" AND ps_tb.`post_type` = "_pods_pod" LIMIT 0, 1', array( $table ) );
				if ( $can_echo_query ) {
					echo $query;
				}
				$items = $wpdb->get_results( $query, ARRAY_A );
				$table_info = array( 'id' => 0, 'name' => '', 'type' => '' );
				if ( count( $items ) > 0 ) {
					$table_info['id']    = $items[0]['ID'];
					$table_info['name']  = $items[0]['post_name'];
					$table_info['title'] = $items[0]['post_title'];
					$table_info['type']  = $items[0]['type'] == '' ? 'pod' : $items[0]['type'];
				}
				//wp_cache_add( $table, $table_info, 'pods_tb_info', MINUTE_IN_SECONDS );
			//}
		}
		return $table_info;
	}
	/**
	 * Find the brief information about a pod table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/find_a_pod_tb/
	 * @param  string $key A key in the Factory's $act_tables variable
	 * @param  string $value A value in the Factory's $act_tables variable
	 * @return array
	 */
	public function find_a_pod_tb( $key, $value ) {
		global $wpdb;
		// if prefix found, remove it from the target tb
		if ( strpos( $value, $wpdb->prefix ) === 0 ) {
			$value = str_replace( $wpdb->prefix, '', $value );
		}
		if ( is_admin() && ! $this->is_frontend_ajax() ) {
			$tables = MVC_Admin_Factory::$act_tables;
		} else {
			$tables = MVC_Front_Factory::$act_tables;
		}
		foreach ( $tables as $table_name => $field_data ) {
			if ( isset( $field_data[ $key ] ) && $field_data[ $key ] == $value ) {
				$index = $table_name;
				return $field_data;
			}
		}
		return array();
	}
	/**
	 * Generate a unique code
	 * If table and column names are specific, check if duplicated in the table
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/unique_code_fn/
	 * @param  integer 		 $chars_length 	Total number of characters (default 8)
	 * @param  string 		 $table 	Data table name
	 * @param  string 		 $column_name 		Data table column name
	 * @param  boolean 		 $is_number_only 	If true, use numbers only. Otherwise use random characters (default false)
	 * @param  boolean|array $formats 	Use to format the unique code (see online documentation for examples) (default false)
	 * @param  boolean 		 $can_echo_query If true the query will be echoed out (default false)
	 * @return string 						The unique code
	 */
	public function unique_code( $chars_length = 8, $table = '' , $column_name = '', $is_number_only = false, $formats = false, $can_echo_query = false ) {
		if ( $table != '' && $column_name != '' ) {
			do {
				if ( $is_number_only ) {
					$min = '1';
					for ( $i = 0; $i < $chars_length - 1; $i ++ ) {
						$min	.= '0';
					}
					$max = '1';
					for ( $i = 0; $i < $chars_length ; $i ++ ) {
						$max	.= '0';
					}
					$id = rand( (int)$min, (int)$max - 1 );
				} else {
					$id = wp_generate_password( $chars_length, false );
				}
				if ( $formats === true ) {
					// default format
					$formats = array('separator' => '-', 'digits' => 4);
				}
				if ( is_array( $formats ) ) {
					if ( isset( $formats['separator'] ) && isset( $formats['digits'] ) ) {
						$id = implode( $formats['separator'], str_split( $id , $formats['digits'] ) );
					}
				}
				$items = $this->simpods_select( array( $column_name => $id ), array( 'target_tb' => $table, 'limit' => 1 ) );
				// if not found, break
				if ( count( $items ) == 0 ) {
					break;
				}
			} while ( count( $items ) > 0);
		}
		return $id;
	}
	/**
	 * Recursive method to create a unique username
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/unique_username_fn/
	 * @param  string $user_name The username to make unique
	 * @return string 				A unique username
	 */
	public function unique_username( $user_name = 'user' ) {
		if ( username_exists( $user_name ) ) {
			$user_names = explode( '-', $user_name );
			$index    = count( $user_names ) - 1;
			if ( is_numeric( $user_names[ $index ] ) ) {
				$user_name_amount = intval( $user_names[ $index ] );
				$user_name_amount++;
				$user_names[ $index ] = $user_name_amount;
				$user_name = implode( '-', $user_names );
			} else {
				$user_name .= '-1';
			}
			// if still exist, recurse
			if ( username_exists( $user_name ) ) {
				$user_name = $this->unique_username( $user_name );
			}
		}
		return $user_name;
	}
	/**
	 * Return the details of a hierachy or a linked area
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/convert_sp_items_fn/
	 * @param  string $categories_serialized     The serialized string from sp_categories field
	 * @param  array  $attributes     array( 'inc_areas' => false, 'inc_cats'  => false ) If ture, the targeted serialized string
	 *                              will be converted into an array
	 * @param  string $the_table_prefix The table prefix
	 * @return array
	 */
	protected function convert_sp_items( $categories_serialized, $attributes = array(), $the_table_prefix = '' ) {
		global $wpdb;
		$attributes =  wp_parse_args( $attributes, array(
			'inc_areas' => false,
			'inc_cats'  => false, // if ture, all cates will be included
		)  );
		// if not empty, it should be the basic table prefix
		if ( $the_table_prefix == '' ) {
			$the_table_prefix = $wpdb->prefix;
		}
		$categories = array();
		$data = maybe_unserialize( $categories_serialized );
		if ( is_admin() && ! $this->is_frontend_ajax() ) {
			$tables = MVC_Admin_Factory::$act_tables;
		} else {
			$tables = MVC_Front_Factory::$act_tables;
		}
		if ( is_array( $data ) ) {
			foreach( $data as $idx => $details ) {
				$exploded_data = explode( '.', $details );
				if ( count( $exploded_data ) > 0 && isset( $tables['pod_' . $exploded_data[0]] ) ) {
					$attrs = array(
						'target_tb' => $the_table_prefix . $tables['pod_' . $exploded_data[0]]['name'],
						'limit'     => 1,
					);
					if ( $attributes['inc_cats'] == true ) {
						$attrs['inc_cats'] = true;
					}
					if ( $attributes['inc_areas'] == true ) {
						$attrs['inc_areas'] = true;
					}
					$category_data = $this->simpods_select(
						array(
							'id' => intval( $exploded_data[1] ),
						),
						$attrs,
						0
					);
					if ( count( $category_data ) > 0 ) {
						$category_data[0]['simpods_pod_table'] = $tables['pod_' . $exploded_data[0]]['name'];
						$category_data[0]['pod_table']		 = $tables['pod_' . $exploded_data[0]];
						array_push( $categories, $category_data[0] );
					}
				}
			}
		}
		return $categories;
	}
	/**
	 * Fetch the information of a Pod field
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/pods_field_settings_fn/
	 * @param  array $attributes array( 'pods_name'  => '', 'field_name' => '' );
	 * @return mixed Empty string if Pods Framework is not available, false if not found, array if found.
	 */
	public function pods_field_settings( $attributes ) {
		if ( !defined( 'PODS_VERSION' ) ) {
			return '';
		}
		$defaults = array(
			'pods_name'  => '',
			'field_name' => '',
		);
		$attributes = wp_parse_args( $attributes, $defaults );
		$pods_api 	= new PodsAPI();
		$pod_data   = $pods_api->load_pod( array( 'name' => esc_sql( $attributes['pods_name'] ) ) );
		if ( !isset( $pod_data['fields'][esc_attr( $attributes['field_name'] )] ) ) {
			return false;
		}
		return $pod_data['fields'][esc_attr( $attributes['field_name'] )];
	}
	/**
	 * Only show the current user's own attachments within the WP media library
	 * @link https://simpodsmvc.me/docs/function-reference/functions/users_own_attachments_fn/
	 * @param  array $query_options The query array sent from the `ajax_query_attachments_args` filter
	 * @return array            Returns the array sent from the filter with the author set to the user's ID if necessary
	 */
	public function users_own_attachments( $query_options = array() ) {
		$user_id = get_current_user_id();
		if ( $user_id ) {
			$query_options['author'] = $user_id;
		}
		return $query_options;
	}
	/**
	 * Parse the variables in a link into an array.
	 * @return array
	 */
	public function parse_url() {
		$site_url = str_replace( array( 'http://' . $_SERVER['SERVER_NAME'], 'https://' . $_SERVER['SERVER_NAME'] ), '', site_url() );
		$slug    = str_replace( $site_url, '', $_SERVER['REQUEST_URI'] );
		$url_exploded     = explode( '?', $slug );
		// remove first slash
		$url_exploded[0] = ltrim( $url_exploded[0], '/\\' );
		// remove last slash
		$url_exploded[0] = rtrim( $url_exploded[0], '/\\' );
		$slugs = explode( '/', $url_exploded[0] );
		return $slugs;
	}
	/**
	 * Validates a date based on a format
	 * @link   https://stackoverflow.com/a/13194398
	 * @param  string $date   The date to be validated
	 * @param  string $format The format the date should be in
	 * @return boolean
	 */
	public function validate_date( $date, $format = 'Y-m-d H:i:s' ) {
		$dt = DateTime::createFromFormat( $format, $date );
		return $dt !== false && !array_sum( $dt->getLastErrors() );
	}
	/**
	 * Formats a date from one format to another
	 * @param  string $date       The date to be formatted
	 * @param  string $format     The format of the output
	 * @param  string $from_format The format to format from
	 * @return string
	 */
	public function format_date( $date, $format = 'Y-m-d H:i:s', $from_format = '' ) {
		if ( $from_format !== '' ) {
			$date_obj 	= DateTime::createFromFormat( $from_format, $date );
			$date 		= $date_obj->format( 'Y-m-d H:i:s' );
		}
		return date( $format, strtotime( $date ) );
	}
	/**
	 * Runs through an array of pods to create them all, returns their IDs in an array e.g. array( 'name' => 'ID' )
	 * @param  array  $pods   Array of pods to create
	 * @param  array  $remove_arr Array of items to remove on Advanced content type creation (e.g. array( 'author', 'permalink' ))
	 * @return array
	 */
	public function create_pods( $pods, $remove_arr = array() ) {
		$pod_ids = array();
		foreach ( $pods as $name_str => $pod_data ) {
			$pod_id = $this->create_pod( $name_str, $pod_data, $remove_arr );
			if ( !is_null( $pod_id ) && is_numeric( $pod_id ) ) {
				$pod_ids[$name_str] = $pod_id;
			}
		}
		return $pod_ids;
	}
	/**
	 * Creates a pod based on an array of parameters.
	 * Will delete items in Advanced content type after creation based on the sent array
	 * @param  string $name_str   The name of the pod to be created
	 * @param  array  $pod_data    The parameters of the pod
	 * @param  array  $remove_arr Array of items to remove from Advanced content type after creation
	 * @return integer|null
	 */
	public function create_pod( $name_str, $pod_data, $remove_arr = array() ) {
		$pods_api = new PodsAPI();
		if ( !$pods_api->pod_exists( array( 'name' => $name_str ) ) ) {
			if ( isset( $pod_data['create_extend'] ) && ( $pod_data['create_extend'] == 'extend' || $pod_data['create_pod_type'] == 'settings' ) ) {
				$pod_id = $pods_api->add_pod( $pod_data );
			} else {
				// Be careful with save_pod, it will create new ids, so it the repeater fields data will lose connection to parents
				$pod_id = $pods_api->save_pod( $pod_data );
			}
			if ( isset( $pod_data['type'] ) && $pod_data['type'] == 'pod' && isset( $pod_data['fields'] ) && is_array( $pod_data['fields'] ) ) {
				foreach ( $remove_arr as $item_str ) {
					$pods_api->delete_field( array(
						'pod'    => $name_str,
						'pod_id' => $pod_id,
						'name'   => $item_str,
					) );
				}
			}
			return $pod_id;
		} else {
			return $pods_api->load_pod( array( 'name' => $name_str ) )['id'];
		}
		return null;
	}
	/**
	 * Created pod fields for multiple pods based on an array
	 * @param  array $fields Array of fields to create. Grouped by pod (e.g. array( 'pod' => array( 'field1' = > array(), 'field1' = > array() ))
	 * @param  array $pods   Array of pods for reference
	 * @param  array $pod_ids Array of pod IDs for linking the field to the correct pod
	 * @return null
	 */
	public function create_fields( $fields, $pods, $pod_ids ) {
		$pods_api = new PodsAPI();
		$sucess      = true;
		foreach ( $fields as $pod_str => $pod_fields ) {
			if ( !array_key_exists( $pod_str, $pod_ids ) ) {
				continue;
			}
			$pod_data   = $pods[ $pod_str ];
			$pod_id 	= $pod_ids[ $pod_str ];
			$count 		= 1;
			foreach ( $pod_fields as  $field => $field_data ) {
				$pod_fields[ $field ]['weight'] = $count;
				$count ++;
			}
			if ( isset( $pod_data['type'] ) && $pod_data['type'] == 'pod' && isset( $pod_data['fields'] ) && is_array( $pod_data['fields'] ) ) {
				$pod_fields['created'] = array(
					'name'    => 'created',
					'label'   => esc_html__( 'Date Created', 'simpods-mvc' ),
					'type'    => 'datetime',
					'weight'  => $count,
					'options' => array(
						'datetime_type' => 'wp',
					),
				);
				$count ++;
				$pod_fields['modified'] = array(
					'name'    => 'modified',
					'label'   => esc_html__( 'Date Modified', 'simpods-mvc' ),
					'type'    => 'datetime',
					'weight'  => $count,
					'options' => array(
						'datetime_type' => 'wp',
					),
				);
			}
			foreach ( $pod_fields as $field => $field_data ) {
				$field_data['pod_id'] = $pod_id;
				$field_data['pod']    = $pod_str;
                $id = $pods_api->save_field( $field_data );
                if( ! $id ){
                    $sucess = false;             
                } else {
                	do_action('simpods_create_pod_fields', $pod_id, $id, $pod_str, $field_data );
                }
			}
		}
		return $sucess;
	}
	/**
	 * Extract the domain name from the siteurl option
	 * @return 	string the domain name of the site
	 * @link 	https://www.warriorforum.com/programming/260618-how-get-root-domain-wordpress.html
	 * @author  phpbbxpert
	 */
	public function get_domain(){
		$domain = get_option('siteurl'); //or home
		
		$domain = str_replace( array('http://','https://'), '', $domain);
		
		$domain = str_replace('www.', '', $domain ); 
		if( strpos( $domain, '/' ) !== false ){
			$domain = strstr( $domain, '/', true ); //PHP5 only, this is in case WP is not root	somehow return emtpy string
		}
			
		return $domain;
	}
	/**
	 * get the closest ratio for Bootstrap responsive iframe
	 * @param  integer $width width of the iframe
	 * @param  integer $height height of the iframe
	 * @return boolean|array if either the width or height is 0, return false, otherwise, return the closest ration
	 */
	public function get_closest_ratio( $width = 0, $height = 0 ){
		if( 0 == $width || 0 == $height ){
			return false;
		}
		$return_data	= array();
		$ratios 		= array(
							array( 'label' => 'least', 'value' 	=> 0 ),
							array( 'label' => '1by1', 'value' 	=> 1),
							array( 'label' => '4by3', 'value' 	=> 1.33),
							array( 'label' => '16by9', 'value' 	=> 1.78),
							array( 'label' => '21by9', 'value' 	=> 2.33),
							array( 'label' => 'most', 'value' 	=> 9999999999), // to keep the programming simple and ensure the loop below won't run out of scope
						  	);
		$ratio_num	= $width / $height;
		for( $i = 0; $i < count( $ratios ); $i ++ ){
			// if the ratio is inbetween two values, work out which one is closer
			if( $ratio_num > $ratios[ $i ][ 'value' ] && $ratio_num <= $ratios[ $i + 1 ][ 'value' ] ){
				$left_num 	= $ratio_num - $ratios[ $i ][ 'value' ];
				$right_num	= $ratios[ $i + 1 ][ 'value' ] - $ratio_num;
				
				if( $left_num > $right_num ){
					$return_data	=	$ratios[ $i + 1 ];
				} else {
					$return_data	=	$ratios[ $i ];
				}
				break;
			}
		}
		return $return_data;
	}	
	/**
	 * Load Simpods plugin files. This function only load registered ajax and controller files from 'simpods_component_plugins'
	 * Model files are handled by simpods_autoload.class.php unless specified. Page specific controllers are handled 
	 * by simpods_admin_fnction.php and simpods_front_functions.php for the convenience to display if a controller is located.
	 * JS, CSS are loaded by mvc_admin_parent_cross_admin->page_specific_enqueue()
	 * @param  array $attributes determine the location in the array
	 * @return null
	 */
	public function load_simpods_plugin_files( $attrs ) {
		global $wp_filesystem;
		$defaults = array(
			'where' => '', //front or admin
			'type' 	=> '', //ctrl or ajax
		);
		$attrs    = wp_parse_args( $attrs, $defaults );
		if( $attrs[ 'where' ] != '' && $attrs[ 'type' ] != '' ){
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			// load simpods component plugins
			$component_plugins = SIMPODS_COMPONENT_PLUGINS; 
			
			if( is_array( $component_plugins ) ) {
				foreach( $component_plugins as $plugin_name => $details ){
					if( is_plugin_active( $plugin_name . '/' . $plugin_name . '.php'  ) ){
						//auto-load cross plugin controller
						$admin_or_front	=	'admin/'; // admin_or_front
						if( $attrs[ 'where' ] == 'front' ) {
							$admin_or_front	=	'';
						}					
						$ctrl_or_ajax	= $attrs[ 'type' ] . '-' . $plugin_name; // controller or ajax 
						$folder_name	= 'ajax';
						if( $attrs[ 'type' ] == 'ctrl' ) {
							$ctrl_or_ajax				= $attrs[ 'type' ] . '-cross-' . $plugin_name;
							$folder_name				= 'control';
						}	
						//cross ajax and controller. e.g. ajax: simpods-booking-ajax, controller: ctrl-cross-simpods-booking.php
						$plugin_file	= WP_PLUGIN_DIR . '/' . $plugin_name . '/mvc/' . $admin_or_front . $folder_name . '/' . $ctrl_or_ajax . '.php';

						if( $wp_filesystem->exists( $plugin_file ) ){ // create instance of the cross one
							include_once( $plugin_file );	
							$cross	= str_replace( '-', '_', $ctrl_or_ajax	);
							$class	= ucwords( $cross, '_' ); // e.g. Ajax_Simpods_Booking or Ctrl_Cross_Simpods_Booking 
							if( class_exists ( $class ) ){
								if( substr( $cross, 0, 5 ) == 'ctrl_' ){
									$cross 	= substr( $cross, 5 ) . '_ctrl'; // move the ctrl_ from left to the end to keep the consistence 
								}								
								
								$reflection  = new ReflectionClass( $class );
								$class_valid = !$reflection->isAbstract() && !$reflection->isInterface() && $reflection->isInstantiable();
								if ( $class_valid ) {
									$GLOBALS[ $cross ] = New $class();	
								}												
							}							
						}	
						// include registered files
						if( isset( $details[ $attrs[ 'where' ] ] ) ){
							if( isset( $details[ $attrs[ 'where' ] ][ $attrs[ 'type' ] ] ) && is_array( $details[ $attrs[ 'where' ] ][ $attrs[ 'type' ] ] ) ){
								foreach( $details[ $attrs[ 'where' ] ][ $attrs[ 'type' ] ] as $name => $path ){
									$plugin_file	= WP_PLUGIN_DIR . '/' . $plugin_name . '/mvc/' . $admin_or_front . $folder_name . '/' . ltrim( $path, '/' );							
									if( $wp_filesystem->exists( $plugin_file ) ){										
										include_once( $plugin_file );									
									}
								}
							}
						}
					}
				
				}
				
			}
		}
	}
	
	// Helper Functions
	/**
	 * Checks whether a provided string is encoded JSON and decodes & returns as appropriate
	 *
	 * @param string  $string The string to attempt to json_decode
	 * @param boolean $assoc  Boolean for whether to return an associative array or not
	 * @return object/array/string Returns object or array if json_decode is successful, returns original passed string if not
	 */
	public function maybe_json_decode( $string, $assoc = false ) {
		if ( $this->is_json( $string ) ) {
			return json_decode( $string, $assoc );
		}
		return $string;
	}

	/**
	 * Checks an array to see if it is an encoded JSON string
	 * 
	 * @link https://subinsb.com/php-check-if-string-is-json/
	 * @param string $string The string to test
	 * @return boolean       Whether the string is JSON is not
	 */
	public function is_json( $string ) {
		return is_string( $string ) && is_array( json_decode( $string, true ) ) ? true : false;
	}
	/**
	 * Explode text by new lines and put the items into a list
	 * 
	 * @param string $text Pure text
	 * @param string $list_type ul|ol
	 */ 
	public function new_line_to_list( $text = '', $list_type = 'ul', $css_class = 'simpods-new-line-to-list' ){
		$text = trim( wp_strip_all_tags ( $text ) );
		$html = '';
		if( $text != '' ){
			$text_arr = explode( PHP_EOL, $text );
			$html = '<ul class="' . $css_class . '">';
			if( $list_type != 'ul' ){
				$html = '<ol class="' . $css_class . '">';
			}
			foreach ( $text_arr as $value ) {
				$html .= '<li>' . $value . '</li>';
			}
			$html .= '</ul >';
			if( $list_type != 'ul' ){
				$html .= '</ol >';
			}
		}	
		return $html;	
	}
	/**
	 * Produce a dynamic progress bar. You will have to do your own JavaScript
	 * 
	 */ 
	public function dynamic_progress_bar(){
		$html = '
			<div class="mg3 simpods-ajax-loader alignright">
				<img src="' . admin_url( 'images/wpspin_light.gif' ) . '"> 
			</div>							
			<div role="progressbar" aria-valuemin="0" aria-valuemax="100" aria-valuenow="0" class="simpods-progressbar">
				<div class="simpods-progressbar-value" style="">
					<div class="simpods-progressbar-percent pdt3">
					0%
					</div>	
				</div>
			</div>
			<div id="simpods-updating-message" class="aligncenter mgb20"></div>
		';
		return $html;
	}
	/**
	 * remove javascript
	 */
	public function remove_js( $content ){
		$content = preg_replace( '/<script\\b[^>]*>(.*?)<\\/script>/is', '', $content );
		return $content;
	}  
	/**
	 * check if a database table exists
	 * 
	 * @param string $table database table name
	 * @param boolean $can_prefix whether to prefix the WordPress table prefix
	 * 
	 * @return boolean, true if found, otherwise false 
	 */ 
	public function check_db_table( $table, $can_prefix = true ) {
		global $wpdb;
		$table = esc_sql( stripslashes( $table ) );
		if ( $can_prefix && stripos( $table, $wpdb->prefix ) !== 0 ) {
			$table 	= $wpdb->prefix . $table;
		}
		$fould = $wpdb->query('SELECT table_name FROM information_schema.tables WHERE table_schema = "' . DB_NAME . '" AND table_name = "' . $table . '"');

		if( ! empty( $fould ) ) {
		   return true;
		} else	{
		   return false;
		}
	}
	/**
	 * Return a random string
	 * 
	 * @param int $length how the string you want
	 * @param array $which_character_set an array with any combination 'number','lowercase','uppercase'; e.g array('number') 
	 * 									 for number only or array('number', 'uppercase') for numbers and uppercases
	 * 
	 * @return string a random string
	 */ 
	public function generate_random_string( $length = 10, $which_character_set = array('number','lowercase','uppercase') ) {
	    $character_sets = array(
	    					'number' 	=> '0123456789',
	    					'lowercase' => 'abcdefghijklmnopqrstuvwxyz',
	    					'uppercase' => 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
	    					);
	    $characters 	= '';
	    if( is_array( $which_character_set ) ) {
		    foreach( $which_character_set as $name ){
		    	if( isset( $character_sets[ $name ] ) ){
		    		$characters .= $character_sets[ $name ];
		    	}
		    }
		} else {
		    foreach( $character_sets as $name => $value ){		    	
		    	$characters .= $character_sets[ $name ];		    	
		    }			
		}
	    $characters_length 	= strlen( $characters );
	    $string 			= '';
	    for ( $i = 0; $i < $length; $i ++ ) {
	        $string .= $characters[ rand(0, $characters_length - 1 ) ];
	    }
	    return $string;
	} 

	/**
	 * Return a random regular expression
	 * 
	 * @param int $times how many chunks of regex you want
	 * @param array $max_length maximum how many characters in a chunk
	 * 
	 * @return string a random regular expression
	 */ 	
	public function generate_random_string_regex( $times = 10, $max_length = 3, $separator = '' ){
	    $sets = array(
					'[0-9]',
					'[a-z]',
					'[A-Z]',
					);		
	    if( $max_length < 1 ){
	    	$max_length = 3;
	    }
	    $regex  = '';
		for ( $i = 0; $i < $times; $i ++ ) {
			$set 	= $sets[ rand( 0, 2 ) ]; 
			$length = rand( 1, $max_length );
			$regex  .= $set . '{' . $length . '}' . $separator;
		}
		$regex = rtrim( $regex, $separator );
		return $regex;
	}  	
	/**
	 * Return a string based on a regular expression
	 * 
	 * @param string $regex a regular expression in this format [scope]{times}, e.g [A-Z]{1}[0-9]{2}[A-Z]{3}[a-z]{2}	 
	 * 
	 * @return string a string
	 */ 			
	//
	public function get_string_by_regex( $regex = '' ){
		if( empty( $regex ) ){
			return false;
		}
		$characters = '';
	    $sets = array(
					'[0-9]' => 'number',
					'[a-z]' => 'lowercase',
					'[A-Z]'	=> 'uppercase',
					);			
		$regex_lv1s = Arrays::string_to_array( trim( $regex ), '}' );
		foreach ( $regex_lv1s as $regex_lv1 ) {
			$regex_lv2s = Arrays::string_to_array( $regex_lv1, '{' ); 
			if( ! empty( $sets[ $regex_lv2s[0] ] ) && ! empty( $regex_lv2s[1] ) && is_numeric( $regex_lv2s[1] ) ){
				
				$characters .= $this->generate_random_string( intval( $regex_lv2s[1] ), array( $sets[ $regex_lv2s[0] ] ) );
			} else { 
				return false; // doesn't meet the format
			}
		}
		return $characters;
	}				
}
// _simpods_deprecated_message( 'test_function', 'You dun goofed', '2.5.1', '3.0.0' );
function _simpods_deprecated_message( $function, $message, $version, $remove_version ) {
	$display_errors = ini_get( 'display_errors' );
	$log_errors = ini_get( 'log_errors' );
	if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
		if ( defined( 'WP_DEBUG_DISPLAY' ) && WP_DEBUG_DISPLAY ) {
			ini_set( 'display_errors', 1 );
		} else {
			ini_set( 'display_errors', 0 );
		}
	}
	$error_message = sprintf( 
		esc_html( '%1$s was deprecated in version %2$s and will be removed in version %3$s. %4$s' ),
		'<strong>' . $function . '</strong>',
		'<strong>' . $version . '</strong>',
		'<strong>' . $remove_version . '</strong>',
		'<strong>' . $message . '</strong>'
	);
	trigger_error( $error_message );
	if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
		if ( defined( 'WP_DEBUG_DISPLAY' ) && WP_DEBUG_DISPLAY ) {
			ini_set( 'display_errors', $display_errors );
		}
	}
}
$GLOBALS['funs'] = new Simpods_Functions();
/**
 * global ones, instant access without class
 */
/**
 * to list array or object in formation, for debugging
 * @param  mixed ...$data data to list
 * @param  boolean use print_r or default to var_dump
 * @return null
 */
if ( !function_exists( 'print' ) ) {
	/**
	 * Print data for debugging purposes
	 * @link   https://simpodsmvc.me/docs/function-reference/functions/predump/
	 * @param  mixed   $data Data to be printed
	 * @return null
	 */
	function predump( $data ) {
		echo '<pre>';
		var_dump( $data );
		echo '</pre>';
	}
	/**
	 * Print data for debugging purposes. Use print_r only	 
	 * @param  mixed   $data Data to be printed	 
	 * @return null
	 */
	function preprint( $data ) {
		echo '<pre>';
		print_r( $data );
		echo '</pre>';
	}
}