<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * Global class for admin and frontend, mainly for action and filters for simpods components. Media uploading usually needs this
 *
 * @version: 1.1.4
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 27/01/2018
 */
class Simpods_Global {
	private $user_logged_in = false;

	//public $htaccss_private	= ''; // General htaccess for protecting files
	public function __construct() {
		global $funs;
		$doamin 		= $funs->get_domain();
		//$this->htaccss_private	= '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://www.' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^http://www.' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://www.' . $doamin . '.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteCond %{HTTP_REFERER} !^https://www.' . $doamin . ':80.*$ [NC]' . PHP_EOL . 'RewriteRule ^ /index.php [NC,L]' . PHP_EOL . '</IfModule>';
// 		<IfModule mod_rewrite.c>
// RewriteEngine On
// RewriteCond %{REQUEST_URI} !^(.+?)/uploads/cpd_public_files/([0-9]{1})-
// RewriteRule ^ /index.php [NC,L]
// </IfModule>

		$this->user_logged_in = is_user_logged_in();
		$this->actions();
		$this->filters();		
		//$this->get_simpods_patterned_code_for_url();
	}
	private function actions() {
		if ( $this->user_logged_in ) {
			add_action( 'wp_ajax_admin_simpodspodsfile_before_upload', array( $this, 'admin_simpodspodsfile_before_upload' ) );
		}
	}
	private function filters() {				

		if ( $this->user_logged_in ) {
			add_filter( 'wp_handle_upload_prefilter', array( $this, 'simpods_handle_upload_prefilter' ), 10, 1 );
			add_filter( 'wp_handle_upload', array( $this, 'simpods_handle_upload' ), 10, 1 );
		} else {
			add_filter('authenticate',  array( $this, 'single_login' ), 0, 3);
		}
	}

	/**
	 * Create the protection files and folders based on the settings from a Simpods File field for Pods.
	 * @return null
	 */
	public function admin_simpodspodsfile_before_upload() {

		global $current_user;
		if ( !isset( $_POST['action'] ) || $_POST['action'] != 'admin_simpodspodsfile_before_upload' || !isset( $current_user->ID ) || !is_user_logged_in() ) {
			return;
		}		
		if ( wp_verify_nonce( $_POST['security'], 'simpods-file-nonce' ) === false || !isset( $_POST['folder'] ) || empty( $_POST['folder'] ) ) {
			wp_die();
		}
		$dir = trim( sanitize_file_name( trim( $_POST['folder'], '/' ) ) );
		// add sub folder
		if ( isset( $_POST['sub_folder'] ) && $_POST['sub_folder'] == 1 ) {
			if ( $dir != '' ) {
				$dir .= '/';
			}
			$dir .= sanitize_file_name( $_POST['sub_folder_type'] . '-' . trim( $_POST['sub_folder_id'], '/' ) );
		}
		$return = array( 
			'feedback' => __('Creating files ajax called. ', 'simpods-mvc' ),
			'session'  => '' 
		);
		update_option( 'simpods_file_upload_folder_' . $current_user->ID, $dir );
		$pods_file = new MVC_Admin_Pods_Files( false );
		$pods_file->create_files( $dir, $_POST['protect_folder'] );
		wp_send_json( $return );
		wp_die();
	}
	/**
	 * Change the upload path based on the settings from a Simpods File field for Pods.
	 * @param  array $file Contains the details of the uploaded file
	 * @return array
	 */
	public function simpods_handle_upload_prefilter( $file_info ) {
		add_filter( 'upload_dir', array( $this, 'simpods_file_upload_dir' ), 10, 1 );
		return $file_info;
	}
	/**
	 * Remove the filter set by a Simpods File Field, so normal file uploads can perform as they should be.
	 * @param  array $file_info Contains the details of the uploaded file
	 * @return array
	 */
	public function simpods_handle_upload( $file_info ) {
		remove_filter( 'upload_dir', array( $this, 'simpods_file_upload_dir' ), 10, 1 );
		return $file_info;
	}
	/**
	 * Return the new upload path details for Simpods File Field
	 * @param  string $path The current upload path details
	 * @return string
	 */
	public function simpods_file_upload_dir( $path ) {
		global $current_user;
		if ( !is_user_logged_in() || !current_user_can( 'upload_files' ) ) {
			return $path;
		}
		$new_dir    = '/';
		$new_folder = get_option( 'simpods_file_upload_folder_' . $current_user->ID, '' );
		if ( !empty( $new_folder ) ) {
			$new_dir = '/' . $new_folder;
		} else {
			return $path;
		}
		//remove default subdir (year/month)
		$path['path']   = str_replace( $path['subdir'], '', $path['path'] ) . $new_dir;
		$path['url']    = str_replace( $path['subdir'], '', $path['url'] ) . $new_dir;
		$path['subdir'] = $new_dir;
		return $path;
	}
	/**
	 * Return the pattern data to use in url. .htaccess file has to be updated after. 
	 * Use create_file_for_patterned_code_for_url instead if you know the file folder	 
	 * 
	 * @param  boolean $instant update immediately or wait when it finds out that the option is more than 30 minutes old.
	 * @return array data of the patterned code
	 */
	public function get_simpods_patterned_code_for_url( $instant = false ){
		global $funs;
		$pattern_option = get_option( 'simpods_patterned_code_for_url', '' );
		$pattern 		= $funs->generate_random_string_regex();

		$return 		= array(
							'pattern_data' 	=> array(
												'pattern' 	=> $pattern,
												'time'		=> time(),
												),
							'new'			=> false,
							);
		if( empty( $pattern_option ) || ! is_array( $pattern_option ) ){
			$pattern_option =  array(
								'pattern' 	=> $pattern,
								'time'		=> time(),
								);
			update_option( 'simpods_patterned_code_for_url', $pattern_option );
			$return['new']	= true;	
			
		} else {
			// How many seconds to delay before updating the values
			$seconds = apply_filters( 'get_simpods_patterned_code_for_url_delay', 1800 );

			if( ( time() - $pattern_option['time'] ) > 1800 || true == $instant ){

				$pattern_option =  array(
								'pattern' 	=> $pattern,
								'time'		=> time(),
							);
				update_option( 'simpods_patterned_code_for_url', $pattern_option );		
				$return['new']	= true;	
			}
			
		}		

		$pattern_option = get_option( 'simpods_patterned_code_for_url' );
		$return['pattern_data']	= $pattern_option;	

		return $return;		
	}
	/**
	 * Creates files/directories and protect them
	 *
	 * @param string  $folder     The folder to change place items
	 * @param string  $protection  How to protect the items
	 * @param boolean $add_upload_dir whether to add upload folder directory before the folder
	 * @return null
	 */
	public function create_file_for_patterned_code_for_url( $folder, $add_upload_dir = true ) {
		global $funs;
		$done = false;
		// Install files and folders for uploading files and prevent hotlinking
		$upload_dir 	= wp_upload_dir();
		if( true === $add_upload_dir ){
			$htaccess_folder	= $upload_dir['basedir'] . '/' . $folder;
		} else {
			$htaccess_folder 	= $folder;
		}
		// Only in a sub-folder of the uploads folder, in case it blocks all files
		if( strpos( $htaccess_folder, $upload_dir['basedir'] ) === false || $folder === '' ){
			return false;
		}

		$file_params  	= array(
							'base'    => $htaccess_folder,
							'file'    => '.htaccess',
							'content' => '',			
						);

		$folders = explode( '/', $folder );
		
		$pattern_params = $this->get_simpods_patterned_code_for_url();
		$pattern_option = $pattern_params['pattern_data'];

		$new 			= false;
		if( ! empty( $pattern_params['new'] ) ){ // create file if it is a new pattern

			$new 		= true;				
			if( ! empty( $pattern_option['pattern'] ) ){ 
				$file_params['content'] = '<IfModule mod_rewrite.c>' . PHP_EOL . 'RewriteEngine On' . PHP_EOL . 'RewriteCond %{QUERY_STRING} !^code=' . $pattern_option['pattern'] . '$' .PHP_EOL . 'RewriteRule ^ /index.php [NC,L]' . PHP_EOL . '</IfModule>';
							
				global $wp_filesystem;

				if ( ! $wp_filesystem->is_dir( $htaccess_folder ) ){
					$wp_filesystem->mkdir( $htaccess_folder  );
				}
			
				if( $file_params['content'] != '' ){ 
					$base 		= str_replace( ABSPATH, $wp_filesystem->abspath(), $file_params['base'] );
					$file_path 	= trailingslashit( $base ) . $file_params['file'];

					if ( $wp_filesystem->exists( $base ) ){
						if ( $wp_filesystem->exists( $file_path ) ){
							$wp_filesystem->delete( $file_path );
						}
					} else {
						$wp_filesystem->mkdir( $base );
					}

					$done = $wp_filesystem->put_contents( $file_path, $file_params['content'], FS_CHMOD_FILE );
				}
			}
		}
		return array( 'pattern_data' => $pattern_option, 'htaccess_folder' => $htaccess_folder, 'new' => $new );
	}	

	/**
	 * The chosen user roles can only log into one device.
	 * 
	 */ 
	public function single_login( $user, $username, $password ){
		global $funs;

		if( ! empty( $funs->settings['simpods_single_login'] ) ){

			$single_login_roles = Arrays::string_to_array( $funs->settings['simpods_single_login'] );
		    $user =  get_user_by('login', $username );

		    if( isset( $user->ID ) ){

		        if( isset( $user->roles ) && is_array( $user->roles ) ) {

					$found = ! empty( array_intersect( $user->roles, $single_login_roles ) );
		            // The user's role is not required for single sign in.
		            if( ! $found ) {			            	
		                return $user;
		            }
		        }

		        // get all sessions for the logging in user
		        $sessions = WP_Session_Tokens::get_instance( $user->ID );

		        // destroy all existing logins since we'll log in soon.
		        $sessions->destroy_all();
		    }
		}
	    return $user;		
	}
}
$GLOBALS['simpods_global'] = new Simpods_Global();
