<?php
/** 
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 20/05/2015
 */
/**
 * DO NOT EDIT
 */
define( 'SIMPODS_VERSION', '3.0.0' );
/**
 * Simpodss Wordpress MVC structure
 *
 * @package Simpods MVC
 * @since 1.0
 */
define( 'SIMPODS_SLUG', 'simpods-mvc' );
define( 'SIMPODS_MVC_AS_PLUGIN', true );
// $path = get_stylesheet_directory_uri() . '';
// if ( get_template_directory() == dirname( __DIR__ ) ) {
//     $path = get_template_directory_uri() . '';
// }
$path = plugins_url() . '/simpods-mvc';
define( 'SIMPODS_URL', $path );
define( 'SIMPODS_DIR', WP_PLUGIN_DIR . '/simpods-mvc' ); 
define( 'SIMPODS_MVC_URL', trailingslashit( $path ) . 'mvc' );
define( 'SIMPODS_MVC_DIR', WP_PLUGIN_DIR . '/simpods-mvc/mvc' );

if ( get_template_directory() === get_stylesheet_directory() ) {
    define( 'CHILD_THEME_ACTIVE', false );
} else { 
    define( 'CHILD_THEME_ACTIVE', true );
} 
if( is_multisite() ){ 
    define( 'MULTISITE_ON', true );
} else {
    define( 'MULTISITE_ON', false ); 
}
//date_default_timezone_set( get_option('timezone_string') );     
/**
 * Activate WP File System. Using the $wp_filesystem object’s methods without requesting and verifying credentials will cause errors.
 * @param  string $target Path to the file or directory
 * @param  false|array $fields This is a list of the form field contents I want passed along between page views like array ('save'), or false
 * @return boolean     if errors occured, return false, else return ture and make it available
 */
function simpods_wp_filesystem( $target, $fields = false ) {
    require_once( ABSPATH . 'wp-admin/includes/file.php' );
    global $pagenow;
    // Normally you leave this an empty string and it figures it out by itself, but you can override the filesystem method here
    $method = '';
    require_once( ABSPATH . 'wp-includes/pluggable.php' );
    // okay, let's see about getting credentials. Add define( 'FS_METHOD', 'direct' ); to wp-config.php if run into problems.
    $url = wp_nonce_url( $pagenow );
    $creds = request_filesystem_credentials( $url, $method, false, false, $fields );
    if ( $creds === false ) {
        // if we get here, then we don't have credentials yet,
        // but have just produced a form for the user to fill in,
        // so stop processing for now
        // stop the normal page form from displaying
        return false;
    }
    // now we have some credentials, try to get the wp_filesystem running
    if ( !WP_Filesystem( $creds ) ) {
        // our credentials were no good, ask the user for them again
        request_filesystem_credentials( $url, $method, true, false, $fields );
        return false;
    }
    return true;
    // now you can use global $wp_filesystem; in your code
}
global $wp_filesystem;
$ready = simpods_wp_filesystem( SIMPODS_MVC_DIR .'/simpods-load-mvc.php' );
if ( $ready ) {
    global $wp_filesystem;
} else {
    exit( 'Failed to load WP file system.');
}

$global_files = array(
    'simpods-arrays',
);
foreach ( $global_files as $global_file ) {
    $global_file = trailingslashit( SIMPODS_MVC_DIR ) . $global_file . '.class.php';
    if ( $wp_filesystem->exists( $global_file ) ) {       
        include_once $global_file;
    }
}
//Backward capacity 'simpods_components_load' was used before ver 3.0.0 now merge components and plugins but set priorities
$components    = get_option( 'simpods_components_load', array() );

define( 'SIMPODS_COMPONENTS', $components );

$component_plugins = get_option( 'simpods_component_plugins', array() ) ;
if( MULTISITE_ON == true ){   
    $component_plugins = wp_parse_args( $component_plugins, get_blog_option( 1, 'simpods_component_plugins_multisite', array() ) );
}

$component_plugins  = Arrays::sort_2d_associated( $component_plugins, 'priority' );
define( 'SIMPODS_COMPONENT_PLUGINS', $component_plugins );

function simpods_load_translations() {
    // Simpods core
    load_theme_textdomain( 'simpods-mvc', SIMPODS_MVC_DIR . '/languages' );
    $component_load = SIMPODS_COMPONENTS; //get_option( 'simpods_components_load' );
    if ( is_array( $component_load ) && array_key_exists( 'lang', $component_load ) && !empty( $component_load['lang'] ) ) {
        foreach ( $component_load['lang'] as $domain => $path ) {
            // Try child theme first, then fallback to parent
            if ( !load_theme_textdomain( $domain, get_stylesheet_directory() . $path ) ) {
                load_theme_textdomain( $domain, get_template_directory() . $path );
            }
        }
    }
}
add_action( 'after_setup_theme', 'simpods_load_translations' );
function simpods_init_redirect() {
    wp_redirect( admin_url( 'admin.php?page=simpods-mvc' ) );
}
add_action( 'after_switch_theme', 'simpods_init_redirect' );
//setup the enviroment. Sometimes we need files for both fronted and admin, like the registration of widgets, therefore we prepare them here. Not designed to be overwritten
$global_files = array(    
    'simpods-functions',
    'simpods-setup',
    'simpods-security',
    'simpods-global',
    'simpods-ajax',
    'simpods-email',
    'simpods-forms',
//    'simpods_scss',
    'simpods-controllers',
    'simpods-autoload'
);
foreach ( $global_files as $global_file ) {
    $global_file = trailingslashit( SIMPODS_MVC_DIR ) . $global_file . '.class.php';
    if ( $wp_filesystem->exists( $global_file ) ) {       
        include_once $global_file;
    }
}
// Required for WP-CLI
global $funs;
//files can be overwriten by child theme
$custom_globals = array(    
    'custom_global'  => '/mvc/custom-global.class.php',
    'custom_ajax'    => '/mvc/custom-ajax.class.php',
);
// if users have their own custom global models, create a variable named $custom_global_models in the same way like $files above
if ( isset( $custom_global_models ) && is_array( $custom_global_models ) ) {
    $custom_globals = array_merge( $custom_globals, $custom_global_models );
}
$is_child_first = true;
$funs->include( $custom_globals, $is_child_first );
if( class_exists( 'Custom_Global' ) ){
    $GLOBALS[ 'custom_global'] = new Custom_Global();
}
if( class_exists( 'Custom_Ajax' ) ){
    new Custom_Ajax();
}

/**
 * global variables
 */
$ctrl       = '';
$blog_ctrl  = '';
// template ajax file
include_once 'simpods-ajax-functions.php';
if ( is_admin() ) {
    // if not frontend ajax
    if ( !$funs->is_frontend_ajax() ) {
        include_once 'simpods-admin-functions.php';
    }
} else {
    // if include the file by an action, variables defined in the controls can't be passed down to the templates
    include_once 'simpods-frontend-functions.php';
}
