<?php

if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}

/**
 * Security
 *
 * @version: 1.1.4
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 30/09/2014
 */
class Simpods_Security {

	public function __construct() {
		$this->actions();
		$this->filters();
	}

	protected function actions() {
		global $funs;
		$simpods_settings = $funs->settings;

		if ( empty( $simpods_settings ) || $simpods_settings['simpods_disable_feeds'] ) {
			add_action( 'do_feed', array( $this, 'disable_feed' ), 1 );
			add_action( 'do_feed_rdf', array( $this, 'disable_feed' ), 1 );
			add_action( 'do_feed_rss', array( $this, 'disable_feed' ), 1 );
			// add_action('do_feed_rss2', array( $this, 'disable_feed'), 1);
			add_action( 'do_feed_atom', array( $this, 'disable_feed' ), 1 );
			add_action( 'do_feed_rss2_comments', array( $this, 'disable_feed' ), 1 );
			add_action( 'do_feed_atom_comments', array( $this, 'disable_feed' ), 1 );
		}

	}

	protected function filters() {
		global $funs;
		$simpods_settings = $funs->settings;

		if ( empty( $simpods_settings ) || $simpods_settings['simpods_login_errors'] ) {
			add_filter( 'login_errors', array( $this, 'login_errors' ) );
		}

		// remove wp version meta tag and from rss feed

		if ( empty( $simpods_settings ) || $simpods_settings['simpods_wp_version_meta'] ) {
			add_filter( 'the_generator', '__return_false' );
		}
		if( ! is_admin() ){ // remove at admin will cause js errors.
			if ( empty( $simpods_settings ) || $simpods_settings['simpods_scripts_versions'] ) {
				add_filter( 'script_loader_src', array( $this, 'remove_script_ver' ), 9999 );
				
			}

			if ( empty( $simpods_settings ) || $simpods_settings['simpods_styles_versions'] ) {
				add_filter( 'style_loader_src', array( $this, 'remove_script_ver' ), 9999 );
			}
		}

	}

	/**
	 * Change the login error message
	 * @return string
	 */
	public function login_errors() {
		return esc_html__( 'Invalid username or password.', 'simpods-mvc' );
	}
	/**
	 * Remove the WordPress version parameter from any enqueued scripts or styles
	 * @param  string $src The source of the script/style
	 * @return string
	 */
	public function remove_script_ver( $src ) {

		if ( strpos( $src, 'ver=' ) ) {
			$src = remove_query_arg( 'ver', $src );
		}

		return $src;
	}

	/**
	 * Remove the feed system for RSS
	 * @return null
	 */
	public function disable_feed() {
		/* translators: %1$s: <a> tag homepage, %2$s closing </a> tag */
		wp_die( sprintf( esc_html__( 'No feed available, please visit our %1$shomepage%2$s', 'simpods-mvc' ), ' <a href="' . home_url() . '">', '</a>' ) );
	}

}

new Simpods_Security();
