<?php
if ( preg_match( '#' . basename( __FILE__ ) . '#', $_SERVER['PHP_SELF'] ) ) {exit( 'You are not allowed to call this page directly.' );}
/**
 * MVC Setup
 *
 * @version: 1.1.5
 * @package: Simpods MVC
 * @author: Dongjie Xu
 * @since: 05/08/2014
 */
class Simpods_Setup {
	public $defaults = array(
		'simpods_settings' => array(
			'simpods_disable_feeds' => 1,
			'simpods_login_errors' => 1,
			'simpods_wp_version_meta' => 1,
			'simpods_scripts_versions' => 1,
			'simpods_styles_versions' => 1,
			'simpods_remove_dashboard_meta' => 1,
			'simpods_feed_links' => 0,
			'simpods_feed_links_extra' => 0,
			'simpods_rsd_link' => 0,
			'simpods_wlwmanifest_link' => 0,
			'simpods_posts_rel_links' => 0,
			'simpods_xhtml_generator' => 0,
			'style_include_bootstrap' => 'bs4',
			'style_include_fontawesome' => 'fa5-js',
			'style_include_match_height' => 1,
			'style_compile_scss' => 0,
			'style_scss_compile_frequency' => 'watch',
			'style_scss_compile_type' => 'Expanded',
			'style_defer_scripts' => 0,
			'style_load_min' => 0,
			'simpods_enable_sorting'	=> '',
			'simpods_single_login'	=> '',
		),
	);
	public function __construct() { 
		$this->parse_settings();
		$this->actions();		
		$this->register_nav();
		$this->optimise_wp();		
	}
	private function parse_settings() {
		global $funs;
		// $defaults = call_user_func_array( 'array_merge', $this->defaults );
		$funs->settings = wp_parse_args( $funs->settings, $this->defaults['simpods_settings'] );
	}
	private function actions() {
		add_action( 'widgets_init', array( $this, 'widgets_init' ) );
		add_action( 'admin_bar_menu', array( $this, 'add_toolbar_items' ), 100 );
		//add_action( 'after_setup_theme', array( $this, 'theme_support' ) );
	}
	/**
	 * Registers the default sidebar and footer widget areas
	 * @return null
	 */
	public function widgets_init() {
		// Sidebars
		register_sidebar( array(
			'name'          => esc_html__( 'Simpods Sidebar Primary', 'simpods-mvc' ),
			'id'            => 'simpods-sidebar-primary',
			'before_widget' => '<section class="simpods-widget widget %1$s %2$s"><div class="widget-inner">',
			'after_widget'  => '</div></section>',
			'before_title'  => '<h3>',
			'after_title'   => '</h3>',
		) );
		register_sidebar( array(
			'name'          => esc_html__( 'Simpods Sidebar Footer', 'simpods-mvc' ),
			'id'            => 'simpods-sidebar-footer',
			'before_widget' => '<section class="simpods-widget widget %1$s %2$s"><div class="widget-inner">',
			'after_widget'  => '</div></section>',
			'before_title'  => '<h3>',
			'after_title'   => '</h3>',
		) );
		// Widgets
		register_sidebar( array(
			'name'          => esc_html__( 'Simpods Sidebar Footer 1', 'simpods-mvc' ),
			'id'            => 'simpods-sidebar-footer-1',
			'before_widget' => '<section class="simpods-widget widget %1$s %2$s"><div class="widget-inner">',
			'after_widget'  => '</div></section>',
			'before_title'  => '<h3>',
			'after_title'   => '</h3>',
		) );
		register_sidebar( array(
			'name'          => esc_html__( 'Simpods Sidebar Footer 2', 'simpods-mvc' ),
			'id'            => 'simpods-sidebar-footer-2',
			'before_widget' => '<section class="simpods-widget widget %1$s %2$s"><div class="widget-inner">',
			'after_widget'  => '</div></section>',
			'before_title'  => '<h3>',
			'after_title'   => '</h3>',
		) );
		register_sidebar( array(
			'name'          => esc_html__( 'Simpods Sidebar Footer 3', 'simpods-mvc' ),
			'id'            => 'simpods-sidebar-footer-3',
			'before_widget' => '<section class="simpods-widget widget %1$s %2$s"><div class="widget-inner">',
			'after_widget'  => '</div></section>',
			'before_title'  => '<h3>',
			'after_title'   => '</h3>',
		) );
		register_sidebar( array(
			'name'          => esc_html__( 'Simpods Sidebar Footer 4', 'simpods-mvc' ),
			'id'            => 'simpods-sidebar-footer-4',
			'before_widget' => '<section class="simpods-widget widget %1$s %2$s"><div class="widget-inner">',
			'after_widget'  => '</div></section>',
			'before_title'  => '<h3>',
			'after_title'   => '</h3>',
		) );
	}
	/**
	 * Add the Simpods MVC link to the admin bar
	 * @param object $admin_bar The admin bar object from WordPress
	 */
	public function add_toolbar_items( $admin_bar ) {
		$simpods_capabilities = get_option( 'simpods_caps', array() );
		$add_item = false;
		foreach ( $simpods_capabilities as $cap_str ) {
			if ( current_user_can( $cap_str ) ) {
				$add_item = true;
			}
		}
		if ( $add_item && current_user_can( 'edit_theme_options' ) ) {
			$admin_bar->add_menu( array(
				'id'    => 'simpods-mvc',
				'title' => '<span class="sim-logo"></span><span style="margin-left:5px;">' . esc_html__( 'Simpods MVC', 'simpods-mvc' ) . '</span>',
				'href'  => admin_url( 'admin.php?page=simpods-mvc' ),
				'meta'  => array(
					'title' => esc_html__( 'Simpods MVC', 'simpods-mvc' ),
				),
			) );
		}
	}
	/**
	 * Setup WP theme support options
	 *
	 * @return void
	 */
	public function theme_support() {
		global $funs;
		add_theme_support( 'post-thumbnails' );
		add_theme_support( 'custom-header' );
		add_theme_support( 'custom-background' );
		add_theme_support( 'title-tag' );
		add_theme_support( 'automatic-feed-links' );
		$args = array(
			'width'         => 1200,
			'height'        => 900,
			'default-image' => $funs->get_final_path( 'mvc/images/', 'header.png', true, false, true, false ),
		);
		add_theme_support( 'custom-header', $args );
	}
	/**
	 * Registers the default navigation menus
	 * @return null
	 */
	private function register_nav() {
		register_nav_menu( 'primary', esc_html__( 'Primary Menu', 'simpods-mvc' ) );
		register_nav_menu( 'top', esc_html__( 'Top Menu', 'simpods-mvc' ) );
		register_nav_menu( 'second', esc_html__( 'Secondary Menu', 'simpods-mvc' ) );
		register_nav_menu( 'third', esc_html__( 'Third Menu', 'simpods-mvc' ) );
		register_nav_menu( 'before_signin', esc_html__( 'Before Sign In', 'simpods-mvc' ) );
	}
	/**
	 * Optimisation functionality for WordPress. Links in with the Simpods Settings menu
	 * @return null
	 */
	private function optimise_wp() {
		global $funs;
		$simpods_settings = $funs->settings;
		if ( empty( $simpods_settings ) || !$simpods_settings['simpods_feed_links'] ) {
			// Display the links to the general feeds: Post and Comment Feed
			remove_action( 'wp_head', 'feed_links', 2 );
		}
		if ( empty( $simpods_settings ) || !$simpods_settings['simpods_feed_links_extra'] ) {
			// Display the links to the extra feeds such as category feeds
			remove_action( 'wp_head', 'feed_links_extra', 3 );
		}
		if ( empty( $simpods_settings ) || !$simpods_settings['simpods_rsd_link'] ) {
			// Display the link to the Really Simple Discovery service endpoint, EditURI link
			remove_action( 'wp_head', 'rsd_link' );
		}
		if ( empty( $simpods_settings ) || !$simpods_settings['simpods_wlwmanifest_link'] ) {
			// Display the link to the Windows Live Writer manifest file.
			remove_action( 'wp_head', 'wlwmanifest_link' );
		}
		if ( empty( $simpods_settings ) || !$simpods_settings['simpods_posts_rel_links'] ) {
			// index link
			remove_action( 'wp_head', 'index_rel_link' );
			// prev link
			remove_action( 'wp_head', 'parent_post_rel_link', 10, 0 );
			// start link
			remove_action( 'wp_head', 'start_post_rel_link', 10, 0 );
			// Display relational links for the posts adjacent to the current post.
			remove_action( 'wp_head', 'adjacent_posts_rel_link_wp_head', 10, 0 );
		}
		if ( empty( $simpods_settings ) || !$simpods_settings['simpods_xhtml_generator'] ) {
			// Display the XHTML generator that is generated on the wp_head hook, WP version
			remove_action( 'wp_head', 'wp_generator' );
		}
		if ( empty( $simpods_settings ) || $simpods_settings['simpods_remove_dashboard_meta'] ) {
			// Remove unnecessary items from the dashboard
			add_action( 'admin_init', array( $this, 'remove_dashboard_meta' ) );
		}
	}
	/**
	 * Removes meta boxes from the admin dashboard
	 * @return null
	 */
	public function remove_dashboard_meta() {
		remove_meta_box( 'dashboard_incoming_links', 'dashboard', 'normal' );
		remove_meta_box( 'dashboard_plugins', 'dashboard', 'normal' );
		remove_meta_box( 'dashboard_primary', 'dashboard', 'normal' );
		remove_meta_box( 'dashboard_secondary', 'dashboard', 'normal' );
		remove_meta_box( 'dashboard_quick_press', 'dashboard', 'side' );
		remove_meta_box( 'dashboard_recent_drafts', 'dashboard', 'side' );
		remove_meta_box( 'dashboard_recent_comments', 'dashboard', 'normal' );
		remove_meta_box( 'dashboard_right_now', 'dashboard', 'normal' );
	}
}
$GLOBALS['simpods_setup'] = new Simpods_Setup();
